
/*****************************************************************************
 * FLT: An OpenFlight file loader
 *
 * Copyright (C) 2001  Michael M. Morrison   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * See the file EXCLUSIONS for further details.
 *
 *****************************************************************************/

/*
 * OpenFlight Parser
 * 
 * Author: Mike Morrison
 *         mmorrison@kqcorp.com
 */

#include "flt.h"

FltRecordEntryName FltNodeEntryName[] = {
	{	"uint32", "type", offsetof( FltNode *, type ) },
	{	"uint32", "length", offsetof( FltNode *, length ) },
	{	"uint32", "treeDepth", offsetof( FltNode *, treeDepth ) },
	{	"FltNode *", "parent", offsetof( FltNode *, parent  ) },
	{	"FltNode *", "prev", offsetof( FltNode *, prev  ) },
	{	"FltNode *", "next", offsetof( FltNode *, next  ) },
	{	"FltNode **", "child", offsetof( FltNode *, child  ) },
	{	"uint32", "numChildren", offsetof( FltNode *, numChildren ) },
	//{	"uint8 [0]", "data", offsetof( FltNode *, data ) },
	{ 0, 0, 0 }
};

FltRecordEntryName FltTextureEntryName[] = {
	{ "FltNode",  "node", offsetof( FltTexture *, node ) },
	{ "char [200]", "ID", offsetof( FltTexture *, ID ) }, 
	{ "uint32", "index", offsetof( FltTexture *, index ) },
	{ "uint32", "xloc", offsetof( FltTexture *, xloc ) },
	{ "uint32", "yloc", offsetof( FltTexture *, yloc ) },
	{ 0, 0, 0 }
};

FltRecordEntryName FltBSPEntryName[] = {
	{ "FltNode",  "node", offsetof( FltBSP *, node ) },
	{ "char [8]", "ID", offsetof( FltBSP *, ID ) }, 
	{ "uint32", "reserved0", offsetof( FltBSP *, reserved0 ) },
	{ "real64", "coefA", offsetof( FltBSP *, coefA ) },
	{ "real64", "coefB", offsetof( FltBSP *, coefB ) },
	{ "real64", "coefC", offsetof( FltBSP *, coefC ) },
	{ "real64", "coefD", offsetof( FltBSP *, coefD ) },
	{ 0, 0, 0 }
};

FltRecordEntryName FltInstanceDefinitionEntryName[] = {
	{ "FltNode",  "node", offsetof( FltInstanceDefinition *, node ) },
	{ "uint16", "reserved0", offsetof( FltInstanceDefinition *, reserved0 ) }, 
	{ "uint16", "instance", offsetof( FltInstanceDefinition *, instance ) }, 
	{ 0, 0, 0 }
};

FltRecordEntryName FltInstanceReferenceEntryName[] = {
	{ "FltNode",  "node", offsetof( FltInstanceReference *, node ) },
	{ "uint16", "reserved0", offsetof( FltInstanceDefinition *, reserved0 ) }, 
	{ "uint16", "instance", offsetof( FltInstanceDefinition *, instance ) }, 
	{ 0, 0, 0 }
};

FltRecordEntryName FltCommentEntryName[] = {
	{ "FltNode",  "node", offsetof( FltComment *, node ) },
	{ "char [256]", "text", offsetof( FltComment *, text ) }, 
	{ 0, 0, 0 }
};

FltRecordEntryName FltLongIDEntryName[] = {
	{ "FltNode",  "node", offsetof( FltLongID *, node ) },
	{ "char [64]", "text", offsetof( FltLongID *, text ) }, 
	{ 0, 0, 0 }
};

FltRecordEntryName FltMatrixEntryName[] = {
	{ "FltNode",  "node", offsetof( FltMatrix *, node ) },
	{ "real32 [16]", "matrix", offsetof( FltMatrix *, matrix ) }, 
	{ 0, 0, 0 }
};

FltRecordEntryName FltExternalReferenceEntryName[] = {
	{ "FltNode",  "node", offsetof( FltExternalReference *, node ) },
	{ "char [200]", "path", offsetof( FltExternalReference *, path ) }, 
	{ "uint8", "reserved0", offsetof( FltExternalReference *, reserved0 ) }, 
	{ "uint8", "reserved1", offsetof( FltExternalReference *, reserved1 ) }, 
	{ "uint16", "reserved2", offsetof( FltExternalReference *, reserved2 ) }, 
	{ "uint32", "flags", offsetof( FltExternalReference *, flags ) }, 
	{ "uint16", "reserved3", offsetof( FltExternalReference *, reserved3 ) }, 
	{ 0, 0, 0 }
};

FltRecordEntryName FltColorPaletteEntryName[] = {
	{ "FltNode", "node", offsetof( FltColorPalette *, node ) },
	{ "uint32 [1024]", "color", offsetof( FltColorPalette *, color ) },
	{ 0, 0, 0 }
};

FltRecordEntryName FltMaterialEntryName[] = {
	{ "FltNode", "node", offsetof( FltMaterial *, node ) },
	{ "char [12]", "ID", offsetof( FltMaterial *, ID ) },
	{ "uint32", "index", offsetof( FltMaterial *, index ) },
	{ "uint32", "flags", offsetof( FltMaterial *, flags ) },
	{ "real32", "ambientRed", offsetof( FltMaterial *, ambientRed ) },
	{ "real32", "ambientGreen", offsetof( FltMaterial *, ambientGreen ) },
	{ "real32", "ambientBlue", offsetof( FltMaterial *, ambientBlue ) },
	{ "real32", "diffuseRed", offsetof( FltMaterial *, diffuseRed ) },
	{ "real32", "diffuseGreen", offsetof( FltMaterial *, diffuseGreen ) },
	{ "real32", "diffuseBlue", offsetof( FltMaterial *, diffuseBlue ) },
	{ "real32", "specularRed", offsetof( FltMaterial *, specularRed ) },
	{ "real32", "specularGreen", offsetof( FltMaterial *, specularGreen ) },
	{ "real32", "specularBlue", offsetof( FltMaterial *, specularBlue ) },
	{ "real32", "emissiveRed", offsetof( FltMaterial *, emissiveRed ) },
	{ "real32", "emissiveGreen", offsetof( FltMaterial *, emissiveGreen ) },
	{ "real32", "emissiveBlue", offsetof( FltMaterial *, emissiveBlue ) },
	{ "real32", "shininess", offsetof( FltMaterial *, shininess ) },
	{ "real32", "alpha", offsetof( FltMaterial *, alpha ) },
	{ "uint32", "spare", offsetof( FltMaterial *, spare ) },
	{ 0, 0, 0 }
};

FltRecordEntryName FltMaterialPaletteEntryName[] = {
	{ "FltNode", "node", offsetof( FltMaterialPalette *, node ) },
	{ "FltMaterial **", "material", offsetof( FltMaterialPalette *, material ) },
	{ "uint32", "numMaterials", offsetof( FltMaterialPalette *, numMaterials ) },
	{ 0, 0, 0 }
};

FltRecordEntryName FltHeaderEntryName[] = {
	{ "FltNode", "node", offsetof( FltHeader *, node) },
	{ "char [8]",	"ID", offsetof( FltHeader *, ID ) },
	{ "uint32",	"formatRevision", offsetof( FltHeader *, formatRevision ) },
	{ "uint32",	"editRevision", offsetof( FltHeader *, editRevision ) },
	{ "char [32]", "dateTime", offsetof( FltHeader *, dateTime ) },
	{	"uint16",	"nextGroupNodeID", offsetof( FltHeader *, nextGroupNodeID ) },
	{	"uint16",	"nextLODNodeID", offsetof( FltHeader *, nextLODNodeID ) },
 { "uint16",	"nextObjectNodeID", offsetof( FltHeader *, nextObjectNodeID ) },
 { "uint16", "nextFaceNodeID", offsetof( FltHeader *, nextFaceNodeID ) },
 { "uint16",	"unitMultiplier", offsetof( FltHeader *, unitMultiplier ) },
 { "uint8",		"coordUnits", offsetof( FltHeader *, coordUnits ) },
 { "uint8",		"setTexWhiteOnNewFaces", offsetof( FltHeader *, setTexWhiteOnNewFaces ) },
 { "uint32",	"flags", offsetof( FltHeader *, flags ) },
 { "uint32 [6]",	"reserved0", offsetof( FltHeader *, reserved0 ) },
 { "uint32",	"projectionType", offsetof( FltHeader *, projectionType ) },
 { "uint32 [7]",	"reserved1", offsetof( FltHeader *, reserved1 ) },
 { "uint16",	"nextDOFNodeID", offsetof( FltHeader *, nextDOFNodeID ) },
 { "uint16",	"vertexStorageLength", offsetof( FltHeader *, vertexStorageLength ) },
 { "uint32",	"databaseOrigin", offsetof( FltHeader *, databaseOrigin ) },
 { "real64",	"swDBCoordX", offsetof( FltHeader *, swDBCoordX ) },
 { "real64",	"swDBCoordY", offsetof( FltHeader *, swDBCoordY ) },
 { "real64",	"deltaXPlace", offsetof( FltHeader *, deltaXPlace ) },
 { "real64",	"deltaYPlace", offsetof( FltHeader *, deltaYPlace ) },
 { "uint16",	"nextSoundNodeID", offsetof( FltHeader *, nextSoundNodeID ) },
 { "uint16",	"nextPathNodeID", offsetof( FltHeader *, nextPathNodeID ) },
 { "uint32 [2]",	"reserved2", offsetof( FltHeader *, reserved2 ) },
 { "uint16",	"nextClipNodeID", offsetof( FltHeader *, nextClipNodeID ) },
 { "uint16",	"nextTextNodeID", offsetof( FltHeader *, nextTextNodeID ) },
 { "uint16",	"nextBSPNodeID", offsetof( FltHeader *, nextBSPNodeID ) },
 { "uint16",	"nextSwitchNodeID", offsetof( FltHeader *, nextSwitchNodeID ) },
 { "uint32",	"reserved3", offsetof( FltHeader *, reserved3 ) },
 { "real64",	"swDBLatitude", offsetof( FltHeader *, swDBLatitude ) },
 { "real64",	"swDBLongitude", offsetof( FltHeader *, swDBLongitude ) },
 { "real64",	"neDBLatitude", offsetof( FltHeader *, neDBLatitude ) },
 { "real64",	"neDBLongitude", offsetof( FltHeader *, neDBLongitude ) },
 { "real64",	"originDBLatitude", offsetof( FltHeader *, originDBLatitude ) },
 { "real64",	"originDBLongitude", offsetof( FltHeader *, originDBLongitude ) },
 { "real64",	"lambertUpperLatitude", offsetof( FltHeader *, lambertUpperLatitude ) },
 { "real64",	"lambertLowerLatitude", offsetof( FltHeader *, lambertLowerLatitude ) },
 { "uint16",	"nextLightSourceNodeID", offsetof( FltHeader *, nextLightSourceNodeID ) },
 { "uint16",	"nextLightPointNodeID", offsetof( FltHeader *, nextLightPointNodeID ) },
 { "uint16",	"nextRoadNodeID", offsetof( FltHeader *, nextRoadNodeID ) },
 { "uint16",	"nextCATNodeID", offsetof( FltHeader *, nextCATNodeID ) },
 { "uint16",	"reserved4", offsetof( FltHeader *, reserved4 ) },
 { "uint16",	"reserved5", offsetof( FltHeader *, reserved5 ) },
 { "uint16",	"reserved6", offsetof( FltHeader *, reserved6 ) },
 { "uint16",	"reserved7", offsetof( FltHeader *, reserved7 ) },
 { "uint32",	"earthEllipsoidModel", offsetof( FltHeader *, earthEllipsoidModel ) },
 { "uint16",	"nextAdaptiveNodeID", offsetof( FltHeader *, nextAdaptiveNodeID ) },
 { "uint16",	"nextCurveNodeID", offsetof( FltHeader *, nextCurveNodeID ) },
 { "uint16",	"reserved8", offsetof( FltHeader *, reserved8 ) },
 { "real64",	"deltaZPlace", offsetof( FltHeader *, deltaZPlace ) },
 { "real64",	"dbRadius", offsetof( FltHeader *, dbRadius ) },
 { "uint16",	"nextMeshNodeID", offsetof( FltHeader *, nextMeshNodeID ) },
 { "uint16",	"reserved9", offsetof( FltHeader *, reserved9 ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltVertexEntryName[] = {
	{	"FltNode", "node", offsetof( FltVertex *, node ) },
	{	"real64", "x", offsetof( FltVertex *, x ) },
	{	"real64", "y", offsetof( FltVertex *, y ) },
	{	"real64", "z", offsetof( FltVertex *, z ) },
	{	"real32", "i", offsetof( FltVertex *, i ) },
	{	"real32", "j", offsetof( FltVertex *, j ) },
	{	"real32", "k", offsetof( FltVertex *, k ) },
	{	"real32", "u", offsetof( FltVertex *, u ) },
	{	"real32", "v", offsetof( FltVertex *, v ) },
	{	"FltVertexFlags", "localFlags", offsetof( FltVertex *, localFlags ) },
	{	"uint16", "colorNameIndex", offsetof( FltVertex *, colorNameIndex ) },
	{	"uint16", "flags", offsetof( FltVertex *, flags ) },
	{	"uint32", "packedColor", offsetof( FltVertex *, packedColor ) },
	{	"uint32", "colorIndex", offsetof( FltVertex *, colorIndex ) },
	{	"uint32", "indx", offsetof( FltVertex *, indx ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltSwitchEntryName[] = {
	{	"FltNode", "node", offsetof( FltSwitch *, node ) },
	{	"char [8]", "ID", offsetof( FltSwitch *, ID ) },
	{	"uint32", "reserved0", offsetof( FltSwitch *, reserved0 ) },
	{	"uint32", "currentMask", offsetof( FltSwitch *, currentMask ) },
	{	"uint32", "numUInt32sPerMask", offsetof( FltSwitch *, numUInt32sPerMask ) },
	{	"uint32", "numMasks", offsetof( FltSwitch *, numMasks ) },
	{	"uint32 *", "masks", offsetof( FltSwitch *, masks  ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltDOFEntryName[] = {
	{	"FltNode", "node", offsetof( FltDOF *, node ) },
	{	"char [8]", "ID", offsetof( FltDOF *, ID ) },
	{	"uint32", "reserved0", offsetof( FltDOF *, reserved0 ) },

	{	"real64", "localOriginX", offsetof( FltDOF *, localOriginX ) },
	{	"real64", "localOriginY", offsetof( FltDOF *, localOriginY ) },
	{	"real64", "localOriginZ", offsetof( FltDOF *, localOriginZ ) },

	{	"real64", "localPointX", offsetof( FltDOF *, localPointX ) },
	{	"real64", "localPointY", offsetof( FltDOF *, localPointY ) },
	{	"real64", "localPointZ", offsetof( FltDOF *, localPointZ ) },

	{	"real64", "localPlanePointX", offsetof( FltDOF *, localPlanePointX ) },
	{	"real64", "localPlanePointY", offsetof( FltDOF *, localPlanePointY ) },
	{	"real64", "localPlanePointZ", offsetof( FltDOF *, localPlanePointZ ) },

	{	"real64", "localMinZ", offsetof( FltDOF *, localMinZ ) },
	{	"real64", "localMaxZ", offsetof( FltDOF *, localMaxZ ) },
	{	"real64", "localCurZ", offsetof( FltDOF *, localCurZ ) },
	{	"real64", "localIncZ", offsetof( FltDOF *, localIncZ ) },

	{	"real64", "localMinY", offsetof( FltDOF *, localMinY ) },
	{	"real64", "localMaxY", offsetof( FltDOF *, localMaxY ) },
	{	"real64", "localCurY", offsetof( FltDOF *, localCurY ) },
	{	"real64", "localIncY", offsetof( FltDOF *, localIncY ) },

	{	"real64", "localMinX", offsetof( FltDOF *, localMinX ) },
	{	"real64", "localMaxX", offsetof( FltDOF *, localMaxX ) },
	{	"real64", "localCurX", offsetof( FltDOF *, localCurX ) },
	{	"real64", "localIncX", offsetof( FltDOF *, localIncX ) },

	{	"real64", "localMinPitch;		// about ", offsetof( FltDOF *, localMinPitch ) },
	{	"real64", "localMaxPitch", offsetof( FltDOF *, localMaxPitch ) },
	{	"real64", "localCurPitch", offsetof( FltDOF *, localCurPitch ) },
	{	"real64", "localIncPitch", offsetof( FltDOF *, localIncPitch ) },

	{	"real64", "localMinRoll;		// about ", offsetof( FltDOF *, localMinRoll ) },
	{	"real64", "localMaxRoll", offsetof( FltDOF *, localMaxRoll ) },
	{	"real64", "localCurRoll", offsetof( FltDOF *, localCurRoll ) },
	{	"real64", "localIncRoll", offsetof( FltDOF *, localIncRoll ) },

	{	"real64", "localMinYaw;		// about ", offsetof( FltDOF *, localMinYaw ) },
	{	"real64", "localMaxYaw", offsetof( FltDOF *, localMaxYaw ) },
	{	"real64", "localCurYaw", offsetof( FltDOF *, localCurYaw ) },
	{	"real64", "localIncYaw", offsetof( FltDOF *, localIncYaw ) },

	{	"real64", "localMinScaleZ", offsetof( FltDOF *, localMinScaleZ ) },
	{	"real64", "localMaxScaleZ", offsetof( FltDOF *, localMaxScaleZ ) },
	{	"real64", "localCurScaleZ", offsetof( FltDOF *, localCurScaleZ ) },
	{	"real64", "localIncScaleZ", offsetof( FltDOF *, localIncScaleZ ) },

	{	"real64", "localMinScaleY", offsetof( FltDOF *, localMinScaleY ) },
	{	"real64", "localMaxScaleY", offsetof( FltDOF *, localMaxScaleY ) },
	{	"real64", "localCurScaleY", offsetof( FltDOF *, localCurScaleY ) },
	{	"real64", "localIncScaleY", offsetof( FltDOF *, localIncScaleY ) },

	{	"real64", "localMinScaleX", offsetof( FltDOF *, localMinScaleX ) },
	{	"real64", "localMaxScaleX", offsetof( FltDOF *, localMaxScaleX ) },
	{	"real64", "localCurScaleX", offsetof( FltDOF *, localCurScaleX ) },
	{	"real64", "localIncScaleX", offsetof( FltDOF *, localIncScaleX ) },

	{	"uint32", "flags", offsetof( FltDOF *, flags ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltLODEntryName[] = {
	{	"FltNode", "node", offsetof( FltLOD *, node ) },
	{	"char [8]", "ID", offsetof( FltLOD *, ID ) },
	{	"uint32", "reserved0", offsetof( FltLOD *, reserved0 ) },
	{	"real64", "switchInDistance", offsetof( FltLOD *, switchInDistance ) },
	{	"real64", "switchOutDistance", offsetof( FltLOD *, switchOutDistance ) },
	{	"uint16", "specialEffectID1", offsetof( FltLOD *, specialEffectID1 ) },
	{	"uint16", "specialEffectID2", offsetof( FltLOD *, specialEffectID2 ) },
	{	"uint32", "flags", offsetof( FltLOD *, flags ) },
	{	"real64", "centerX", offsetof( FltLOD *, centerX ) },
	{	"real64", "centerY", offsetof( FltLOD *, centerY ) },
	{	"real64", "centerZ", offsetof( FltLOD *, centerZ ) },
	{	"real64", "transitionRange", offsetof( FltLOD *, transitionRange ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltVertexPaletteEntryName[] = {
	{	"FltNode", "node", offsetof( FltVertexPalette *, node ) },
	{	"uint32", "numVerts", offsetof( FltVertexPalette *, numVerts ) },
	{	"FltVertex **", "verts", offsetof( FltVertexPalette *, verts  ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltVertexListEntryName[] = {
	{	"FltNode", "node", offsetof( FltVertexList *, node ) },
	{	"uint32", "numVerts", offsetof( FltVertexList *, numVerts ) },
	{	"FltVertex **", "list", offsetof( FltVertexList *, list  ) },
	{	"uint32 *", "indexList", offsetof( FltVertexList *, indexList  ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltGroupEntryName[] = {
	{	"FltNode", "node", offsetof( FltGroup *, node ) },
	{	"char [8]", "ID", offsetof( FltGroup *, ID ) },
	{	"uint16", "relativePriority", offsetof( FltGroup *, relativePriority ) },
	{	"uint16", "reserved0", offsetof( FltGroup *, reserved0 ) },
	{	"uint32", "flags", offsetof( FltGroup *, flags ) },
	{	"uint16", "specialEffectID1", offsetof( FltGroup *, specialEffectID1 ) },
	{	"uint16", "specialEffectID2", offsetof( FltGroup *, specialEffectID2 ) },
	{	"uint16", "significance", offsetof( FltGroup *, significance ) },
	{	"uint8",  "layerCode", offsetof( FltGroup *, layerCode ) },
	{	"uint8",  "reserved1", offsetof( FltGroup *, reserved1 ) },
	{	"uint32", "reserved2", offsetof( FltGroup *, reserved2 ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltObjectEntryName[] = {
	{	"FltNode", "node", offsetof( FltObject *, node ) },
	{	"char [8]", "ID", offsetof( FltObject *, ID ) },
	{	"uint32", "flags", offsetof( FltObject *, flags ) },
	{	"uint16", "relativePriority", offsetof( FltObject *, relativePriority ) },
	{	"uint16", "transparency", offsetof( FltObject *, transparency ) },
	{	"uint16", "specialEffectID1", offsetof( FltObject *, specialEffectID1 ) },
	{	"uint16", "specialEffectID2", offsetof( FltObject *, specialEffectID2 ) },
	{	"uint16", "significance", offsetof( FltObject *, significance ) },
	{	"uint16", "reserved0", offsetof( FltObject *, reserved0 ) },
 { 0, 0, 0 }
};

FltRecordEntryName FltFaceEntryName[] = {
	{	"FltNode", "node", offsetof( FltFace *, node ) },
	{	"char [8]", "ID", offsetof( FltFace *, ID ) },
	{	"uint32", "irColorCode", offsetof( FltFace *, irColorCode ) },
	{	"uint16", "relativePriority", offsetof( FltFace *, relativePriority ) },
	{	"uint8",  "drawType", offsetof( FltFace *, drawType ) },
	{	"uint8",  "textureWhite", offsetof( FltFace *, textureWhite ) },
	{	"uint16", "colorNameIndex", offsetof( FltFace *, colorNameIndex ) },
	{	"uint16", "alternateColorNameIndex", offsetof( FltFace *, alternateColorNameIndex ) },
	{	"uint8",  "reserved0", offsetof( FltFace *, reserved0 ) },
	{	"uint8",  "billboardFlags", offsetof( FltFace *, billboardFlags ) },
	{	"int16",  "detailTexturePatternIndex", offsetof( FltFace *, detailTexturePatternIndex ) },
	{	"int16",  "texturePatternIndex", offsetof( FltFace *, texturePatternIndex)},
	{	"int16",  "materialIndex", offsetof( FltFace *, materialIndex ) },
	{	"uint16", "surfaceMaterialCode", offsetof( FltFace *, surfaceMaterialCode)},
	{	"uint16", "featureID", offsetof( FltFace *, featureID ) },
	{	"uint32", "irMaterialCode", offsetof( FltFace *, irMaterialCode ) },
	{	"uint16", "transparency", offsetof( FltFace *, transparency ) },
	{	"uint8",  "LODGenerationControl", offsetof( FltFace *, LODGenerationControl ) },
	{	"uint8",  "lineStyleIndex", offsetof( FltFace *, lineStyleIndex ) },
	{	"uint32", "miscFlags", offsetof( FltFace *, miscFlags ) },
	{	"uint8",  "lightMode", offsetof( FltFace *, lightMode ) },
	{	"uint8",  "reserved1", offsetof( FltFace *, reserved1 ) },
	{	"uint16", "reserved2", offsetof( FltFace *, reserved2 ) },
	{	"uint32", "reserved3", offsetof( FltFace *, reserved3 ) },
	{	"uint32", "packedColorPrimary", offsetof( FltFace *, packedColorPrimary ) },
	{	"uint32", "packedColorAlternate", offsetof( FltFace *, packedColorAlternate ) },
	{	"uint16", "textureMappingIndex", offsetof( FltFace *, textureMappingIndex ) },
	{	"uint16", "reserved4", offsetof( FltFace *, reserved4 ) },
	{	"uint32", "primaryColorIndex", offsetof( FltFace *, primaryColorIndex ) },
	{	"uint32", "alternateColorIndex", offsetof( FltFace *, alternateColorIndex ) },
	{	"uint16", "reserved5", offsetof( FltFace *, reserved5 ) },
	{	"uint16", "reserved6", offsetof( FltFace *, reserved6 ) },
 { 0, 0, 0 }
};



