#!/usr/local/bin/python
'''
daily.py -- script for creating daily project tarball
for http://sgl.sf.net
Norman Vine nhv@users.sf.net

COPY THIS FILE TO YOUR SOURCEFORGE HOME
AND INSTALL CRONTAB per below

SCRIPT AS WRITTEN EXPECTS THIS DIRECTORY TO EXIST
/home/groups/s/sg/sgl/htdocs/dist

CHANGE DIST_DIR variable below if desired

MUST UNCOMMENT LAST TWO LINES FOR THIS TO ACTUALLY
MOVE THE TARBALL TO ITS FINAL LOCATION

## crontab entry to run this daily at 23:30 PDT ##
30 23 * * *    python ~/daily.py

I find it is easiest to create crontab locally and
then use scp or sftp to copy it to my SourceForge directory
then use crontab replace option
ie assuming I have copied the crontab entry above to crontab.txt
% crontab crontab.txt
will install crontab.txt as active crontab
use crontab -l to list crontab entry

To manually run script
% python daily.py
'''
import os, sys, string, traceback, re, shutil
from types import *

project = 'sgl'
project2 = 'fltlib'

USER = 'anonymous'
DIST_DIR = 'dist'

# 
def get_projectdir(d):
    return '/home/groups/%s/%s/%s'%(d[:1],d[:2],d)

## CHANGE THIS to reflect the directory you want the tarball put
def get_distdir(d):
    return '%s/htdocs/%s'%(get_projectdir(d), DIST_DIR)
#    
def get_path():
    for i in os.environ.keys():
        if string.upper(i)=='PATH':
            return string.split(os.environ[i],os.pathsep)
    return []
#
def find_exe(exe):
    if sys.platform=='win32':
        exe='%s.exe'%exe

    for p in get_path():
        f = os.path.join(p,exe)
        if os.path.isfile(f):
            return f

    print "Can't find %s anywhere on the path" % exe
    return None
#
def recursive_rmdir(d):
    'destroy directory d'
    if os.path.isdir(d):
        for p in os.listdir(d):
            fn = os.path.join(d,p)
            if os.path.isfile(fn):
                os.remove(fn)
            else:
                recursive_rmdir(fn)
        os.rmdir(d)
#
def CVS_remove(d):
    'destroy CVS subdirs'
    if os.path.isdir(d):
        if os.path.split(d)[1]=='CVS':
            recursive_rmdir(d)
            for p in os.listdir(d):
                fn = os.path.join(d,p)
                if os.path.isdir(fn):
                    if p=='CVS':
                        recursive_rmdir(fn)
                    else:
                        CVS_remove(fn)
#
def safe_remove(p):
    if os.path.isfile(p): os.remove(p)
#
def do_exec(cmd, cmdname=None):
    i=os.popen(cmd,'r')
    print i.read()
    i = i.close()
    if i is not None:
        if cmdname is not None:
            print 'Error: %s '+ cmdname or cmd
        sys.exit(1)

#
def cvs_checkout():
    'only export the src directory'
    recursive_rmdir(project)
    recursive_rmdir(project2)

    cvs = find_exe('cvs')
    if cvs is None:
        os.exit(1)

    os.environ['CVSROOT']=':pserver:%s@cvs.%s.sourceforge.net:/cvsroot/%s'%(USER,project,project)
#### USE THIS LINE TO KEEP CVS MARKINGS    
#    do_exec(cvs+' co %s' % project, 'the checkout phase')
#    do_exec(cvs+' co %s' % project2, 'the checkout phase')
#### USE THESE LINES TO NOT HAVE CVS MARKINGS    
    do_exec(cvs+' export -r HEAD -l %s' % project, 'the 1st checkout phase')
    do_exec(cvs+' export -r HEAD %s/sgl' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/docs' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/make' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/make.msdev' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sgl3ds' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sgldb' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sglflt' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sgljpeg' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sglobj' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sgltiff' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sgltxp' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sglpng' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sglmd3' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sglwrl' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/sglu' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD %s/testsgl' % project, 'the 2nd checkout phase')
#### UNCOMMENT THIS LINE TO HAVE MODELS included in tarball   
#    do_exec(cvs+' export -r HEAD %s/models' % project, 'the 2nd checkout phase')
    do_exec(cvs+' export -r HEAD -l %s' % project2, 'the 1st checkout phase (fltlib)')

    
#                        
def do_zip():
    'create .tgz file archives of project'
    tarfile = 'current.tgz'

    tar = find_exe('tar')
    if tar is not None:
        safe_remove(tarfile)
        do_exec('%s czvf %s %s %s' % (tar, tarfile, project, project2))

    recursive_rmdir(project)
    recursive_rmdir(project2)
    return tarfile
#
def get_testdir(d):
    return '/src/%s'%(d)
    
#
if __name__=='__main__':
    cvs_checkout()
    tarfile = do_zip()

###### Make sure the destination directory EXISTS FIRST        
    shutil.copy(tarfile,get_distdir(project))
    safe_remove(tarfile)
