##############################################################################
# Global Variable Setup Section
##############################################################################


default: all

GLOBAL := true

MOC := moc

libdir := lib/platform.$(SGL_PLATFORM)
objdir := obj/platform.$(SGL_PLATFORM)
exedir := bin/platform.$(SGL_PLATFORM)

include make/make.platform.$(SGL_PLATFORM)

# List all the modules (that should normally be compiled) in the src tree.
# Order does not matter.
modules := \
	sgl \
	sgldb \
	sglobj \
	sgl3ds \
	sglmd3

ifndef NO_TXP
modules += sgltxp
endif

ifdef LDTIFF
modules += sgltiff
endif

ifdef LDJPEG
modules += sgljpeg
endif

ifdef LDPNG
modules += sglpng
endif

ifdef LDFLT
modules += sglflt
endif

ifdef LDWRL
modules += sglwrl
endif

ifdef LDGLUT
modules += \
	sglu \
	testsgl
endif

ifdef LDQT
modules += \
	sglqt \
        sglqt/examples/SharedContext
endif


cleanGoal := $(strip $(filter %Clean %clean,$(MAKECMDGOALS)))

ifeq ("$(cleanGoal)","")
mkcmd1 = if [ ! -d $(libdir) ]; then mkdir -p $(libdir); fi;
mkcmd1 += if [ ! -d $(objdir) ]; then mkdir -p $(objdir); fi;
mkcmd1 += if [ ! -d $(exedir) ]; then mkdir -p $(exedir); fi;
junk2 := $(shell $(mkcmd1))
endif

include $(addsuffix /Makefile,$(modules))

moduleNames := $(notdir $(modules))

incs := $(addprefix -I,$(modules))

##############################################################################
# Implicit Rule Section
##############################################################################

vpath %.d $(objdir)
vpath %.cpp $(modules)
vpath %.hpp $(modules)
vpath %.c $(modules)
vpath %.o $(objdir)
vpath %.exe $(exedir)
vpath %.$(LIBSFX) $(libdir)

$(objdir)/%.o : %.cpp
	@echo _____________________________________________________________
	@echo Compiling $@
	$(CXX) $(incs) $(CXXFLAGS) -D$(AreaName)_DLL_FILE $< $(OBJNAMEARG)$@

#	$(CXX) $(incs) $(CXXFLAGS) -D$(notdir $(<D))_DLL_FILE $< $(OBJNAMEARG)$@

$(objdir)/%.o : %.c
	@echo _____________________________________________________________
	@echo Compiling $@
	$(CC) $(incs) $(CXXFLAGS) -D$(AreaName)_DLL_FILE $< $(OBJNAMEARG)$@

$(exedir)/%.exe : %.cpp %.o
	@echo _____________________________________________________________
	@echo Linking $@
	$(LINKER) $(filter %.o,$^) $(EXENAMEARG)$@ $(LINKDIRARG)$(libdir) $($(notdir $(<D))ExeLibs) $(LDFLAGS)

%.d:
	@echo _____________________________________________________________
	@echo Computing dependencies for $*
	make/depend.pl $@ $? -- $(incs) $(DEPENDFLAGS)

moc_%.cpp: %.hpp
	@echo _____________________________________________________________
	@echo Generating $@
	$(MOC) $< -o $@

##############################################################################
# Global Target Section
##############################################################################

.PHONY: default rmdirs all clean distclean force true

rmdirs:
	if [ -d lib ]; then rm -rf lib; fi;
	if [ -d obj ]; then rm -rf obj; fi;
	if [ -d bin ]; then rm -rf bin; fi;

all: $(addsuffix Tree,$(moduleNames))

clean: $(addsuffix Clean,$(moduleNames))

distclean: rmdirs

force:
	$(MAKE) all FORCE=true

true:
