/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sgl.cpp
 *  Created: 12 September 1999
 *  Summary: sgl setup code
 *****************************************************************************/

#if defined(WIN32)
# include <windows.h>
#endif

#include <sgl.h>
#include <sglOStream.hpp>
#include <sglBitmapFont.hpp>
#include <sglLock.hpp>
#include <sglStatelet.hpp>

#include <string>

#if defined(vxw)
# include <sysSymTbl.h>
# include <symLib.h>
# include <loadLib.h>
#elif defined(MAC_OSX)
#elif !defined(WIN32)
# include <dlfcn.h>
#endif

bool    sgl::s_initialized = false;

GLint   sgl::s_max_prefetch_indices = 0;
GLint   sgl::s_max_prefetch_vertices = 0;
void  (APIENTRY *sgl::drawRangeElements)(GLenum, GLuint, GLuint,
                                         GLsizei, GLenum, const GLvoid *) = 0;

GLint   sgl::s_max_texture_size = 64;
GLint   sgl::s_clamp_to_edge = GL_CLAMP;
GLenum  sgl::s_rescale_normal = GL_NORMALIZE;
float   sgl::s_max_texture_max_anisotropy = 1.0f;
GLint   sgl::s_num_texture_units = 1;

void (APIENTRY *sgl::setActiveTextureUnit)(GLenum);
void (APIENTRY *sgl::setClientActiveTextureUnit)(GLenum);
sgl::multiTexCoordFunc sgl::multiTexCoord1sv;
sgl::multiTexCoordFunc sgl::multiTexCoord2sv;
sgl::multiTexCoordFunc sgl::multiTexCoord3sv;
sgl::multiTexCoordFunc sgl::multiTexCoord4sv;
sgl::multiTexCoordFunc sgl::multiTexCoord1iv;
sgl::multiTexCoordFunc sgl::multiTexCoord2iv;
sgl::multiTexCoordFunc sgl::multiTexCoord3iv;
sgl::multiTexCoordFunc sgl::multiTexCoord4iv;
sgl::multiTexCoordFunc sgl::multiTexCoord1fv;
sgl::multiTexCoordFunc sgl::multiTexCoord2fv;
sgl::multiTexCoordFunc sgl::multiTexCoord3fv;
sgl::multiTexCoordFunc sgl::multiTexCoord4fv;
sgl::multiTexCoordFunc sgl::multiTexCoord1dv;
sgl::multiTexCoordFunc sgl::multiTexCoord2dv;
sgl::multiTexCoordFunc sgl::multiTexCoord3dv;
sgl::multiTexCoordFunc sgl::multiTexCoord4dv;

unsigned int sgl::s_num_statelet_types = sglStatelet::eNUM_BASIC_STATES;

GLenum  sgl::s_stencil_inc_wrap = GL_INCR;
GLenum  sgl::s_stencil_dec_wrap = GL_DECR;
GLint   sgl::s_num_stencil_bits = 0;

bool    sgl::s_multisample_capable = false;

bool    sgl::s_texture_env_add_capable = false;
bool    sgl::s_texture_env_combine_capable = false;
bool    sgl::s_texture_env_dot3_capable = false;

bool    sgl::s_occlusion_capable = false;
GLenum  sgl::s_occlusion_enable_flag = (GLenum)0x8165; //GL_OCCLUSION_TEST_HP
GLenum  sgl::s_occlusion_result_flag = (GLenum)0x8166; //GL_OCCLUSION_RESULT_HP

bool    sgl::s_texture_compression_capable = false;
sgl::CompressionVendor sgl::s_texture_compression_vendor =
                                                     eCOMPRESSION_VENDOR_ARB;
void (APIENTRY *sgl::compressedTexImage2D)(GLenum, GLint, GLenum,
           GLsizei, GLsizei, GLint, GLsizei, const GLvoid *);
void (APIENTRY *sgl::getCompressedTexImage)(GLenum, GLint, GLvoid *);

bool    sgl::s_auto_mipmap_capable = false;
GLenum  sgl::s_auto_mipmap_enable_flag = (GLenum)0x8191; //GENERATE_MIPMAP_SGIS

bool    sgl::s_texture_lod_bias_capable = false;
float   sgl::s_max_texture_lod_bias = 0.0f;

bool    sgl::s_texture_lod_capable = false;

bool    sgl::s_fog_distance_capable = false;

bool    sgl::s_point_parameters_capable = false;
float   sgl::s_point_default_size_min = 0;
float   sgl::s_point_default_size_max = 0;
float   sgl::s_point_default_fade_threshold = 0;
void (APIENTRY *sgl::pointParameterf)(GLenum, GLfloat);
void (APIENTRY *sgl::pointParameterfv)(GLenum, GLfloat *);

bool    sgl::s_texture_cube_map_capable = false;
GLint   sgl::s_max_cube_map_texture_size = 0;

GLint   sgl::s_max_lights = 8;  // the minimum value guaranteed

vector<GLuint>  sgl::s_unused_display_lists;
sglLock         sgl::s_dlist_lock;

vector<GLuint>  sgl::s_unused_texture_objs;
sglLock         sgl::s_tex_lock;

//----------------------------------------------------------------------------
// Query for GL limits and extensions here.
void sgl::initialize(ostream* default_stream)
{
   sglOStream::setStream(default_stream);
   sglPrintInfo() << "Initializing SGL..." << endl;

   string vendor((const char*)glGetString(GL_VENDOR));
   sglPrintInfo() << "GL_VENDOR: " << vendor << endl;

   string renderer((const char*)glGetString(GL_RENDERER));
   sglPrintInfo() << "GL_RENDERER: " << renderer << endl;

   string extensions((const char*)glGetString(GL_EXTENSIONS));
   extensions += string(" ");
   sglPrintInfo() << "GL Extensions: " << extensions << "." << endl;

   float version = atof((const char*)glGetString(GL_VERSION));
   sglPrintInfo() << "GL version: " << version << endl;

   float range[2];
   glGetFloatv(GL_LINE_WIDTH_RANGE, range);
   sglPrintInfo() << "line width range: " << range[0] << ", "
                  << range[1] << endl;

   glGetIntegerv(GL_MAX_TEXTURE_SIZE, &s_max_texture_size);
   sglPrintInfo() << "max_texture_size = " << s_max_texture_size << endl;

   if (version >= 1.2f)
   {
      s_clamp_to_edge = 0x812F; // GL_CLAMP_TO_EDGE
      s_rescale_normal = (GLenum) 0x803A; // GL_RESCALE_NORMAL
   }
   else
   {
      if (extensions.find("GL_EXT_texture_edge_clamp ") < string::npos ||
          extensions.find("GL_SGIS_texture_edge_clamp ") < string::npos)
         s_clamp_to_edge = 0x812F; // GL_CLAMP_TO_EDGE_EXT
      if (extensions.find("GL_EXT_rescale_normal ") < string::npos)
         s_rescale_normal = (GLenum) 0x803A; // GL_RESCALE_NORMAL_EXT
   }

   if (version >= 1.3f ||
       extensions.find("GL_ARB_multitexture ") < string::npos)
   {
      glGetIntegerv((GLenum)0x84E2, //GL_MAX_TEXTURE_UNITS_ARB
                    &s_num_texture_units);
      if (s_num_texture_units < 1)
         s_num_texture_units = 1;

      if (s_num_texture_units > 1)
      {
         setActiveTextureUnit =
            (void (APIENTRY *)(GLenum))getGLFuncPtr("glActiveTexture");
         if (setActiveTextureUnit == NULL)
         {
            setActiveTextureUnit =
               (void (APIENTRY *)(GLenum))getGLFuncPtr("glActiveTextureARB");
         }
         setClientActiveTextureUnit =
            (void (APIENTRY *)(GLenum))getGLFuncPtr("glClientActiveTexture");
         if (setClientActiveTextureUnit == NULL)
         {
            setClientActiveTextureUnit =
               (void (APIENTRY *)(GLenum))getGLFuncPtr(
                  "glClientActiveTextureARB");
         }
         multiTexCoord1sv = getMultiTexFunc("1sv");
         multiTexCoord2sv = getMultiTexFunc("2sv");
         multiTexCoord3sv = getMultiTexFunc("3sv");
         multiTexCoord4sv = getMultiTexFunc("4sv");
         multiTexCoord1iv = getMultiTexFunc("1iv");
         multiTexCoord2iv = getMultiTexFunc("2iv");
         multiTexCoord3iv = getMultiTexFunc("3iv");
         multiTexCoord4iv = getMultiTexFunc("4iv");
         multiTexCoord1fv = getMultiTexFunc("1fv");
         multiTexCoord2fv = getMultiTexFunc("2fv");
         multiTexCoord3fv = getMultiTexFunc("3fv");
         multiTexCoord4fv = getMultiTexFunc("4fv");
         multiTexCoord1dv = getMultiTexFunc("1dv");
         multiTexCoord2dv = getMultiTexFunc("2dv");
         multiTexCoord3dv = getMultiTexFunc("3dv");
         multiTexCoord4dv = getMultiTexFunc("4dv");

         if (setActiveTextureUnit == NULL ||
             setClientActiveTextureUnit == NULL ||
             multiTexCoord1sv == NULL ||
             multiTexCoord2sv == NULL ||
             multiTexCoord3sv == NULL ||
             multiTexCoord4sv == NULL ||
             multiTexCoord1iv == NULL ||
             multiTexCoord2iv == NULL ||
             multiTexCoord3iv == NULL ||
             multiTexCoord4iv == NULL ||
             multiTexCoord1fv == NULL ||
             multiTexCoord2fv == NULL ||
             multiTexCoord3fv == NULL ||
             multiTexCoord4fv == NULL ||
             multiTexCoord1dv == NULL ||
             multiTexCoord2dv == NULL ||
             multiTexCoord3dv == NULL ||
             multiTexCoord4dv == NULL)
         {
            s_num_texture_units = 1;
         }
      }

      sglPrintInfo() << "number of texture units = "
                     << s_num_texture_units << endl;

      if (s_num_texture_units > 1 && getenv("SGL_NUM_TEXTURE_UNITS"))
      {
         int num = atoi(getenv("SGL_NUM_TEXTURE_UNITS"));
         if (num > 0 && num < s_num_texture_units)
         {
            s_num_texture_units = num;
            sglPrintInfo() << "** Forcing number of texture units = "
                           << s_num_texture_units << " **" << endl;
         }
      }
   }
   // Add sglTexture, sglTexEnv, sglTexGen, and sglTextureMatrix statelets.
   s_num_statelet_types += (s_num_texture_units-1) * 4;


   if (version >= 1.2f ||
       extensions.find("GL_EXT_draw_range_elements ") < string::npos)
   {
      drawRangeElements =
         (void (APIENTRY *)(GLenum, GLuint, GLuint,
                            GLsizei, GLenum, const GLvoid *))getGLFuncPtr(
                               "glDrawRangeElements");
      if (drawRangeElements)
      {
         glGetIntegerv((GLenum)0x80E9, // GL_MAX_ELEMENTS_INDICES
                       &s_max_prefetch_indices);
         glGetIntegerv((GLenum)0x80E8, // GL_MAX_ELEMENTS_VERTICES
                       &s_max_prefetch_vertices);
         sglPrintInfo() << "max_prefetch_indices = "
                        << s_max_prefetch_indices << endl;
         sglPrintInfo() << "max_prefetch_vertices = "
                        << s_max_prefetch_vertices << endl;

         if (getenv("SGL_DISABLE_VERTEX_PREFETCH"))
         {
            s_max_prefetch_indices = 0;
            s_max_prefetch_vertices = 0;
            sglPrintInfo() << "** Forcing vertex prefetching off **" << endl;
         }
      }
   }

   if (extensions.find("GL_EXT_texture_filter_anisotropic ") < string::npos)
   {
      glGetFloatv((GLenum)0x84FF, // MAX_TEXTURE_MAX_ANISOTROPY_EXT
                  &s_max_texture_max_anisotropy);
      sglPrintInfo() << "max_texture_max_anisotropy = "
                     << s_max_texture_max_anisotropy << endl;
   }

   // search for texture environment add extension
   if (version >= 1.3f ||
       extensions.find("GL_EXT_texture_env_add ") < string::npos ||
       extensions.find("GL_ARB_texture_env_add ") < string::npos)
   {
      s_texture_env_add_capable = true;

      sglPrintInfo() << "TexEnv GL_ADD extension supported." << endl;
   }

   // search for texture environment combine extension
   if (version >= 1.3f ||
       extensions.find("GL_ARB_texture_env_combine ") < string::npos)
   {
      // Note that the GL_EXT_texture_env_combine extension is not quite the
      // same as the ARB version.

      s_texture_env_combine_capable = true;

      sglPrintInfo() << "TexEnv GL_COMBINE extension supported." << endl;
   }

   // search for texture environment dot3 extension
   if (version >= 1.3f ||
       extensions.find("GL_ARB_texture_env_dot3 ") < string::npos)
   {
      // Note that the GL_EXT_texture_env_dot3 extension is not quite the
      // same as the ARB version.

      s_texture_env_dot3_capable = true;

      sglPrintInfo() << "TexEnv dot3 extension supported." << endl;
   }

   // search for fog dist eq
   if (extensions.find("GL_NV_fog_distance ") < string::npos)
   {
      s_fog_distance_capable = true;

      sglPrintInfo() << "nVidia fog distance equation supported" << endl;
   }

   // search for texture level of detail extension
   if ( (extensions.find("GL_SGIS_texture_lod ") < string::npos) ||
        (extensions.find("GL_EXT_texture_lod ") < string::npos) )
   {
      s_texture_lod_capable = true;

      sglPrintInfo() << "texture level of detail supported" << endl;
   }

   // search for global texture level of detail bias
   if ( extensions.find("GL_EXT_texture_lod_bias ") < string::npos )
   {
      s_texture_lod_bias_capable = true;

      // clear errors
      while( glGetError() != GL_NO_ERROR );

      //
      // NOTE: Some vendors do not appropriately implement the next
      //       enum.  Therefore, we check for an error and supply a
      //       reasonable default in this case.  When the drivers are
      //       fixed, we will pick up the new value.

      glGetFloatv( GL_MAX_TEXTURE_LOD_BIAS_EXT, &s_max_texture_lod_bias );

      //
      // If you get the warning below, complain to your vendor that they
      // have incorrectly implemented the extension, as the above enum
      // is not supported.
      //

      if( glGetError() == GL_INVALID_ENUM )
      {
         sglPrintWarning() <<
         "EXT_texture_lod_bias broken: Default MAX(20) supplied.  See sgl.cpp."
                        << endl;
         s_max_texture_lod_bias = 20.0;
      }

      sglPrintInfo() << "max_texture_lod_bias = "
                     << s_max_texture_lod_bias << endl;
   }

   // search for auto-mipmap extension
   if ( (extensions.find("GL_EXT_generate_mipmap ") < string::npos) ||
        (extensions.find("GL_SGIS_generate_mipmap ") < string::npos) )
   {
      s_auto_mipmap_capable = true;
      sglPrintInfo() << "auto mipmap generation supported" << endl;

      if (s_auto_mipmap_capable && getenv("SGL_DISABLE_HARDWARE_MIPMAP"))
      {
         s_auto_mipmap_capable = false;
         sglPrintInfo()
            << "** Forcing hardware mipmap generation off **" << endl;
      }
   }

   // search for texture compression extension
   if ( (extensions.find("GL_EXT_texture_compression_s3tc ") < string::npos) ||
        (extensions.find("GL_S3_st3c ") < string::npos) )
   {
	   // S3 is preferred, since it provides more control over quality
      s_texture_compression_capable = true;
      s_texture_compression_vendor = eCOMPRESSION_VENDOR_S3;

      sglPrintInfo() << "S3TC texture compression supported" << endl;
   }
   else if ( version >= 1.3f ||
             extensions.find("GL_ARB_texture_compression ") < string::npos )
   {
      s_texture_compression_capable = true;
      s_texture_compression_vendor = eCOMPRESSION_VENDOR_ARB;

      sglPrintInfo() << "ARB texture compression supported" << endl;
   }

   if (s_texture_compression_capable)
   {
      compressedTexImage2D = (void (APIENTRY *)(GLenum, GLint, GLenum,
           GLsizei, GLsizei, GLint, GLsizei, const GLvoid *))
           getGLFuncPtr("glCompressedTexImage2DARB");

      getCompressedTexImage = (void (APIENTRY *)(GLenum, GLint, GLvoid *))
           getGLFuncPtr("glGetCompressedTexImageARB");
   }

   if (extensions.find("GL_EXT_stencil_wrap ") < string::npos)
   {
      s_stencil_inc_wrap = (GLenum)0x8507; // INCR_WRAP_EXT
      s_stencil_dec_wrap = (GLenum)0x8508; // DECR_WRAP_EXT
      sglPrintInfo() << "stencil_wrap supported" << endl;
   }
   glGetIntegerv(GL_STENCIL_BITS, &s_num_stencil_bits);
   sglPrintInfo() << "num_stencil_bits = " << s_num_stencil_bits << endl;

   if (extensions.find("GL_EXT_occlusion_test ") < string::npos ||
       extensions.find("GL_HP_occlusion_test ") < string::npos)
      s_occlusion_capable = true;

   if (s_clamp_to_edge != GL_CLAMP)
      sglPrintInfo() << "clamp_to_edge supported" << endl;
   if (s_rescale_normal != GL_NORMALIZE)
      sglPrintInfo() << "rescale_normal supported" << endl;
   if (s_occlusion_capable)
      sglPrintInfo() << "occlusion_cull supported" << endl;

   if (version >= 1.3f ||
       extensions.find("GL_ARB_multisample ") < string::npos ||
       extensions.find("GL_EXT_multisample ") < string::npos ||
       extensions.find("GL_SGIS_multisample ") < string::npos)
   {
      sglPrintInfo() << "multisample supported by driver" << endl;

      GLint val;
      glGetIntegerv((GLenum)0x80A8, &val); // GL_SAMPLE_BUFFERS_EXT
      if (val > 0)
      {
         sglPrintInfo() << "   " << "sample buffers = " << val << endl;
         glGetIntegerv((GLenum)0x80A9, &val); // GL_SAMPLES_EXT
         if (val > 0)
         {
            sglPrintInfo() << "   " << "samples per pixel = " << val << endl;
            s_multisample_capable = true;
         }
      }

      if (!s_multisample_capable)
         sglPrintInfo() << "multisample not supported by visual" << endl;
   }

   // search for point parameters extension
   if ( (extensions.find("GL_EXT_point_parameters ") < string::npos) ||
        (extensions.find("GL_SGIS_point_parameters ") < string::npos) )
   {
      s_point_parameters_capable = true;

      // extract the funcs
      pointParameterf = (void (APIENTRY *)(GLenum, GLfloat))
                               getGLFuncPtr("glPointParameterfEXT");

      pointParameterfv = (void (APIENTRY *)(GLenum, GLfloat *))
                               getGLFuncPtr("glPointParameterfvEXT");

      glGetFloatv( GL_POINT_SIZE_MIN_EXT, &s_point_default_size_min );
      glGetFloatv( GL_POINT_SIZE_MAX_EXT, &s_point_default_size_max );
      glGetFloatv( GL_POINT_FADE_THRESHOLD_SIZE_EXT,
                                          &s_point_default_fade_threshold);

      sglPrintInfo() << "Point Parameters supported:" << endl;
      sglPrintInfo() << "   MinSize: " << s_point_default_size_min
                     << " MaxSize: " << s_point_default_size_max
                     << " FadeThreshold: " << s_point_default_fade_threshold
                     << endl;
   }

   // search for cube map extension
   if ((extensions.find("GL_ARB_texture_cube_map ") < string::npos) ||
       (extensions.find("GL_EXT_texture_cube_map ") < string::npos) ||
       (extensions.find("GL_NV_texture_cube_map ") < string::npos))
   {
      s_texture_cube_map_capable = true;

      glGetIntegerv(GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB,
                    &s_max_cube_map_texture_size);

      sglPrintInfo() << "Cube map supported." << endl;
      sglPrintInfo() << "   max_cube_map_texture_size = "
                     << s_max_cube_map_texture_size << endl;
   }

   // query for the number of supported GL_LIGHTs - is this necessary?
   glGetIntegerv(GL_MAX_LIGHTS, &s_max_lights);
   sglPrintInfo() << "number of supported lights = " << s_max_lights << endl;

   // Setup the default font.
   sglBitmapFont::setDefaultFont(new sglBitmapFont8x13());

   s_initialized = true;
   sglPrintInfo() << "Done initializing SGL." << endl;
}


//----------------------------------------------------------------------------
void *sgl::getDLLHandle(const char *dll_name)
{
#if defined(WIN32)
   string tmp = string(dll_name) + ".dll";
   HINSTANCE dll = LoadLibrary(tmp.c_str());
   if (!dll)
   {
      tmp = string("lib") + tmp;
      dll = LoadLibrary(tmp.c_str());
   }
   return (void*)dll;
#elif defined(vxw)
   string tmp("lib");
   tmp += dll_name;
   MODULE_ID mod =
      moduleFindByName((char *)(tmp + ".relobj").c_str());
   if (!mod)
      mod = moduleFindByName((char *)(tmp + ".ro").c_str());
   if (!mod)
      mod = moduleFindByName((char *)(tmp + ".o").c_str());
   if (!mod)
      mod = moduleFindByName((char *)tmp.c_str());
   if (!mod)
      mod = moduleFindByName((char *)dll_name);
   return (void*)mod;
#elif defined(MAC_OSX)
   return 0;
#else
   string tmp = string("lib") + dll_name + ".so";
   return dlopen(tmp.c_str(), RTLD_LAZY);
#endif
}

//----------------------------------------------------------------------------
void *sgl::getFuncPtr(void *dll, const char *func_name)
{
#if defined(WIN32)
   if (dll)
      return GetProcAddress((HINSTANCE)dll, func_name);
   else
      return wglGetProcAddress(func_name);
#elif defined(vxw)
   char *func;
   SYM_TYPE type;
   if (symFindByName(sysSymTbl, (char*)func_name, &func, &type) == OK)
       return func;
   else
       return NULL;
#elif defined(MAC_OSX)
   return 0;
#else
   if (dll)
      return dlsym(dll, func_name);
   else
      return NULL;
#endif
}

//----------------------------------------------------------------------------
void *sgl::getGLFuncPtr(const char *func_name)
{
#if defined(WIN32) || defined(vxw)
   return getFuncPtr(NULL, func_name);
#elif defined( MAC_OSX )
   return 0;
#else
   static void *s_openGL_dll = getDLLHandle("GL");
   return getFuncPtr(s_openGL_dll, func_name);
#endif
}


//----------------------------------------------------------------------------
GLuint sgl::getDisplayList()
{
   GLuint dlist = 0;

   s_dlist_lock.acquire();
   if (!sgl::s_unused_display_lists.empty())
   {
      dlist = s_unused_display_lists.back();
      s_unused_display_lists.pop_back();
      s_dlist_lock.release();
   }
   else
   {
      s_dlist_lock.release();
      if ((dlist = glGenLists (1)) == 0)
         sglPrintWarning() << "Error: could not generate display list" << endl;
   }
   return dlist;
}


//----------------------------------------------------------------------------
void sgl::releaseDisplayList(GLuint dlist)
{
   if (dlist)
   {
      s_dlist_lock.acquire();
      s_unused_display_lists.push_back(dlist);
      s_dlist_lock.release();
   }
}


//----------------------------------------------------------------------------
void sgl::deleteUnusedDisplayLists()
{
   s_dlist_lock.acquire();
   while (!s_unused_display_lists.empty())
   {
      glDeleteLists(s_unused_display_lists.back(), 1);
      s_unused_display_lists.pop_back();
   }
   s_dlist_lock.release();
}


//----------------------------------------------------------------------------
GLuint sgl::getTextureObj()
{
   GLuint tex_obj = 0;
   glGenTextures(1, &tex_obj);
   return tex_obj;
}


//----------------------------------------------------------------------------
void sgl::releaseTextureObj(GLuint tex_obj)
{
   s_tex_lock.acquire();
   if (tex_obj)
      s_unused_texture_objs.push_back(tex_obj);
   s_tex_lock.release();
}


//----------------------------------------------------------------------------
void sgl::deleteUnusedTextureObjs()
{
   s_tex_lock.acquire();
   while (!s_unused_texture_objs.empty())
   {
      GLuint tex_obj = s_unused_texture_objs.back();
      glDeleteTextures(1, &tex_obj);
      s_unused_texture_objs.pop_back();
   }
   s_tex_lock.release();
}


//----------------------------------------------------------------------------
unsigned int sgl::nextPower2(unsigned int val)
{
   unsigned int pow = 0;
   for (unsigned int tmp = val; tmp > 1; ++pow) {tmp >>= 1;}
   if (1u << pow == val)
      return val;
   else
      return 1 << (pow+1);
}


//----------------------------------------------------------------------------
sgl::multiTexCoordFunc sgl::getMultiTexFunc(const char *type)
{
   string name("glMultiTexCoord");
   name += type;
   void *tmp = getGLFuncPtr(name.c_str());
   if (tmp == NULL)
      tmp = getGLFuncPtr((name+"ARB").c_str());

   return (multiTexCoordFunc)tmp;
}
