/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglBezierCurveSet.cpp
 *   Author: Tom Stimson
 *  Created: 13 Feb 2000
 *  Summary: 
 *****************************************************************************/

#include <sglBezierCurveSet.hpp>
#include <sglOStream.hpp>


sglBezierCurveSet::Map1Func sglBezierCurveSet::s_vfunc_table[] = {
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyVerts3f,
   &sglBezierCurveSet::applyVerts3d,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyVerts4f,
   &sglBezierCurveSet::applyVerts4d
};

sglBezierCurveSet::Map1Func sglBezierCurveSet::s_nfunc_table[] = {
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyNorms3f,
   &sglBezierCurveSet::applyNorms3d
};

sglBezierCurveSet::Map1Func sglBezierCurveSet::s_cfunc_table[] = {
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyColors4f,
   &sglBezierCurveSet::applyColors4d
};

sglBezierCurveSet::Map1Func sglBezierCurveSet::s_tfunc_table[] = {
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyTexCoords1f,
   &sglBezierCurveSet::applyTexCoords1d,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyTexCoords2f,
   &sglBezierCurveSet::applyTexCoords2d,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyTexCoords3f,
   &sglBezierCurveSet::applyTexCoords3d,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::invalidFunc,
   &sglBezierCurveSet::applyTexCoords4f,
   &sglBezierCurveSet::applyTexCoords4d
};


//----------------------------------------------------------------------------
sglBezierCurveSet::sglBezierCurveSet()
      : sglGeoSet(),
        m_vorder(4),
        m_norder(4),
        m_corder(4),
        m_torder(4),
        m_min(0),
        m_max(1.0f),
        m_segments(10)
{
   sglBezierCurveSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglBezierCurveSet::~sglBezierCurveSet()
{
}

//----------------------------------------------------------------------------
//  default for strip/poly primitives only
bool sglBezierCurveSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field =
      eNUMPRIMS|eVERTICES;

   if ((m_compute_bounds&test_field) == test_field)
   {
      m_bbox.makeEmpty();

      sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

      sglMultiTypePtr::iterator it = m_vertices.begin();
      int inc = m_vertices.getInc();
      sglMultiTypePtr::iterator end = it + inc * m_num_prims * m_vorder;
      for (; it != end; it += inc)
      {
         func(&m_bbox, it);
      }

      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::setTexCoordBind(BindEnum bind)
{
   m_mode &= ~eTEXCOORDS;

   switch (bind)
   {
   case eOVERALL:
       m_mode |= eTEXCOORDS_OVERALL;
       break;
   case ePER_PRIM:
       m_mode |= eTEXCOORDS_PER_PRIM;
       break;
   case ePER_VERTEX:
       m_mode |= eTEXCOORDS_PER_VERTEX;
       break;
   default:
       break;
   }

   dirty(eDLIST, true);
}
//----------------------------------------------------------------------------
sglGeoSet::BindEnum sglBezierCurveSet::getTexCoordBind() const
{
   BindEnum result = eOFF;

   switch (m_mode & eTEXCOORDS)
   {
   case eTEXCOORDS_OVERALL:
       result = eOVERALL;
       break;
   case eTEXCOORDS_PER_PRIM:
       result = ePER_PRIM;
       break;
   case eTEXCOORDS_PER_VERTEX:
       result = ePER_VERTEX;
       break;
   default:
       sglPrintWarning() << "sglBezierCurveSet::getTexCoordBind error: "
            << "invalid tex coord binding." << endl;
       exit(1);
       break;
   }

   return result;
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::setCoordOrder(GLint order)
{
   m_vorder = order;
   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::setTexCoordOrder(GLint order)
{
   m_torder = order;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::setNormalOrder(GLint order)
{
   m_norder = order;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::setColorOrder(GLint order)
{
   m_corder = order;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::setRange(float min, float max)
{
   m_min = min;
   m_max = max;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::getRange(float &min, float &max)
{
   min = m_min;
   max = m_max;
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::setNumSegments(unsigned int num)
{
   m_segments = num;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::updateDrawFunc()
{
   m_draw_func = reinterpret_cast<DrawFunc>(&sglBezierCurveSet::drawGeometry);
}

//----------------------------------------------------------------------------
void sglBezierCurveSet::drawGeometry(
   const vector<sglTexCoords> &tex_coords_list) const
{
   // OpenGL evaluators do not support multitexturing.
   const sglTexCoords *tex_coords =
      (tex_coords_list.empty() ? NULL : &tex_coords_list[0]);
   if (sgl::s_num_texture_units > 1)
      sgl::setActiveTextureUnit(GL_TEXTURE0);

   float min = m_min;
   float max = m_max;
   float range = max - min;
   float overallmax = min + (range * (float)m_num_prims);
   float overallmin = min;

   float nmin, nmax, nrange;
   float tmin, tmax, trange;
   float cmin, cmax, crange;
   int ninc, tinc, cinc;

   nmin = tmin = cmin = overallmin;
   nmax = tmax = cmax = overallmax;
   ninc = tinc = cinc = 0;
   nrange = trange = crange = -range;

   GLint vorder = m_vorder;
   Map1Func vfunc = s_vfunc_table[m_vertices.getTypeIndex()];
   int vstride = m_vertices.getInc();
   int vinc = vorder * vstride;
   if (m_vertices.getDataType() == sglDataPtr::eFLOAT)
      vstride /= sizeof(float);
   else
      vstride /= sizeof(double);

   GLenum vtarget = GL_MAP1_VERTEX_3;
   if (m_vertices.getSize() == 4)
      vtarget = GL_MAP1_VERTEX_4;
   glEnable(vtarget);


   GLint norder = m_norder;
   Map1Func nfunc = s_nfunc_table[m_normals.getTypeIndex()];
   int nstride = m_normals.getInc();
   if (nfunc != invalidFunc)
   {
      if (m_normals.getDataType() == sglDataPtr::eFLOAT)
         nstride /= sizeof(float);
      else
         nstride /= sizeof(double);

      switch (m_mode & eNORMALS)
      {
         case eNORMALS_OVERALL:
            glEnable(GL_MAP1_NORMAL);
            break;
         case eNORMALS_PER_PRIM:
            glEnable(GL_MAP1_NORMAL);
            nmin = min;
            nmax = max;
            nrange = 0;
            ninc = 0;
            break;
         case eNORMALS_PER_VERTEX:
            glEnable(GL_MAP1_NORMAL);
            nmin = min;
            nmax = max;
            nrange = 0;
            ninc = norder * m_normals.getInc();
            break;
         default:
            break;
      }
   }

   GLint corder = m_corder;
   Map1Func cfunc = s_cfunc_table[m_colors.getTypeIndex()];
   int cstride = m_colors.getInc();
   if (cfunc != invalidFunc)
   {
      if (m_colors.getDataType() == sglDataPtr::eFLOAT)
         cstride /= sizeof(float);
      else
         cstride /= sizeof(double);

      switch (m_mode & eCOLORS)
      {
         case eCOLORS_OVERALL:
            glEnable(GL_MAP1_COLOR_4);
            break;
         case eCOLORS_PER_PRIM:
            glEnable(GL_MAP1_COLOR_4);
            cmin = min;
            cmax = max;
            crange = 0;
            cinc = 0;
            break;
         case eCOLORS_PER_VERTEX:
            glEnable(GL_MAP1_COLOR_4);
            cmin = min;
            cmax = max;
            crange = 0;
            cinc = corder * m_colors.getInc();
            break;
         default:
            break;
      }
   }

   Map1Func tfunc = NULL;
   GLint torder = m_torder;
   GLenum ttarget = GL_MAP1_TEXTURE_COORD_1;
   int tstride = 1;
   sglMultiTypePtr::iterator tex = NULL;
   if (tex_coords && tex_coords->isValid() && tfunc != invalidFunc)
   {
      tex = tex_coords->begin();
      tfunc = s_tfunc_table[tex_coords->getTypeIndex()];
      ttarget += tex_coords->getSize() - 1;
      tstride = tex_coords->getInc();

      if (tex_coords->getDataType() == sglDataPtr::eFLOAT)
         tstride /= sizeof(float);
      else
         tstride /= sizeof(double);

      switch (m_mode & eTEXCOORDS)
      {
         case eTEXCOORDS_OVERALL:
            glEnable(ttarget);
            break;
         case eTEXCOORDS_PER_PRIM:
            glEnable(ttarget);
            tmin = min;
            tmax = max;
            trange = 0;
            tinc = 0;
            break;
         case eTEXCOORDS_PER_VERTEX:
            glEnable(ttarget);
            tmin = min;
            tmax = max;
            trange = 0;
            tinc = torder * tstride;
            break;
         default:
            tex = NULL;
            break;
      }
   }

   glMapGrid1f(m_segments, min, max);

   sglMultiTypePtr::iterator verts = m_vertices.begin();
   sglMultiTypePtr::iterator last = verts + (m_num_prims * vinc);
   sglMultiTypePtr::iterator norms = m_normals.begin();
   sglMultiTypePtr::iterator color = m_colors.begin();

   while (verts != last)
   {
      vfunc(min, max, vstride, vorder, verts);
      verts += vinc;
      if (norms)
      {
         nfunc(nmin, nmax, nstride, norder, norms);
         norms += ninc;
         nmin += nrange; nmax += nrange;
      }
      if (color)
      {
         cfunc(cmin, cmax, cstride, corder, color);
         color += cinc;
         cmin += crange; cmax += crange;
      }
      if (tex)
      {
         tfunc(tmin, tmax, tstride, torder, tex);
         tex += tinc;
         tmin += trange; tmax += trange;
      }

       glEvalMesh1(GL_LINE, 0, m_segments);
   }

   glDisable(vtarget);
   glDisable(GL_MAP1_NORMAL);
   glDisable(ttarget);
   glDisable(GL_MAP1_COLOR_4);
}


//----------------------------------------------------------------------------
void sglBezierCurveSet::invalidFunc(
   float, float, int, int, const void*)
{
   sglPrintWarning() << "sglBezierCurveSet::invalidFunc() called" << endl;
}

void sglBezierCurveSet::applyVerts3f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_VERTEX_3, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyVerts4f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_VERTEX_4, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyVerts3d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_VERTEX_3, min, max, stride, order, (const double*)ptr);
}

void sglBezierCurveSet::applyVerts4d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_VERTEX_4, min, max, stride, order, (const double*)ptr);
}

void sglBezierCurveSet::applyNorms3f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_NORMAL, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyNorms3d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_NORMAL, min, max, stride, order, (const double*)ptr);
}

void sglBezierCurveSet::applyColors4f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_COLOR_4, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyColors4d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_COLOR_4, min, max, stride, order, (const double*)ptr);
}

void sglBezierCurveSet::applyTexCoords1f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_TEXTURE_COORD_1, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyTexCoords1d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_TEXTURE_COORD_1, min, max, stride, order, (const double*)ptr);
}

void sglBezierCurveSet::applyTexCoords2f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_TEXTURE_COORD_2, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyTexCoords2d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_TEXTURE_COORD_2, min, max, stride, order, (const double*)ptr);
}

void sglBezierCurveSet::applyTexCoords3f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_TEXTURE_COORD_3, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyTexCoords3d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_TEXTURE_COORD_3, min, max, stride, order, (const double*)ptr);
}

void sglBezierCurveSet::applyTexCoords4f(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1f(GL_MAP1_TEXTURE_COORD_4, min, max, stride, order, (const float*)ptr);
}

void sglBezierCurveSet::applyTexCoords4d(
   float min, float max, int stride, int order, const void* ptr)
{
   glMap1d(GL_MAP1_TEXTURE_COORD_4, min, max, stride, order, (const double*)ptr);
}
