/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglBezierSurfaceSet.cpp
 *   Author: Tom Stimson
 *  Created: 30 April 2000
 *  Summary: 
 *****************************************************************************/

#include <sglBezierSurfaceSet.hpp>
#include <sglOStream.hpp>


sglBezierSurfaceSet::Map2Func sglBezierSurfaceSet::s_vfunc_table[] = {
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyVerts3f,
   &sglBezierSurfaceSet::applyVerts3d,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyVerts4f,
   &sglBezierSurfaceSet::applyVerts4d
};

sglBezierSurfaceSet::Map2Func sglBezierSurfaceSet::s_nfunc_table[] = {
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyNorms3f,
   &sglBezierSurfaceSet::applyNorms3d
};

sglBezierSurfaceSet::Map2Func sglBezierSurfaceSet::s_cfunc_table[] = {
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyColors4f,
   &sglBezierSurfaceSet::applyColors4d
};

sglBezierSurfaceSet::Map2Func sglBezierSurfaceSet::s_tfunc_table[] = {
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyTexCoords1f,
   &sglBezierSurfaceSet::applyTexCoords1d,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyTexCoords2f,
   &sglBezierSurfaceSet::applyTexCoords2d,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyTexCoords3f,
   &sglBezierSurfaceSet::applyTexCoords3d,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::invalidFunc,
   &sglBezierSurfaceSet::applyTexCoords4f,
   &sglBezierSurfaceSet::applyTexCoords4d
};


//----------------------------------------------------------------------------
sglBezierSurfaceSet::sglBezierSurfaceSet()
      : sglGeoSet(),
        m_vorder_u(4),
        m_vorder_v(4),
        m_norder_u(4),
        m_norder_v(4),
        m_corder_u(4),
        m_corder_v(4),
        m_torder_u(4),
        m_torder_v(4),
        m_min_u(0),
        m_min_v(0),
        m_max_u(1.0f),
        m_max_v(1.0f),
        m_segments_u(10),
        m_segments_v(10)
{
   sglBezierSurfaceSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglBezierSurfaceSet::~sglBezierSurfaceSet()
{
}

//----------------------------------------------------------------------------
//  default for strip/poly primitives only
bool sglBezierSurfaceSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field =
      eNUMPRIMS|eVERTICES;

   if ((m_compute_bounds&test_field) == test_field)
   {
      m_bbox.makeEmpty();

      sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

      sglMultiTypePtr::iterator it = m_vertices.begin();
      int inc = m_vertices.getInc();
      sglMultiTypePtr::iterator end =
         it + inc * m_num_prims * m_vorder_u * m_vorder_v;
      for (; it != end; it += inc)
      {
         func(&m_bbox, it);
      }

      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::setTexCoordBind(BindEnum bind)
{
   m_mode &= ~eTEXCOORDS;

   switch (bind)
   {
   case eOVERALL:
       m_mode |= eTEXCOORDS_OVERALL;
       break;
   case ePER_PRIM:
       m_mode |= eTEXCOORDS_PER_PRIM;
       break;
   case ePER_VERTEX:
       m_mode |= eTEXCOORDS_PER_VERTEX;
       break;
   default:
       break;
   }

   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
sglGeoSet::BindEnum sglBezierSurfaceSet::getTexCoordBind() const
{
   BindEnum result = eOFF;

   switch (m_mode & eTEXCOORDS)
   {
   case eTEXCOORDS_OVERALL:
       result = eOVERALL;
       break;
   case eTEXCOORDS_PER_PRIM:
       result = ePER_PRIM;
       break;
   case eTEXCOORDS_PER_VERTEX:
       result = ePER_VERTEX;
       break;
   default:
       sglPrintWarning() << "sglBezierSurfaceSet::getTexCoordBind error: "
            << "invalid tex coord binding." << endl;
       exit(1);
       break;
   }

   return result;
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::setCoordOrder(GLint order_u, GLint order_v)
{
   m_vorder_u = order_u;
   m_vorder_v = order_v;
   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::setTexCoordOrder(GLint order_u, GLint order_v)
{
   m_torder_u = order_u;
   m_torder_v = order_v;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::setNormalOrder(GLint order_u, GLint order_v)
{
   m_norder_u = order_u;
   m_norder_v = order_v;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::setColorOrder(GLint order_u, GLint order_v)
{
   m_corder_u = order_u;
   m_corder_v = order_v;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::setRange(float min_u, float max_u,
                                   float min_v, float max_v)
{
   m_min_u = min_u;
   m_max_u = max_u;
   m_min_v = min_v;
   m_max_v = max_v;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::getRange(float &min_u, float &max_u,
                                   float &min_v, float &max_v)
{
   min_u = m_min_u;
   max_u = m_max_u;
   min_v = m_min_v;
   max_v = m_max_v;
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::setNumSegments(unsigned int num_u,
                                         unsigned int num_v)
{
   m_segments_u = num_u;
   m_segments_v = num_v;
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::updateDrawFunc()
{
   m_draw_func = reinterpret_cast<DrawFunc>(
      &sglBezierSurfaceSet::drawGeometry);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::drawGeometry(
   const vector<sglTexCoords> &tex_coords_list) const
{
   // OpenGL evaluators do not support multitexturing.
   const sglTexCoords *tex_coords =
      (tex_coords_list.empty() ? NULL : &tex_coords_list[0]);
   if (sgl::s_num_texture_units > 1)
      sgl::setActiveTextureUnit(GL_TEXTURE0);

   float min_u = m_min_u;
   float max_u = m_max_u;
   float min_v = m_min_v;
   float max_v = m_max_v;
   float range_u = max_u - min_u;

   float nmin, nmax, nrange;
   float tmin, tmax, trange;
   float cmin, cmax, crange;
   int vinc, ninc, tinc, cinc;

   nmin = tmin = cmin = min_u;
   nmax = tmax = cmax = min_u + (range_u * (float)m_num_prims);
   ninc = tinc = cinc = 0;
   nrange = trange = crange = -range_u;

   Map2Func vfunc = s_vfunc_table[m_vertices.getTypeIndex()];
   int vstride_v = m_vertices.getInc();
   if (m_vertices.getDataType() == sglDataPtr::eFLOAT)
      vstride_v /= sizeof(float);
   else
      vstride_v /= sizeof(double);
   int vstride_u = vstride_v * m_vorder_v;
   vinc = m_vorder_u * m_vorder_v * m_vertices.getInc();

   GLenum vtarget = GL_MAP2_VERTEX_3;
   if (m_vertices.getSize() == 4)
      vtarget = GL_MAP2_VERTEX_4;
   glEnable(vtarget);


   Map2Func nfunc = s_nfunc_table[m_normals.getTypeIndex()];
   int nstride_v, nstride_u;
   if (nfunc != invalidFunc)
   {
      nstride_v = m_normals.getInc();
      if (m_normals.getDataType() == sglDataPtr::eFLOAT)
         nstride_v /= sizeof(float);
      else
         nstride_v /= sizeof(double);
      nstride_u = nstride_v * m_norder_v;

      switch (m_mode & eNORMALS)
      {
         case eNORMALS_OVERALL:
            glEnable(GL_MAP2_NORMAL);
            break;
         case eNORMALS_PER_PRIM:
            glEnable(GL_MAP2_NORMAL);
            nmin = min_u;
            nmax = max_u;
            nrange = 0;
            ninc = 0;
            break;
         case eNORMALS_PER_VERTEX:
            glEnable(GL_MAP2_NORMAL);
            nmin = min_u;
            nmax = max_u;
            nrange = 0;
            ninc = m_norder_u * m_norder_v * m_normals.getInc();
            break;
         default:
            break;
      }
   }

   Map2Func cfunc = s_cfunc_table[m_colors.getTypeIndex()];
   int cstride_v, cstride_u;
   if (cfunc != invalidFunc)
   {
      cstride_v = m_colors.getInc();
      if (m_colors.getDataType() == sglDataPtr::eFLOAT)
         cstride_v /= sizeof(float);
      else
         cstride_v /= sizeof(double);
      cstride_u = cstride_v * m_corder_v;

      switch (m_mode & eCOLORS)
      {
         case eCOLORS_OVERALL:
            glEnable(GL_MAP2_COLOR_4);
            break;
         case eCOLORS_PER_PRIM:
            glEnable(GL_MAP2_COLOR_4);
            cmin = min_u;
            cmax = max_u;
            crange = 0;
            cinc = 0;
            break;
         case eCOLORS_PER_VERTEX:
            glEnable(GL_MAP2_COLOR_4);
            cmin = min_u;
            cmax = max_u;
            crange = 0;
            cinc = m_corder_u * m_corder_v * m_colors.getInc();
            break;
         default:
            break;
      }
   }

   Map2Func tfunc = NULL;
   GLenum ttarget = GL_MAP2_TEXTURE_COORD_1;
   int tstride_v = 1;
   sglMultiTypePtr::iterator tex = NULL;
   if (tex_coords && tex_coords->isValid() && tfunc != invalidFunc)
   {
      tex = tex_coords->begin();
      tfunc = s_tfunc_table[tex_coords->getTypeIndex()];
      ttarget += tex_coords->getSize() - 1;
      tstride_v = tex_coords->getInc();

      if (tex_coords->getDataType() == sglDataPtr::eFLOAT)
         tstride_v /= sizeof(float);
      else
         tstride_v /= sizeof(double);

      switch (m_mode & eTEXCOORDS)
      {
         case eTEXCOORDS_OVERALL:
            glEnable(ttarget);
            break;
         case eTEXCOORDS_PER_PRIM:
            glEnable(ttarget);
            tmin = min_u;
            tmax = max_u;
            trange = 0;
            tinc = 0;
            break;
         case eTEXCOORDS_PER_VERTEX:
            glEnable(GL_MAP2_TEXTURE_COORD_2);
            tmin = min_u;
            tmax = max_u;
            trange = 0;
            tinc = m_torder_u * m_torder_v * tstride_v;
            break;
         default:
            tex = NULL;
            break;
      }
   }
   int tstride_u = tstride_v * m_torder_v;

   glEnable(vtarget);
   glMapGrid2f(m_segments_u, min_u, max_u, m_segments_v, min_v, max_v);

   sglMultiTypePtr::iterator verts = m_vertices.begin();
   sglMultiTypePtr::iterator last = verts + (m_num_prims * vinc);
   sglMultiTypePtr::iterator norms = m_normals.begin();
   sglMultiTypePtr::iterator color = m_colors.begin();

   while (verts != last)
   {
      vfunc(min_u, max_u, vstride_u, m_vorder_u,
            min_v, max_v, vstride_v, m_vorder_v, verts);
      verts += vinc;
      if (norms)
      {
         nfunc(nmin, nmax, nstride_u, m_norder_u,
               min_v, max_v, nstride_v, m_norder_v, norms);
         norms += ninc;
         nmin += nrange; nmax += nrange;
      }
      if (color)
      {
         cfunc(cmin, cmax, cstride_u, m_corder_u,
               min_v, max_v, cstride_v, m_corder_v, color);
         color += cinc;
         cmin += crange; cmax += crange;
      }
      if (tex)
      {
         tfunc(tmin, tmax, tstride_u, m_torder_u,
               min_v, max_v, tstride_v, m_torder_v, tex);
         tex += tinc;
         tmin += trange; tmax += trange;
      }

      glEvalMesh2(GL_FILL, 0, m_segments_u, 0, m_segments_v);
   }

   glDisable(vtarget);
   glDisable(GL_MAP2_NORMAL);
   glDisable(ttarget);
   glDisable(GL_MAP2_COLOR_4);
}

//----------------------------------------------------------------------------
void sglBezierSurfaceSet::addStats(sglStats& stats) const
{
   sglStats::PrimEnum prim_type = getPrimType();
   unsigned int prim_size = getPrimSize();

   stats.addGSets(prim_type, 1);
   stats.addPrims(prim_type, m_num_prims);
   stats.addVertices(prim_type, m_num_prims*prim_size);

   stats.addTriangles(prim_type, 2*m_num_prims*m_segments_u*m_segments_v);
}


//----------------------------------------------------------------------------
void sglBezierSurfaceSet::invalidFunc(
   float, float, int, int, float, float, int, int, const void*)
{
   sglPrintWarning() << "sglBezierSurfaceSet::invalidFunc() called" << endl;
}

void sglBezierSurfaceSet::applyVerts3f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_VERTEX_3, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyVerts4f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_VERTEX_4, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyVerts3d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_VERTEX_3, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}

void sglBezierSurfaceSet::applyVerts4d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_VERTEX_4, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}

void sglBezierSurfaceSet::applyNorms3f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_NORMAL, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyNorms3d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_NORMAL, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}

void sglBezierSurfaceSet::applyColors4f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_COLOR_4, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyColors4d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_COLOR_4, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords1f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_TEXTURE_COORD_1, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords1d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_TEXTURE_COORD_1, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords2f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_TEXTURE_COORD_2, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords2d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_TEXTURE_COORD_2, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords3f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_TEXTURE_COORD_3, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords3d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_TEXTURE_COORD_3, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords4f(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2f(GL_MAP2_TEXTURE_COORD_4, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const float*)ptr);
}

void sglBezierSurfaceSet::applyTexCoords4d(
   float min_u, float max_u, int stride_u, int order_u,
   float min_v, float max_v, int stride_v, int order_v, const void* ptr)
{
   glMap2d(GL_MAP2_TEXTURE_COORD_4, min_u, max_u, stride_u, order_u,
           min_v, max_v, stride_v, order_v, (const double*)ptr);
}
