/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglBitmapFontWGL.hpp
 *  Created: 13 April 2001 (separated from sglBitmapFont.hpp)
 *  Summary: Windows bitmap font class.  To reduce dependency of sgl.h on
 *           windows.h (through sglBitmapFont.hpp)
 *****************************************************************************/

#ifndef __SGL_BITMAP_FONT_WGL_HPP
#define __SGL_BITMAP_FONT_WGL_HPP

#if defined(WIN32)

#include <windows.h>
#include <sglBitMapFont.hpp>
#include <string>

//============================================================================

/// Font that uses wgl to rasterize an Win32 font.

class SGL_DLL_API sglBitmapFontWGL : public sglBitmapFont
{
public:
   /** Construct a font from the given Win32 font.
       NOTE: A gl context should be current when this method is called.
       @param font the Win32 font
   */
   sglBitmapFontWGL(HFONT font);

   /// NOTE: A gl context should be current when this method is called.
   ~sglBitmapFontWGL();

   /** Create a copy of this font.  The copy will have display lists
       that are valid for the current context.
       @return a copy of this font
   */
   virtual sglBitmapFont* clone() const;

   /** Renders a bitmap string at the current raster position.
       Characters should be limited to the ASCII range 0-127.
       @param str the string to render
       @param len the length of the string.  If len is less than zero,
              strlen() will be used to determine the length.
   */
   virtual void rasterString(const char *str, int len = -1) const;

   /** Gets the height of the font in pixels.
       @return the height of the font in pixels
   */
   virtual unsigned int getHeight() const;

   /** Gets the extent of the font below the baseline in pixels.
       @return the extent of the font below the baseline in pixels.
   */
   virtual unsigned int getDescent() const;

   /** Gets the width of the given character.
       @param c the character whose width will be returned
       @return the width of the given character
   */
   virtual unsigned int getCharWidth(char c) const;

private:  // not implemented
   sglBitmapFontWGL();
   sglBitmapFontWGL(const sglBitmapFontWGL &);
   sglBitmapFontWGL &operator=(const sglBitmapFontWGL &);

private:
   HFONT        m_font;
   GLuint       m_list_base;
   unsigned int m_height;
   unsigned int m_descent;
   unsigned int m_widths[128];
};

#endif // WIN32
#endif
