/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglBoxBound.cpp
 *  Created: 26 December 1998
 *  Summary: bounding boxes for view frustum culling
 *****************************************************************************/

#include <sglBoxBound.hpp>

sglBoxBound::ExpandToIncludeFunc
sglBoxBound::s_expand_to_include_func_table[] = {
   NULL,
   NULL,
   &sglBoxBound::expandToInclude2s,
   NULL,
   &sglBoxBound::expandToInclude2i,
   NULL,
   &sglBoxBound::expandToInclude2f,
   &sglBoxBound::expandToInclude2d,
   NULL,
   NULL,
   &sglBoxBound::expandToInclude3s,
   NULL,
   &sglBoxBound::expandToInclude3i,
   NULL,
   &sglBoxBound::expandToInclude3f,
   &sglBoxBound::expandToInclude3d,
   NULL,
   NULL,
   &sglBoxBound::expandToInclude4s,
   NULL,
   &sglBoxBound::expandToInclude4i,
   NULL,
   &sglBoxBound::expandToInclude4f,
   &sglBoxBound::expandToInclude4d
};


//----------------------------------------------------------------------------
sglBoxBound::sglBoxBound()
{
   makeEmpty();
}

//----------------------------------------------------------------------------
sglBoxBound::sglBoxBound(const sglBoxBound &rhs)
{
   m_min[0] = rhs.m_min[0];
   m_min[1] = rhs.m_min[1];
   m_min[2] = rhs.m_min[2];

   m_max[0] = rhs.m_max[0];
   m_max[1] = rhs.m_max[1];
   m_max[2] = rhs.m_max[2];
}

//----------------------------------------------------------------------------
sglBoxBound &sglBoxBound::operator=(const sglBoxBound &rhs)
{
   // check for assignment to self
   if (this != &rhs)
   {
      m_min[0] = rhs.m_min[0];
      m_min[1] = rhs.m_min[1];
      m_min[2] = rhs.m_min[2];

      m_max[0] = rhs.m_max[0];
      m_max[1] = rhs.m_max[1];
      m_max[2] = rhs.m_max[2];
   }

   return *this;
}

//----------------------------------------------------------------------------
void sglBoxBound::makeEmpty()
{
   m_min[0] = m_min[1] = m_min[2] = FLT_MAX; // allows expandToInclude
   m_max[0] = m_max[1] = m_max[2] = -FLT_MAX; // allows expandToInclude
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude(const sglBoxBound &box)
{
   // if box is not empty update this
   if (!box.isEmpty())
   {
      if (isEmpty())
      {
         // if this one is empty set this one to child's
         *this = box;
      }
      else
      {
         // else do the per element bound compares
         if (m_min[0] > box.m_min[0]) m_min[0] = box.m_min[0];
         if (m_max[0] < box.m_max[0]) m_max[0] = box.m_max[0];

         if (m_min[1] > box.m_min[1]) m_min[1] = box.m_min[1];
         if (m_max[1] < box.m_max[1]) m_max[1] = box.m_max[1];

         if (m_min[2] > box.m_min[2]) m_min[2] = box.m_min[2];
         if (m_max[2] < box.m_max[2]) m_max[2] = box.m_max[2];
      }
   }
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude(const sglVec3f &point)
{
    if (m_min[0] > point[0]) m_min[0] = point[0];
    if (m_max[0] < point[0]) m_max[0] = point[0];

    if (m_min[1] > point[1]) m_min[1] = point[1];
    if (m_max[1] < point[1]) m_max[1] = point[1];

    if (m_min[2] > point[2]) m_min[2] = point[2];
    if (m_max[2] < point[2]) m_max[2] = point[2];
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude2s(sglBoxBound *bbox, const void *ptr)
{
   short *point = (short*)ptr;
   float x = (float)point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = (float)point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   if (bbox->m_min[2] > 0) bbox->m_min[2] = 0;
   if (bbox->m_max[2] < 0) bbox->m_max[2] = 0;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude2i(sglBoxBound *bbox, const void *ptr)
{
   int *point = (int*)ptr;
   float x = (float)point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = (float)point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   if (bbox->m_min[2] > 0) bbox->m_min[2] = 0;
   if (bbox->m_max[2] < 0) bbox->m_max[2] = 0;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude2f(sglBoxBound *bbox, const void *ptr)
{
   float *point = (float*)ptr;
   float x = point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   if (bbox->m_min[2] > 0) bbox->m_min[2] = 0;
   if (bbox->m_max[2] < 0) bbox->m_max[2] = 0;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude2d(sglBoxBound *bbox, const void *ptr)
{
   double *point = (double*)ptr;
   float x = (float)point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = (float)point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   if (bbox->m_min[2] > 0) bbox->m_min[2] = 0;
   if (bbox->m_max[2] < 0) bbox->m_max[2] = 0;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude3s(sglBoxBound *bbox, const void *ptr)
{
   short *point = (short*)ptr;
   float x = (float)point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = (float)point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = (float)point[2];
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude3i(sglBoxBound *bbox, const void *ptr)
{
   int *point = (int*)ptr;
   float x = (float)point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = (float)point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = (float)point[2];
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude3f(sglBoxBound *bbox, const void *ptr)
{
   float *point = (float*)ptr;
   float x = point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = point[2];
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude3d(sglBoxBound *bbox, const void *ptr)
{
   double *point = (double*)ptr;
   float x = (float)point[0];
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = (float)point[1];
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = (float)point[2];
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude4s(sglBoxBound *bbox, const void *ptr)
{
   short *point = (short*)ptr;
   float w = (float)point[3];
   float x = ((float)point[0])/w;
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = ((float)point[1])/w;
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = ((float)point[2])/w;
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude4i(sglBoxBound *bbox, const void *ptr)
{
   int *point = (int*)ptr;
   float w = (float)point[3];
   float x = ((float)point[0])/w;
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = ((float)point[1])/w;
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = ((float)point[2])/w;
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude4f(sglBoxBound *bbox, const void *ptr)
{
   float *point = (float*)ptr;
   float w = point[3];
   float x = point[0]/w;
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = point[1]/w;
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = point[2]/w;
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::expandToInclude4d(sglBoxBound *bbox, const void *ptr)
{
   double *point = (double*)ptr;
   float w = (float)point[3];
   float x = ((float)point[0])/w;
   if (bbox->m_min[0] > x) bbox->m_min[0] = x;
   if (bbox->m_max[0] < x) bbox->m_max[0] = x;

   float y = ((float)point[1])/w;
   if (bbox->m_min[1] > y) bbox->m_min[1] = y;
   if (bbox->m_max[1] < y) bbox->m_max[1] = y;

   float z = ((float)point[2])/w;
   if (bbox->m_min[2] > z) bbox->m_min[2] = z;
   if (bbox->m_max[2] < z) bbox->m_max[2] = z;
}

//----------------------------------------------------------------------------
void sglBoxBound::drawImmediate() const
{
   glBegin(GL_LINE_STRIP);
   glVertex3f(m_min[0], m_min[1], m_min[2]);
   glVertex3f(m_min[0], m_max[1], m_min[2]);
   glVertex3f(m_min[0], m_max[1], m_max[2]);
   glVertex3f(m_max[0], m_max[1], m_max[2]);
   glEnd();

   glBegin(GL_LINE_STRIP);
   glVertex3f(m_min[0], m_max[1], m_min[2]);
   glVertex3f(m_max[0], m_max[1], m_min[2]);
   glVertex3f(m_max[0], m_max[1], m_max[2]);
   glVertex3f(m_max[0], m_min[1], m_max[2]);
   glEnd();

   glBegin(GL_LINE_STRIP);
   glVertex3f(m_max[0], m_max[1], m_min[2]);
   glVertex3f(m_max[0], m_min[1], m_min[2]);
   glVertex3f(m_max[0], m_min[1], m_max[2]);
   glVertex3f(m_min[0], m_min[1], m_max[2]);
   glEnd();

   glBegin(GL_LINE_STRIP);
   glVertex3f(m_max[0], m_min[1], m_min[2]);
   glVertex3f(m_min[0], m_min[1], m_min[2]);
   glVertex3f(m_min[0], m_min[1], m_max[2]);
   glVertex3f(m_min[0], m_max[1], m_max[2]);
   glEnd();
}

//----------------------------------------------------------------------------
void sglBoxBound::drawOcclusionBox() const
{
   glBegin(GL_TRIANGLE_STRIP);
   glVertex3f(m_max[0], m_min[1], m_min[2]);
   glVertex3f(m_min[0], m_min[1], m_min[2]);
   glVertex3f(m_max[0], m_max[1], m_min[2]);
   glVertex3f(m_min[0], m_max[1], m_min[2]);
   glVertex3f(m_max[0], m_max[1], m_max[2]);
   glVertex3f(m_min[0], m_max[1], m_max[2]);
   glVertex3f(m_max[0], m_min[1], m_max[2]);
   glVertex3f(m_min[0], m_min[1], m_max[2]);
   glEnd();
   glBegin(GL_TRIANGLE_STRIP);
   glVertex3f(m_max[0], m_max[1], m_min[2]);
   glVertex3f(m_max[0], m_max[1], m_max[2]);
   glVertex3f(m_max[0], m_min[1], m_min[2]);
   glVertex3f(m_max[0], m_min[1], m_max[2]);
   glVertex3f(m_min[0], m_min[1], m_min[2]);
   glVertex3f(m_min[0], m_min[1], m_max[2]);
   glVertex3f(m_min[0], m_max[1], m_min[2]);
   glVertex3f(m_min[0], m_max[1], m_max[2]);
   glEnd();
}

//----------------------------------------------------------------------------
//    Summary: friend
// Parameters: 
//    Returns: 
//----------------------------------------------------------------------------
ostream &operator<<(ostream &ostrm, const sglBoxBound &box)
{
   if (box.isEmpty())
      return (ostrm << "bbox (empty)");

   return (ostrm << "bbox (" << box.m_min << ")->(" << box.m_max << ')');
}
