/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglCallback.cpp
 *  Created: 21 February 1999
 *  Summary:
 *****************************************************************************/

#include <sglCallback.hpp>
#include <sglCull.hpp>

void defaultCallback(void *) {}

//----------------------------------------------------------------------------
sglCallback::sglCallback() : sglGroup(),
   m_callback_func(&defaultCallback),
   m_callback_data(NULL)
{
}

//----------------------------------------------------------------------------
sglCallback::~sglCallback()
{
}

//----------------------------------------------------------------------------
void sglCallback::setCallbackFunc(sglCallbackFunc cb_func)
{
   if (cb_func)
      m_callback_func = cb_func;
   else
      m_callback_func = &defaultCallback;
}

//----------------------------------------------------------------------------
void sglCallback::cull(sglCull<float> &trav_state,
                       unsigned int cull_flags) const
{
   (*m_callback_func)(m_callback_data);

   sglGroup::cull(trav_state, cull_flags);
}

//----------------------------------------------------------------------------
void sglCallback::cull(sglCull<double> &trav_state,
                       unsigned int cull_flags) const
{
   (*m_callback_func)(m_callback_data);

   sglGroup::cull(trav_state, cull_flags);
}

//----------------------------------------------------------------------------
sglNode *sglCallback::clone(unsigned int mode) const
{
   sglCallback *root = new sglCallback;
   sglCallback::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglCallback::copyTo(sglCallback *dst, unsigned int mode) const
{
   sglGroup::copyTo(dst, mode);

   // transfer stuff
   dst->setCallbackFunc(m_callback_func);
   dst->setCallbackData(m_callback_data);
}

//----------------------------------------------------------------------------
void sglCallback::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglGroup::printInfo(ostrm, indent_string);
   ostrm << indent_string << "sglCallback: function = " << m_callback_func
         << endl;
   ostrm << indent_string << "                 data = " << m_callback_data
         << endl;
}
