/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglCallback.hpp
 *  Created: 21 February 1999
 *  Summary: execute a user defined callback during the cull traversal.
 *
 *           **** THIS DOES NOT WORK AS A DRAW CALLBACK ANY MORE ****
 *
 *        With access to the source code and ability to subclass nodes, I
 *        question the usefulness of this class. - mcmillan
 *****************************************************************************/

#ifndef __SGL_CALLBACK_HPP
#define __SGL_CALLBACK_HPP

#include <sgl.h>
#include <sglGroup.hpp>

//============================================================================

/// The callback function signature
typedef void (*sglCallbackFunc)(void *);

//============================================================================

/**

A subclass of sglGroup that executes a callback function during the cull
traversal.  During cull it will first call the user-specified callback function
(passing in the user-specified callback data pointer).  Then it will perform
sglGroup's cull function.  It acts as a pre-cull callback.

<i> This seems like a pretty useless class.  I leave it in the distribution
because it seems relatively benign.  In reality, the callback mechanism has not
been given proper consideration yet.  In some other scene graphs there exist
concepts like pre- and post- cull callbacks that actually affect the results
of the cull.  (There are also pre- and post-draw traversals which have
yet been implemented in SGL, but I digress).  Really, if you want to do
something special you should implement your own sglGroup subclass. <i>

*/

class SGL_DLL_API sglCallback : public sglGroup
{
public:
   /// default constructor
   sglCallback();
   /// virtual destructor
   virtual ~sglCallback();

   /** Get a pointer to the current callback function.
       @returns pointer to the callback function, or NULL if one has not been
       set.
   */
   sglCallbackFunc getCallbackFunc() const { return m_callback_func; }

   /** Set the callback function.
       @param cb_func Callback function pointer.  Passing NULL will clear the
       callback function.
   */
   void setCallbackFunc(sglCallbackFunc cb_func);

   /** Get a pointer to the callback data that is sent as a parameter to the
       callback function.
       @returns pointer to callback data, or NULL if not set.
   */
   void *getCallbackData() const { return m_callback_data; }

   /** Set the callback data pointer.
       @param data pointer to callback data.  NULL to clear.
   */
   void setCallbackData(void *data) { m_callback_data = data; }

   // see sglNode::clone() for details.
   virtual sglNode *clone(unsigned int mode) const;

   // see sglObject::printInfo() for details.
   virtual void printInfo(ostream &ostrm, const char *indent_string) const;

protected:
   // compute the bounding sphere using sglGroup version
   //virtual void computeBound();

   // sglNode::cull(f) for details
   virtual void cull(sglCull<float> &trav_state,
                     unsigned int cull_flags) const;

   // sglNode::cull(d) for details
   virtual void cull(sglCull<double> &trav_state,
                     unsigned int cull_flags) const;

   void copyTo(sglCallback *dst, unsigned int mode) const;

private:  // not implemented
   sglCallback(const sglCallback &);
   sglCallback &operator=(const sglCallback &);

protected:
   sglCallbackFunc m_callback_func;
   void *m_callback_data;
};

#endif
