/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglCurrState.hpp
 *   Author: Tom Stimson
 *  Created: 15 July 2000
 *  Summary:
 *****************************************************************************/

#ifndef __SGL_CURR_STATE_HPP
#define __SGL_CURR_STATE_HPP

#include <sgl.h>

#include <sglDrawable.hpp>


// A complete state.
struct SGL_DLL_API sglFullState
{
public:
   sglFullState()
         : m_statelet(sgl::s_num_statelet_types, (const sglStatelet*)NULL) {}
   ~sglFullState() {}

   void clear()
      {
         for (unsigned int ii = 0; ii < sgl::s_num_statelet_types; ++ii)
            m_statelet[ii] = NULL;
         m_tex_coords = NULL;
         m_dlist = 0;
      }

   // Call this for each drawable rendered
   void addStats(sglStats& stats) const;

private: // not implemented
   sglFullState(const sglFullState &);
   sglFullState& operator=(const sglFullState &);

public:
   vector<const sglStatelet*>  m_statelet;
   const vector<sglTexCoords> *m_tex_coords;
   GLuint                      m_dlist;
};

class SGL_DLL_API sglCurrState
{
public:
   sglCurrState()
      : m_point_smooth(false), m_line_smooth(false), m_multisample(false) {}
   ~sglCurrState() {}

   void apply(const sglFullState *new_state);
   void statsApply(const sglFullState *new_state, sglStats& stats);
   void draw(const sglDrawable *drawable);
   void force(const sglFullState *new_state);

   void checkBlend();
   void checkDepthMask();

private:
   void checkPointSmooth(bool point_type);
   void checkLineSmooth(bool line_type);
   void checkMultisample(bool line_type, bool fill_type);
   void drawGeometry(const sglDrawable *drawable);

   // not implemented
   sglCurrState(const sglCurrState &);
   sglCurrState &operator=(const sglCurrState &);

private:
   sglFullState m_state;
   bool         m_point_smooth;
   bool         m_line_smooth;
   bool         m_multisample;
};


#endif
