/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglDefaultStateNode.cpp
 *   Author: Tom Stimson
 *  Created: 21 July 2000
 *****************************************************************************/

#include <sglDefaultStateNode.hpp>
#include <sglCull.hpp>

//----------------------------------------------------------------------------
sglDefaultStateNode::sglDefaultStateNode() : sglGroup()
{
}

//----------------------------------------------------------------------------
sglDefaultStateNode::~sglDefaultStateNode()
{
   for (vector<sglStatelet*>::const_iterator ii = m_statelets.begin();
        ii != m_statelets.end(); ++ii)
   {
      (*ii)->deref();
      if ((*ii)->getRefCount() == 0)
         delete *ii;
   }
}

//----------------------------------------------------------------------------
void sglDefaultStateNode::setStatelets(const vector<sglStatelet*> &statelets)
{
   vector<sglStatelet*>::const_iterator end = statelets.end();
   vector<sglStatelet*>::const_iterator ii = statelets.begin();
   for (; ii != end; ++ii)
   {
      (*ii)->ref();
   }
   end = m_statelets.end();
   for (ii = m_statelets.begin(); ii != end; ++ii)
   {
      (*ii)->deref();
      if ((*ii)->getRefCount() == 0)
         delete *ii;
   }
   m_statelets = statelets;
}

//----------------------------------------------------------------------------
void sglDefaultStateNode::cull(sglCull<float> &trav_state,
                               unsigned int cull_flags) const
{
   vector<sglStatelet*>::const_iterator end = m_statelets.end();
   for (vector<sglStatelet*>::const_iterator i = m_statelets.begin();
        i != end; ++i)
      trav_state.pushDefaultStatelet(*i);

   sglGroup::cull(trav_state, cull_flags);

   for (unsigned int j=0; j<m_statelets.size(); ++j)
      trav_state.popDefaultStatelet();
}

//----------------------------------------------------------------------------
void sglDefaultStateNode::cull(sglCull<double> &trav_state,
                               unsigned int cull_flags) const
{
   vector<sglStatelet*>::const_iterator end = m_statelets.end();
   for (vector<sglStatelet*>::const_iterator i = m_statelets.begin();
        i != end; ++i)
      trav_state.pushDefaultStatelet(*i);

   sglGroup::cull(trav_state, cull_flags);

   for (unsigned int j=0; j<m_statelets.size(); ++j)
      trav_state.popDefaultStatelet();
}

//----------------------------------------------------------------------------
sglNode *sglDefaultStateNode::clone(unsigned int mode) const
{
   sglDefaultStateNode *root = new sglDefaultStateNode;
   sglDefaultStateNode::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglDefaultStateNode::copyTo(sglDefaultStateNode *dst,
                                 unsigned int mode) const
{
   sglGroup::copyTo(dst, mode);

   // transfer stuff
   dst->setStatelets(m_statelets);
}

//----------------------------------------------------------------------------
void sglDefaultStateNode::printInfo(ostream &ostrm,
                                     const char *indent_string) const
{
   sglGroup::printInfo(ostrm, indent_string);
   ostrm << indent_string << "sglDefaultStateNode: "
         << m_statelets.size() << " statelets" << endl;
}
