/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglDirectionalLight.cpp
 *   Author: Scott McMillan
 *  Created: 9 December 2000
 *****************************************************************************/

#include <sglDirectionalLight.hpp>

//----------------------------------------------------------------------------
sglDirectionalLight::sglDirectionalLight() : sglLight(),
   m_direction(0.0f, 0.0f, 1.0f, 0.0f)
{
}

//----------------------------------------------------------------------------
sglDirectionalLight::~sglDirectionalLight()
{
}

//----------------------------------------------------------------------------
void sglDirectionalLight::setDirection(const sglVec3f &dir)
{
   m_direction.set(dir[0], dir[1], dir[2], 0.0f);
   m_direction.normalize();
}

//----------------------------------------------------------------------------
void sglDirectionalLight::getDirection(sglVec3f &dir) const
{
   dir.set(m_direction[0], m_direction[1], m_direction[2]);
}

//----------------------------------------------------------------------------
void sglDirectionalLight::setup(GLenum gl_light_number) const
{
   sglLight::setup(gl_light_number);
   glLightfv(gl_light_number, GL_POSITION, m_direction.getPtr());
}

//----------------------------------------------------------------------------
sglNode *sglDirectionalLight::clone(unsigned int mode) const
{
   sglDirectionalLight *root = new sglDirectionalLight;
   sglDirectionalLight::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglDirectionalLight::computeBound()
{
   m_sphere.setCenter(0., 0., 0.);
   m_sphere.setRadius(-1.0);
   m_dirty &= ~eBOUND;
}

//----------------------------------------------------------------------------
void sglDirectionalLight::copyTo(sglDirectionalLight *dst,
                                 unsigned int mode) const
{
   sglLight::copyTo(dst, mode);

   // transfer stuff
   sglVec3f dir;
   getDirection(dir);
   dst->setDirection(dir);
}

//----------------------------------------------------------------------------
void sglDirectionalLight::printInfo(ostream &ostrm,
                                    const char *indent_string) const
{
   sglLight::printInfo(ostrm, indent_string);
   ostrm << indent_string << "direction: " << m_direction[0] << ", "
         << m_direction[1] << ", " << m_direction[2] << endl;
}
