/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglDiscriminator.cpp
 *  Created: 21 February 1999
 *  Summary:
 *****************************************************************************/

#include <sglDiscriminator.hpp>
#include <sglCull.hpp>

//----------------------------------------------------------------------------
sglDiscriminator::sglDiscriminator() : sglGroup(),
   m_mask(0)
{
}

//----------------------------------------------------------------------------
sglDiscriminator::~sglDiscriminator()
{
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglDiscriminator::intersect(
   sglIntersectf &segset) const
{
   sglBound::IntersectResultEnum result = sglBound::eOUTSIDE;

   if (m_mask & segset.getDiscriminatorMask())
   {
      result = sglGroup::intersect(segset);
   }
   return result;
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglDiscriminator::intersect(
   sglIntersectd &segset) const
{
   sglBound::IntersectResultEnum result = sglBound::eOUTSIDE;

   if (m_mask & segset.getDiscriminatorMask())
   {
      result = sglGroup::intersect(segset);
   }
   return result;
}

//----------------------------------------------------------------------------
void sglDiscriminator::pick(sglPickf &pick_state,
                            unsigned int cull_flags) const
{
   if (m_mask & pick_state.getDiscriminatorMask())
      sglGroup::pick(pick_state, cull_flags);
}

//----------------------------------------------------------------------------
void sglDiscriminator::pick(sglPickd &pick_state,
                            unsigned int cull_flags) const
{
   if (m_mask & pick_state.getDiscriminatorMask())
      sglGroup::pick(pick_state, cull_flags);
}

//----------------------------------------------------------------------------
void sglDiscriminator::cull(sglCull<float> &trav_state,
                            unsigned int cull_flags) const
{
   if (m_mask & trav_state.getDiscriminatorMask())
      sglGroup::cull(trav_state, cull_flags);
}

//----------------------------------------------------------------------------
void sglDiscriminator::cull(sglCull<double> &trav_state,
                            unsigned int cull_flags) const
{
   if (m_mask & trav_state.getDiscriminatorMask())
      sglGroup::cull(trav_state, cull_flags);
}

//----------------------------------------------------------------------------
sglNode *sglDiscriminator::clone(unsigned int mode) const
{
   sglDiscriminator *root = new sglDiscriminator;
   sglDiscriminator::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglDiscriminator::copyTo(sglDiscriminator *dst, unsigned int mode) const
{
   sglGroup::copyTo(dst, mode);

   // transfer stuff
   dst->setMask(m_mask);
}

//----------------------------------------------------------------------------
void sglDiscriminator::printInfo(ostream &ostrm,
                                 const char *indent_string) const
{
   sglGroup::printInfo(ostrm, indent_string);
   ostrm << indent_string << "sglDiscriminator: mask = " << hex << m_mask
         << dec << endl;
}
