/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglFog.cpp
 *   Author: Scott McMillan and Michael Morrison
 *  Created: 28 November 2000
 *****************************************************************************/

#include <sglFog.hpp>
#include <sglOStream.hpp>

//----------------------------------------------------------------------------
sglFog::sglFog()
      : sglStatelet(eFOG),
        m_mode(eOFF),
        m_start(0.f),
        m_end(1.f),
        m_density(1.f),
        m_distance_equation(eEYE_PLANE_ABSOLUTE),
        m_color(0.0f, 0.0f, 0.0f, 0.0f)
{
   setIndex();
}

//----------------------------------------------------------------------------
sglFog::~sglFog()
{
}

//----------------------------------------------------------------------------
void sglFog::setDistanceEquation(DistanceEqEnum eq)
{
   if( sgl::s_fog_distance_capable )
   {
      m_distance_equation = eq;
      setIndex();
   }
}

//----------------------------------------------------------------------------
void sglFog::setMode(ModeEnum mode)
{
   m_mode = mode;
   setIndex();
}

//----------------------------------------------------------------------------
void sglFog::setExtent(float start, float end)
{
   if (start >= 0.f && end >= 0.f && end > start)
   {
      m_start = start;
      m_end = end;
   }
}

//----------------------------------------------------------------------------
void sglFog::setDensity(float density)
{
   if (density >= 0.f)
      m_density = density;
}

//----------------------------------------------------------------------------
void sglFog::apply(sglCurrState*) const
{
   if (m_mode == eOFF)
   {
      glDisable(GL_FOG);
   }
   else
   {
      glFogi(GL_FOG_MODE, (GLenum)m_mode);
      glFogfv(GL_FOG_COLOR, m_color.getPtr());
      glFogf(GL_FOG_DENSITY, m_density);
      glFogf(GL_FOG_START, m_start);
      glFogf(GL_FOG_END, m_end);
      if (sgl::s_fog_distance_capable)
         glFogi( GL_FOG_DISTANCE_MODE_NV, m_distance_equation ); 
      glEnable(GL_FOG);
   }
}

//----------------------------------------------------------------------------
void sglFog::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglStatelet::printInfo(ostrm, indent_string);
   switch (m_mode)
   {
      case eOFF:
         ostrm << indent_string << "fog mode: disabled" << endl;
         break;
      case eLINEAR:
         ostrm << indent_string << "fog mode: LINEAR" << endl;
         break;
      case eEXP:
         ostrm << indent_string << "fog mode: EXP" << endl;
         break;
      case eEXP2:
         ostrm << indent_string << "fog mode: EXP2" << endl;
         break;
   }

   ostrm << indent_string << "color:   " << m_color << endl;
   ostrm << indent_string << "density: " << m_density << endl;
   ostrm << indent_string << "start:   " << m_start << endl;
   ostrm << indent_string << "end:     " << m_end << endl;
   if( sgl::s_fog_distance_capable )
   {
      ostrm << indent_string << "EQ:     ";
      if (m_distance_equation == eEYE_PLANE_ABSOLUTE) 
			   ostrm << "eEYE_PLANE_ABSOLUTE";
      else
			   ostrm << "eEYE_RADIAL";
      ostrm << endl;
   }
}
