/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglGeoSet.cpp
 *  Created: 13 August 1997
 *  Summary: base class for all geosets
 *****************************************************************************/

#include <sglGeoSet.hpp>
#include <sglGeode.hpp>
#include <sglOStream.hpp>


sglDrawable::DrawFunc sglGeoSet::s_func_table[];

#ifdef INLINE_BOXBOUND
sglGeoSet::BBFunc sglGeoSet::s_bb_func_table[] = {
   NULL,
   NULL,
   &sglGeoSet::buildBoxBound2s,
   NULL,
   &sglGeoSet::buildBoxBound2i,
   NULL,
   &sglGeoSet::buildBoxBound2f,
   &sglGeoSet::buildBoxBound2d,
   NULL,
   NULL,
   &sglGeoSet::buildBoxBound3s,
   NULL,
   &sglGeoSet::buildBoxBound3i,
   NULL,
   &sglGeoSet::buildBoxBound3f,
   &sglGeoSet::buildBoxBound3d,
   NULL,
   NULL,
   &sglGeoSet::buildBoxBound4s,
   NULL,
   &sglGeoSet::buildBoxBound4i,
   NULL,
   &sglGeoSet::buildBoxBound4f,
   &sglGeoSet::buildBoxBound4d
};
#endif


//----------------------------------------------------------------------------
class sglGeoSet::Initializer
{
public:
   Initializer() { sglGeoSet::setupFuncTable(); }
};
sglGeoSet::Initializer sglGeoSet::sInitializer;


//----------------------------------------------------------------------------
void sglGeoSet::setupFuncTable()
{
   s_func_table[eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVT);
   s_func_table[eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVNoT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVNpT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoNoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoNpT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpNoT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpNpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvNpT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvNvT);

#ifndef SGL_NO_VERTEX_ARRAYS
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCoNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCpNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoSet::drawVCvNvTm);
#endif
}


//----------------------------------------------------------------------------
sglGeoSet::sglGeoSet()
      : sglDrawable(),
        m_mode(eCOLORS_NONE|eNORMALS_NONE),
        m_num_prims(0),
        m_vertices(),
        m_normals(),
        m_colors(),
        m_compute_bounds(0)
{
#ifndef SGL_NO_VERTEX_ARRAYS
   if (sgl::s_num_texture_units > 1)
      m_mode |= eMULTI_TEXTURE;
#endif

   sglGeoSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglGeoSet::~sglGeoSet()
{
}

#ifdef INLINE_BOXBOUND
//----------------------------------------------------------------------------
template <class T>
static inline
void sglGeoSetBuildBoxBound2(
   sglBoxBound &bbox, T *ptr, int vert_inc, int num_verts)
{
   T min_x, min_y;
   T max_x, max_y;

   min_x = max_x = ptr[0];
   min_y = max_y = ptr[1];

   int inc = vert_inc / sizeof(T);
   T *end = ptr + inc * num_verts;
   for (ptr += inc; ptr != end; ptr += inc)
   {
      T x = ptr[0];
      if (x < min_x) min_x = x;
      if (x > max_x) max_x = x;

      T y = ptr[1];
      if (y < min_y) min_y = y;
      if (y > max_y) max_y = y;
   }

   bbox.setMin((float)min_x, (float)min_y, 0);
   bbox.setMax((float)max_x, (float)max_y, 0);
}

//----------------------------------------------------------------------------
template <class T>
static inline
void sglGeoSetBuildBoxBound3(
   sglBoxBound &bbox, T *ptr, int vert_inc, int num_verts)
{
   T min_x, min_y, min_z;
   T max_x, max_y, max_z;

   min_x = max_x = ptr[0];
   min_y = max_y = ptr[1];
   min_z = max_z = ptr[2];

   int inc = vert_inc / sizeof(T);
   T *end = ptr + inc * num_verts;
   for (ptr += inc; ptr != end; ptr += inc)
   {
      T x = ptr[0];
      if (x < min_x) min_x = x;
      if (x > max_x) max_x = x;

      T y = ptr[1];
      if (y < min_y) min_y = y;
      if (y > max_y) max_y = y;

      T z = ptr[2];
      if (z < min_z) min_z = z;
      if (z > max_z) max_z = z;
   }

   bbox.setMin((float)min_x, (float)min_y, (float)min_z);
   bbox.setMax((float)max_x, (float)max_y, (float)max_z);
}

//----------------------------------------------------------------------------
template <class T>
static inline
void sglGeoSetBuildBoxBound4(
   sglBoxBound &bbox, T *ptr, int vert_inc, int num_verts)
{
   float min_x, min_y, min_z;
   float max_x, max_y, max_z;

   float w = (float)ptr[3];
   min_x = max_x = ((float)ptr[0])/w;
   min_y = max_y = ((float)ptr[1])/w;
   min_z = max_z = ((float)ptr[2])/w;

   int inc = vert_inc / sizeof(T);
   T *end = ptr + inc * num_verts;
   for (ptr += inc; ptr != end; ptr += inc)
   {
      float w = (float)ptr[3];
      float x = ((float)ptr[0])/w;
      if (x < min_x) min_x = x;
      if (x > max_x) max_x = x;

      float y = ((float)ptr[1])/w;
      if (y < min_y) min_y = y;
      if (y > max_y) max_y = y;

      float z = ((float)ptr[2])/w;
      if (z < min_z) min_z = z;
      if (z > max_z) max_z = z;
   }

   bbox.setMin(min_x, min_y, min_z);
   bbox.setMax(max_x, max_y, max_z);
}

void sglGeoSet::buildBoxBound2s(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound2(bbox, (short*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound2i(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound2(bbox, (int*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound2f(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound2(bbox, (float*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound2d(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound2(bbox, (double*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound3s(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound3(bbox, (short*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound3i(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound3(bbox, (int*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound3f(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound3(bbox, (float*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound3d(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound3(bbox, (double*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound4s(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound4(bbox, (short*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound4i(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound4(bbox, (int*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound4f(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound4(bbox, (float*)ptr, vert_inc, num_verts);
}
void sglGeoSet::buildBoxBound4d(
   sglBoxBound &bbox, void *ptr, int vert_inc, int num_verts)
{
   sglGeoSetBuildBoxBound4(bbox, (double*)ptr, vert_inc, num_verts);
}
#endif

//----------------------------------------------------------------------------
//  default for strip/poly primitives only
bool sglGeoSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field = eNUMPRIMS|eVERTICES;

   if ((m_compute_bounds&test_field) == test_field)
   {
#ifdef INLINE_BOXBOUND
      BBFunc bound_func = s_bb_func_table[m_vertices.getTypeIndex()];
      bound_func(m_bbox, m_vertices.begin(), m_vertices.getInc(),
                 m_num_prims * getPrimSize());
#else
      m_bbox.makeEmpty();

      sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

      sglMultiTypePtr::iterator it = m_vertices.begin();
      int inc = m_vertices.getInc();
      sglMultiTypePtr::iterator end = it + inc * m_num_prims * getPrimSize();
      for (; it != end; it += inc)
      {
         func(&m_bbox, it);
      }
#endif
      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}

//----------------------------------------------------------------------------
void sglGeoSet::setNumPrims(unsigned int num)
{
   m_num_prims = num;
   if (num > 0)
   {
      m_compute_bounds |= eNUMPRIMS;
   }
   else
   {
      m_compute_bounds &= ~eNUMPRIMS;
   }
   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
void sglGeoSet::setCoordList(sglDataPtr *data,
                             sglDataPtr::DataTypeEnum type,
                             int size,
                             int offset,
                             int stride)
{
   m_vertices.set(data, type, size, offset, stride);
   if (m_vertices.isValid())
      m_compute_bounds |= eVERTICES;
   else
      m_compute_bounds &= ~eVERTICES;

   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
void sglGeoSet::setNormalList(BindEnum bind,
                              sglDataPtr *data,
                              sglDataPtr::DataTypeEnum type,
                              int offset,
                              int stride)
{
   m_mode &= ~eNORMALS;

   m_normals.set(data, type, 3, offset, stride);
   if (m_normals.isValid() && bind != eOFF)
   {
      switch (bind)
      {
         case eOVERALL:
            m_mode |= eNORMALS_OVERALL;
            break;
         case ePER_PRIM:
            m_mode |= eNORMALS_PER_PRIM;
            break;
         case ePER_VERTEX:
            m_mode |= eNORMALS_PER_VERTEX;
            break;
         default:
            break;
      }
   }
   updateDrawFunc();
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglGeoSet::setColorList(BindEnum bind,
                             sglDataPtr *data,
                             sglDataPtr::DataTypeEnum type,
                             int size,
                             int offset,
                             int stride)
{
   m_mode &= ~eCOLORS;

   m_colors.set(data, type, size, offset, stride);
   if (m_colors.isValid() && bind != eOFF)
   {
      switch (bind)
      {
         case eOVERALL:
            m_mode |= eCOLORS_OVERALL;
            break;
         case ePER_PRIM:
            m_mode |= eCOLORS_PER_PRIM;
            break;
         case ePER_VERTEX:
            m_mode |= eCOLORS_PER_VERTEX;
            break;
         default:
            break;
      }
   }
   updateDrawFunc();
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
sglGeoSet::BindEnum sglGeoSet::getNormalBind() const
{
   BindEnum result = eOFF;
   switch (m_mode & eNORMALS)
   {
      case eNORMALS_OVERALL:
         result = eOVERALL;
         break;
      case eNORMALS_PER_PRIM:
         result = ePER_PRIM;
         break;
      case eNORMALS_PER_VERTEX:
         result = ePER_VERTEX;
         break;
      default:
         break;
   }

   return result;
}

//----------------------------------------------------------------------------
sglGeoSet::BindEnum sglGeoSet::getColorBind() const
{
   BindEnum result = eOFF;
   switch (m_mode & eCOLORS)
   {
      case eCOLORS_OVERALL:
         result = eOVERALL;
         break;
      case eCOLORS_PER_PRIM:
         result = ePER_PRIM;
         break;
      case eCOLORS_PER_VERTEX:
         result = ePER_VERTEX;
         break;
      default:
         break;
   }

   return result;
}

//----------------------------------------------------------------------------
void sglGeoSet::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglDrawable::printInfo(ostrm, indent_string);

   ostrm << indent_string << "sglGeoSet mode 0x" << hex << m_mode
         << dec << " [need to add stuff], " << endl;
}

//----------------------------------------------------------------------------
void sglGeoSet::addStats(sglStats& stats) const
{
   sglStats::PrimEnum prim_type = getPrimType();
   unsigned int prim_size = getPrimSize();

   stats.addGSets(prim_type, 1);
   stats.addPrims(prim_type, m_num_prims);
   stats.addVertices(prim_type, m_num_prims*prim_size);

   if (prim_size == 3)
   {
      stats.addTriangles(prim_type, (float)m_num_prims);
   }
   else if (prim_size == 4)
   {
      stats.addTriangles(prim_type, (float)(m_num_prims*2));
   }
}

//----------------------------------------------------------------------------
bool sglGeoSet::isValid() const
{
   return m_vertices.isValid();
}

//----------------------------------------------------------------------------
void sglGeoSet::updateDrawFunc()
{
   unsigned int mode = m_mode;

#ifdef SGL_NO_VERTEX_ARRAYS
   mode &= ~eMULTI_TEXTURE;
#endif

   m_draw_func = s_func_table[mode];
}


#define normalIterator() \
   sglMultiTypePtr::iterator nit = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc();

#define normalApplyNext() \
   nfunc(nit); \
   nit += ninc;

#define colorIterator() \
   sglMultiTypePtr::iterator cit = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc();

#define colorApplyNext() \
   cfunc(cit); \
   cit += cinc;

#ifndef SGL_NO_VERTEX_ARRAYS

#define drawV() \
{ \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVNo() \
{ \
   m_normals.apply0(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVNp() \
{ \
   normalIterator(); \
   const GLenum primType = getGLPrimType(); \
   for (unsigned int j=0; j<num_elems; j += prim_size) \
   { \
      normalApplyNext(); \
      glDrawArrays(primType, j, prim_size); \
   } \
}

#define drawVNv() \
{ \
   m_normals.applyArray(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVCo() \
{ \
   m_colors.apply0(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVCoNo() \
{ \
   m_colors.apply0(); \
   m_normals.apply0(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVCoNp() \
{ \
   m_colors.apply0(); \
   normalIterator(); \
   const GLenum primType = getGLPrimType(); \
   for (unsigned int j=0; j<num_elems; j += prim_size) \
   { \
      normalApplyNext(); \
      glDrawArrays(primType, j, prim_size); \
   } \
}

#define drawVCoNv() \
{ \
   m_normals.applyArray(); \
   m_colors.apply0(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVCp() \
{ \
   colorIterator(); \
   const GLenum primType = getGLPrimType(); \
   for (unsigned int j=0; j<num_elems; j += prim_size) \
   { \
      colorApplyNext(); \
      glDrawArrays(primType, j, prim_size); \
   } \
}

#define drawVCpNo() \
{ \
   m_normals.apply0(); \
   colorIterator(); \
   const GLenum primType = getGLPrimType(); \
   for (unsigned int j=0; j<num_elems; j += prim_size) \
   { \
      colorApplyNext(); \
      glDrawArrays(primType, j, prim_size); \
   } \
}

#define drawVCpNp() \
{ \
   colorIterator(); \
   normalIterator(); \
   const GLenum primType = getGLPrimType(); \
   for (unsigned int j=0; j<num_elems; j += prim_size) \
   { \
      colorApplyNext(); \
      normalApplyNext(); \
      glDrawArrays(primType, j, prim_size); \
   } \
}

#define drawVCpNv() \
{ \
   m_normals.applyArray(); \
   colorIterator(); \
   const GLenum primType = getGLPrimType(); \
   for (unsigned int j=0; j<num_elems; j += prim_size) \
   { \
      colorApplyNext(); \
      glDrawArrays(primType, j, prim_size); \
   } \
}

#define drawVCv() \
{ \
   m_colors.applyArray(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVCvNo() \
{ \
   m_colors.applyArray(); \
   m_normals.apply0(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define drawVCvNp() \
{ \
   m_colors.applyArray(); \
   normalIterator(); \
   const GLenum primType = getGLPrimType(); \
   for (unsigned int j=0; j<num_elems; j += prim_size) \
   { \
      normalApplyNext(); \
      glDrawArrays(primType, j, prim_size); \
   } \
}

#define drawVCvNv() \
{ \
   m_colors.applyArray(); \
   m_normals.applyArray(); \
   glDrawArrays(getGLPrimType(), 0, num_elems); \
}

#define DRAWFUNC(a) \
void sglGeoSet::a##T(const vector<sglTexCoords> &tex_coords) const\
{ \
   const unsigned int prim_size = getPrimSize(); \
   const unsigned int num_elems = m_num_prims * prim_size; \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySet(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
} \
void sglGeoSet::a##Tm(const vector<sglTexCoords> &tex_coords) const\
{ \
   const unsigned int prim_size = getPrimSize(); \
   const unsigned int num_elems = m_num_prims * prim_size; \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySetMulti(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
}

DRAWFUNC(drawV)
DRAWFUNC(drawVNo)
DRAWFUNC(drawVNp)
DRAWFUNC(drawVNv)
DRAWFUNC(drawVCo)
DRAWFUNC(drawVCoNo)
DRAWFUNC(drawVCoNp)
DRAWFUNC(drawVCoNv)
DRAWFUNC(drawVCp)
DRAWFUNC(drawVCpNo)
DRAWFUNC(drawVCpNp)
DRAWFUNC(drawVCpNv)
DRAWFUNC(drawVCv)
DRAWFUNC(drawVCvNo)
DRAWFUNC(drawVCvNp)
DRAWFUNC(drawVCvNv)


#else

#define vertexIterator() \
   sglMultiTypePtr::iterator vit = m_vertices.begin(); \
   const sglMultiTypePtr::FuncType1 vfunc = m_vertices.getFunc(); \
   const int vinc = m_vertices.getInc(); \
   const unsigned int prim_size = getPrimSize(); \
   const sglMultiTypePtr::iterator vend = vit + vinc * m_num_prims * prim_size;

#define vertexApplyNext() \
   vfunc(vit); \
   vit += vinc;

#define loopBegin() \
   for (const sglMultiTypePtr::iterator loop_end = vit + vinc * prim_size; \
        vit != loop_end;) \
   {

#define loopEnd() \
      vertexApplyNext() \
   }

#define texIterator() \
   unsigned int tex_index = 0; \
   GLenum num_tex = GL_TEXTURE0 + \
      SGL_MIN((GLenum)sgl::s_num_texture_units, (GLenum)tex_coords.size());

#define texApplyNext() \
   sglTexCoords::applySet(tex_coords, tex_index, num_tex); \
   ++tex_index;

//----------------------------------------------------------------------------
void sglGeoSet::drawVT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         normalApplyNext();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCoNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCoNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCoNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         normalApplyNext();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCpNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCpNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCpNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         colorApplyNext();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         colorApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCvNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         colorApplyNext();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         colorApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCvNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglGeoSet::drawVCvNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (vit != vend)
      {
         colorApplyNext();
         normalApplyNext();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (vit != vend)
      {
         colorApplyNext();
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

#endif
