/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglGeoSetDataPtr.cpp
 *   Author: Tom Stimson
 *  Created: 6 April 2002
 *****************************************************************************/

#include <sglGeoSetDataPtr.hpp>
#include <sglOStream.hpp>


sglGeoSetDataPtr::FuncType1 sglVertexPtr::s_func[] = {
   invalidFunc,
   invalidFunc,
   (FuncType1)glVertex2sv,
   invalidFunc,
   (FuncType1)glVertex2iv,
   invalidFunc,
   (FuncType1)glVertex2fv,
   (FuncType1)glVertex2dv,
   invalidFunc,
   invalidFunc,
   (FuncType1)glVertex3sv,
   invalidFunc,
   (FuncType1)glVertex3iv,
   invalidFunc,
   (FuncType1)glVertex3fv,
   (FuncType1)glVertex3dv,
   invalidFunc,
   invalidFunc,
   (FuncType1)glVertex4sv,
   invalidFunc,
   (FuncType1)glVertex4iv,
   invalidFunc,
   (FuncType1)glVertex4fv,
   (FuncType1)glVertex4dv
};

sglGeoSetDataPtr::FuncType1 sglNormalPtr::s_func[] = {
   (FuncType1)glNormal3bv,
   invalidFunc,
   (FuncType1)glNormal3sv,
   invalidFunc,
   (FuncType1)glNormal3iv,
   invalidFunc,
   (FuncType1)glNormal3fv,
   (FuncType1)glNormal3dv
};

sglGeoSetDataPtr::FuncType1 sglColorPtr::s_func[] = {
   (FuncType1)glColor3bv,
   (FuncType1)glColor3ubv,
   (FuncType1)glColor3sv,
   (FuncType1)glColor3usv,
   (FuncType1)glColor3iv,
   (FuncType1)glColor3uiv,
   (FuncType1)glColor3fv,
   (FuncType1)glColor3dv,
   (FuncType1)glColor4bv,
   (FuncType1)glColor4ubv,
   (FuncType1)glColor4sv,
   (FuncType1)glColor4usv,
   (FuncType1)glColor4iv,
   (FuncType1)glColor4uiv,
   (FuncType1)glColor4fv,
   (FuncType1)glColor4dv
};


//----------------------------------------------------------------------------
void sglGeoSetDataPtr::set(sglDataPtr *data, int stride)
{
   derefDataPtr();
   if (data) data->ref();

   m_data_ptr = data;
   m_stride = stride;
}


//----------------------------------------------------------------------------
void sglGeoSetDataPtr::derefDataPtr()
{
   if (m_data_ptr)
   {
      m_data_ptr->deref();
      if (m_data_ptr->getRefCount() == 0)
      {
         delete m_data_ptr;
      }
   }
}


//----------------------------------------------------------------------------
void sglGeoSetDataPtr::invalidFunc(const void*)
{
   sglPrintWarning() << "sglGeoSetDataPtr::invalidFunc() called" << endl;
}


//----------------------------------------------------------------------------
void sglMultiTypePtr::set(sglDataPtr *data, sglDataPtr::DataTypeEnum type,
                          int size, int offset, int stride)
{
   sglGeoSetDataPtr::set(data, stride);

   m_type = type;
   m_gltype = sglDataPtr::getGLType(type);
   m_size = size;
   if (stride == 0)
      m_byte_inc = sglDataPtr::getSizeOfType(type) * size;
   else
      m_byte_inc = stride;

   if (m_data_ptr && m_data_ptr->getData() && stride >= 0)
   {
      m_start = (char*)m_data_ptr->getData() + offset;
      setFunc();
   }
   else
   {
      m_start = NULL;
      m_func = invalidFunc;
   }
}


//----------------------------------------------------------------------------
void sglVertexPtr::setFunc()
{
   if (m_size > 1 && m_size < 5)
      m_func = s_func[getTypeIndex()];
   else
      m_func = invalidFunc;
}


//----------------------------------------------------------------------------
void sglNormalPtr::setFunc()
{
   if (m_size == 3)
      m_func = s_func[getTypeIndex()];
   else
      m_func = invalidFunc;
}


//----------------------------------------------------------------------------
void sglColorPtr::setFunc()
{
   if (m_size == 3 || m_size == 4)
      m_func = s_func[getTypeIndex()];
   else
      m_func = invalidFunc;
}

//----------------------------------------------------------------------------
void sglIndexPtr::set(sglDataPtr *data, int offset, int stride)
{
   sglGeoSetDataPtr::set(data, stride);

   if (stride == 0)
      m_inc = 1;
   else
   {
      int tmp = stride/sizeof(sglIndexType);
      if (tmp * (int)sizeof(sglIndexType) != stride)
         sglPrintWarning() << "sglIndexPtr::set() stride: " << stride
                           << " is not a multiple of index type size" << endl;

      m_inc = tmp;
   }

   if (m_data_ptr && m_data_ptr->getData())
   {
      m_start = (iterator)((char*)m_data_ptr->getData() + offset);
   }
   else
   {
      m_start = NULL;
   }
}

