/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglGeoStripSet.cpp
 *  Created: 13 August 1997
 *****************************************************************************/

#include <sgl.h>
#include <sglGeoStripSet.hpp>
#include <sglOStream.hpp>


sglDrawable::DrawFunc sglGeoStripSet::s_func_table[];

//----------------------------------------------------------------------------
class sglGeoStripSet::Initializer
{
public:
   Initializer() { sglGeoStripSet::setupFuncTable(); }
};
sglGeoStripSet::Initializer sglGeoStripSet::sInitializer;


//----------------------------------------------------------------------------
void sglGeoStripSet::setupFuncTable()
{
   s_func_table[eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVT);
   s_func_table[eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVNoT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVNpT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoNoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoNpT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpNoT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpNpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvNpT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvNvT);

#ifndef SGL_NO_VERTEX_ARRAYS
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCoNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCpNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglGeoStripSet::drawVCvNvTm);
#endif
}

//----------------------------------------------------------------------------
sglGeoStripSet::sglGeoStripSet() : sglGeoSet(),
      m_lengths()
{
   sglGeoStripSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglGeoStripSet::~sglGeoStripSet()
{
}

//----------------------------------------------------------------------------
bool sglGeoStripSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field =
      eNUMPRIMS|eLENGTHS|eVERTICES;

   if ((m_compute_bounds&test_field) == test_field)
   {
      m_bbox.makeEmpty();

      sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

      sglMultiTypePtr::iterator vit = m_vertices.begin();
      int vinc = m_vertices.getInc();
      sglIndexPtr::iterator lit = m_lengths.begin();
      int linc = m_lengths.getInc();
      sglIndexPtr::iterator lend = lit + linc * m_num_prims;
      for (; lit != lend; lit += linc)
      {
         sglMultiTypePtr::iterator vend = vit + vinc * *lit;
         for (; vit != vend; vit += vinc)
         {
            func(&m_bbox, vit);
         }
      }
      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::setPrimLengths(sglDataPtr *data,
                                    int offset,
                                    int stride)
{
   m_lengths.set(data, offset, stride);
   if (m_lengths.isValid())
      m_compute_bounds |= eLENGTHS;
   else
      m_compute_bounds &= ~eLENGTHS;

   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
void sglGeoStripSet::addStats(sglStats &stats) const
{
   sglStats::PrimEnum prim_type = getPrimType();

   stats.addGSets(prim_type, 1);
   stats.addPrims(prim_type, m_num_prims);

   sglIndexPtr::iterator lit = m_lengths.begin();
   int linc = m_lengths.getInc();
   sglIndexPtr::iterator lend = lit + linc * m_num_prims;

   switch (prim_type)
   {
      case sglStats::eTRIFANS:
      case sglStats::eTRISTRIPS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            if (*lit > 2)
               stats.addTriangles(prim_type, (*lit - 2));
         }
         break;
      case sglStats::eQUADSTRIPS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            unsigned int num = (*lit - 2)/2;
            if (num > 0)
               stats.addTriangles(prim_type, (num*2));
         }
         break;
      case sglStats::ePOLYS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            if (*lit > 2)
               stats.addTriangles(prim_type, (*lit-2));
         }
         break;
      default:
         for (; lit != lend; lit += linc)
            stats.addVertices(prim_type, *lit);
         break;
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::updateDrawFunc()
{
   unsigned int mode = m_mode;

#ifdef SGL_NO_VERTEX_ARRAYS
   mode &= ~eMULTI_TEXTURE;
#endif

   m_draw_func = s_func_table[mode];
}


#define normalIterator() \
   sglMultiTypePtr::iterator nit = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc();

#define normalApplyNext() \
   nfunc(nit); \
   nit += ninc;

#define colorIterator() \
   sglMultiTypePtr::iterator cit = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc();

#define colorApplyNext() \
   cfunc(cit); \
   cit += cinc;

#define lengthLoop() \
   sglIndexPtr::iterator lit = m_lengths.begin(); \
   const int linc = m_lengths.getInc(); \
   const sglIndexPtr::iterator lend = lit + linc * m_num_prims; \
   for (; lit != lend; lit += linc)

#ifndef SGL_NO_VERTEX_ARRAYS

#define drawV() \
{ \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVNo() \
{ \
   m_normals.apply0(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVNp() \
{ \
   normalIterator() \
   lengthLoop() \
   { \
      normalApplyNext() \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVNv() \
{ \
   m_normals.applyArray(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCo() \
{ \
   m_colors.apply0(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCoNo() \
{ \
   m_colors.apply0(); \
   m_normals.apply0(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCoNp() \
{ \
   m_colors.apply0(); \
   normalIterator() \
   lengthLoop() \
   { \
      normalApplyNext() \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCoNv() \
{ \
   m_colors.apply0(); \
   m_normals.applyArray(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCp() \
{ \
   colorIterator() \
   lengthLoop() \
   { \
      colorApplyNext() \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCpNo() \
{ \
   m_normals.apply0(); \
   colorIterator() \
   lengthLoop() \
   { \
      colorApplyNext() \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCpNp() \
{ \
   colorIterator() \
   normalIterator() \
   lengthLoop() \
   { \
      colorApplyNext() \
      normalApplyNext() \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCpNv() \
{ \
   m_normals.applyArray(); \
   colorIterator() \
   lengthLoop() \
   { \
      colorApplyNext() \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCv() \
{ \
   m_colors.applyArray(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCvNo() \
{ \
   m_colors.applyArray(); \
   m_normals.apply0(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCvNp() \
{ \
   m_colors.applyArray(); \
   normalIterator() \
   lengthLoop() \
   { \
      normalApplyNext() \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define drawVCvNv() \
{ \
   m_colors.applyArray(); \
   m_normals.applyArray(); \
   lengthLoop() \
   { \
      sglIndexType length = *lit; \
      glDrawArrays(primType, index, length); \
      index += length; \
   } \
}

#define DRAWFUNC(a) \
void sglGeoStripSet::a##T(const vector<sglTexCoords> &tex_coords) const\
{ \
   const GLenum primType = getGLPrimType(); \
   unsigned int index = 0; \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySet(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
} \
void sglGeoStripSet::a##Tm(const vector<sglTexCoords> &tex_coords) const\
{ \
   const GLenum primType = getGLPrimType(); \
   unsigned int index = 0; \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySetMulti(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
}

DRAWFUNC(drawV)
DRAWFUNC(drawVNo)
DRAWFUNC(drawVNp)
DRAWFUNC(drawVNv)
DRAWFUNC(drawVCo)
DRAWFUNC(drawVCoNo)
DRAWFUNC(drawVCoNp)
DRAWFUNC(drawVCoNv)
DRAWFUNC(drawVCp)
DRAWFUNC(drawVCpNo)
DRAWFUNC(drawVCpNp)
DRAWFUNC(drawVCpNv)
DRAWFUNC(drawVCv)
DRAWFUNC(drawVCvNo)
DRAWFUNC(drawVCvNp)
DRAWFUNC(drawVCvNv)

#else

#define lengthIterator() \
   sglIndexPtr::iterator lit = m_lengths.begin(); \
   const int linc = m_lengths.getInc(); \
   const sglIndexPtr::iterator lend = lit + linc * m_num_prims;

#define vertexIterator() \
   const GLenum primType = getGLPrimType(); \
   sglMultiTypePtr::iterator vit = m_vertices.begin(); \
   const sglMultiTypePtr::FuncType1 vfunc = m_vertices.getFunc(); \
   const int vinc = m_vertices.getInc();

#define vertexApplyNext() \
   vfunc(vit); \
   vit += vinc;

#define texIterator() \
   unsigned int tex_index = 0; \
   GLenum num_tex = GL_TEXTURE0 + \
      SGL_MIN((GLenum)sgl::s_num_texture_units, (GLenum)tex_coords.size());

#define texApplyNext() \
   sglTexCoords::applySet(tex_coords, tex_index, num_tex); \
   ++tex_index;

#define loopBegin() \
   glBegin(primType); \
   for (const sglMultiTypePtr::iterator loop_end = vit + vinc * *lit; \
        vit != loop_end;) \
   {

#define loopEnd() \
      vertexApplyNext() \
   } \
   glEnd();

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCoNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCoNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCoNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCpNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCpNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCpNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCvNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCvNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglGeoStripSet::drawVCvNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

#endif
