/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglIndexedBitMaskSwitch.cpp
 *   Author: Michael Morrison
 *  Created: October 2001
 *  Summary: selects which child (or none or all children) to be traversed
 *****************************************************************************/

#include <sgl.h>
#include <sglIndexedBitMaskSwitch.hpp>
#include <sglOStream.hpp>

//----------------------------------------------------------------------------
sglIndexedBitMaskSwitch::sglIndexedBitMaskSwitch()
      : sglBitMaskSwitch(),
        m_switch_index(0) // defaults to a sglBitMaskSwitch
{
}

//----------------------------------------------------------------------------
sglIndexedBitMaskSwitch::~sglIndexedBitMaskSwitch()
{
}

//----------------------------------------------------------------------------
sglNode *sglIndexedBitMaskSwitch::clone(unsigned int mode) const
{
   sglIndexedBitMaskSwitch *root = new sglIndexedBitMaskSwitch;
   sglIndexedBitMaskSwitch::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglIndexedBitMaskSwitch::copyTo(sglIndexedBitMaskSwitch *dst,
                                     unsigned int mode) const
{
   sglBitMaskSwitch::copyTo(dst, mode);

   // transfer stuff
   vector<unsigned int>::const_iterator iter;
   for( iter = m_index.begin(); iter != m_index.end(); ++iter )
      dst->addMask( *iter );

   dst->setIndex(m_switch_index);
}

//----------------------------------------------------------------------------
void sglIndexedBitMaskSwitch::printInfo(ostream &ostrm,
                                        const char *indent_string) const
{
   sglBitMaskSwitch::printInfo(ostrm, indent_string);

   ostrm << indent_string << "sglIndexedBitMaskSwitch index = "
         << m_switch_index << endl;

   vector<unsigned int>::const_iterator iter;
   for( iter = m_index.begin(); iter != m_index.end(); ++iter )
      ostrm << indent_string << indent_string << "mask: " << hex
            << *iter << dec << endl;
}
