/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglIndexedGeoSet.cpp
 *  Created: 22 October 1997
 *  Summary: base class for all indexed sglGeoSets (contains the indices)
 *****************************************************************************/

#include <sglIndexedGeoSet.hpp>


//----------------------------------------------------------------------------
sglIndexedGeoSet::sglIndexedGeoSet() : sglGeoSet(),
   m_vindex(),
   m_nindex(),
   m_tindex(),
   m_cindex()
{
}


//----------------------------------------------------------------------------
sglIndexedGeoSet::~sglIndexedGeoSet()
{
}

//----------------------------------------------------------------------------
void sglIndexedGeoSet::setCoordIndexList(sglDataPtr *data,
                                         int offset,
                                         int stride)
{
   m_vindex.set(data, offset, stride);
   if (m_vindex.isValid())
      m_compute_bounds |= eVINDICES;
   else
      m_compute_bounds &= ~eVINDICES;

   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
void sglIndexedGeoSet::setTexCoordIndexList(sglDataPtr *data,
                                            int offset,
                                            int stride)
{
   m_tindex.set(data, offset, stride);
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglIndexedGeoSet::setNormalIndexList(sglDataPtr *data,
                                          int offset,
                                          int stride)
{
   m_nindex.set(data, offset, stride);
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
void sglIndexedGeoSet::setColorIndexList(sglDataPtr *data,
                                         int offset,
                                         int stride)
{
   m_cindex.set(data, offset, stride);
   dirty(eDLIST, true);
}

//----------------------------------------------------------------------------
bool sglIndexedGeoSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field = (eNUMPRIMS|
                                    eVERTICES|eVINDICES);

   if ((m_compute_bounds&test_field) == test_field)
   {
      m_bbox.makeEmpty();

      sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

      sglMultiTypePtr::iterator vbegin = m_vertices.begin();
      int vinc = m_vertices.getInc();

      sglIndexPtr::iterator iit = m_vindex.begin();
      int iinc = m_vindex.getInc();
      sglIndexPtr::iterator iend = iit + iinc * m_num_prims * getPrimSize();
      while (iit != iend)
      {
         func(&m_bbox, vbegin + vinc * *iit);
         iit += iinc;
      }

      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}
