/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglIndexedGeoStripSet.cpp
 *  Created: 28 August 1999
 *****************************************************************************/

#include <sgl.h>
#include <sglIndexedGeoStripSet.hpp>
#include <sglOStream.hpp>


sglDrawable::DrawFunc sglIndexedGeoStripSet::s_func_table[];

//----------------------------------------------------------------------------
class sglIndexedGeoStripSet::Initializer
{
public:
   Initializer() { sglIndexedGeoStripSet::setupFuncTable(); }
};
sglIndexedGeoStripSet::Initializer sglIndexedGeoStripSet::sInitializer;


//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::setupFuncTable()
{
   s_func_table[eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVT);
   s_func_table[eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVNoT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVNpT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCoNoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCoNpT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCoNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCpNoT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCpNpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCpNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCvNpT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedGeoStripSet::drawVCvNvT);
}


//----------------------------------------------------------------------------
sglIndexedGeoStripSet::sglIndexedGeoStripSet() :
    sglIndexedGeoSet(),
    m_lengths()
{
   sglIndexedGeoStripSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglIndexedGeoStripSet::~sglIndexedGeoStripSet()
{

}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::setPrimLengths(sglDataPtr *data,
                                           int offset,
                                           int stride)
{
   m_lengths.set(data, offset, stride);
   if (m_lengths.isValid())
      m_compute_bounds |= eLENGTHS;
   else
      m_compute_bounds &= ~eLENGTHS;

   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
bool sglIndexedGeoStripSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field = (eNUMPRIMS|eLENGTHS|
                                    eVERTICES|eVINDICES);

   if ((m_compute_bounds&test_field) == test_field)
   {
      m_bbox.makeEmpty();

      const sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

      sglIndexPtr::iterator iit = m_vindex.begin();
      const int iinc = m_vindex.getInc();

      const sglMultiTypePtr::iterator vbegin = m_vertices.begin();
      const int vinc = m_vertices.getInc();

      sglIndexPtr::iterator lit = m_lengths.begin();
      const int linc = m_lengths.getInc();
      const sglIndexPtr::iterator lend = lit + linc * m_num_prims;
      for (; lit != lend; lit += linc)
      {
         for (const sglIndexPtr::iterator iend = iit + iinc * *lit;
              iit != iend; iit += iinc)
         {
            func(&m_bbox, vbegin + vinc * *iit);
         }
      }

      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::addStats(sglStats& stats) const
{
   sglStats::PrimEnum prim_type = getPrimType();
   stats.addGSets(prim_type, 1);
   stats.addPrims(prim_type, m_num_prims);

   sglIndexPtr::iterator lit = m_lengths.begin();
   const int linc = m_lengths.getInc();
   const sglIndexPtr::iterator lend = lit + linc * m_num_prims;

   switch (prim_type)
   {
      case sglStats::eINDEXED_TRIFANS:
      case sglStats::eINDEXED_TRISTRIPS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            if (*lit > 2)
               stats.addTriangles(prim_type, (*lit - 2));
         }
         break;
      case sglStats::eINDEXED_QUADSTRIPS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            sglIndexType num = (*lit - 2)/2;
            if (num > 0)
               stats.addTriangles(prim_type, (num*2));
         }
         break;
      case sglStats::eINDEXED_POLYS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            if (*lit > 2)
               stats.addTriangles(prim_type, (*lit-2));
         }
         break;
      default:
         for (; lit != lend; lit += linc)
            stats.addVertices(prim_type, *lit);
         break;
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::updateDrawFunc()
{
   unsigned int mode = m_mode & (eNORMALS|eCOLORS);
   m_draw_func = s_func_table[mode];
}

#define lengthIterator() \
   sglIndexPtr::iterator lit = m_lengths.begin(); \
   const int linc = m_lengths.getInc(); \
   const sglIndexPtr::iterator lend = lit + linc * m_num_prims;

#define vertexIterator() \
   const GLenum primType = getGLPrimType(); \
   const sglMultiTypePtr::iterator vbegin = m_vertices.begin(); \
   const sglMultiTypePtr::FuncType1 vfunc = m_vertices.getFunc(); \
   const int vinc = m_vertices.getInc(); \
   sglIndexPtr::iterator viit = m_vindex.begin(); \
   const int viinc = m_vindex.getInc();

#define loopBegin() \
   glBegin(primType); \
   for (const sglIndexPtr::iterator viend = viit + viinc * *lit; \
        viit != viend; viit += viinc) \
   {

#define loopEnd() \
      vfunc(vbegin + vinc * *viit); \
   } \
   glEnd();

#define normalIterator() \
   const sglMultiTypePtr::iterator nbegin = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc(); \
   sglIndexPtr::iterator niit = m_nindex.begin(); \
   const int niinc = m_nindex.getInc();

#define normalApplyNext() \
   nfunc(nbegin + ninc * *niit); \
   niit += niinc;

#define colorIterator() \
   const sglMultiTypePtr::iterator cbegin = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc(); \
   sglIndexPtr::iterator ciit = m_cindex.begin(); \
   const int ciinc = m_cindex.getInc();

#define colorApplyNext() \
   cfunc(cbegin + cinc * *ciit); \
   ciit += ciinc;

#define texIterator() \
   sglIndexPtr::iterator tiit = m_tindex.begin(); \
   const int tiinc = m_tindex.getInc(); \
   GLenum num_tex = GL_TEXTURE0 + \
      SGL_MIN((GLenum)sgl::s_num_texture_units, (GLenum)tex_coords.size());

#define texApplyNext() \
   sglTexCoords::applySet(tex_coords, *tiit, num_tex); \
   tiit += tiinc;


//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCoNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCoNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCoNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCpNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCpNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCpNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCvNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCvNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglIndexedGeoStripSet::drawVCvNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         normalApplyNext();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNext();
         normalApplyNext();
         texApplyNext();
         loopEnd();
      }
   }
}
