/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglIndexedPointSet.cpp
 *  Created: 22 October 1997
 *  Summary: the indexed points primitives drawing code
 *****************************************************************************/

#include <sgl.h>
#include <sglIndexedPointSet.hpp>
#include <sglOStream.hpp>


sglDrawable::DrawFunc sglIndexedPointSet::s_func_table[];

//----------------------------------------------------------------------------
class sglIndexedPointSet::Initializer
{
public:
   Initializer() { sglIndexedPointSet::setupFuncTable(); }
};
sglIndexedPointSet::Initializer sglIndexedPointSet::sInitializer;


//----------------------------------------------------------------------------
void sglIndexedPointSet::setupFuncTable()
{
   s_func_table[eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVT);
   s_func_table[eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVNoT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVNvT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCoNoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCoNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCoNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedPointSet::drawVCvNvT);
}


//----------------------------------------------------------------------------
sglIndexedPointSet::sglIndexedPointSet()
      : sglIndexedGeoSet()
{
   sglIndexedPointSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglIndexedPointSet::~sglIndexedPointSet()
{
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::updateDrawFunc()
{
   unsigned int mode = m_mode & (eNORMALS|eCOLORS);
   m_draw_func = s_func_table[mode];
}


#define vertexIterator() \
   const sglMultiTypePtr::iterator vbegin = m_vertices.begin(); \
   const sglMultiTypePtr::FuncType1 vfunc = m_vertices.getFunc(); \
   const int vinc = m_vertices.getInc(); \
   sglIndexPtr::iterator viit = m_vindex.begin(); \
   const int viinc = m_vindex.getInc(); \
   const sglIndexPtr::iterator viend = viit + viinc * m_num_prims;

#define vertexApplyNext() \
   vfunc(vbegin + vinc * *viit); \
   viit += viinc;

#define normalIterator() \
   const sglMultiTypePtr::iterator nbegin = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc(); \
   sglIndexPtr::iterator niit = m_nindex.begin(); \
   const int niinc = m_nindex.getInc();

#define normalApplyNext() \
   nfunc(nbegin + ninc * *niit); \
   niit += niinc;

#define colorIterator() \
   const sglMultiTypePtr::iterator cbegin = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc(); \
   sglIndexPtr::iterator ciit = m_cindex.begin(); \
   const int ciinc = m_cindex.getInc();

#define colorApplyNext() \
   cfunc(cbegin + cinc * *ciit); \
   ciit += ciinc;

#define texIterator() \
   sglIndexPtr::iterator tiit = m_tindex.begin(); \
   const int tiinc = m_tindex.getInc(); \
   GLenum num_tex = GL_TEXTURE0 + \
      SGL_MIN((GLenum)sgl::s_num_texture_units, (GLenum)tex_coords.size());

#define texApplyNext() \
   sglTexCoords::applySet(tex_coords, *tiit, num_tex); \
   tiit += tiinc;


//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         normalApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVCoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVCoNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVCoNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         normalApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVCvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         colorApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         colorApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVCvNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         colorApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         colorApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedPointSet::drawVCvNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_POINTS);
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}
