/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglIndexedPointSet.hpp
 *  Created: 22 October 1997
 *  Summary: Approximates a pfGeoSet which contains geometry data and indices
 *         : for a set of points, and pointers to the graphics state associated
 *         : with it (sglGeoState). 
 *****************************************************************************/

#ifndef __SGL_INDEXED_POINT_SET_HPP
#define __SGL_INDEXED_POINT_SET_HPP

#include <sgl.h>
#include <sglIndexedGeoSet.hpp>

//============================================================================

/** geometry data (with index lists) for points.
 */

class SGL_DLL_API sglIndexedPointSet : public sglIndexedGeoSet
{
public:
   /// default constructor
   sglIndexedPointSet();
   /// virtual destructor
   virtual ~sglIndexedPointSet();

   /// retrieve the type of primitive
   sglStats::PrimEnum getPrimType() const {return sglStats::eINDEXED_POINTS;}
   ///
   GLenum getGLPrimType() const {return GL_POINTS;}
   ///
   unsigned int getPrimSize() const {return 1;}
   ///
   AAEnum getAAType() const {return ePOINT_AA;}

protected:
   virtual void updateDrawFunc();

private: // not implemented
   sglIndexedPointSet(const sglIndexedPointSet &);
   sglIndexedPointSet &operator=(const sglIndexedPointSet &);

   void drawVT(const vector<sglTexCoords> &tex_coords) const;
   void drawVNoT(const vector<sglTexCoords> &tex_coords) const;
   void drawVNvT(const vector<sglTexCoords> &tex_coords) const;
   void drawVCoT(const vector<sglTexCoords> &tex_coords) const;
   void drawVCoNoT(const vector<sglTexCoords> &tex_coords) const;
   void drawVCoNvT(const vector<sglTexCoords> &tex_coords) const;
   void drawVCvT(const vector<sglTexCoords> &tex_coords) const;
   void drawVCvNoT(const vector<sglTexCoords> &tex_coords) const;
   void drawVCvNvT(const vector<sglTexCoords> &tex_coords) const;

   class Initializer;
   friend class Initializer;
   static void setupFuncTable();

private:
   static DrawFunc s_func_table[(eCOLORS|eNORMALS)+1];
   static Initializer sInitializer;
};

#endif
