/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglIndexedTriangleFanSet.hpp
 *  Created: 12 August 1997
 *  Summary: Approximates a pfGeoSet which contains geometry data and indices
 *         : for triangle fans, and pointers to the graphics state associated
 *         : with it (sglGeoState). 
 *****************************************************************************/

#ifndef __SGL_INDEXED_TRIANGLE_FAN_SET_HPP
#define __SGL_INDEXED_TRIANGLE_FAN_SET_HPP

#include <sgl.h>
#include <sglIndexedGeoStripSet.hpp>

//============================================================================

/** geometry data (with index lists for triangle fans.
 */

class SGL_DLL_API sglIndexedTriangleFanSet : public sglIndexedGeoStripSet
{
public:
   /// default constructor
   sglIndexedTriangleFanSet();
   /// virtual destructor
   virtual ~sglIndexedTriangleFanSet();

   /// retrieve the type of primitive
   sglStats::PrimEnum getPrimType() const {return sglStats::eINDEXED_TRIFANS;}
   ///
   GLenum getGLPrimType() const {return GL_TRIANGLE_FAN;}
   ///
   AAEnum getAAType() const {return ePOLYGON_AA;}

   ///
   virtual sglBound::IntersectResultEnum intersectPrim(sglIntersectf &segset);
   ///
   virtual sglBound::IntersectResultEnum intersectPrim(sglIntersectd &segset);

private: // not implemented
   sglIndexedTriangleFanSet(const sglIndexedTriangleFanSet &);
   sglIndexedTriangleFanSet &operator=(const sglIndexedTriangleFanSet &);
};

#endif
