/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglIndexedTriangleSet.cpp
 *  Created: 22 October 1997
 *  Summary: the indexed tris primitives drawing code
 *****************************************************************************/

#include <sgl.h>
#include <sglIndexedTriangleSet.hpp>
#include <sglOStream.hpp>


sglDrawable::DrawFunc sglIndexedTriangleSet::s_func_table[];

//----------------------------------------------------------------------------
class sglIndexedTriangleSet::Initializer
{
public:
   Initializer() { sglIndexedTriangleSet::setupFuncTable(); }
};
sglIndexedTriangleSet::Initializer sglIndexedTriangleSet::sInitializer;


//----------------------------------------------------------------------------
void sglIndexedTriangleSet::setupFuncTable()
{
   s_func_table[eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVT);
   s_func_table[eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVNoT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVNpT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCoNoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCoNpT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCoNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCpNoT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCpNpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCpNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCvNpT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglIndexedTriangleSet::drawVCvNvT);
}


//----------------------------------------------------------------------------
sglIndexedTriangleSet::sglIndexedTriangleSet() : sglIndexedGeoSet()
{
   sglIndexedTriangleSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglIndexedTriangleSet::~sglIndexedTriangleSet()
{
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglIndexedTriangleSet::intersectPrim(
   sglIntersectf &segset)
{
   return segset.intersectIndexedTriangleSet(
      m_bbox, m_vertices, m_vindex, m_num_prims);
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglIndexedTriangleSet::intersectPrim(
   sglIntersectd &segset)
{
   return segset.intersectIndexedTriangleSet(
      m_bbox, m_vertices, m_vindex, m_num_prims);
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::updateDrawFunc()
{
   unsigned int mode = m_mode & (eNORMALS|eCOLORS);
   m_draw_func = s_func_table[mode];
}


#define vertexIterator() \
   const sglMultiTypePtr::iterator vbegin = m_vertices.begin(); \
   const sglMultiTypePtr::FuncType1 vfunc = m_vertices.getFunc(); \
   const int vinc = m_vertices.getInc(); \
   sglIndexPtr::iterator viit = m_vindex.begin(); \
   const int viinc = m_vindex.getInc(); \
   const sglIndexPtr::iterator viend = viit + viinc * m_num_prims * 3;

#define vertexApplyNext() \
   vfunc(vbegin + vinc * *viit); \
   viit += viinc;

#define normalIterator() \
   const sglMultiTypePtr::iterator nbegin = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc(); \
   sglIndexPtr::iterator niit = m_nindex.begin(); \
   const int niinc = m_nindex.getInc();

#define normalApplyNext() \
   nfunc(nbegin + ninc * *niit); \
   niit += niinc;

#define colorIterator() \
   const sglMultiTypePtr::iterator cbegin = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc(); \
   sglIndexPtr::iterator ciit = m_cindex.begin(); \
   const int ciinc = m_cindex.getInc();

#define colorApplyNext() \
   cfunc(cbegin + cinc * *ciit); \
   ciit += ciinc;

#define texIterator() \
   sglIndexPtr::iterator tiit = m_tindex.begin(); \
   const int tiinc = m_tindex.getInc(); \
   GLenum num_tex = GL_TEXTURE0 + \
      SGL_MIN((GLenum)sgl::s_num_texture_units, (GLenum)tex_coords.size());

#define texApplyNext() \
   sglTexCoords::applySet(tex_coords, *tiit, num_tex); \
   tiit += tiinc;


//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();

         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();
         vertexApplyNext();

         normalApplyNext();
         vertexApplyNext();

         normalApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCoNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCoNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();

         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCoNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();
         vertexApplyNext();

         normalApplyNext();
         vertexApplyNext();

         normalApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();

         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCpNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();

         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCpNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();

         vertexApplyNext();
         vertexApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();

         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCpNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();

         normalApplyNext();
         vertexApplyNext();

         normalApplyNext();
         vertexApplyNext();

         normalApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();

         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         vertexApplyNext();

         colorApplyNext();
         vertexApplyNext();

         colorApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCvNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         vertexApplyNext();

         colorApplyNext();
         vertexApplyNext();

         colorApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCvNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();

         colorApplyNext();
         vertexApplyNext();

         colorApplyNext();
         vertexApplyNext();

         colorApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         normalApplyNext();

         colorApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}

//----------------------------------------------------------------------------
void sglIndexedTriangleSet::drawVCvNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();
         vertexApplyNext();

         colorApplyNext();
         normalApplyNext();
         vertexApplyNext();

         colorApplyNext();
         normalApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
   else
   {
      texIterator();
      glBegin(GL_TRIANGLES);
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();

         colorApplyNext();
         normalApplyNext();
         texApplyNext();
         vertexApplyNext();
      }
      glEnd();
   }
}
