/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLODStateNode.cpp
 *   Author: Scott McMillan
 *  Created: 3 February 2000
 *  Summary:
 *****************************************************************************/

#include <sglLODStateNode.hpp>
#include <sglCull.hpp>

//----------------------------------------------------------------------------
sglLODStateNode::sglLODStateNode() : sglGroup(),
   m_lod_state()
{
}

//----------------------------------------------------------------------------
sglLODStateNode::~sglLODStateNode()
{
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglLODStateNode::intersect(
   sglIntersectf &segset) const
{
   if (m_child_list.empty())
   {
      return sglBound::eOUTSIDE;
   }
   else if (segset.getMode()&sglProbe::eLOD_VIEW)
   {
      float old_lod_scale  = segset.getLODScale();
      float old_lod_offset = segset.getLODOffset();

      segset.setLODScale(old_lod_scale * m_lod_state.getRangeScale());
      // HACK - is this the correct formula?
      segset.setLODOffset(
         (old_lod_offset*m_lod_state.getRangeScale()) +
         m_lod_state.getRangeOffset());

      sglBound::IntersectResultEnum result = sglGroup::intersect(segset);

      segset.setLODScale(old_lod_scale);
      segset.setLODOffset(old_lod_offset);

      return result;
   }
   else
   {
      return sglGroup::intersect(segset);
   }
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglLODStateNode::intersect(
   sglIntersectd &segset) const
{
   if (m_child_list.empty())
   {
      return sglBound::eOUTSIDE;
   }
   else if (segset.getMode()&sglProbe::eLOD_VIEW)
   {
      float old_lod_scale  = segset.getLODScale();
      float old_lod_offset = segset.getLODOffset();

      segset.setLODScale(old_lod_scale * m_lod_state.getRangeScale());
      // HACK - is this the correct formula?
      segset.setLODOffset(
         (old_lod_offset*m_lod_state.getRangeScale()) +
         m_lod_state.getRangeOffset());

      sglBound::IntersectResultEnum result = sglGroup::intersect(segset);

      segset.setLODScale(old_lod_scale);
      segset.setLODOffset(old_lod_offset);

      return result;
   }
   else
   {
      return sglGroup::intersect(segset);
   }
}

//----------------------------------------------------------------------------
void sglLODStateNode::pick(sglPickf &pick_state,
                           unsigned int cull_flags) const
{
   if (pick_state.getMode() & sglProbe::eLOD_VIEW)
   {
      float old_lod_scale  = pick_state.getLODScale();
      float old_lod_offset = pick_state.getLODOffset();

      pick_state.setLODScale(old_lod_scale * m_lod_state.getRangeScale());

      // HACK - is this the correct formula?
      pick_state.setLODOffset((old_lod_offset*m_lod_state.getRangeScale()) +
                              m_lod_state.getRangeOffset());

      sglGroup::pick(pick_state, cull_flags);

      pick_state.setLODScale(old_lod_scale);
      pick_state.setLODOffset(old_lod_offset);
   }
   else
   {
      sglGroup::pick(pick_state, cull_flags);
   }
}

//----------------------------------------------------------------------------
void sglLODStateNode::pick(sglPickd &pick_state,
                           unsigned int cull_flags) const
{
   if (pick_state.getMode() & sglProbe::eLOD_VIEW)
   {
      float old_lod_scale  = pick_state.getLODScale();
      float old_lod_offset = pick_state.getLODOffset();

      pick_state.setLODScale(old_lod_scale * m_lod_state.getRangeScale());

      // HACK - is this the correct formula?
      pick_state.setLODOffset((old_lod_offset*m_lod_state.getRangeScale()) +
                              m_lod_state.getRangeOffset());

      sglGroup::pick(pick_state, cull_flags);

      pick_state.setLODScale(old_lod_scale);
      pick_state.setLODOffset(old_lod_offset);
   }
   else
   {
      sglGroup::pick(pick_state, cull_flags);
   }
}

//----------------------------------------------------------------------------
void sglLODStateNode::cull(sglCull<float> &trav_state,
                           unsigned int cull_flags) const
{
   float old_lod_scale  = trav_state.getLODScale();
   float old_lod_offset = trav_state.getLODOffset();

   trav_state.setLODScale(old_lod_scale * m_lod_state.getRangeScale());

   // HACK - is this the correct formula?
   trav_state.setLODOffset(
      (old_lod_offset*m_lod_state.getRangeScale()) +
      m_lod_state.getRangeOffset());

   sglGroup::cull(trav_state, cull_flags);

   trav_state.setLODScale(old_lod_scale);
   trav_state.setLODOffset(old_lod_offset);
}

//----------------------------------------------------------------------------
void sglLODStateNode::cull(sglCull<double> &trav_state,
                           unsigned int cull_flags) const
{
   float old_lod_scale  = trav_state.getLODScale();
   float old_lod_offset = trav_state.getLODOffset();

   trav_state.setLODScale(old_lod_scale * m_lod_state.getRangeScale());

   // HACK - is this the correct formula?
   trav_state.setLODOffset(
      (old_lod_offset*m_lod_state.getRangeScale()) +
      m_lod_state.getRangeOffset());

   sglGroup::cull(trav_state, cull_flags);

   trav_state.setLODScale(old_lod_scale);
   trav_state.setLODOffset(old_lod_offset);
}

//----------------------------------------------------------------------------
sglNode *sglLODStateNode::clone(unsigned int mode) const
{
   sglLODStateNode *root = new sglLODStateNode;
   sglLODStateNode::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglLODStateNode::copyTo(sglLODStateNode *dst, unsigned int mode) const
{
   sglGroup::copyTo(dst, mode);

   // transfer stuff
   dst->setRangeScale(getRangeScale());
   dst->setRangeOffset(getRangeOffset());
}

//----------------------------------------------------------------------------
void sglLODStateNode::printInfo(ostream &ostrm,
                                const char *indent_string) const
{
   sglGroup::printInfo(ostrm, indent_string);
   ostrm << indent_string << "sglLODStateNode:" << endl;
   m_lod_state.printInfo(ostrm, indent_string);
}
