/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLight.cpp
 *   Author: Scott McMillan
 *  Created: 28 November 2000
 *****************************************************************************/

#include <sglLight.hpp>
#include <sglOStream.hpp>
#include <sglCull.hpp>

//----------------------------------------------------------------------------
sglLight::sglLight() : sglNode(),
   m_on_flag(true),
   m_ambient(0.0f, 0.0f, 0.0f, 1.0f),
   m_diffuse(1.0f, 1.0f, 1.0f, 1.0f),
   m_specular(1.0f, 1.0f, 1.0f, 1.0f)
{
}

//----------------------------------------------------------------------------
sglLight::~sglLight()
{
}

//----------------------------------------------------------------------------
void sglLight::setOn(bool on)
{
   if (on != m_on_flag)
      m_on_flag = on;
}

//----------------------------------------------------------------------------
void sglLight::cull(sglCull<float> &trav_state,
                    unsigned int cull_flags) const
{
   if (!m_on_flag)
      return;

   if (cull_flags && (m_sphere.intersect(trav_state.getFrustum(),
                                         cull_flags) == sglBound::eOUTSIDE))
   {
      return; // draw nothing
   }

   trav_state.addLight(this);
}

//----------------------------------------------------------------------------
void sglLight::cull(sglCull<double> &trav_state,
                    unsigned int cull_flags) const
{
   if (!m_on_flag)
      return;

   if (cull_flags && (m_sphere.intersect(trav_state.getFrustum(),
                                         cull_flags) == sglBound::eOUTSIDE))
   {
      return; // draw nothing
   }

   trav_state.addLight(this);
}

//----------------------------------------------------------------------------
void sglLight::setup(GLenum gl_light_number) const
{
   // How inefficient is it to apply everything?
   glLightfv(gl_light_number, GL_AMBIENT, m_ambient.getPtr());
   glLightfv(gl_light_number, GL_DIFFUSE, m_diffuse.getPtr());
   glLightfv(gl_light_number, GL_SPECULAR, m_specular.getPtr());

   // hack disable certain modes
   glLightf(gl_light_number, GL_SPOT_CUTOFF, 180.0);

   // I don't think I need to reset the attenuation parameters here.
}

//----------------------------------------------------------------------------
void sglLight::copyTo(sglLight *dst, unsigned int mode) const
{
   sglNode::copyTo(dst, mode);

   // if the bounding mode is dynamic then set it properly here.  Opposite that
   // in sglNode::copyTo(..)
   if (m_bound_mode == sglBound::eDYNAMIC)
   {
      dst->setBound(&m_sphere, sglBound::eDYNAMIC);
   }

   // transfer stuff
   dst->setOn(m_on_flag);
   dst->setAmbient(m_ambient);
   dst->setDiffuse(m_diffuse);
   dst->setSpecular(m_specular);
}

//----------------------------------------------------------------------------
void sglLight::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglNode::printInfo(ostrm, indent_string);
   ostrm << indent_string << "light " << (m_on_flag ? "enabled" : "disabled")
         << endl;
   ostrm << indent_string << "ambient:  " << m_ambient << endl;
   ostrm << indent_string << "diffuse:  " << m_diffuse << endl;
   ostrm << indent_string << "specular: " << m_specular << endl;
}
