/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglMatrix.hpp
 *  Created: 2 January 1999
 *  Summary: sglMat[234]f matrix classes and all helpful operations
 *****************************************************************************/

#ifndef __SGL_MATRIX_HPP
#define __SGL_MATRIX_HPP

#include <sgl.h>
#include <sglVector.hpp>

#include <sglMat2.hpp>
#include <sglMat3.hpp>
#include <sglMat4.hpp>

// ---------------------------------------------------------------------------
// Forward references and convenience types
// ---------------------------------------------------------------------------

typedef sglMat2<float>  sglMat2f;
typedef sglMat2<double> sglMat2d;
typedef sglMat3<float>  sglMat3f;
typedef sglMat3<double> sglMat3d;
typedef sglMat4<float>  sglMat4f;
typedef sglMat4<double> sglMat4d;

// ===========================================================================
// identity Matrices for convenience

const sglMat2f sglIdentMat2f(1.0f, 0.0f,
                             0.0f, 1.0f);
const sglMat3f sglIdentMat3f(1.0f, 0.0f, 0.0f,
                             0.0f, 1.0f, 0.0f,
                             0.0f, 0.0f, 1.0f);
const sglMat4f sglIdentMat4f(1.0f, 0.0f, 0.0f, 0.0f,
                             0.0f, 1.0f, 0.0f, 0.0f,
                             0.0f, 0.0f, 1.0f, 0.0f,
                             0.0f, 0.0f, 0.0f, 1.0f);

const sglMat2d sglIdentMat2d(1.0, 0.0,
                             0.0, 1.0);
const sglMat3d sglIdentMat3d(1.0, 0.0, 0.0,
                             0.0, 1.0, 0.0,
                             0.0, 0.0, 1.0);
const sglMat4d sglIdentMat4d(1.0, 0.0, 0.0, 0.0,
                             0.0, 1.0, 0.0, 0.0,
                             0.0, 0.0, 1.0, 0.0,
                             0.0, 0.0, 0.0, 1.0);

// ===========================================================================
// OpenGL convenience functions

inline void sglLoadMatrix(const sglMat4f& mat) { glLoadMatrixf(mat.getPtr()); }
inline void sglLoadMatrix(const sglMat4d& mat) { glLoadMatrixd(mat.getPtr()); }

inline void sglMultMatrix(const sglMat4f& mat) { glMultMatrixf(mat.getPtr()); }
inline void sglMultMatrix(const sglMat4d& mat) { glMultMatrixd(mat.getPtr()); }

#endif
