/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglMonoIndexedGeoSet.cpp
 *  Created: 28 August 1999
 *****************************************************************************/

#include <sgl.h>
#include <sglMonoIndexedGeoSet.hpp>
#include <sglOStream.hpp>


sglDrawable::DrawFunc sglMonoIndexedGeoSet::s_func_table[];

//----------------------------------------------------------------------------
class sglMonoIndexedGeoSet::Initializer
{
public:
   Initializer() { sglMonoIndexedGeoSet::setupFuncTable(); }
};
sglMonoIndexedGeoSet::Initializer sglMonoIndexedGeoSet::sInitializer;


//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::setupFuncTable()
{
   s_func_table[eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVT);
   s_func_table[eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNoT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNpT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNpT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNoT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNpT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNvT);

#ifndef SGL_NO_VERTEX_ARRAYS
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNvTm);

   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVT);
   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVNoT);
   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNpT);
   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVNvT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCoT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCoNoT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNpT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCoNvT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNoT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNpT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNvT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCvT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCvNoT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNpT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCvNvT);

   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVNvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCoNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCoNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCoNvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCpNvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCvNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawVCvNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoSet::drawPrefetchVCvNvTm);
#endif
}


//----------------------------------------------------------------------------
sglMonoIndexedGeoSet::sglMonoIndexedGeoSet() : sglGeoSet(),
   m_index(),
   m_disable_prefetch(false)
#ifndef SGL_NO_VERTEX_ARRAYS
   , m_index_min(),
   m_index_max()
#endif
{
#ifndef SGL_NO_VERTEX_ARRAYS
   m_index_min.reserve(1);
   m_index_max.reserve(1);
   m_index_min.push_back(0);
   m_index_max.push_back(0);
#endif

   sglMonoIndexedGeoSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglMonoIndexedGeoSet::~sglMonoIndexedGeoSet()
{

}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::setDisablePrefetch(bool val)
{
   m_disable_prefetch = val;
   updateDrawFunc();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::setIndexList(sglDataPtr *data,
                                        int offset,
                                        int stride)
{
   m_index.set(data, offset, stride);
   if (m_index.isValid())
      m_compute_bounds |= eVINDICES;
   else
      m_compute_bounds &= ~eVINDICES;

   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::computeNoBounds()
{
   m_dirty &= ~eBOUND;

   // Disable prefetching, because it's the simplist thing we could do.
   // Other options would be to always recompute the index min/max values,
   // but that is a lot of work and is not something that users who select
   // static bounds would want done everytime this is called (when the
   // vertices or the indices change).  Alternatively, we could create a
   // new dirty bit (eINDEX), so that the index min/max values are only
   // computed when the indices changes, and not when the vertices change.
   if (m_mode & ePREFETCH)
   {
      m_mode &= ~ePREFETCH;
      updateDrawFunc();
      dirty(eDLIST, true);
   }
}

//----------------------------------------------------------------------------
bool sglMonoIndexedGeoSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field = (eNUMPRIMS|eVERTICES|eVINDICES);

   if ((m_compute_bounds&test_field) == test_field)
   {
      m_bbox.makeEmpty();

      const sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

      sglIndexPtr::iterator iit = m_index.begin();
#ifndef SGL_NO_VERTEX_ARRAYS
      sglIndexType index_min = *iit;
      sglIndexType index_max = *iit;
#endif

      const sglMultiTypePtr::iterator vbegin = m_vertices.begin();
      const int vinc = m_vertices.getInc();

      const unsigned int prim_size = getPrimSize();
      const int iinc = m_index.getInc();
      const sglIndexPtr::iterator iend = iit + iinc * m_num_prims * prim_size;
      while (iit != iend)
      {
         const sglIndexType index = *iit;

#ifndef SGL_NO_VERTEX_ARRAYS
         if (index < index_min)
            index_min = index;
         if (index > index_max)
            index_max = index;
#endif

         func(&m_bbox, vbegin + vinc * index);

         iit += iinc;
      }
#ifndef SGL_NO_VERTEX_ARRAYS
      m_index_min[0] = index_min;
      m_index_max[0] = index_max;

      const unsigned int index_range = (index_max - index_min) + 1;
      const unsigned int num_indices = m_num_prims * prim_size;
      if (num_indices <= (unsigned int)sgl::s_max_prefetch_indices &&
          index_range <= (unsigned int)sgl::s_max_prefetch_vertices &&
          index_range <= num_indices)
      {
         if (!(m_mode & ePREFETCH))
         {
            m_mode |= ePREFETCH;
            updateDrawFunc();
            dirty(eDLIST, true);
         }
      }
      else
      {
         if (m_mode & ePREFETCH)
         {
            m_mode &= ~ePREFETCH;
            updateDrawFunc();
            dirty(eDLIST, true);
         }
      }
#endif

      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::updateDrawFunc()
{
   unsigned int mode = m_mode;

#ifndef SGL_NO_VERTEX_ARRAYS
   if (m_disable_prefetch)
      mode &= ~ePREFETCH;
#else
   mode &= ~(eMULTI_TEXTURE|ePREFETCH);
#endif

   m_draw_func = s_func_table[mode];
}


#define normalIterator() \
   sglMultiTypePtr::iterator nit = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc();

#define normalApplyNext() \
   nfunc(nit); \
   nit += ninc;

#define colorIterator() \
   sglMultiTypePtr::iterator cit = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc();

#define colorApplyNext() \
   cfunc(cit); \
   cit += cinc;

#define indexIterator() \
   sglIndexPtr::iterator iit = m_index.begin(); \
   const int iinc = m_index.getInc(); \
   const sglIndexPtr::iterator iend = iit + num_elems * iinc;

#define primLoop() \
      for (const sglIndexPtr::iterator prim_end = iit + iinc * prim_size; \
           iit != prim_end; iit += iinc) \
         glArrayElement(*iit);


#ifndef SGL_NO_VERTEX_ARRAYS

#define drawPrefetchV() \
{ \
   sgl::drawRangeElements(getGLPrimType(), \
                          m_index_min[0], m_index_max[0], \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVNo() \
{ \
   m_normals.apply0(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVNv() \
{ \
   m_normals.applyArray(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVCo() \
{ \
   m_colors.apply0(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVCoNo() \
{ \
   m_colors.apply0(); \
   m_normals.apply0(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVCoNv() \
{ \
   m_colors.apply0(); \
   m_normals.applyArray(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVCv() \
{ \
   m_colors.applyArray(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVCvNo() \
{ \
   m_colors.applyArray(); \
   m_normals.apply0(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawPrefetchVCvNv() \
{ \
   m_colors.applyArray(); \
   m_normals.applyArray(); \
   sgl::drawRangeElements(getGLPrimType(),  \
                          m_index_min[0], m_index_max[0],  \
                          num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawV() \
{ \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVNo() \
{ \
   m_normals.apply0(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVNp() \
{ \
   normalIterator() \
   indexIterator() \
   glBegin(getGLPrimType()); \
   while (iit != iend) \
   { \
      normalApplyNext() \
      primLoop() \
   } \
   glEnd(); \
}


#define drawVNv() \
{ \
   m_normals.applyArray(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVCo() \
{ \
   m_colors.apply0(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVCoNo() \
{ \
   m_colors.apply0(); \
   m_normals.apply0(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVCoNp() \
{ \
   m_colors.apply0(); \
   normalIterator() \
   indexIterator() \
   glBegin(getGLPrimType()); \
   while (iit != iend) \
   { \
      normalApplyNext() \
      primLoop() \
   } \
   glEnd(); \
}

#define drawVCoNv() \
{ \
   m_normals.applyArray(); \
   m_colors.apply0(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVCp() \
{ \
   colorIterator() \
   indexIterator() \
   glBegin(getGLPrimType()); \
   while (iit != iend) \
   { \
      colorApplyNext() \
      primLoop() \
   } \
   glEnd(); \
}

#define drawVCpNo() \
{ \
   m_normals.apply0(); \
   colorIterator() \
   indexIterator() \
   glBegin(getGLPrimType()); \
   while (iit != iend) \
   { \
      colorApplyNext() \
      primLoop() \
   } \
   glEnd(); \
}

#define drawVCpNp() \
{ \
   colorIterator() \
   normalIterator() \
   indexIterator() \
   glBegin(getGLPrimType()); \
   while (iit != iend) \
   { \
      colorApplyNext() \
      normalApplyNext() \
      primLoop() \
   } \
   glEnd(); \
}

#define drawVCpNv() \
{ \
   m_normals.applyArray(); \
   colorIterator() \
   indexIterator() \
   glBegin(getGLPrimType()); \
   while (iit != iend) \
   { \
      colorApplyNext() \
      primLoop() \
   } \
   glEnd(); \
}

#define drawVCv() \
{ \
   m_colors.applyArray(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVCvNo() \
{ \
   m_colors.applyArray(); \
   m_normals.apply0(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}

#define drawVCvNp() \
{ \
   m_colors.applyArray(); \
   normalIterator() \
   indexIterator() \
   glBegin(getGLPrimType()); \
   while (iit != iend) \
   { \
      normalApplyNext() \
      primLoop() \
   } \
   glEnd(); \
}

#define drawVCvNv() \
{ \
   m_colors.applyArray(); \
   m_normals.applyArray(); \
   glDrawElements(getGLPrimType(), num_elems, GLINDEXTYPE, m_index.begin()); \
}


#define DRAWFUNC(a) \
void sglMonoIndexedGeoSet::a##T( \
   const vector<sglTexCoords> &tex_coords) const\
{ \
   const unsigned int prim_size = getPrimSize(); \
   const unsigned int num_elems = m_num_prims * prim_size; \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySet(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
} \
void sglMonoIndexedGeoSet::a##Tm( \
   const vector<sglTexCoords> &tex_coords) const\
{ \
   const unsigned int prim_size = getPrimSize(); \
   const unsigned int num_elems = m_num_prims * prim_size; \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySetMulti(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
}

DRAWFUNC(drawPrefetchV)
DRAWFUNC(drawPrefetchVNo)
DRAWFUNC(drawPrefetchVNv)
DRAWFUNC(drawPrefetchVCo)
DRAWFUNC(drawPrefetchVCoNo)
DRAWFUNC(drawPrefetchVCoNv)
DRAWFUNC(drawPrefetchVCv)
DRAWFUNC(drawPrefetchVCvNo)
DRAWFUNC(drawPrefetchVCvNv)
DRAWFUNC(drawV)
DRAWFUNC(drawVNo)
DRAWFUNC(drawVNp)
DRAWFUNC(drawVNv)
DRAWFUNC(drawVCo)
DRAWFUNC(drawVCoNo)
DRAWFUNC(drawVCoNp)
DRAWFUNC(drawVCoNv)
DRAWFUNC(drawVCp)
DRAWFUNC(drawVCpNo)
DRAWFUNC(drawVCpNp)
DRAWFUNC(drawVCpNv)
DRAWFUNC(drawVCv)
DRAWFUNC(drawVCvNo)
DRAWFUNC(drawVCvNp)
DRAWFUNC(drawVCvNv)


#else

#define vertexIterator() \
   const sglMultiTypePtr::iterator vbegin = m_vertices.begin(); \
   const sglMultiTypePtr::FuncType1 vfunc = m_vertices.getFunc(); \
   const int vinc = m_vertices.getInc(); \
   const unsigned int prim_size = getPrimSize(); \
   sglIndexPtr::iterator viit = m_index.begin(); \
   const int viinc = m_index.getInc(); \
   const sglIndexPtr::iterator viend = viit + viinc * m_num_prims * prim_size;

#define vertexApplyNext() \
   vfunc(vbegin + vinc * *viit); \
   viit += viinc;

#define loopBegin() \
   for (const sglIndexPtr::iterator loop_end = viit + viinc * prim_size; \
        viit != loop_end;) \
   {

#define loopEnd() \
      vertexApplyNext() \
   }

#define normalIteratorIndexed() \
   const sglMultiTypePtr::iterator nbegin = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc();

#define normalApplyNextIndexed() \
   nfunc(nbegin + ninc * *viit);

#define colorIteratorIndexed() \
   const sglMultiTypePtr::iterator cbegin = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc();

#define colorApplyNextIndexed() \
   cfunc(cbegin + cinc * *viit);

#define texIterator() \
   GLenum num_tex = GL_TEXTURE0 + \
      SGL_MIN((GLenum)sgl::s_num_texture_units, (GLenum)tex_coords.size());

#define texApplyNext() \
   sglTexCoords::applySet(tex_coords, *viit, num_tex);

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   normalIteratorIndexed();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         normalApplyNextIndexed();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         normalApplyNextIndexed();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCoNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCoNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCoNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   m_colors.apply0();
   normalIteratorIndexed();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         normalApplyNextIndexed();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         normalApplyNextIndexed();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCpNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCpNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCpNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIterator();
   normalIteratorIndexed();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIteratorIndexed();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         colorApplyNextIndexed();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         colorApplyNextIndexed();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCvNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIteratorIndexed();
   m_normals.apply0();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         colorApplyNextIndexed();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         colorApplyNextIndexed();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCvNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIteratorIndexed();
   normalIterator();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
   glEnd();
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoSet::drawVCvNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   vertexIterator();
   colorIteratorIndexed();
   normalIteratorIndexed();

   glBegin(getGLPrimType());
   if (tex_coords.empty())
   {
      while (viit != viend)
      {
         colorApplyNextIndexed();
         normalApplyNextIndexed();
         vertexApplyNext();
      }
   }
   else
   {
      texIterator();
      while (viit != viend)
      {
         colorApplyNextIndexed();
         normalApplyNextIndexed();
         texApplyNext();
         vertexApplyNext();
      }
   }
   glEnd();
}

#endif
