/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglMonoIndexedGeoStripSet.cpp
 *  Created: 28 August 1999
 *****************************************************************************/

#include <sgl.h>
#include <sglMonoIndexedGeoStripSet.hpp>
#include <sglOStream.hpp>


sglDrawable::DrawFunc sglMonoIndexedGeoStripSet::s_func_table[];

//----------------------------------------------------------------------------
class sglMonoIndexedGeoStripSet::Initializer
{
public:
   Initializer() { sglMonoIndexedGeoStripSet::setupFuncTable(); }
};
sglMonoIndexedGeoStripSet::Initializer sglMonoIndexedGeoStripSet::sInitializer;


//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::setupFuncTable()
{
   s_func_table[eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVT);
   s_func_table[eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVNoT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVNpT);
   s_func_table[eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVNvT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoNoT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoNpT);
   s_func_table[eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoNvT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpNoT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpNpT);
   s_func_table[eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpNvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvNoT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvNpT);
   s_func_table[eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvNvT);

#ifndef SGL_NO_VERTEX_ARRAYS
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCoNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCpNvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvNoTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvNpTm);
   s_func_table[eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawVCvNvTm);

   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVT);
   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVNoT);
   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVNpT);
   s_func_table[ePREFETCH|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVNvT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoNoT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoNpT);
   s_func_table[ePREFETCH|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoNvT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpNoT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpNpT);
   s_func_table[ePREFETCH|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpNvT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvNoT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvNpT);
   s_func_table[ePREFETCH|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvNvT);

   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_NONE|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVNvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_OVERALL|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCoNvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_PRIM|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCpNvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_NONE] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_OVERALL] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvNoTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_PRIM] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvNpTm);
   s_func_table[ePREFETCH|eMULTI_TEXTURE|eCOLORS_PER_VERTEX|eNORMALS_PER_VERTEX] =
      reinterpret_cast<DrawFunc>(&sglMonoIndexedGeoStripSet::drawPrefetchVCvNvTm);
#endif
}


//----------------------------------------------------------------------------
sglMonoIndexedGeoStripSet::sglMonoIndexedGeoStripSet() :
    sglMonoIndexedGeoSet(),
    m_lengths()
{
   sglMonoIndexedGeoStripSet::updateDrawFunc();
}

//----------------------------------------------------------------------------
sglMonoIndexedGeoStripSet::~sglMonoIndexedGeoStripSet()
{

}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::setPrimLengths(sglDataPtr *data,
                                               int offset,
                                               int stride)
{
   m_lengths.set(data, offset, stride);
   if (m_lengths.isValid())
      m_compute_bounds |= eLENGTHS;
   else
      m_compute_bounds &= ~eLENGTHS;

   dirty(eBOUND|eDLIST, true);
}

//----------------------------------------------------------------------------
bool sglMonoIndexedGeoStripSet::computeBounds()
{
   m_dirty &= ~eBOUND;

   // make sure all the components are present
   const unsigned int test_field = (eNUMPRIMS|eLENGTHS|
                                    eVERTICES|eVINDICES);

   if ((m_compute_bounds&test_field) == test_field)
   {
#ifndef SGL_NO_VERTEX_ARRAYS
      unsigned int num_prims = m_num_prims;
      if (m_index_min.size() != num_prims)
      {
         m_index_min.clear();
         m_index_min.insert(m_index_min.end(), num_prims, 0);
      }
      if (m_index_max.size() != num_prims)
      {
         m_index_max.clear();
         m_index_max.insert(m_index_max.end(), num_prims, 0);
      }
      vector<sglIndexType>::iterator minit = m_index_min.begin();
      vector<sglIndexType>::iterator maxit = m_index_max.begin();
#endif

      m_bbox.makeEmpty();

      sglBoxBound::ExpandToIncludeFunc func =
         sglBoxBound::getExpandToIncludeFunc(m_vertices.getTypeIndex());

#ifndef SGL_NO_VERTEX_ARRAYS
      unsigned int num_prefetchable = 0;
#endif

      sglIndexPtr::iterator iit = m_index.begin();
      int iinc = m_index.getInc();

      sglMultiTypePtr::iterator vbegin = m_vertices.begin();
      int vinc = m_vertices.getInc();

      sglIndexPtr::iterator lit = m_lengths.begin();
      int linc = m_lengths.getInc();
      sglIndexPtr::iterator lend = lit + linc * m_num_prims;
      for (; lit != lend; lit += linc)
      {
         sglIndexType index = *iit;
#ifndef SGL_NO_VERTEX_ARRAYS
         sglIndexType min_index = index;
         sglIndexType max_index = index;
#endif

         sglIndexType length = *lit;
         for (sglIndexPtr::iterator iend = iit + iinc * length;
              iit != iend; iit += iinc)
         {
            index = *iit;

#ifndef SGL_NO_VERTEX_ARRAYS
            if (index < min_index)
               min_index = index;
            if (index > max_index)
               max_index = index;
#endif

            func(&m_bbox, vbegin + vinc * index);
         }

#ifndef SGL_NO_VERTEX_ARRAYS
         *minit = min_index;
         *maxit = max_index;
         ++minit;
         ++maxit;

         sglIndexType index_range = (max_index - min_index) + 1;
         if (length <= (sglIndexType)sgl::s_max_prefetch_indices &&
             index_range <= (sglIndexType)sgl::s_max_prefetch_vertices &&
             index_range <= length)
         {
            ++num_prefetchable;
         }
#endif
      }

#ifndef SGL_NO_VERTEX_ARRAYS
      // We _should_ turn on and off prefetching for each primitive,
      // but I don't want to add an extra if test inside the for loop in
      // the draw function.  Instead, we'll only turn on prefetching
      // if the vast majority of primitives are prefetchable.
      if ((float)num_prefetchable >= (float)num_prims * .75f)
      {
         if (!(m_mode & ePREFETCH))
         {
            m_mode |= ePREFETCH;
            updateDrawFunc();
            dirty(eDLIST, true);
         }
      }
      else
      {
         if (m_mode & ePREFETCH)
         {
            m_mode &= ~ePREFETCH;
            updateDrawFunc();
            dirty(eDLIST, true);
         }
      }
#endif

      return true;
   }
   else
   {
      m_bbox.makeEmpty();
      return false;
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::addStats(sglStats &stats) const
{
   sglStats::PrimEnum prim_type = getPrimType();

   stats.addGSets(prim_type, 1);
   stats.addPrims(prim_type, m_num_prims);

   sglIndexPtr::iterator lit = m_lengths.begin();
   int linc = m_lengths.getInc();
   sglIndexPtr::iterator lend = lit + linc * m_num_prims;

   switch (prim_type)
   {
      case sglStats::eMONO_INDEXED_TRIFANS:
      case sglStats::eMONO_INDEXED_TRISTRIPS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            if (*lit > 2)
               stats.addTriangles(prim_type, (*lit - 2));
         }
         break;
      case sglStats::eMONO_INDEXED_QUADSTRIPS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            sglIndexType num = (*lit - 2)/2;
            if (num > 0)
               stats.addTriangles(prim_type, (num*2));
         }
         break;
      case sglStats::eMONO_INDEXED_POLYS:
         for (; lit != lend; lit += linc)
         {
            stats.addVertices(prim_type, *lit);
            if (*lit > 2)
               stats.addTriangles(prim_type, (*lit-2));
         }
         break;
      default:
         for (; lit != lend; lit += linc)
            stats.addVertices(prim_type, *lit);
         break;
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::updateDrawFunc()
{
   unsigned int mode = m_mode;

#ifndef SGL_NO_VERTEX_ARRAYS
   if (m_disable_prefetch)
      mode &= ~ePREFETCH;
#else
   mode &= ~(eMULTI_TEXTURE|ePREFETCH);
#endif

   m_draw_func = s_func_table[mode];
}

#define normalIterator() \
   sglMultiTypePtr::iterator nit = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc();

#define normalApplyNext() \
   nfunc(nit); \
   nit += ninc;

#define colorIterator() \
   sglMultiTypePtr::iterator cit = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc();

#define colorApplyNext() \
   cfunc(cit); \
   cit += cinc;

#define prefetchLoop() \
   vector<sglIndexType>::const_iterator minit = m_index_min.begin(); \
   vector<sglIndexType>::const_iterator maxit = m_index_max.begin(); \
   sglIndexPtr::iterator lit = m_lengths.begin(); \
   const int linc = m_lengths.getInc(); \
   const sglIndexPtr::iterator lend = lit + linc * m_num_prims; \
   for (; lit != lend; lit += linc, ++minit, ++maxit)

#define prefetchDraw() \
      const sglIndexType length = *lit; \
      sgl::drawRangeElements(primType, *minit, *maxit, length, \
                             GLINDEXTYPE, iit); \
      iit += length * iinc;

#define lengthLoop() \
   sglIndexPtr::iterator lit = m_lengths.begin(); \
   const int linc = m_lengths.getInc(); \
   const sglIndexPtr::iterator lend = lit + linc * m_num_prims; \
   for (; lit != lend; lit += linc)

#define drawElements() \
      const sglIndexType length = *lit; \
      glDrawElements(primType, length, GLINDEXTYPE, iit); \
      iit += length * iinc;


#ifndef SGL_NO_VERTEX_ARRAYS

#define drawPrefetchV() \
{ \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVNo() \
{ \
   m_normals.apply0(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVNp() \
{ \
   normalIterator() \
   prefetchLoop() \
   { \
      normalApplyNext(); \
      prefetchDraw() \
   } \
}

#define drawPrefetchVNv() \
{ \
   m_normals.applyArray(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCo() \
{ \
   m_colors.apply0(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCoNo() \
{ \
   m_colors.apply0(); \
   m_normals.apply0(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCoNp() \
{ \
   m_colors.apply0(); \
   normalIterator() \
   prefetchLoop() \
   { \
      normalApplyNext() \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCoNv() \
{ \
   m_colors.apply0(); \
   m_normals.applyArray(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCp() \
{ \
   colorIterator() \
   prefetchLoop() \
   { \
      colorApplyNext() \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCpNo() \
{ \
   colorIterator() \
   m_normals.apply0(); \
   prefetchLoop() \
   { \
      colorApplyNext() \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCpNp() \
{ \
   colorIterator() \
   normalIterator() \
   prefetchLoop() \
   { \
      colorApplyNext() \
      normalApplyNext() \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCpNv() \
{ \
   colorIterator() \
   m_normals.applyArray(); \
   prefetchLoop() \
   { \
      colorApplyNext() \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCv() \
{ \
   m_colors.applyArray(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCvNo() \
{ \
   m_colors.applyArray(); \
   m_normals.apply0(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCvNp() \
{ \
   m_colors.applyArray(); \
   normalIterator() \
   prefetchLoop() \
   { \
      normalApplyNext() \
      prefetchDraw() \
   } \
}

#define drawPrefetchVCvNv() \
{ \
   m_colors.applyArray(); \
   m_normals.applyArray(); \
   prefetchLoop() \
   { \
      prefetchDraw() \
   } \
}

#define drawV() \
{ \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVNo() \
{ \
   m_normals.apply0(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVNp() \
{ \
   normalIterator() \
   lengthLoop() \
   { \
      normalApplyNext() \
      drawElements() \
   } \
}

#define drawVNv() \
{ \
   m_normals.applyArray(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVCo() \
{ \
   m_colors.apply0(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVCoNo() \
{ \
   m_colors.apply0(); \
   m_normals.apply0(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVCoNp() \
{ \
   m_colors.apply0(); \
   normalIterator() \
   lengthLoop() \
   { \
      normalApplyNext() \
      drawElements() \
   } \
}

#define drawVCoNv() \
{ \
   m_colors.apply0(); \
   m_normals.applyArray(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVCp() \
{ \
   colorIterator() \
   lengthLoop() \
   { \
      colorApplyNext() \
      drawElements() \
   } \
}

#define drawVCpNo() \
{ \
   colorIterator() \
   m_normals.apply0(); \
   lengthLoop() \
   { \
      colorApplyNext() \
      drawElements() \
   } \
}

#define drawVCpNp() \
{ \
   colorIterator() \
   normalIterator() \
   lengthLoop() \
   { \
      colorApplyNext() \
      normalApplyNext() \
      drawElements() \
   } \
}

#define drawVCpNv() \
{ \
   colorIterator() \
   m_normals.applyArray(); \
   lengthLoop() \
   { \
      colorApplyNext() \
      drawElements() \
   } \
}

#define drawVCv() \
{ \
   m_colors.applyArray(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVCvNo() \
{ \
   m_colors.applyArray(); \
   m_normals.apply0(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}

#define drawVCvNp() \
{ \
   m_colors.applyArray(); \
   normalIterator() \
   lengthLoop() \
   { \
      normalApplyNext() \
      drawElements() \
   } \
}

#define drawVCvNv() \
{ \
   m_colors.applyArray(); \
   m_normals.applyArray(); \
   lengthLoop() \
   { \
      drawElements() \
   } \
}


#define DRAWFUNC(a) \
void sglMonoIndexedGeoStripSet::a##T( \
   const vector<sglTexCoords> &tex_coords) const\
{ \
   const GLenum primType = getGLPrimType(); \
   sglIndexPtr::iterator iit = m_index.begin(); \
   const int iinc = m_index.getInc(); \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySet(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
} \
void sglMonoIndexedGeoStripSet::a##Tm( \
   const vector<sglTexCoords> &tex_coords) const\
{ \
   const GLenum primType = getGLPrimType(); \
   sglIndexPtr::iterator iit = m_index.begin(); \
   const int iinc = m_index.getInc(); \
   glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT); \
   sglTexCoords::applyArraySetMulti(tex_coords); \
   m_vertices.applyArray(); \
   a(); \
   glPopClientAttrib(); \
}

DRAWFUNC(drawPrefetchV)
DRAWFUNC(drawPrefetchVNo)
DRAWFUNC(drawPrefetchVNp)
DRAWFUNC(drawPrefetchVNv)
DRAWFUNC(drawPrefetchVCo)
DRAWFUNC(drawPrefetchVCoNo)
DRAWFUNC(drawPrefetchVCoNp)
DRAWFUNC(drawPrefetchVCoNv)
DRAWFUNC(drawPrefetchVCp)
DRAWFUNC(drawPrefetchVCpNo)
DRAWFUNC(drawPrefetchVCpNp)
DRAWFUNC(drawPrefetchVCpNv)
DRAWFUNC(drawPrefetchVCv)
DRAWFUNC(drawPrefetchVCvNo)
DRAWFUNC(drawPrefetchVCvNp)
DRAWFUNC(drawPrefetchVCvNv)
DRAWFUNC(drawV)
DRAWFUNC(drawVNo)
DRAWFUNC(drawVNp)
DRAWFUNC(drawVNv)
DRAWFUNC(drawVCo)
DRAWFUNC(drawVCoNo)
DRAWFUNC(drawVCoNp)
DRAWFUNC(drawVCoNv)
DRAWFUNC(drawVCp)
DRAWFUNC(drawVCpNo)
DRAWFUNC(drawVCpNp)
DRAWFUNC(drawVCpNv)
DRAWFUNC(drawVCv)
DRAWFUNC(drawVCvNo)
DRAWFUNC(drawVCvNp)
DRAWFUNC(drawVCvNv)

#else

#define lengthIterator() \
   sglIndexPtr::iterator lit = m_lengths.begin(); \
   const int linc = m_lengths.getInc(); \
   const sglIndexPtr::iterator lend = lit + linc * m_num_prims;

#define vertexIterator() \
   const GLenum primType = getGLPrimType(); \
   const sglMultiTypePtr::iterator vbegin = m_vertices.begin(); \
   const sglMultiTypePtr::FuncType1 vfunc = m_vertices.getFunc(); \
   const int vinc = m_vertices.getInc(); \
   sglIndexPtr::iterator viit = m_index.begin(); \
   const int viinc = m_index.getInc();

#define loopBegin() \
   glBegin(primType); \
   for (const sglIndexPtr::iterator viend = viit + viinc * *lit; \
        viit != viend; viit += viinc) \
   {

#define loopEnd() \
      vfunc(vbegin + vinc * *viit); \
   } \
   glEnd();

#define normalIteratorIndexed() \
   const sglMultiTypePtr::iterator nbegin = m_normals.begin(); \
   const sglMultiTypePtr::FuncType1 nfunc = m_normals.getFunc(); \
   const int ninc = m_normals.getInc();

#define normalApplyNextIndexed() \
   nfunc(nbegin + ninc * *viit);

#define colorIteratorIndexed() \
   const sglMultiTypePtr::iterator cbegin = m_colors.begin(); \
   const sglMultiTypePtr::FuncType1 cfunc = m_colors.getFunc(); \
   const int cinc = m_colors.getInc();

#define colorApplyNextIndexed() \
   cfunc(cbegin + cinc * *viit);

#define texIterator() \
   GLenum num_tex = GL_TEXTURE0 + \
      SGL_MIN((GLenum)sgl::s_num_texture_units, (GLenum)tex_coords.size());

#define texApplyNext() \
   sglTexCoords::applySet(tex_coords, *viit, num_tex);


//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   normalIteratorIndexed();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCoNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCoNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCoNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   m_colors.apply0();
   normalIteratorIndexed();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         normalApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCpNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCpNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         normalApplyNext();
         loopBegin();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCpNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIterator();
   normalIteratorIndexed();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         colorApplyNext();
         loopBegin();
         normalApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIteratorIndexed();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCvNoT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIteratorIndexed();
   m_normals.apply0();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCvNpT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIteratorIndexed();
   normalIterator();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         normalApplyNext();
         loopBegin();
         colorApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
}

//----------------------------------------------------------------------------
void sglMonoIndexedGeoStripSet::drawVCvNvT(
   const vector<sglTexCoords> &tex_coords) const
{
   lengthIterator();
   vertexIterator();
   colorIteratorIndexed();
   normalIteratorIndexed();

   if (tex_coords.empty())
   {
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNextIndexed();
         normalApplyNextIndexed();
         loopEnd();
      }
   }
   else
   {
      texIterator();
      for (; lit != lend; lit += linc)
      {
         loopBegin();
         colorApplyNextIndexed();
         normalApplyNextIndexed();
         texApplyNext();
         loopEnd();
      }
   }
}

#endif
