/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglObject.cpp
 *  Created: 30 December 1997
 *  Summary: Base class for all scene graph objects (not just nodes)
 *****************************************************************************/

#include <sgl.h>
#include <string.h>
#include <typeinfo>
#include <sglObject.hpp>
#include <sglOStream.hpp>

//----------------------------------------------------------------------------
sglObject::sglObject() :
      m_ref_count(0),
      m_user_data(NULL),
      m_name()
{
}

//----------------------------------------------------------------------------
sglObject::~sglObject()
{
   if (m_ref_count)
   {
      sglPrintWarning() << "~sgl warning: deleting sglObject at " << this
                        << " with count = " << m_ref_count << endl
                        << "<" << endl
                        << *this
                        << ">" << endl;
   }
}

//----------------------------------------------------------------------------
void sglObject::copyTo(sglObject *dst, unsigned int mode) const
{
   if (this != dst)
   {
      dst->setName(m_name);

      if (mode & eCLONE_USER_DATA)
      {
         dst->setUserData(m_user_data);
      }
   }
}

//----------------------------------------------------------------------------
void sglObject::printInfo(ostream &ostrm, const char *indent_string) const
{
   ostrm << indent_string << "Type: " << typeid(*this).name() << endl;
   ostrm << indent_string << "Node address: " << this << endl;
   ostrm << indent_string << "Reference count = " << m_ref_count << endl;
   ostrm << indent_string << "name = " << m_name << endl;
   ostrm << indent_string << "user_data = " << m_user_data << endl;
   if (m_user_data)
      m_user_data->printInfo(ostrm, indent_string);
}
