/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglOrderedGroup.cpp
 *  Created: 14 November 2001
 *  Summary: Group that enforces a strict rendering order.
 *****************************************************************************/

#include <sglOrderedGroup.hpp>
#include <sglCull.hpp>


//----------------------------------------------------------------------------
sglOrderedGroup::sglOrderedGroup() : sglGroup()
{
}

//----------------------------------------------------------------------------
sglOrderedGroup::~sglOrderedGroup()
{
}


//----------------------------------------------------------------------------
void sglOrderedGroup::cull(sglCull<float> &trav_state,
                           unsigned int cull_flags) const
{
   if (cull_flags && (m_sphere.intersect(trav_state.getFrustum(),
                                         cull_flags) == sglBound::eOUTSIDE))
   {
      return; // draw nothing
   }

   if (m_child_list.size() > 1)
   {
      trav_state.pushDrawablePool();

      for (vector<sglNode*>::const_iterator it = m_child_list.begin();
           it != m_child_list.end(); ++it)
      {
         (*it)->cull(trav_state, cull_flags);
      }

      trav_state.popDrawablePool();
   }
   else
   {
      for (vector<sglNode*>::const_iterator it = m_child_list.begin();
           it != m_child_list.end(); ++it)
      {
         (*it)->cull(trav_state, cull_flags);
      }
   }
}

//----------------------------------------------------------------------------
void sglOrderedGroup::cull(sglCull<double> &trav_state,
                           unsigned int cull_flags) const
{
   if (cull_flags && (m_sphere.intersect(trav_state.getFrustum(),
                                         cull_flags) == sglBound::eOUTSIDE))
   {
      return; // draw nothing
   }

   if (m_child_list.size() > 1)
   {
      trav_state.pushDrawablePool();

      for (vector<sglNode*>::const_iterator it = m_child_list.begin();
           it != m_child_list.end(); ++it)
      {
         (*it)->cull(trav_state, cull_flags);
      }

      trav_state.popDrawablePool();
   }
   else
   {
      for (vector<sglNode*>::const_iterator it = m_child_list.begin();
           it != m_child_list.end(); ++it)
      {
         (*it)->cull(trav_state, cull_flags);
      }
   }
}

//----------------------------------------------------------------------------
sglNode *sglOrderedGroup::clone(unsigned int mode) const
{
   sglOrderedGroup *root = new sglOrderedGroup();
   sglOrderedGroup::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglOrderedGroup::copyTo(sglOrderedGroup *dst, unsigned int mode) const
{
   sglGroup::copyTo(dst, mode);
}

//----------------------------------------------------------------------------
void sglOrderedGroup::printInfo(ostream &ostrm,
                                const char *indent_string) const
{
   sglGroup::printInfo(ostrm, indent_string);
}
