/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglOrderedGroup.hpp
 *  Created: 14 November 2001
 *  Summary: Group that enforces a strict rendering order.
 *****************************************************************************/

#ifndef __SGL_ORDERED_GROUP_HPP
#define __SGL_ORDERED_GROUP_HPP

#include <sgl.h>
#include <sglGroup.hpp>

//----------------------------------------------------------------------------

/**

An sglOrderedGroup node is a specialized group that ensures that the
subtree of child i is rendered before the subtree of i+1.  This node disrupts
the state sorting process, but allows certain multipass algorithms to be
performed within a single draw traversal.  Other than this feature, this node
behaves (and uses the interface for) sglGroup.

*/

class SGL_DLL_API sglOrderedGroup : public sglGroup
{
public:
   /** Constructor
   */
   sglOrderedGroup();
   /// virtual destructor
   virtual ~sglOrderedGroup();

   // See sglNode::clone() for details.
   virtual sglNode *clone(unsigned int mode) const;

   // See sglObject::printInfo() for details.
   virtual void printInfo(ostream &ostrm, const char *indent_string) const;

protected:
   // See sglNode::cull(f) for details
   virtual void cull(sglCull<float> &trav_state,
                     unsigned int cull_flags) const;

   // See sglNode::cull(d) for details
   virtual void cull(sglCull<double> &trav_state,
                     unsigned int cull_flags) const;

   void copyTo(sglOrderedGroup *dst, unsigned int mode) const;

private:  // not implemented
   sglOrderedGroup(const sglOrderedGroup &);
   sglOrderedGroup &operator=(const sglOrderedGroup &);
};

#endif
