/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglOrthographicCamera.cpp
 *   Author: Scott McMillan
 *  Created: 19 December 2000
 *****************************************************************************/

#include <sglOrthographicCamera.hpp>
#include <sglOStream.hpp>

//----------------------------------------------------------------------------
sglOrthographicCamera::sglOrthographicCamera() :
      sglCamera(),
      m_left(-10.0),
      m_right(10.0),
      m_bottom(-10.0),
      m_top(10.0)
{
   buildFrustum();
}
      
//----------------------------------------------------------------------------
sglOrthographicCamera::~sglOrthographicCamera()
{
}

//----------------------------------------------------------------------------
void sglOrthographicCamera::setOrtho(double left, double right,
                                     double bottom, double top,
                                     double near_clip, double far_clip)
{
   m_left = left;
   m_right = right;
   m_bottom = bottom;
   m_top = top;
   m_near = near_clip;
   m_far = far_clip;

   buildFrustum();
}

//----------------------------------------------------------------------------
void sglOrthographicCamera::getOrtho(double &left, double &right,
                                     double &bottom, double &top,
                                     double &near_clip,
                                     double &far_clip) const
{
   left = m_left;
   right = m_right;
   bottom = m_bottom;
   top = m_top;
   near_clip = m_near;
   far_clip = m_far;
}

//----------------------------------------------------------------------------
void sglOrthographicCamera::buildFrustum()
{
   if (!m_frustum.buildOrtho(m_left, m_right, m_bottom, m_top, m_near, m_far))
   {
      sglPrintWarning() << "sglOrthographicCamera error: "
                        << "sglFrustum::buildOrtho failed" << endl;
   }
}

//----------------------------------------------------------------------------
void sglOrthographicCamera::applyProjection() const
{
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(m_left, m_right, m_bottom, m_top, m_near, m_far);
}

//----------------------------------------------------------------------------
void sglOrthographicCamera::printInfo(ostream &ostrm,
                                      const char *indent_string) const
{
   sglCamera::printInfo(ostrm, indent_string);
   ostrm << indent_string << "right/left = " << m_right << "/" << m_left
         << endl;
   ostrm << indent_string << "bottom/top = " << m_bottom << "/" << m_top
         << endl;
}

