/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglPick.cpp
 *  Created: 18 April 2001
 *****************************************************************************/

#include <sglPick.hpp>
#include <sglGeode.hpp>
#include <sglFrustum.hpp>

//----------------------------------------------------------------------------
template <class T>
sglPick<T>::sglPick(vector<PickResult*> *result_pool)
      : sglProbeTemplate<T>(),
        m_orig_pick_frustum(),
        m_current_pick_frustum(),
        m_results(),
        m_result_pool(result_pool)
{
}

//----------------------------------------------------------------------------
template <class T>
sglPick<T>::~sglPick()
{
   reset();
}

//----------------------------------------------------------------------------
template <class T>
void sglPick<T>::reset()
{
   sglProbeTemplate<T>::reset();

   // clear out any previous results
   while (!m_results.empty())
   {
      PickResult *result = m_results.back();
      m_results.pop_back();
      if (result)
      {
         if (m_result_pool)
         {
            result->reset();
            m_result_pool->push_back(result);
         }
         else
         {
            delete result;
         }
      }
   }
}

//----------------------------------------------------------------------------

template <class T>
void sglPick<T>::setPickFrustum(const sglMat4<T>    &pick_matrix,
                                const sglFrustum<T> &pick_frustum)
{
   m_orig_pick_frustum = pick_frustum;

   sglMat4<T> inv_pick_mat;
   inv_pick_mat.inverseAffine(pick_matrix);
   m_current_pick_frustum.mul(pick_frustum, inv_pick_mat);
}

//----------------------------------------------------------------------------

template <class T>
void sglPick<T>::addHit(const sglGeode *geode, const sglDrawable *geometry)
{
   PickResult *result = NULL;

   if (m_result_pool && !m_result_pool->empty())
   {
      result = m_result_pool->back();
      m_result_pool->pop_back();
   }

   if (result == NULL) // separate if, in case m_result_pool filled with NULLs
   {
      result = new PickResult;
   }

   if (m_mode&eGEODE)
      result->m_geode    = geode;
   if (m_mode&eGEOMETRY)
      result->m_geometry = geometry;
   if (m_mode&ePATH)
      result->m_path     = m_path;
   if (m_mode&eTRANSFORM)
      result->m_xform    = m_modelview_matrix;

   m_results.push_back(result);
}

template class SGL_DLL_API sglPick<double>;
template class SGL_DLL_API sglPick<float>;
