/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglQuadSet.cpp
 *  Created: 22 October 1997
 *  Summary: the quad primitives drawing code
 *****************************************************************************/

#include <sgl.h>
#include <sglQuadSet.hpp>

//----------------------------------------------------------------------------
sglQuadSet::sglQuadSet() : sglGeoSet()
{
}

//----------------------------------------------------------------------------
sglQuadSet::~sglQuadSet()
{
}

//----------------------------------------------------------------------------
template <class T>
static inline sglBound::IntersectResultEnum
intersectQuadSet(
   sglIntersect<T> &segset,
   const sglBoxBound &bbox,
   const sglVertexPtr &vertices,
   unsigned int num_prims)
{
   // check against the bounding box
   sglSegment<T> seg(segset.getSegment());
   sglBound::IntersectResultEnum result = bbox.intersect(seg);
   if (result == sglBound::eOUTSIDE)
   {
      return result;
   }

   // else result == INSIDE | INTERSECT
   // the bounding box test trims the segment so we have to restore it
   seg = segset.getSegment();
   result = sglBound::eOUTSIDE;

   const typename sglIntersect<T>::IntersectTriangleFunc isect_func =
      sglIntersect<T>::getIntersectTriangleFunc(vertices.getTypeIndex());

   sglMultiTypePtr::iterator it = vertices.begin();
   const int inc = vertices.getInc();
   const sglMultiTypePtr::iterator end = it + inc * num_prims * 4;
   while (it != end)
   {
      const sglMultiTypePtr::iterator it2 = it + inc;
      const sglMultiTypePtr::iterator it3 = it2 + inc;
      const sglMultiTypePtr::iterator it4 = it3 + inc;

      isect_func(segset, seg, it, it2, it3, result);
      isect_func(segset, seg, it, it3, it4, result);

      it = it4 + inc;
   }

   if (result == sglBound::eINTERSECT)
   {
      segset.setSegment(seg);
   }

   return result;
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglQuadSet::intersectPrim(sglIntersectf &segset)
{
   return intersectQuadSet( segset, m_bbox, m_vertices, m_num_prims);
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglQuadSet::intersectPrim(sglIntersectd &segset)
{
   return intersectQuadSet( segset, m_bbox, m_vertices, m_num_prims);
}
