/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglQuadStripSet.cpp
 *  Created: 22 October 1997
 *  Summary: the quad strip primitives drawing code
 *****************************************************************************/

#include <sgl.h>
#include <sglQuadStripSet.hpp>

//----------------------------------------------------------------------------
sglQuadStripSet::sglQuadStripSet() : sglGeoStripSet()
{
}

//----------------------------------------------------------------------------
sglQuadStripSet::~sglQuadStripSet()
{
}

//----------------------------------------------------------------------------
template <class T>
static inline sglBound::IntersectResultEnum
intersectQuadStripSet(
   sglIntersect<T> &segset,
   const sglBoxBound &bbox,
   const sglVertexPtr &vertices,
   const sglIndexPtr &lengths,
   unsigned int num_prims)
{
   // check against the bounding box
   sglSegment<T> seg(segset.getSegment());
   sglBound::IntersectResultEnum result = bbox.intersect(seg);
   if (result == sglBound::eOUTSIDE)
   {
      return result;
   }

   // else result == INSIDE | INTERSECT
   // the bounding box test trims the segment so we have to restore it
   seg = segset.getSegment();
   result = sglBound::eOUTSIDE;

   const typename sglIntersect<T>::IntersectTriangleFunc isect_func =
      sglIntersect<T>::getIntersectTriangleFunc(vertices.getTypeIndex());

   sglMultiTypePtr::iterator it = vertices.begin();
   const int inc = vertices.getInc();

   sglIndexPtr::iterator lit = lengths.begin();
   const int linc = lengths.getInc();
   const sglIndexPtr::iterator lend = lit + linc * num_prims;
   for (; lit != lend; lit += linc)
   {
      const sglIndexType length = *lit;
      if (length >= 4)
      {
         sglMultiTypePtr::iterator it0 = it;
         sglMultiTypePtr::iterator it1 = it0 + inc;
         sglMultiTypePtr::iterator it2 = it1 + inc;

         const sglMultiTypePtr::iterator end = it0 + (length&(~0x1)) * inc;
         while (it2 != end)
         {
            const sglMultiTypePtr::iterator it3 = it2 + inc;
            isect_func(segset, seg, it0, it1, it2, result);
            isect_func(segset, seg, it2, it1, it3, result);
            it0 = it2;
            it1 = it3;
            it2 = it3 + inc;
         }
      }
      it += inc * length;
   }

   if (result == sglBound::eINTERSECT)
   {
      segset.setSegment(seg);
   }

   return result;
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglQuadStripSet::intersectPrim(
   sglIntersectf &segset)
{
   return intersectQuadStripSet(
      segset, m_bbox, m_vertices, m_lengths, m_num_prims);
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum sglQuadStripSet::intersectPrim(
   sglIntersectd &segset)
{
   return intersectQuadStripSet(
      segset, m_bbox, m_vertices, m_lengths, m_num_prims);
}
