/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglQuadStripSet.hpp
 *  Created: 12 August 1997
 *  Summary: Approximates a pfGeoSet which contains geometry data for strips of
 *         : quads, and pointers to the graphics state associated with it
 *         : (sglGeoState). 
 *****************************************************************************/

#ifndef __SGL_QUAD_STRIP_SET_HPP
#define __SGL_QUAD_STRIP_SET_HPP

#include <sgl.h>
#include <sglGeoStripSet.hpp>

//============================================================================

/** contains geometry data for strips of quads
 */

class SGL_DLL_API sglQuadStripSet : public sglGeoStripSet
{
public:
   /// default constructor
   sglQuadStripSet();
   /// virtual destructor
   virtual ~sglQuadStripSet();

   /// retrieve the type of primitive
   sglStats::PrimEnum getPrimType() const {return sglStats::eQUADSTRIPS;}
   ///
   GLenum getGLPrimType() const {return GL_QUAD_STRIP;}
   ///
   AAEnum getAAType() const {return ePOLYGON_AA;}

   ///
   virtual sglBound::IntersectResultEnum intersectPrim(sglIntersectf &segset);
   ///
   virtual sglBound::IntersectResultEnum intersectPrim(sglIntersectd &segset);

private:
   sglQuadStripSet(const sglQuadStripSet &);
   sglQuadStripSet &operator=(const sglQuadStripSet &);
};

#endif
