/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglSpotLight.cpp
 *   Author: Scott McMillan
 *  Created: 9 December 2000
 *****************************************************************************/

#include <sglSpotLight.hpp>

//----------------------------------------------------------------------------
sglSpotLight::sglSpotLight() : sglPositionalLight(),
   m_direction(0.0f, 0.0f, -1.0f),
   m_exponent(0.0f),
   m_cutoff((float)(M_PI*0.5))
{
}

//----------------------------------------------------------------------------
sglSpotLight::~sglSpotLight()
{
}

//----------------------------------------------------------------------------
void sglSpotLight::setDirection(const sglVec3f &dir)
{
   if (dir.dot(dir) > 0.0f)
   {
      m_direction = dir;
      m_direction.normalize();
   }
}

//----------------------------------------------------------------------------
void sglSpotLight::setExponent(float exponent)
{
   if (exponent >= 0.0f)
      m_exponent = exponent;
}

//----------------------------------------------------------------------------
void sglSpotLight::setCutoff(float cutoff)
{
   if (cutoff >= 0.0f && cutoff <= 0.5f*M_PI)
   {
      m_cutoff = cutoff;
   }
}

//----------------------------------------------------------------------------
void sglSpotLight::setup(GLenum gl_light_number) const
{
   // must come first b/c sglLight temporarily disables via cutoff
   sglPositionalLight::setup(gl_light_number);

   glLightfv(gl_light_number, GL_SPOT_DIRECTION, m_direction.getPtr());
   glLightf(gl_light_number, GL_SPOT_EXPONENT, m_exponent);
   glLightf(gl_light_number, GL_SPOT_CUTOFF, m_cutoff*180.0f/M_PI);
}

//----------------------------------------------------------------------------
sglNode *sglSpotLight::clone(unsigned int mode) const
{
   sglSpotLight *root = new sglSpotLight;
   sglSpotLight::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglSpotLight::copyTo(sglSpotLight *dst, unsigned int mode) const
{
   sglPositionalLight::copyTo(dst, mode);

   // transfer stuff
   dst->setDirection(m_direction);
   dst->setCutoff(m_cutoff);
   dst->setExponent(m_exponent);
}

//----------------------------------------------------------------------------
void sglSpotLight::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglPositionalLight::printInfo(ostrm, indent_string);
   ostrm << indent_string << "spot direction: " << m_direction << endl;
   ostrm << indent_string << "exponent: " << m_exponent << endl;
   ostrm << indent_string << "cutoff: " << m_cutoff << endl;
}
