/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglStats.hpp
 *   Author: Tom Stimson
 *  Created: 2 April 2000
 *  Summary: Structure for maintain rendering statistics.
 *****************************************************************************/

#ifndef __SGL_STATS_HPP
#define __SGL_STATS_HPP

#include <sgl.h>


//============================================================================

// forward declare
class sglTexture;

/** Container for draw traversal statistics collection.
 */

class SGL_DLL_API sglStats
{
public:
   ///
   enum PrimEnum {ePOINTS,
                  eLINES,
                  eLINESTRIPS,
                  eTRIS,
                  eQUADS,
                  eTRISTRIPS,
                  eTRIFANS,
                  eQUADSTRIPS,
                  ePOLYS,
                  eINDEXED_POINTS,
                  eINDEXED_LINES,
                  eINDEXED_LINESTRIPS,
                  eINDEXED_TRIS,
                  eINDEXED_QUADS,
                  eINDEXED_TRISTRIPS,
                  eINDEXED_TRIFANS,
                  eINDEXED_QUADSTRIPS,
                  eINDEXED_POLYS,
                  eMONO_INDEXED_POINTS,
                  eMONO_INDEXED_LINES,
                  eMONO_INDEXED_LINESTRIPS,
                  eMONO_INDEXED_TRIS,
                  eMONO_INDEXED_QUADS,
                  eMONO_INDEXED_TRISTRIPS,
                  eMONO_INDEXED_TRIFANS,
                  eMONO_INDEXED_QUADSTRIPS,
                  eMONO_INDEXED_POLYS,
                  eINTERLEAVED_POINTS,
                  eINTERLEAVED_LINES,
                  eINTERLEAVED_LINESTRIPS,
                  eINTERLEAVED_LINELOOPS,
                  eINTERLEAVED_TRIS,
                  eINTERLEAVED_QUADS,
                  eINTERLEAVED_TRISTRIPS,
                  eINTERLEAVED_TRIFANS,
                  eINTERLEAVED_QUADSTRIPS,
                  eINTERLEAVED_POLYS,
                  eBEZIER_CURVES,
                  eBEZIER_SURFACES,
                  eNUM_PRIM_TYPES};

public:
   /// default constructor
   sglStats() { reset(); }

   /// destructor
   ~sglStats() {};

   /// copy contructor
   sglStats(const sglStats &);

   /// assignment operator
   sglStats& operator=(const sglStats &);

   /// += operator
   sglStats& operator+=(const sglStats &);

   /// Reset all the counters to zero.
   void reset();

   /// Compute the average number of gsets, etc..., per frame.
   void average();

   ///
   void addNumLights(float num_used, float num_desired)
      {
         m_num_lights_used += num_used;
         m_num_lights_desired += num_desired;
      }
   ///
   float getNumLightsUsed() const    { return m_num_lights_used; }
   ///
   float getNumLightsDesired() const { return m_num_lights_desired; }

   ///
   void addFrames(float num) { m_frames += num; m_textures.clear(); }
   ///
   float getFrames() const { return m_frames; }

   ///
   void addTransforms(float num) { m_transforms += num; }
   ///
   float getTransforms() const { return m_transforms; }

   ///
   void addStateChanges(float num) { m_state_changes += num; }
   ///
   float getStateChanges() const { return m_state_changes; }

   ///
   void addTotalGSets(float num) { m_total_gsets += num; }
   ///
   float getTotalGSets() const { return m_total_gsets; }

   ///
   void addDListedGSets(float num) { m_dlisted_gsets += num; }
   ///
   float getDListedGSets() const { return m_dlisted_gsets; }

   ///
   void addTransparencyEnabledGSets(float num)
      { m_transparency_enabled_gsets += num; }
   ///
   float getTransparencyEnabledGSets() const
      { return m_transparency_enabled_gsets; }

   ///
   void addOcclusionEnabledGSets(float num)
      { m_occlusion_enabled_gsets += num; }
   ///
   float getOcclusionEnabledGSets() const
      { return m_occlusion_enabled_gsets; }

   ///
   void addOccludedGSets(float num) { m_occluded_gsets += num; }
   ///
   float getOccludedGSets() const { return m_occluded_gsets; }

   ///
   void addLitGSets(float num) { m_lit_gsets += num; }
   ///
   float getLitGSets() const { return m_lit_gsets; }

   ///
   void addTexturedGSets(float num) { m_textured_gsets += num; }
   ///
   float getTexturedGSets() const { return m_textured_gsets; }

   ///
   void addTexture(const sglTexture *tex);
   ///
   float getNumTextures() { return m_num_textures; }
   ///
   float getIdealTextureMemory() { return m_ideal_tex_mem; }
   ///
   float getLikelyTextureMemory() { return m_likely_tex_mem; }

   ///
   void addText(float num) { m_text += num; }
   ///
   float getText() const { return m_text; }

   ///
   void addGSets(PrimEnum type, float num) { m_gsets[type] += num; }
   ///
   float getGSets(PrimEnum type) { return m_gsets[type]; }

   ///
   void addPrims(PrimEnum type, float num) { m_prims[type] += num; }
   ///
   float getPrims(PrimEnum type) { return m_prims[type]; }

   ///
   void addVertices(PrimEnum type, float num) { m_vertices[type] += num; }
   ///
   float getVertices(PrimEnum type) { return m_vertices[type]; }

   ///
   void addTriangles(PrimEnum type, float num) { m_triangles[type] += num; }
   ///
   float getTriangles(PrimEnum type) { return m_triangles[type]; }

   ///
   void printInfo(ostream &ostrm, const char *indent_string) const;

public:
   static const char*   s_geometry_type_names[];

private:
   float                m_frames;
   float                m_transforms;
   float                m_state_changes;
   float                m_total_gsets;
   float                m_dlisted_gsets;
   float                m_transparency_enabled_gsets;
   float                m_occlusion_enabled_gsets;
   float                m_occluded_gsets;
   float                m_lit_gsets;
   float                m_textured_gsets;
   vector<const sglTexture*>  m_textures;
   float                m_num_textures;
   float                m_ideal_tex_mem;
   float                m_likely_tex_mem;
   float                m_text;
   float                m_gsets[eNUM_PRIM_TYPES];
   float                m_prims[eNUM_PRIM_TYPES];
   float                m_vertices[eNUM_PRIM_TYPES];
   float                m_triangles[eNUM_PRIM_TYPES];

   float                m_num_lights_used;
   float                m_num_lights_desired;
};

#endif
