/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTexCoords.cpp
 *   Author: Tom Stimson
 *  Created: 8 June 2002
 *****************************************************************************/

#include <sglTexCoords.hpp>


sglTexCoords::FuncType1 sglTexCoords::s_func1[] = {
   NULL,
   NULL,
   (FuncType1)glTexCoord1sv,
   NULL,
   (FuncType1)glTexCoord1iv,
   NULL,
   (FuncType1)glTexCoord1fv,
   (FuncType1)glTexCoord1dv,

   NULL,
   NULL,
   (FuncType1)glTexCoord2sv,
   NULL,
   (FuncType1)glTexCoord2iv,
   NULL,
   (FuncType1)glTexCoord2fv,
   (FuncType1)glTexCoord2dv,

   NULL,
   NULL,
   (FuncType1)glTexCoord3sv,
   NULL,
   (FuncType1)glTexCoord3iv,
   NULL,
   (FuncType1)glTexCoord3fv,
   (FuncType1)glTexCoord3dv,

   NULL,
   NULL,
   (FuncType1)glTexCoord4sv,
   NULL,
   (FuncType1)glTexCoord4iv,
   NULL,
   (FuncType1)glTexCoord4fv,
   (FuncType1)glTexCoord4dv
};

sglTexCoords::FuncType2* sglTexCoords::s_func2[] = {
   NULL,
   NULL,
   (FuncType2*)&sgl::multiTexCoord1sv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord1iv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord1fv,
   (FuncType2*)&sgl::multiTexCoord1dv,

   NULL,
   NULL,
   (FuncType2*)&sgl::multiTexCoord2sv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord2iv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord2fv,
   (FuncType2*)&sgl::multiTexCoord2dv,

   NULL,
   NULL,
   (FuncType2*)&sgl::multiTexCoord3sv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord3iv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord3fv,
   (FuncType2*)&sgl::multiTexCoord3dv,

   NULL,
   NULL,
   (FuncType2*)&sgl::multiTexCoord4sv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord4iv,
   NULL,
   (FuncType2*)&sgl::multiTexCoord4fv,
   (FuncType2*)&sgl::multiTexCoord4dv,
};


//----------------------------------------------------------------------------
sglTexCoords &sglTexCoords::operator=(const sglTexCoords &rhs)
{
   if (this != &rhs)
   {
      derefDataPtr();

      m_data_ptr = rhs.m_data_ptr;
      m_type = rhs.m_type;
      m_size = rhs.m_size;
      m_stride = rhs.m_stride;
      m_gltype = rhs.m_gltype;
      m_byte_inc = rhs.m_byte_inc;
      m_start = rhs.m_start;
      m_func1 = rhs.m_func1;
      m_func2 = rhs.m_func2;

      if (m_data_ptr) m_data_ptr->ref();
   }
   return *this;
}


//----------------------------------------------------------------------------
void sglTexCoords::set(sglDataPtr *data,
                       sglDataPtr::DataTypeEnum type,
                       int size,
                       int offset,
                       int stride)
{
   derefDataPtr();
   if (data) data->ref();

   m_data_ptr = data;
   m_type = type;
   m_size = size;
   m_stride = stride;
   m_gltype = sglDataPtr::getGLType(type);

   if (stride == 0)
      m_byte_inc = sglDataPtr::getSizeOfType(type) * size;
   else
      m_byte_inc = stride;

   if (m_data_ptr && m_data_ptr->getData() && stride >= 0 &&
       size > 0 && size <= 4)
   {
      int indx = 8 * (size-1) + type;
      m_func1 = s_func1[indx];
      m_func2 = *s_func2[indx];
   }
   else
   {
      m_func1 = NULL;
      m_func2 = NULL;
   }

   if (m_func1)
      m_start = (char*)m_data_ptr->getData() + offset;
   else
      m_start = NULL;
}


//----------------------------------------------------------------------------
void sglTexCoords::derefDataPtr()
{
   if (m_data_ptr)
   {
      m_data_ptr->deref();
      if (m_data_ptr->getRefCount() == 0)
      {
         delete m_data_ptr;
      }
   }
}


//----------------------------------------------------------------------------
void sglTexCoords::printInfo(ostream &ostrm, const char *indent_string) const
{
   ostrm << indent_string << "m_data_ptr = " << m_data_ptr
         << ", m_type = " << (int)m_type
         << ", m_size = " << m_size
         << ", m_stride = " << m_stride << endl;
}
