/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTimespec.cpp
 *   Author: Scott McMillan
 *  Created: 1 June 2001
 *  Summary:
 *****************************************************************************/

#include <sglTimespec.hpp>

#if defined(WIN32)
#   include <sys/timeb.h>
#else
#  include <unistd.h>
#  if !defined(_POSIX_TIMERS) || defined(linux)
#   include <sys/time.h>
#  endif
#endif

//----------------------------------------------------------------------------
sglTimespec& sglTimespec::operator+=(const sglTimespec &ts)
{
   m_sec += ts.m_sec;
   long nsec  = m_nsec + ts.m_nsec;
   if (nsec >= 1000000000)
   {
      ++m_sec;
      nsec -= 1000000000;
   }
   m_nsec = nsec;
   return *this;
}

//----------------------------------------------------------------------------
sglTimespec &sglTimespec::operator-=(const sglTimespec &ts)
{
   m_sec -= ts.m_sec;
   long nsec = m_nsec - ts.m_nsec;
   if (nsec < 0)
   {
      --m_sec;
      nsec += 1000000000;
   }

   m_nsec = nsec;
   return *this;
}

//----------------------------------------------------------------------------
const sglTimespec sglTimespec::operator+(const sglTimespec &rhs) const
{
   sglTimespec tmp = *this;
   tmp += rhs;
   return tmp;
}

//----------------------------------------------------------------------------
const sglTimespec sglTimespec::operator-(const sglTimespec &rhs) const
{
   sglTimespec tmp = *this;
   tmp -= rhs;
   return tmp;
}

//----------------------------------------------------------------------------
void sglTimespec::getSysTime(sglTimespec &ts)
{
#if defined(WIN32)
    struct _timeb timebuffer;
    _ftime(&timebuffer);

    ts.m_sec = timebuffer.time;
    ts.m_nsec = timebuffer.millitm*1000000;

#elif defined(_POSIX_TIMERS) && !defined(linux)
    timespec ts1;
    if (clock_gettime(CLOCK_REALTIME, &ts1) != 0)
    {
        throw ts1;
    }
    ts.m_sec  = ts1.tv_sec;
    ts.m_nsec = ts1.tv_nsec;

#else
    struct timeval tv;
    gettimeofday(&tv, NULL);
    ts.m_sec = tv.tv_sec;
    ts.m_nsec = 1000*tv.tv_usec;
#endif
}

//----------------------------------------------------------------------------
ostream& operator<<(ostream& strm, const sglTimespec& rhs)
{
    return strm << rhs.m_sec << "." << setw(9) << setfill('0')
                << rhs.m_nsec;
}
