/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTimespec.hpp
 *  Created: 1 June 2001
 *  Summary: Cross platform timing
 *****************************************************************************/

#ifndef __SGL_TIMESPEC_HPP
#define __SGL_TIMESPEC_HPP

#include <sgl.h>
#include <time.h>

//----------------------------------------------------------------------------

class SGL_DLL_API sglTimespec
{
public:
   sglTimespec() : m_sec(0), m_nsec(0) {}
   sglTimespec(time_t sec, long nsec) : m_sec(sec), m_nsec(nsec) {}
   sglTimespec(const sglTimespec &rhs) : m_sec(rhs.m_sec), m_nsec(rhs.m_nsec){}
   sglTimespec(double sec) : m_sec((time_t)sec), m_nsec(0)
      {
         m_nsec = (long)(fabs(sec - m_sec)*1000000000);
      }
   ~sglTimespec() {}

   sglTimespec &operator=(const sglTimespec &rhs)
      {
         m_sec = rhs.m_sec; m_nsec = rhs.m_nsec; return *this;
      }

   sglTimespec& operator+=(const sglTimespec& rhs);
   sglTimespec& operator-=(const sglTimespec& rhs);

   bool operator<(const sglTimespec& rhs)
      {
         if (m_sec < rhs.m_sec)
            return true;
         else if (m_sec > rhs.m_sec)
            return false;
         else if (m_nsec < rhs.m_nsec) // m_sec == rhs.m_sec in this case
            return true;

         return false;
      }

   const sglTimespec operator+(const sglTimespec& rhs) const;
   const sglTimespec operator-(const sglTimespec& rhs) const;

   static double convertToDouble(const sglTimespec &ts)
      { return ((double)ts.m_sec) + (1.0e-9*((double)ts.m_nsec)); }

   static void getSysTime(sglTimespec &ts);

public:
   long m_sec;
   long m_nsec;
};

// *************************************************************************
SGL_DLL_API
ostream& operator<<(ostream& strm, const sglTimespec& rhs);

#endif
