/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTraverser.hpp
 *   Author: Scott McMillan
 *  Created: 3 June 2001
 *  Summary: Base class for all traversal classes
 *****************************************************************************/

#ifndef __SGL_TRAVERSER_HPP
#define __SGL_TRAVERSER_HPP

#include <sgl.h>
#include <sglTimespec.hpp>

//----------------------------------------------------------------------------

/**

Base class for all traversers (sglCull and sglProbe currently)

*/

class SGL_DLL_API sglTraverser
{
public:
   sglTraverser()
         : m_discriminator_mask(~0x0u),
           m_lod_scale(1.0f),
           m_lod_offset(0.0f),
           m_frame_count(0),
           m_frame_time() {}

   virtual ~sglTraverser() {};

   ///
   void setDiscriminatorMask(unsigned int mask) {m_discriminator_mask = mask;}
   ///
   unsigned int getDiscriminatorMask() const    {return m_discriminator_mask;}

   ///
   void setLODScale(float lod_scale) { m_lod_scale = lod_scale; }
   ///
   float getLODScale() const         { return m_lod_scale; }

   ///
   void setLODOffset(float lod_offset) { m_lod_offset = lod_offset; }
   ///
   float getLODOffset() const          { return m_lod_offset; }

   ///
   void setFrameCount(unsigned int count) { m_frame_count = count; }
   ///
   unsigned int getFrameCount() const { return m_frame_count; }

   ///
   void setFrameTime(const sglTimespec &ftime) { m_frame_time = ftime; }
   ///
   const sglTimespec &getFrameTime() const { return m_frame_time; }

private: // Methods - NOT IMPLEMENTED
   sglTraverser(const sglTraverser &);
   sglTraverser &operator=(const sglTraverser &);

protected: // Variables
   unsigned int m_discriminator_mask;
   float        m_lod_scale;
   float        m_lod_offset;

   unsigned int m_frame_count;
   sglTimespec  m_frame_time;
};

#endif
