/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTriangleStripSet.hpp
 *  Created: 12 August 1997
 *  Summary: Approximates a pfGeoSet which contains geometry data, and
 *         : pointers to the graphics state associated with it (sglGeoState).
 *****************************************************************************/

#ifndef __SGL_TRIANGLE_STRIP_SET_HPP
#define __SGL_TRIANGLE_STRIP_SET_HPP

#include <sgl.h>
#include <sglGeoStripSet.hpp>

//============================================================================

/** Approximates a pfGeoSet which contains geometry data, and
    pointers to the graphics state associated with it (sglGeoState).
*/

class SGL_DLL_API sglTriangleStripSet : public sglGeoStripSet
{
public:
   /// default constructor
   sglTriangleStripSet();
   /// virtual destructor
   virtual ~sglTriangleStripSet();

   /// retrieve the type of primitive
   sglStats::PrimEnum getPrimType() const {return sglStats::eTRISTRIPS;}
   ///
   GLenum getGLPrimType() const {return GL_TRIANGLE_STRIP;}
   ///
   AAEnum getAAType() const {return ePOLYGON_AA;}

   ///
   virtual sglBound::IntersectResultEnum intersectPrim(sglIntersectf &segset);
   ///
   virtual sglBound::IntersectResultEnum intersectPrim(sglIntersectd &segset);

private: // not implemented
   sglTriangleStripSet(const sglTriangleStripSet &);
   sglTriangleStripSet& operator=(const sglTriangleStripSet &);
};

#endif
