/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglUserData.hpp
 *  Created: 7 August 1999
 *  Summary: Base class from which application user data should be subclassed
 *           so that dynamic_cast operators can be safely performed.
 *****************************************************************************/

#ifndef __SGL_USERDATA_HPP
#define __SGL_USERDATA_HPP

#include <sgl.h>

/** Base class from which application user data should be subclassed
    so that dynamic_cast operators can be safely performed.
*/

class SGL_DLL_API sglUserData
{
public:
   ///
   sglUserData() {}
   ///
   virtual ~sglUserData() {}

   /** Output the state of this data to the specified ostream
       @param ostrm the ostream to which the output is sent
       @param indent_string the string (usually spaces) that is output at the
                            beginning of every line of output
   */
   virtual void printInfo(ostream &, const char *) const {}

private:
   sglUserData(const sglUserData &); // not implemented
   sglUserData& operator=(const sglUserData &); // not implemented
};
//----------------------------------------------------------------------------

inline ostream& operator<<(ostream& ostrm, const sglUserData& object)
{
   object.printInfo(ostrm, " ");
   return ostrm;
}

#endif
