/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglViewPlatform.cpp
 *   Author: Scott McMillan
 *  Created: 16 December 2000
 *****************************************************************************/

#include <sglViewPlatform.hpp>
#include <sglGroup.hpp>
#include <sglOStream.hpp>

sglMat4f rotx_matrix(1.0f,  0.0f,  0.0f,  0.0f,
                     0.0f,  0.0f, -1.0f,  0.0f,
                     0.0f,  1.0f,  0.0f,  0.0f,
                     0.0f,  0.0f,  0.0f,  1.0f);

//----------------------------------------------------------------------------
sglViewPlatform::sglViewPlatform(sglCamera &camera) : sglNode(),
   m_rotx_flag(false),
   m_camera(&camera)
{
   // make the bounding sphere empty (and static) by default
   sglSphereBoundf bsph;
   bsph.setRadius(0.0f);
   setBound(&bsph, sglBound::eSTATIC);

   m_camera->ref();
}

//----------------------------------------------------------------------------
sglViewPlatform::~sglViewPlatform()
{
   m_camera->deref();
}

//----------------------------------------------------------------------------
void sglViewPlatform::setCamera(sglCamera &camera)
{
   m_camera->deref();
   m_camera = &camera;
   m_camera->ref();
}

//----------------------------------------------------------------------------
// necessary?
void sglViewPlatform::computeBound()
{
   // is this necessary?  yes when the user sets the bounds to dynamic
   m_sphere.setCenter(0., 0., 0.);
   m_sphere.setRadius(0.0);
   m_dirty &= ~eBOUND;
}

//----------------------------------------------------------------------------
void sglViewPlatform::computeViewMatrix(sglMat4f &curr_mat) const
{
   curr_mat.buildIdentity();

   sglNode *curr_node = getParent(0);

   while (curr_node != NULL)
   {
      curr_node->computeTxToParent(curr_mat);
      curr_node = curr_node->getParent(0);
   }

   if (m_rotx_flag)
   {
      curr_mat *= rotx_matrix;
   }
}

//----------------------------------------------------------------------------
void sglViewPlatform::computeViewMatrix(sglMat4d &curr_mat) const
{
   curr_mat.buildIdentity();

   sglNode *curr_node = getParent(0);

   while (curr_node != NULL)
   {
      curr_node->computeTxToParent(curr_mat);
      curr_node = curr_node->getParent(0);
   }

   if (m_rotx_flag)
   {
      curr_mat *= rotx_matrix;
   }
}

//----------------------------------------------------------------------------
sglNode *sglViewPlatform::clone(unsigned int mode) const
{
   // HACK need mode bits for cloning cameras - see copyTo()
   sglViewPlatform *root = new sglViewPlatform(getCamera());
   sglViewPlatform::copyTo(root, mode);
   return root;
}

//----------------------------------------------------------------------------
void sglViewPlatform::copyTo(sglViewPlatform *dst, unsigned int mode) const
{
   sglNode::copyTo(dst, mode);

   // transfer stuff
   dst->setEnableZUp(m_rotx_flag);

   //if (mode & sglObject::eCLONE_CAMERAS)
   //   sglCamera *camera = camera->clone(mode);
}

//----------------------------------------------------------------------------
void sglViewPlatform::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglNode::printInfo(ostrm, indent_string);
   ostrm << indent_string << "rotx_mode "
         << (m_rotx_flag ? "enabled" : "disabled") << endl;

   ostrm << indent_string << "....................................." << endl;
   m_camera->printInfo(ostrm, indent_string);
   ostrm << indent_string << "....................................." << endl;
}
