/*****************************************************************************
 * SGL: A SceneGraph Library, Version 1.0
 * Copyright (C) 1997-2001  Scott McMillan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLoadModel3ds.hpp
 *   Author: Scott McMillan
 *  Created: 16 July 1999
 *  Summary: 3D Studio format 3D model loader
 *****************************************************************************
 *  Adapted from PLIB 1.2.0
 *  File: ssgLoad3ds.cxx
 *  Written by Per Liedman (liedman@home.se)
 *  Last updated: 2000-04-06
 *****************************************************************************/

#ifndef __SGL_LOAD_MODEL_3DS_HPP
#define __SGL_LOAD_MODEL_3DS_HPP

#if defined(WIN32) && defined(_DLL)
// The next define will come from the makefile for archive objects.
#ifdef sgl3ds_DLL_FILE
#define SGL3DS_DLL_API __declspec(dllexport)
#else
#define SGL3DS_DLL_API __declspec(dllimport)
#endif
#else
#define SGL3DS_DLL_API
#endif


#include <sgl.h>
#include <sglVector.hpp>
#include <sglTransform.hpp>
#include <stdio.h>

class sglGroup;
class sgl3dsMaterial;

class SGL3DS_DLL_API sglLoadModel3ds // : public sglLoadModel
{
public:
   sglLoadModel3ds();
   virtual ~sglLoadModel3ds();

   virtual sglNode *load(const string &filename);

   struct Chunk
   {
      unsigned short id;
      Chunk *subchunks;
      bool (*parse_func) (sglLoadModel3ds &, FILE* , unsigned int );
   };

   static bool parseMaterial      (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseOBJBlock      (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseRGB1          (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseRGB2          (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseMaterialName  (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseAmbient       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseDiffuse       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseSpecular      (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseShininess     (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseTransparency  (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseDoublesided   (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parse8Bytes        (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseNBytes        (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseVertList      (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseFaceList      (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseMapList       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseTraMatrix     (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseTriMesh       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseSmoothList    (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseFaceMaterials (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseMapName       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseMapOptions    (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseUScale        (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseVScale        (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseUOffset       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseVOffset       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseOneUnit       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool parseVersion       (sglLoadModel3ds &ldr,
                                   FILE *fd, unsigned int length);
   static bool identifyFaceMaterials(sglLoadModel3ds &ldr,
                                     FILE *fd, unsigned int length);
   static void addLeaf(sglLoadModel3ds &ldr,
                       sgl3dsMaterial *material, int listed_faces,
                       unsigned short *face_indices);

protected:
   sglLoadModel3ds(const sglLoadModel3ds &); // not implemented
   sglLoadModel3ds &operator==(const sglLoadModel3ds &);  // not implemented

private:    // Methods
   bool parseChunks(FILE *fd, unsigned int length, Chunk *chunk_list);
   void smoothNormals(bool use_smooth_list);
   void freeTrimesh();

private:    // Variables
   vector<sgl3dsMaterial*> m_material_list;
   sgl3dsMaterial *m_current_material;

   bool m_double_sided_flag;     // is there some double sided material?
   //unsigned int m_num_objects;
   //unsigned int m_num_materials;
   //unsigned int m_num_textures;

   sglGroup *m_top_object;

   unsigned int *m_vertex_index;
   //unsigned int *m_normal_index;
   //unsigned int  m_num_vertices;
   unsigned int  m_num_faces;
   unsigned int *m_smooth_list;

   sglTransformf *m_current_transform;

   sglVec3f *m_vertex_list;
   sglVec3f *m_face_normals;
   sglVec3f *m_vertex_normals;
   sglVec2f *m_texcrd_list;
   bool m_smooth_found;
   bool m_facemat_found;

   GLenum m_color_mode;
};


#endif
