/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglAxes.cpp
 *   Author: Scott McMillan
 *  Created: 23 January 2001
 *****************************************************************************/

#include <sglAxes.hpp>

//----------------------------------------------------------------------------
sglAxes::sglAxes(float size)
      : sglLineSet(),
        m_size(size)   
{
   if (m_size <= 0.0f)
      m_size = 1.0f;

   makeCoordinateAxes(m_size);
}

//----------------------------------------------------------------------------
sglAxes::~sglAxes()
{
}

//----------------------------------------------------------------------------
void sglAxes::makeCoordinateAxes(float size)
{
   sglVec3f *vertex = new sglVec3f[6];
   sglVec4f *color  = new sglVec4f[3];

   color[0].set(1.0f, 0.0f, 0.0f, 1.0f);
   color[1].set(0.0f, 1.0f, 0.0f, 1.0f);
   color[2].set(0.0f, 0.0f, 1.0f, 1.0f);

   vertex[0].set(0.0f, 0.0f, 0.0f);
   vertex[1].set(size, 0.0f, 0.0f);
   vertex[2].set(0.0f, 0.0f, 0.0f);
   vertex[3].set(0.0f, size, 0.0f);
   vertex[4].set(0.0f, 0.0f, 0.0f);
   vertex[5].set(0.0f, 0.0f, size);

   setCoordList(new sglDeleteableDataPtr<sglVec3f>(vertex));
   setColorList(sglGeoSet::ePER_PRIM,
                new sglDeleteableDataPtr<sglVec4f>(color));
   setNumPrims(3);
}

//----------------------------------------------------------------------------
void sglAxes::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglGeoSet::printInfo(ostrm, indent_string);
   ostrm << indent_string << "size: " << m_size << endl;
}
