/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglCone.hpp
 *   Author: Scott McMillan
 *  Created: 23 January 2001
 *  Summary: an leaf sglNode containing the geometry for a cone.  The height
 *           and base radius can be specified.  The base lies in the x-y plane
 *           with the center at the origin and the height extends along the
 *           +z-axis. 
 *****************************************************************************/

#ifndef SGL_CONE_HPP
#define SGL_CONE_HPP

#include <sgldb.h>
#include <sglTriangleStripSet.hpp>

//----------------------------------------------------------------------------

class SGLDB_DLL_API sglCone : public sglTriangleStripSet
{
public:
   sglCone(unsigned int num_sections,
           float height = 1.0f, float radius = 1.0f);
   virtual ~sglCone();

   virtual void printInfo(ostream &ostrm, const char *indent_string) const;

protected:

private:
   sglCone();                           // not implemented
   sglCone(const sglCone &);            // not implemented
   sglCone &operator=(const sglCone &); // not implemented

   void makeCone(unsigned int num_sections,
                 float height, float radius);

private:
   unsigned int         m_num_sections;
   float                m_height;
   float                m_radius;
};

#endif
