/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglCube.cpp
 *   Author: Scott McMillan
 *  Created: 23 January 2001
 *****************************************************************************/

#include <sglCube.hpp>

//----------------------------------------------------------------------------
sglCube::sglCube(float size_x, float size_y, float size_z)
      : sglQuadSet(),
        m_size(size_x, size_y, size_z)
{
   makeCube(m_size);
}

//----------------------------------------------------------------------------
sglCube::sglCube(const sglVec3f &size)
      : sglQuadSet(),
        m_size(size)
{
   makeCube(m_size);
}

//----------------------------------------------------------------------------
sglCube::~sglCube()
{
}

//----------------------------------------------------------------------------
void sglCube::makeCube(const sglVec3f &s)
{
   sglVec3f *vertex = new sglVec3f[24];
   sglVec3f *normal = new sglVec3f[6];

   // ceiling; normal z = -1
   vertex[0].set( s[0], 0.0f, 0.0f);    normal[0].set(0.0f, 0.0f, -1.0f);
   vertex[1].set( 0.0f, 0.0f, 0.0f);
   vertex[2].set( 0.0f, s[1], 0.0f);
   vertex[3].set( s[0], s[1], 0.0f);

   // wall 1; normal y = +1
   vertex[4].set( s[0], s[1], s[2]);     normal[1].set(0.0f, 1.0f, 0.0f);
   vertex[5].set( s[0], s[1], 0.0f);
   vertex[6].set( 0.0f, s[1], 0.0f);
   vertex[7].set( 0.0f, s[1], s[2]);

   // wall 2; normal y = -1
   vertex[8].set( 0.0f, 0.0f, s[2]);     normal[2].set(0.0f, -1.0f, 0.0f);
   vertex[9].set( 0.0f, 0.0f, 0.0f);
   vertex[10].set( s[0], 0.0f, 0.0f);
   vertex[11].set( s[0], 0.0f, s[2]);

   // wall 3; normal x = +1
   vertex[12].set( s[0], 0.0f, s[2]);     normal[3].set(1.0f, 0.0f, 0.0f);
   vertex[13].set( s[0], 0.0f, 0.0f);
   vertex[14].set( s[0], s[1], 0.0f);
   vertex[15].set( s[0], s[1], s[2]);

   // wall 4; normal x = -1
   vertex[16].set( 0.0f, s[1], s[2]);    normal[4].set(-1.0f, 0.0f, 0.0f);
   vertex[17].set( 0.0f, s[1], 0.0f);
   vertex[18].set( 0.0f, 0.0f, 0.0f);
   vertex[19].set( 0.0f, 0.0f, s[2]);

   //floor; normal z = +1
   vertex[20].set( 0.0f, s[1], s[2]);     normal[5].set(0.0f, 0.0f, 1.0f);
   vertex[21].set( 0.0f, 0.0f, s[2]);
   vertex[22].set( s[0], 0.0f, s[2]);
   vertex[23].set( s[0], s[1], s[2]);

   setNumPrims(6);
   setCoordList(new sglDeleteableDataPtr<sglVec3f>(vertex));
   setNormalList(sglGeoSet::ePER_PRIM,
                 new sglDeleteableDataPtr<sglVec3f>(normal));
}

//----------------------------------------------------------------------------
void sglCube::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglGeoSet::printInfo(ostrm, indent_string);
   ostrm << indent_string << "size: " << m_size << endl;
}

