/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglHashTable.hpp
 *  Created: 16 July 1997
 *  Summary: create a hash table
 *****************************************************************************/

#ifndef __SGL_HASH_LIST_HPP
#define __SGL_HASH_LIST_HPP

#include <sgldb.h>
#include <vector>

//----------------------------------------------------------------------------
/// Hash table
template <class T>
class SGLDB_DLL_API sglHashTable
{
   typedef typename vector<T*>::iterator hash_iterator;
public:
   sglHashTable(unsigned int hashsize) :
      m_hashsize(hashsize)
      {
         m_hash_list = new vector<T*>[hashsize];
      }

   /// Destructor
   virtual ~sglHashTable() { delete[] m_hash_list; }

   /// find hash table entry with same hash value as t, return NULL if none
   T* search(T* t)
      {
         unsigned int pos = t->hash(m_hashsize);
         for (hash_iterator i = m_hash_list[pos].begin();
              i != m_hash_list[pos].end(); ++i)
         {
            if (t->isEqual(*i))
               return *i;
         }

         return NULL;
      }

   /// find hash table entry with same hash value as t, and after u
   /// return NULL if none
   T* searchagain(T* t, T* u)
      {
         unsigned int pos = t->hash(m_hashsize);
         bool foundu = false;
         for (hash_iterator i = m_hash_list[pos].begin();
              i != m_hash_list[pos].end(); ++i)
         {
            if (u == *i)
               foundu = true;
            else if (foundu && t->isEqual(*i))
               return *i;
         }

         return NULL;
      }

   /// add t to hash table
   void add(T* t)
      {
         unsigned int pos = t->hash(m_hashsize);
         m_hash_list[pos].push_back(t);
      }

private:
   /// default constructor forbidden
   sglHashTable();

   /// copy constructor forbidden
   sglHashTable(const sglHashTable &);

   /// assignment operator forbidden
   sglHashTable &operator=(const sglHashTable &);

private:
   /// the number of hash values to use
   unsigned int m_hashsize;

   /** Array of T lists.  The array index equals the hash value.
       Each element is a list containing Ts with the same hash value.
    */
   vector<T*> *m_hash_list;
};

#endif
