/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLoadFile_cmb.cpp
 *  Created: 12 July 1999
 *  Summary: simple model format loader for dynamechs 2.1 formats
 *****************************************************************************/

#include <sgldb.h>
#include <sglLoadFile_cmb.hpp>
#include <sglModelLoader.hpp>

#include <sglGroup.hpp>
#include <sglOStream.hpp>

//----------------------------------------------------------------------------

sglNode *sglLoadFile_cmb(const string &model_filename,
                         unsigned int mode)
{
   ifstream data_ptr;
   data_ptr.open(model_filename.c_str());
   if (!data_ptr)
   {
      sglPrintWarning() << "sglLoadFile_cmb: Error unable to open data file: "
                        << model_filename << endl;
      return NULL;
   }

   sglGroup *root = NULL;

   unsigned int num_files;
   data_ptr >> num_files;

   if (num_files > 0)
   {
      root = new sglGroup();

      char filename[256];
      for (unsigned int i=0; i<num_files; i++)
      {
         // read a filename inside double quotes
         if (data_ptr.getline(filename, 255, '\042'))
         {
            data_ptr.getline(filename, 255, '\042');
         }

         if (strlen(filename) > 0)
         {
            sglNode *model = sglModelLoader::loadModel(filename, mode);

            if (model)
               root->addChild(model);
         }
      }

      if (root->getNumChildren() == 0)
      {
         delete root;
         root = NULL;
      }
   }

   return root;
}
