/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglSphere.cpp
 *   Author: Scott McMillan
 *  Created: 23 January 2001
 *****************************************************************************/

#include <sglSphere.hpp>

//----------------------------------------------------------------------------
sglSphere::sglSphere(unsigned int num_slices, unsigned int num_stacks,
                     float radius)
      : sglMonoIndexedTriangleStripSet(),
        m_num_slices(3),
        m_num_stacks(2),
        m_radius(1.0f)
{
   if (num_slices > 3)  m_num_slices = num_slices;
   if (num_stacks > 2)  m_num_stacks = num_stacks;
   if (m_radius > 0.0f) m_radius = radius;

   makeSphere(m_num_slices, m_num_stacks, radius);
}

//----------------------------------------------------------------------------
sglSphere::~sglSphere()
{
}

//----------------------------------------------------------------------------
void sglSphere::makeSphere(unsigned int num_slices, unsigned int num_stacks,
                           float radius)
{
   ++num_slices;
   ++num_stacks;

   sglVec3f *vertex = new sglVec3f[num_slices*num_stacks];
   sglVec3f *normal = new sglVec3f[num_slices*num_stacks];

   sglIndexType ix, index = 0;
   for (ix=0; ix<num_stacks; ++ix)
   {
      for (sglIndexType iy=0; iy<num_slices; ++iy)
      {
         double angle1 = 2.0*M_PI*(double)iy/((double)(num_slices - 1));
         float cs = (float)cos(angle1);
         float ss = (float)sin(angle1);

         double angle2 = M_PI*(-0.5+(double)ix/((double)(num_stacks-1)));
         float ct = (float)cos(angle2);
         float st = (float)sin(angle2);

         normal[index].set(cs*ct, ss*ct, st);
         vertex[index].set(radius*normal[index][0],
                           radius*normal[index][1],
                           radius*normal[index][2]);
         //m_tcoord[index].set((float)ix/(float)(num_slices - 1),
         //                    (float)iy/(float)(num_stacks - 1));
         ++index;
      }
   }

   index = 0;
   sglIndexType *length = new sglIndexType[num_slices - 1];
   sglIndexType *indices = new sglIndexType[(num_slices - 1)*2*num_stacks];

   for (ix=0; ix<num_slices-1; ++ix)
   {
      length[ix] = 2*num_stacks;
      for (sglIndexType iy=0; iy<num_stacks; ++iy)
      {
         indices[index] = ix +     num_slices*iy;
         index++;
         indices[index] = ix + 1 + num_slices*iy;
         index++;
      }
   }

   setNumPrims(num_slices - 1);
   setPrimLengths(new sglDeleteableDataPtr<sglIndexType>(length));
   setIndexList(new sglDeleteableDataPtr<sglIndexType>(indices));
   setCoordList(new sglDeleteableDataPtr<sglVec3f>(vertex));
   setNormalList(sglGeoSet::ePER_VERTEX,
                 new sglDeleteableDataPtr<sglVec3f>(normal));
}

//----------------------------------------------------------------------------
void sglSphere::printInfo(ostream &ostrm, const char *indent_string) const
{
   sglGeoSet::printInfo(ostrm, indent_string);
   ostrm << indent_string << "stacks, slices: " << m_num_stacks << ", "
         << m_num_slices << endl;
   ostrm << indent_string << "radius: " << m_radius << endl;
}

