/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglVertStruct.hpp
 *  Project: obj loader/stripifier/morphing
 *  Created: 17 July 1997
 *  Summary: vertStruct objects
 *****************************************************************************/

#ifndef __SGL_VERT_STRUCT_HPP
#define __SGL_VERT_STRUCT_HPP

#include <sgldb.h>

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructV
{
public:
   /// vertex index
   unsigned int vi;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi;}
   static bool hasNormals() {return false;}
   unsigned int getNormal() {return UINT_MAX;}
   void setNormal(unsigned int) {};
};

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructVC
{
public:
   /// vertex index
   unsigned int vi;
   /// color index
   unsigned int ci;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi + ci;}
   static bool hasNormals() {return false;}
   unsigned int getNormal() {return UINT_MAX;}
   void setNormal(unsigned int) {};
};

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructVN
{
public:
   /// vertex index
   unsigned int vi;
   /// normal index
   unsigned int ni;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi + ni;}
   static bool hasNormals() {return true;}
   unsigned int getNormal() {return ni;}
   void setNormal(unsigned int n) {ni = n;}
};

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructVNC
{
public:
   /// vertex index
   unsigned int vi;
   /// normal index
   unsigned int ni;
   /// color index
   unsigned int ci;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi + ni + ci;}
   static bool hasNormals() {return true;}
   unsigned int getNormal() {return ni;}
   void setNormal(unsigned int n) {ni = n;}
};

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructVT
{
public:
   /// vertex index
   unsigned int vi;
   /// texture coordinate index
   unsigned int ti;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi + ti;}
   static bool hasNormals() {return false;}
   unsigned int getNormal() {return UINT_MAX;}
   void setNormal(unsigned int) {};
};

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructVTC
{
public:
   /// vertex index
   unsigned int vi;
   /// texture coordinate index
   unsigned int ti;
   /// color index
   unsigned int ci;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi + ti + ci;}
   static bool hasNormals() {return false;}
   unsigned int getNormal() {return UINT_MAX;}
   void setNormal(unsigned int) {};
};

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructVNT
{
public:
   /// vertex index
   unsigned int vi;
   /// normal index
   unsigned int ni;
   /// texture coordinate index
   unsigned int ti;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi + ni + ti;}
   static bool hasNormals() {return true;}
   unsigned int getNormal() {return ni;}
   void setNormal(unsigned int n) {ni = n;}
};

//============================================================================

/// Vertex data struct
class SGLDB_DLL_API sglVertStructVNTC
{
public:
   /// vertex index
   unsigned int vi;
   /// normal index
   unsigned int ni;
   /// texture coordinate index
   unsigned int ti;
   /// color index
   unsigned int ci;

   unsigned int hash() {return vi;}
   unsigned int edgeHash() {return vi + ni + ti + ci;}
   static bool hasNormals() {return true;}
   unsigned int getNormal() {return ni;}
   void setNormal(unsigned int n) {ni = n;}
};

//============================================================================



SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructVNTC& vert);

SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructVNT& vert);

SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructVNC& vert);

SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructVN& vert);

SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructVTC& vert);

SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructVT& vert);

SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructVC& vert);

SGLDB_DLL_API
ostream& operator<<(ostream& ostrm, const sglVertStructV& vert);

SGLDB_DLL_API
bool operator==(const sglVertStructV& lhs, const sglVertStructV& rhs);

SGLDB_DLL_API
bool operator==(const sglVertStructVC& lhs, const sglVertStructVC& rhs);

SGLDB_DLL_API
bool operator==(const sglVertStructVN& lhs, const sglVertStructVN& rhs);

SGLDB_DLL_API
bool operator==(const sglVertStructVNC& lhs, const sglVertStructVNC& rhs);

SGLDB_DLL_API
bool operator==(const sglVertStructVT& lhs, const sglVertStructVT& rhs);

SGLDB_DLL_API
bool operator==(const sglVertStructVTC& lhs, const sglVertStructVTC& rhs);

SGLDB_DLL_API
bool operator==(const sglVertStructVNT& lhs, const sglVertStructVNT& rhs);

SGLDB_DLL_API
bool operator==(const sglVertStructVNTC& lhs, const sglVertStructVNTC& rhs);

#endif
