/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 * SGL's FLT loader: Copyright (C) 2001  Michael M. Morrison
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: ObjOptimizer.cpp
 *  Created: November 2001
 *  Author : Michael M. Morrison
 *  Summary: Sorts state, and stripifies at the OpenFLT Object level.
 *****************************************************************************/

#include "ObjOptimizer.hpp"
#include "sglTriStripper.hpp"

ObjectOptimizer::ObjectOptimizer( char * objName )
{
	mName = new string( objName );
}

ObjectOptimizer:: ~ObjectOptimizer()
{
	ObjectOptimizer::mapIter m;

	for( m = mStateBin.begin(); m != mStateBin.end(); ++m ) {
		// delete the attr and vertstruct
		delete m->first;
		delete m->second;
	}

	delete mName;
}

OOStateAttr *
ObjectOptimizer::findStateAttr( int sf,
																unsigned int primaryColor,
																unsigned int texture,
																unsigned int material )
{
	ObjectOptimizer::mapIter m;

	for( m = mStateBin.begin(); m != mStateBin.end(); ++m ) {
		OOStateAttr * attr = m->first;

		if( attr->isEqual( sf, primaryColor, texture, material ) )
			return attr;
	}

	return 0;
}

OOStateAttr *
ObjectOptimizer::addStateAttr( int sf,
																unsigned int primaryColor,
																unsigned int texture,
																unsigned int material )
{
	OOStateAttr * attr = new OOStateAttr();

	attr->addPrimaryColor( primaryColor );
	attr->addTexture( texture );
	attr->addMaterial( material );
	attr->addStateFlags( sf );

	return attr;
}

void 
ObjectOptimizer::addPrimitive(	int stateFlags,
															 	vector<unsigned int> & indexList,
																unsigned int primaryColor = OOStateAttr::NONE,
																unsigned int texture = OOStateAttr::NONE,
																unsigned int material = OOStateAttr::NONE )
{
	// convert into sglVertStructV vector
	vector<sglVertStructV> tmpList;

	for( unsigned int i = 0; i < indexList.size(); i++ ) {
		sglVertStructV tmp;

		tmp.vi = indexList[i];
		tmpList.push_back( tmp );

		// save for later
		mRemappedIndices.push_back( indexList[i] );
	}

	sglTriStripper<sglVertStructV> *v = 0;
	OOStateAttr * attr = 0;
	
	// see if this StateAttr is already on the list
	attr = findStateAttr( stateFlags, primaryColor, texture, material );

	if( attr ) {

		// we know it is on the list
		ObjectOptimizer::mapIter m = mStateBin.find( attr );

		v = m->second;

	} else {

		attr = addStateAttr( stateFlags, primaryColor, texture, material );

		v = new sglTriStripper<sglVertStructV>();

		mStateBin[ attr ] = v;

	}

	v->addPolyFace( tmpList );
}

void 
ObjectOptimizer::optimize( void )
{
	ObjectOptimizer::mapIter m;
	list<unsigned int> reduced;
	unsigned int idx = 0;

	// stripify all primitives in each bin
	for( m = mStateBin.begin(); m != mStateBin.end(); ++m ) {
		// stripify it
		m->second->triStripify( false );
	}

	// create a unified index list for all strips in this object
	// new list will be the list index entry points

	for( idx = 0; idx < mRemappedIndices.size(); idx++ )
		reduced.push_back( mRemappedIndices[idx] );

	reduced.sort();
	reduced.unique();
	mRemappedIndices.clear();

	// create a reverse index map too
	list<unsigned int>::const_iterator it = reduced.begin();

	for( idx = 0; idx < reduced.size(); idx++, ++it ) {
		mReverseIndexMap[ *it ] = idx;
		mRemappedIndices.push_back( *it );
	}
}

#if 0
static bool
sortFunc( const SortVertex3D & first, const SortVertex3D & second )
{
	if( first < second )
		return true;
	else
		return false;
}
#endif

void
ObjectOptimizer::vertexListOptimize( void )
{
}

void
ObjectOptimizer::vertexListBegin( void )
{
#if 0
	mVertexSortList.clear();
	mVertexXlateMap.clear();
#endif
}

void
ObjectOptimizer::vertexListEnd( void )
{
#if 0
	mVertexSortList.sort

	sort( mVertexSortList.begin(), mVertexSortList.end(), sortFunc );
#endif
}

void
ObjectOptimizer::vertexListEntry( unsigned int idx, float x, float y, float z )
{
#if 0
	SortVertex3D * sv3d = new SortVertex3D( idx, x, y, z );

	mVertexSortList.push_back( sv3d );
#endif
}


