/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLoadImage_jpeg.hpp
 *   Author: Tom Stimson
 *  Created: January 2002
 *  Summary: read JPEG format images using external library
 *****************************************************************************/

#ifndef __SGL_LOAD_IMAGE_JPEG_HPP
#define __SGL_LOAD_IMAGE_JPEG_HPP

#include <sgljpeg.h>
#include <sglTexture.hpp>

// Helper function to load a .jpg file.
// Returns a pointer to the data and sets the number of components and size
// parameters.  Will return NULL if there was a problem loading the image
extern "C"
{
   SGLJPEG_DLL_API void *loadImageJPG(const string &filepath,
                                      unsigned int &width,
                                      unsigned int &height,
                                      sglTexture::ExternalFormatEnum &format,
                                      sglTexture::TypeEnum &type);
   SGLJPEG_DLL_API void *loadImageJPEG(const string &filepath,
                                       unsigned int &width,
                                       unsigned int &height,
                                       sglTexture::ExternalFormatEnum &format,
                                       sglTexture::TypeEnum &type);
}

// Helper function to write a .jpg file.
// Returns false if an error occured while writing the file.
extern "C"
{
   SGLJPEG_DLL_API bool writeImageJPG(const string &filepath,
                                      const void *image,
                                      unsigned int width,
                                      unsigned int height,
                                      sglTexture::ExternalFormatEnum format,
                                      sglTexture::TypeEnum type);
   SGLJPEG_DLL_API bool writeImageJPEG(const string &filepath,
                                       const void *image,
                                       unsigned int width,
                                       unsigned int height,
                                       sglTexture::ExternalFormatEnum format,
                                       sglTexture::TypeEnum type);
}

#endif
