/* vi:set ts=3 sw=3 expandtab: */
/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 * SGL's MD3  loader Copyright (C) 2002  Michael M. Morrison  All Rights
 *   Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: md3file.h
 *  Created: March 2002
 *  Author : Michael M. Morrison
 *  Summary: Quake 3 MD3 file format loader.
 *           Based on file format specs by "Mental Vortex"
 *****************************************************************************/

#ifndef __MD3_H_
#define __MD3_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

//
// MD3 Header
//
typedef struct {
   char ID[4];             // id of file, always "IDP3"
   int  Version;           // always 15?
   char fileName[68];      // name of file?
   int   numBoneFrames;    // number of boneframes
   int   numTags;          // number of tags
   int   numMeshes;        // number of meshes
   int   maxTextures;      // max textures used in file
   int   headerSize;       // length of header
   int   startTag;         // starting position of tag structure
   int   endTag;           // ending position of tag structures/start of mesh
   int   fileSize;         // size of file
} MD3Header;

//
// MD3 Tag
//
typedef struct {
   char tagName[64];       // name of tag
   float position[3];      // position of tag
   float orientation[3*3]; // orientation matrix of tag
} MD3Tag;

//
// MD3 BoneFrame
//
typedef struct {
   float mins[3];
   float maxs[3];
   float position[3];
   float scale;
   char  creator[16];
} MD3BoneFrame;

//
// MD3 MeshHeader
//
typedef struct {
   char  ID[4];         // id, 'IDP3'
   char  name[68];      // name of mesh
   int   numMeshFrames; // number of mesh frames
   int   numTextures;   // number of textures
   int   numVerts;      // number of verticies
   int   numTris;       // number of triangle indices
   int   startTri;      // start of triangle data relative to mesh header
   int   headerSize;    // size of this header
   int   startTxVec;    // start of texvector data relative to mesh header
   int   startVertex;   // start of vertex data relative to mesh header
   int   meshSize;      // size of mesh
} MD3MeshHeader;

//
// MD3 Texture
//
typedef struct {
   char hak;           // not sure why this is necessary
   char name[67];      // name of texture file
} MD3Texture;

//
// MD3 Triangle
//
typedef struct {
   int  indices[3];     // vertex indices
} MD3Triangle;

//
// MD3 UVCoord
//
typedef struct {
   float  uv[2];        // uv texcoords
} MD3UVCoord;

//
// MD3 Vertex
//
typedef struct {
   signed short coord[3];     // XYZ Coords
   unsigned char envTex[2];   // env mapping texcoords?
} MD3Vertex;

//
// MD3 Mesh
//
typedef struct {
   MD3MeshHeader header;
   MD3Texture * texture;
   MD3Triangle * indices;
   MD3UVCoord * uvCoord;
   MD3Vertex * vertex;
} MD3Mesh;

typedef struct {
   MD3Header header;
   MD3Tag    * tag;
   MD3BoneFrame * boneFrame;
   MD3Mesh * mesh;
   FILE * file;
} MD3File;

// loader funcs
extern MD3File * md3Open( const char * name );
extern int md3Parse( MD3File * file );
extern void md3Close( MD3File * file );
extern void md3Free( MD3File * file );

#ifdef __cplusplus
}
#endif

#endif /* _MD3_H_ */
