/* vi:set ts=3 sw=3 expandtab: */
/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 * SGL's MD3 loader Copyright (C) 2002  Michael M. Morrison  All Rights
 *   Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLoadFile_md3.hpp
 *  Created: March 2002
 *  Author : Michael M. Morrison
 *  Summary: Quake 3 MD3 file format loader.
 *****************************************************************************/

#ifndef __SGL_LOAD_FILE_MD3_HPP
#define __SGL_LOAD_FILE_MD3_HPP

#include <sglmd3.h>
#include <sglModelLoader.hpp>
#include <sglTexture2D.hpp>
#include "md3file.h"

#include <map>

class sglGroup;

//============================================================================

class SGLMD3_DLL_API sglLoadFile_md3
{
public:
   /// default constructor
   sglLoadFile_md3();
   /// virtual destructor
   virtual ~sglLoadFile_md3();

   /// load a file, build a scene graph, and get the sglGeode* pointing to it
   sglNode *loadFile(const string &filename, unsigned int mode);

private:
   /// Copy constructor forbidden
   sglLoadFile_md3(const sglLoadFile_md3 &);
   /// Assignment operator forbidden
   sglLoadFile_md3 &operator=(const sglLoadFile_md3 &);

   void buildScene( sglGroup *parent );

   sglTexture2D * lookupTexture( const char * );
   sglTexture2D * createTexture( const string & );

private:

   string m_alternate_path;
   MD3File * m_md3;

private:

   // texture cache
   static map<string, sglTexture2D*> s_tex_map;

   // default minification and magnification filters
   sglTexture::FilterEnum m_default_min_filter;
   sglTexture::FilterEnum m_default_mag_filter;

   bool   m_compress_textures;   // compress textures during load
   sglTexture::CompressionLevelEnum m_default_compression_level;

   bool   m_verbose;
};

#endif /* __SGL_LOAD_FILE_MD3_HPP */
