/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * Copyright 2000, Scott McMillan
 *****************************************************************************
 *     File: sglObjMtl.cpp
 *  Created: 30 September 1997
 *  Summary: 
 *****************************************************************************/

#include <sglobj.h>
#include <sglTexture2D.hpp>
#include <sglObjMtl.hpp>
#include <sglOStream.hpp>
#include <sglImageLoader.hpp>

class sglObjTexture : public sglTexture2D
{
public:
   sglObjTexture() {};
   ~sglObjTexture()
      {
         void *image;
         ExternalFormatEnum format;
         TypeEnum type;
         unsigned int num_s, num_t;
         bool border;
         getImage(image, format, type, num_s, num_t, border);
         if (image)
            free(image);
      }
};


//----------------------------------------------------------------------------
//    Summary: default constructor initializes to default OpenGL material
sglObjMtl::sglObjMtl() :
      m_material(new sglMaterial),
      m_texture(NULL),
      m_refl_tex(NULL)
{
   m_tex_scale[0] = m_tex_scale[1] = 1.0f;
   m_rtex_scale[0] = m_rtex_scale[1] = 1.0f;
}

//----------------------------------------------------------------------------
sglObjMtl::~sglObjMtl()
{
}

//----------------------------------------------------------------------------
bool sglObjMtl::loadTexture(
   const string &filename, float scale_u, float scale_v)
{
   /* FIXME - currently if a base texture has already be set, and an
      invalid one is set here the old definition is lost */

   if (m_texture)
   {
      sglPrintWarning() << "Warning: redefining texture "
                        << m_texture->getName()
                        << " to " << filename << endl;
      delete m_texture;
      m_tex_scale[0] = m_tex_scale[1] = 1.0f;
      m_texture = NULL;
   }

   sglTexture::ExternalFormatEnum format;
   sglTexture::TypeEnum type;
   unsigned int num_s, num_t;
   void* image =
      sglImageLoader::loadImage(filename, num_s, num_t, format, type);
   if (image && num_s && num_t)
   {
      sglTexture2D *tex = new sglObjTexture;
      tex->setImage(image, format, type, num_s, num_t, false);
      m_texture = tex;
      m_tex_scale[0] = scale_u;
      m_tex_scale[1] = scale_v;

      return true;
   }

   return false;
}

//----------------------------------------------------------------------------
bool sglObjMtl::loadReflTexture(
   const string &filename, float scale_u, float scale_v)
{
   /* FIXME - currently if a reflective texture has already be set, and an
      invalid one is set here the old definition is lost */

   if (m_refl_tex)
   {
      sglPrintWarning() << "Warning: redefining refl_tex "
                        << m_refl_tex->getName()
                        << " to " << filename << endl;
      delete m_refl_tex;
      m_rtex_scale[0] = m_rtex_scale[1] = 1.0f;
      m_refl_tex = NULL;
   }

   sglTexture::ExternalFormatEnum format;
   sglTexture::TypeEnum type;
   unsigned int num_s, num_t;
   void* image =
      sglImageLoader::loadImage(filename, num_s, num_t, format, type);
   if (image && num_s && num_t)
   {
      sglTexture2D *tex = new sglObjTexture;
      tex->setImage(image, format, type, num_s, num_t, false);
      m_refl_tex = tex;
      m_rtex_scale[0] = scale_u;
      m_rtex_scale[1] = scale_v;

      return true;
   }

   return false;
}
