/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglObjMtl.hpp
 *  Created: 30 September 1997
 *  Summary: 
 *****************************************************************************/

#ifndef __SGL_OBJ_MTL_HPP
#define __SGL_OBJ_MTL_HPP

#include <sglobj.h>
#include <sglVector.hpp>
#include <sglTexture.hpp>
#include <sglMaterial.hpp>

class SGLOBJ_DLL_API sglObjMtl
{
public:
   /// default constructor
   sglObjMtl();
   ///  destructor - not virtual - DO NOT SUBCLASS
   ~sglObjMtl();

   /// Specify the material name
   void setName(const char *name) { strcpy(m_name, name); }
   /// get the material name
   const char *getName() { return m_name; }

   /// get a handle to the material object
   sglMaterial *getMaterial() {return m_material;}

   /// not sure if this actually loads the image data yet.
   bool loadTexture(const string &filename, float scale_u, float scale_v);
   /// not sure if this actually loads the image data yet.
   bool loadReflTexture(const string &filename, float scale_u, float scale_v);


   /// get a handle to the struct containing texture info
   sglTexture *getTexture(float *su, float *sv)
      {
         *su = m_tex_scale[0]; *sv = m_tex_scale[1];
         return m_texture;
      }

   /// get a handle to the struct containing reflective texture info
   sglTexture *getReflTexture(float *su, float *sv)
      {
         *su = m_rtex_scale[0]; *sv = m_rtex_scale[1];
         return m_refl_tex;
      }

   /// make OpenGL calls to set up this material
   void draw();
   void undraw();

private:
   /// allocate a texture structure
   sglTexture *allocateTexture();

   /// copy constructor forbidden
   sglObjMtl(const sglObjMtl &);

   // assignment operator forbidden
   sglObjMtl &operator=(const sglObjMtl &);

private:
   /// material name
   char m_name[256];

   /// the actual material object
   sglMaterial *m_material;

   /// structure containing texture info
   sglTexture *m_texture;
   float       m_tex_scale[2];
   /// structure containing reflective texture info
   sglTexture *m_refl_tex;
   float       m_rtex_scale[2];
};

#endif
