/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglObjMtlMgr.hpp 
 *   Author: Scott McMillan
 *  Created: 30 September 1997
 *  Summary: The material manager that maintains a list of mtllib files, and
 *           searches them in order when queried for named materials
 *****************************************************************************/

#ifndef __SGL_OBJ_MTL_MGR_HPP
#define __SGL_OBJ_MTL_MGR_HPP

#include <sglobj.h>
#include <string>
#include <vector>
#include <sglObjMtl.hpp>

class SGLOBJ_DLL_API sglObjMtlMgr
{
public:
   /// default constructor
   sglObjMtlMgr();
   /// virtual destructor
   virtual ~sglObjMtlMgr();

   /// add a new filename to the mtllib list, return TRUE if successful
   bool addMtlLib(const char *filename);

   /** get a handle to a material object if it is not found in the mtllibs,
       a handle to a default material is returned.
    */
   sglObjMtl *getMtl(const char *mtl_name);

   /// add a material to the list
   bool addMtl(sglObjMtl *mtl);

private:
   /** search the mtllib filename for the material name, parse and create a
       material (if found and return a handle to it
    */
   sglObjMtl *parseMtlLib(const string &filename, const char *mtl_name);

   /// parse a texture entry
   void parseTexture(char *next, string &tex_filename, float &su, float &sv);

   /// copy constructor forbidden
   sglObjMtlMgr(const sglObjMtlMgr &);
   // assignment operator forbidden
   sglObjMtlMgr &operator=(const sglObjMtlMgr &);

private:
   /// number of mtllibs currently stored in the list
   //unsigned int m_num_mtllibs;
   /// mtllib list
   //char **m_mtllib_list;
   vector<string> m_mtllib_list;

   /// number of currently active materials
   //unsigned int m_num_mtls;
   /// list of pointers to currently active materials
   //sglObjMtl **m_mtl_list;
   vector<sglObjMtl*> m_mtl_list;
};

#endif
