/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

#include <sglLoadFile_txp.hpp>
#include <sglDataLoader.hpp>
#include <sglTrPageArchive.hpp>
#include <sglTrPageTest.hpp>
#include <sglOStream.hpp>

//----------------------------------------------------------------------------

extern "C"
{
    SGLTXP_DLL_API
    void *loadModelTXP(const string &filepath, unsigned int /* mode */)
    {
        sglTrPageGroup *scenegraph = new sglTrPageGroup();
        scenegraph->loadFile(filepath);
        return scenegraph;
    }
}

//----------------------------------------------------------------------------
sglTrPageGroup::sglTrPageGroup()
      : sglGroup()
{
   m_archive = new sglTrPageArchive;
}

//----------------------------------------------------------------------------
sglTrPageGroup::~sglTrPageGroup()
{
   delete m_archive;
}

//----------------------------------------------------------------------------
bool sglTrPageGroup::loadFile(const string &filename)
{
   bool ret = false;

   // search the SGL data path
   string foundname;
   if (sglDataLoader::findFile(filename, NULL, foundname))
   {
      if (m_archive->OpenFile(foundname))
      {
         sglPrintInfo() << "sglTrPageGroup::loadFile(): loading archive: "
                        << foundname << endl;
         m_archive->LoadMaterials();
         m_archive->LoadModels();

         sglPrintInfo() << "sglTrPageGroup::loadFile(): loading geometry"
                        << endl;
         addChild(m_archive->LoadAllTiles());

         sglPrintInfo() << "sglTrPageGroup::loadFile(): loaded archive: "
                        << foundname << endl;
         //sgluOutputTree(sceneGraph, cout, 3);
         ret = true;
      }
      else
      {
         sglPrintWarning() << "Failed to load archive: " << foundname << endl;
      }
   }
   else
   {
      sglPrintWarning() <<"sglTrPageGroup::loadFile() failed to find archive: "
                        << foundname << endl;
   }
   return ret;
}
