/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLoadFile_txp.hpp
 *   Author: Bryan Walsh
 *  Created: 5 October 2001
 *  Summary: Terrex terrain page loader for SGL
 *****************************************************************************/

#ifndef __SGL_LOAD_FILE_TXP_HPP
#define __SGL_LOAD_FILE_TXP_HPP

#include <trpage_sys.h>
#include <sgltxp.h>
#include <sglGroup.hpp>
#include <string>

class sglTrPageArchive;

//----------------------------------------------------------------------------
/**
   Subclass of sglGroup that has function to load the Terrex Terrain Page
   archive and add nodes to this group.  The user may instantiate one of these
   classes explicitly and load an archive, or the user can depend on the
   plug-in capability of the sglModelLoader to load a .txp file (the
   recommended approach).
*/

class SGLTXP_DLL_API sglTrPageGroup : public sglGroup
{
public:
   /// constructor.
   sglTrPageGroup();
   /// virtual destructor.
   virtual ~sglTrPageGroup();

   /** Load a Terrex Terrain Page archive.  This node will become the root of
       any scenegraph that is created to represent the archive.
       @param filename Filename of the *.txp file (optionally with path).
       @return true if the load is successful; otherwise false.
   */
   bool loadFile(const string &filename);

private: // not implemented
   sglTrPageGroup(const sglTrPageGroup &);
   sglTrPageGroup &operator=(const sglTrPageGroup &);

private:
   sglTrPageArchive *m_archive;
};

#endif
