/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTrPageArchive.hpp
 *   Author: Bryan Walsh
 *  Created: 5 October 2001
 *  Summary: Implementation of sglTrPageArchive object.  This class parses a
 *           TerraPage tile into SGL's scene graph structure.
 *
 * This loader is based on/modified from Terrain Experts Performer Loader,
 * and was ported to SGL by Bryan Walsh / bryanw at earthlink dot net
 *
 * That loader is redistributed under the terms listed on Terrain Experts
 * website (www.terrex.com/www/pages/technology/technologypage.htm)
 *
 * "TerraPage is provided as an Open Source format for use by anyone...
 * We supply the TerraPage C++ source code free of charge.  Anyone
 * can use it and redistribute it as needed (including our competitors).
 * We do, however, ask that you keep the TERREX copyrights intact."
 *
 * Copyright Terrain Experts Inc. 1999.
 * All Rights Reserved.
 *
 *****************************************************************************/

#ifndef __SGL_TRPAGE_ARCHIVE_HPP
#define __SGL_TRPAGE_ARCHIVE_HPP

#include <trpage_sys.h>
#include <sgl.h>
#include <string>
#include <trpage_read.h>
#include <sglTrPageParser.hpp>

//----------------------------------------------------------------------------

/**
   SGL TerraPage archive. This class assists in setting up a Terra Page scene
   graph.
*/

class sglTrPageArchive : public trpgr_Archive
{
public:
   /// Constructor - sets up all callbacks for parsinga
   sglTrPageArchive();

   /// destructor
   ~sglTrPageArchive();

   /** Open an archive and read the header (take no other action).
       @param filepath Filename of archive to open (optionally with a path).
       @return true if file opened successfully and the header was correct.
   */
   bool OpenFile(const string &filepath);

   /** To avoid SGI compiler warnings
    */
   virtual bool OpenFile(const char *filepath)
      { return OpenFile(string(filepath)); }

   /// Load and create textures and materials
   void LoadMaterials();

   // Load and create models - not implemented yet.
   bool LoadModels();

   /** Load a tile and return an sglGroup with children.
       @param x Tile location input - x dimension.
       @param y Tile location input - y dimension.
       @param lod Tile LOD level input.
       @return The parent ID of this tile to let you hook it into the scene
               graph.
   */
   sglGroup *LoadTile(int x,int y,int lod,int &parent);

   /** Load all the tiles and assemble them into a scene graph.
       Don't use this if you're paging.  However, it is an
       example of how to hook the scenegraph together.
       @return The parent of the complete scene graph.
   */
   sglGroup *LoadAllTiles();

protected:
   // Helper class used to parse a tile into a scene graph
   sglTrPageParser *parse;

   // These are the texture, material, and model lists
   vector<sglTexture2D *> m_textures;
   vector<tmpGeoState>    m_gstates;
   vector<sglNode *>      m_models;
   string                 m_alternate_path;
};

#endif
