/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTrPageParser.hpp
 *   Author: Bryan Walsh
 *  Created: 5 October 2001
 *  Summary: Implementation of sglTrPageParser object.
 *
 * This loader is based on/modified from Terrain Experts Performer Loader,
 * and was ported to SGL by Bryan Walsh / bryanw at earthlink dot net
 *
 * That loader is redistributed under the terms listed on Terrain Experts
 * website (www.terrex.com/www/pages/technology/technologypage.htm)
 *
 * "TerraPage is provided as an Open Source format for use by anyone...
 * We supply the TerraPage C++ source code free of charge.  Anyone
 * can use it and redistribute it as needed (including our competitors).
 * We do, however, ask that you keep the TERREX copyrights intact."
 *
 * Copyright Terrain Experts Inc. 1999.
 * All Rights Reserved.
 *
 *****************************************************************************/

#ifndef __SGL_TRPAGE_PARSE_HPP
#define __SGL_TRPAGE_PARSE_HPP

#include <sglVector.hpp>
#include <trpage_read.h>

class sglGroup;
class sglNode;
class sglMaterial;
class sglTexture;
class sglTexture2D;
class sglTexEnv;
class sglLighting;
class sglTransparency;
class sglCullMode;

//----------------------------------------------------------------------------
// encapsulates the contents equivalent to a pfGeoState
struct tmpGeoState
{
   sglTexture2D    *tex;
   sglTexEnv       *texEnv;
   sglLighting     *lighting;
   sglTransparency *transparency;
   sglCullMode     *cullMode;
   sglMaterial     *material;    // new - necessary?
};

//----------------------------------------------------------------------------
class sglTrPageParser : public trpgSceneParser
{
public:
   sglTrPageParser();
   ~sglTrPageParser();

   // Call this to parse a buffer
   sglGroup *ParseScene(trpgReadBuffer &,
                        vector<tmpGeoState> &,
                        vector<sglNode *> &);

   // Return the parent of a recently parsed tile
   int GetParentID() { return parentID; }
   void SetParentID(int id) { parentID = id; }
   // Return a reference to the tile header (after a tile has been read)
   trpgTileHeader *GetTileHeaderRef();

   // Return the current top node (used during parsing)
   sglGroup *GetCurrTop();

   // Return the current material list (passed in to ParseScene())
   vector<tmpGeoState> *GetMaterials() { return materials; }
   vector<sglNode *> *GetModels() { return models; }

   // Add the given pfGroup to the group list
   bool AddToGroupList(int id,sglGroup *);

   // Return the group list
   // Note: should be const
   vector<sglGroup *> *GetGroupList() { return &groupList; }

   // Set the maximum number of groups (once per archive)
   void SetMaxGroupID(int);

protected:
   bool StartChildren(void *);
   bool EndChildren(void *);

protected:
   sglGroup *currTop;      // Current parent group
   sglGroup *top;          // Top group
   trpgTileHeader tileHead;        // Dump tile header here
   int parentID;        // If there was an attach node, this is
                        // the tile's parent ID.  -1 otherwise
   vector<tmpGeoState> *materials;
   vector<sglGroup *> groupList;
   vector<sglNode *> *models;
};


// **********************************
// TerraPage SGL scene parser helpers
// **********************************

//----------------------------------------------------------------------------
class geomRead : public trpgr_Callback {
public:
   geomRead(sglTrPageParser *in_parse);
   ~geomRead();
   void *Parse(trpgToken tok,trpgReadBuffer &buf);
protected:
   sglTrPageParser *parse;
   sglVec3f *m_verts;
   sglVec2f *m_texCoord;
   sglVec3f *m_norms;
   unsigned int *m_primLens;
   vector <sglVec3f *> m_vec3_malloc_list;
   vector <sglVec2f *> m_vec2_malloc_list;
   vector <unsigned int *> m_uint_malloc_list;
};

//----------------------------------------------------------------------------
class groupRead : public trpgr_Callback {
public:
   groupRead(sglTrPageParser *in_parse);
   void *Parse(trpgToken tok,trpgReadBuffer &buf);
protected:
   sglTrPageParser *parse;
};

//----------------------------------------------------------------------------
class attachRead : public trpgr_Callback {
public:
   attachRead(sglTrPageParser *in_parse);
   void *Parse(trpgToken tok,trpgReadBuffer &buf);
protected:
   sglTrPageParser *parse;
};

//----------------------------------------------------------------------------
class billboardRead : public trpgr_Callback {
public:
   billboardRead(sglTrPageParser *in_parse);
   void *Parse(trpgToken tok,trpgReadBuffer &buf);
protected:
   sglTrPageParser *parse;
};

//----------------------------------------------------------------------------
class lodRead : public trpgr_Callback {
public:
   lodRead(sglTrPageParser *in_parse);
   void *Parse(trpgToken tok,trpgReadBuffer &buf);
protected:
   sglTrPageParser *parse;
};

//----------------------------------------------------------------------------
// Read and convert a model reference
class modelRefRead : public trpgr_Callback {
public:
   modelRefRead(sglTrPageParser *in_parse);
   void *Parse(trpgToken tok,trpgReadBuffer &buf);
protected:
   sglTrPageParser *parse;
};

//----------------------------------------------------------------------------
class tileHeaderRead : public trpgr_Callback {
public:
   tileHeaderRead(sglTrPageParser *in_parse);
   void *Parse(trpgToken tok,trpgReadBuffer &buf);
protected:
   sglTrPageParser *parse;
};

#endif
