/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglTrPageTest.hpp
 *   Author: Bryan Walsh
 *  Created: 19 October 2001
 *  Summary: Terrex terrain page loader unit test for SGL
 *****************************************************************************/

#include <trpage_read.h>
#include <trpage_scene.h>
#include <sglTrPageTest.hpp>
#include <sglDataLoader.hpp>
#include <sglOStream.hpp>


//----------------------------------------------------------------------------
sglTrPageTest::sglTrPageTest()
{

}

//----------------------------------------------------------------------------
sglTrPageTest::~sglTrPageTest()
{

}

//----------------------------------------------------------------------------
bool sglTrPageTest::testLoader(const string &filepath)
{
   bool ret = true;
   trpgr_Archive archive;

   string alternate_path = sglDataLoader::extractDirectoryFromPath(filepath);
   string name = sglDataLoader::extractFilenameFromPath(filepath);

   archive.SetDirectory(alternate_path.c_str());

   if (archive.OpenFile(name.c_str()))
   {
      sglPrintWarning() << "sglTrPageTest::testLoader(): "
                        << "Opened archive: " << name.c_str() << endl;
      if (testHeader(archive)) {
         sglPrintWarning() << "sglTrPageTest::testLoader(): "
                           << "Read header" << endl;
         if (testTiles(archive))
         {
            sglPrintWarning() << "sglTrPageTest::testLoader(): "
                              << "Read tiles" << endl;
         }
         else
         {
            sglPrintWarning() << "sglTrPageTest::testLoader(): "
                              << "Failed to read tiles" << endl;
         }
      }
      else
      {
         sglPrintWarning() << "sglTrPageTest::testLoader(): "
                           << "Failed to read header" << endl;
         ret = false;
      }
   }
   else
   {
      sglPrintWarning() << "sglTrPageTest::testLoader() error: "
                        << "couldn't open archive: " << name.c_str() << endl;
      ret = false;
   }
   return ret;
}

//----------------------------------------------------------------------------
bool sglTrPageTest::testHeader(trpgr_Archive &archive)
{
   bool ret = true;
   if (!archive.ReadHeader())
   {
      ret = false;
   }
   return ret;
}

//----------------------------------------------------------------------------
bool sglTrPageTest::testTiles(trpgr_Archive &archive)
{
   int numLod;
   trpg2iPoint tileSize;
   trpgSceneGraphParser parse;
   trpgReadNode *scene;
   trpgSceneGraphParser::GroupMap gmap;

   if (!archive.isValid()) return false;

   const trpgHeader *head = archive.GetHeader();
   head->GetNumLods(numLod);

   // Iterate over the lods
   int nl,x,y;
   trpgMemReadBuffer buf(archive.GetEndian());
   trpg2dPoint ll,ur;
   for (nl = 0; nl < numLod; nl++)
   {
      head->GetLodSize(nl,tileSize);
      // Iterate over the tiles within those
      for (x = 0; x < tileSize.x; x++)
      {
         for (y = 0; y < tileSize.y; y++)
         {
            archive.trpgGetTileMBR(x,y,nl,ll,ur);
            if (archive.ReadTile(x,y,nl,buf))
            {
               // Parse it
               scene = parse.ParseScene(buf,gmap);
               if (scene) delete scene;
            }
         }
      }
   }
   return true;
}
