/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglu.h
 *  Created: 11 November 1998
 *  Summary: SGL Utility library...miscellaneous objects and functions.
 *****************************************************************************/

#ifndef __SGLU_H
#define __SGLU_H

#if defined(WIN32) && defined(_DLL)
// The next define will come from the makefile for archive objects.
#ifdef sglu_DLL_FILE
#define SGLU_DLL_API __declspec(dllexport)
#else
#define SGLU_DLL_API __declspec(dllimport)
#endif
#else
#define SGLU_DLL_API
#endif

#include <sgl.h>
#include <sglNode.hpp>

// ---------------------------------------------------------------------
inline void cross(float a[3], float b[3], float c[3])
{
   c[0] = a[1]*b[2] - a[2]*b[1];
   c[1] = a[2]*b[0] - a[0]*b[2];
   c[2] = a[0]*b[1] - a[1]*b[0];
}

//----------------------------------------------------------------------------
inline float normalize(float v[3])
{
    float norm = sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);

    if (norm > 0.0)
    {
       v[0] /= norm;
       v[1] /= norm;
       v[2] /= norm;
    }

    return norm;
}

//----------------------------------------------------------------------------
inline void compute_face_normal(float v0[3], float v1[3], float v2[3],
                                float normal[3])
{
    float a[3], b[3];
    register int i;

    for (i=0; i<3; i++)
    {
        a[i] = v1[i] - v0[i];
        b[i] = v2[i] - v0[i];
    }

    cross(a, b, normal);
    normalize(normal);

}

//--------------------------------------------------------------------------

SGLU_DLL_API void sgluOutputTree(sglNode *root,
                                 ostream &ostrm,
                                 unsigned int indent_spaces);
SGLU_DLL_API void sgluOutputPath(sglNode *end_point,
                                 ostream &ostrm,
                                 unsigned int indent_spaces);

/** This function traverses the tree rooted at root, finds all sglTextures and
    binds them.  This should only be called with a GL graphics context
    current.
    @param root Root node of subgraph to be traversed.
*/
SGLU_DLL_API void sgluBindTextures(sglNode *root);

/** This functions traverses the tree rooted at root, searching for the object
    with the specified name.
    @param name Name of object to be found
    @param root Root object of subgraph to be traversed.  Can be an sglNode or
       sglDrawable.
    @return Pointer to first object encountered with the given name, or NULL if
       none was found.
*/
SGLU_DLL_API sglObject *sgluFindObject(const string &name, sglObject *root);

#endif
