/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sgluMouse.cpp
 *  Created: 22 March 1997
 *  Summary: a struct to store state of the mouse in the current window
 *           -- was dmGLMouse.cpp
 *****************************************************************************/

#include <GL/glut.h>
#include <sglu.h>
#include <sgluMouse.hpp>
#include <sglOStream.hpp>

sgluMouse *s_mouse = NULL;

//----------------------------------------------------------------------------
void sgluMouse::handleEntry(int state)
{
   if (state == GLUT_ENTERED)
   {
      s_mouse->in_window_flag = TRUE;
      sglPrintDebug() << "mouse entered window" << endl;
   }
   else if (state == GLUT_LEFT)
   {
      s_mouse->in_window_flag = FALSE;
      sglPrintDebug() << "mouse left window" << endl;
   }
   else
   {
      sglPrintWarning() << "sgluMouse::handleEntry Error: got unknown state: "
                        << state << endl;
   }
}

//----------------------------------------------------------------------------
void sgluMouse::handleMouse(int button, int state, int x, int y)
{
   if (state == GLUT_DOWN)
   {
      sglPrintDebug() << "button press: " << button << endl;
      switch (button)
      {
         case GLUT_LEFT_BUTTON:
            s_mouse->button_flags |= sgluMouse::eLEFT_DOWN;
            break;

         case GLUT_MIDDLE_BUTTON:
            s_mouse->button_flags |= sgluMouse::eMIDDLE_DOWN;
            break;

         case GLUT_RIGHT_BUTTON:
            s_mouse->button_flags |= sgluMouse::eRIGHT_DOWN;
            break;
      }
   }
   else // GLUT_UP
   {
      sglPrintDebug() << "button release: " << button << endl;
      switch (button)
      {
         case GLUT_LEFT_BUTTON:
            s_mouse->button_flags &= ~sgluMouse::eLEFT_DOWN;
            break;

         case GLUT_MIDDLE_BUTTON:
            s_mouse->button_flags &= ~sgluMouse::eMIDDLE_DOWN;
            break;

         case GLUT_RIGHT_BUTTON:
            s_mouse->button_flags &= ~sgluMouse::eRIGHT_DOWN;
            break;
      }
   }

   s_mouse->xwin = x;
   s_mouse->ywin = y;
   s_mouse->xchan = (2.0*((GLfloat) s_mouse->xwin) - s_mouse->win_size_x)/
                     (GLfloat) s_mouse->win_size_x;
   s_mouse->ychan = (2.0*((GLfloat) s_mouse->ywin) - s_mouse->win_size_y)/
                     (GLfloat) s_mouse->win_size_y;

   sglPrintDebug() << "Pos: " << x << ", " << y << "  Button state: "
                   << s_mouse->button_flags << endl;
}

//----------------------------------------------------------------------------
void sgluMouse::handleMotion(int x, int y)
{
   s_mouse->xwin = x;
   s_mouse->ywin = y;

   s_mouse->xchan = (2.0*((GLfloat) s_mouse->xwin) - s_mouse->win_size_x)/
                     (GLfloat) s_mouse->win_size_x;
   s_mouse->ychan = (2.0*((GLfloat) s_mouse->ywin) - s_mouse->win_size_y)/
                     (GLfloat) s_mouse->win_size_y;

   //sglPrintDebug() << "Motion Pos: " << x << ", " << y << endl;
}

//----------------------------------------------------------------------------
void sgluMouse::handlePassiveMotion(int x, int y)
{
   s_mouse->xwin = x;
   s_mouse->ywin = y;

   s_mouse->xchan = (2.0*((GLfloat) s_mouse->xwin) - s_mouse->win_size_x)/
                     (GLfloat) s_mouse->win_size_x;
   s_mouse->ychan = (2.0*((GLfloat) s_mouse->ywin) - s_mouse->win_size_y)/
                     (GLfloat) s_mouse->win_size_y;
}

//----------------------------------------------------------------------------
sgluMouse *sgluMouse::initialize()
{
   // create one if not yet created; otherwise, return the previous one
   if (!s_mouse)
   {
      s_mouse = new sgluMouse;

      if (glutGetWindow())
      {
          glutEntryFunc(handleEntry);
          glutMouseFunc(handleMouse);
          glutMotionFunc(handleMotion);
          glutPassiveMotionFunc(handlePassiveMotion);
      }
      else
      {
          sglPrintWarning()
             << "sgluMouse::initialize() error: no window.  "
             << "Possible explanations include: " << endl
             << "  - glutInit has not been called in this thread" << endl
             << "  - you need to create and link with a shared library "
             << "libglut.so" << endl;
      }

      s_mouse->in_window_flag = FALSE;
      s_mouse->button_flags = 0x0;
      s_mouse->xwin = s_mouse->ywin = 0;
      s_mouse->xchan = s_mouse->ychan = 0.0;
      s_mouse->win_size_x = s_mouse->win_size_y = 1;
   }

   return s_mouse;
}
