/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sgluMouse.hpp
 *  Created: 22 March 1997
 *  Summary: a struct to store state of the mouse in the current window.
 *           -- was dmGLMouse.hpp
 *****************************************************************************/

#ifndef __SGLU_MOUSE_HPP
#define __SGLU_MOUSE_HPP

#include <sglu.h>

//----------------------------------------------------------------------------

struct SGLU_DLL_API sgluMouse
{
   enum {eLEFT_DOWN   = 0x01,
         eMIDDLE_DOWN = 0x02,
         eRIGHT_DOWN  = 0x04};

   int win_size_x;
   int win_size_y;
                             
   int in_window_flag;  // TRUE if mouse is in window
   int button_flags;    // bitmasks 4,2,1 for right,middle,left buttons
   int xwin, ywin;      // position within window
   float xchan, ychan;  // normalized window position

   // not thread safe - only one mouse handle exists and is returned by the 
   static sgluMouse* initialize();

   ~sgluMouse() {};

   static void handleEntry(int state);
   static void handleMouse(int button, int state, int x, int y);
   static void handleMotion(int x, int y);
   static void handlePassiveMotion(int x, int y);

protected:
   sgluMouse() {};

private: // not implemented
   sgluMouse(const sgluMouse &);
   sgluMouse &operator=(const sgluMouse &);
};

#endif


