/* vi:set ts=3 sw=3 expandtab: */
/* I do not believe that the "original" vi has 'expandtab' (TABs=spaces) */
/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 * SGL's WRL loader: Copyright (C) 2001  Michael M. Morrison
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLoadFile_wrl.cpp
 *  Created: April 2002
 *  Author : Michael M. Morrison
 *  Summary: VRML 2.0 file loader.
 *****************************************************************************/

#include <sglLoadFile_wrl.hpp>
#include <sglImageLoader.hpp>

#include <sglGroup.hpp>
#include <sglTransform.hpp>
#include <sglLOD.hpp>
#include <sglSwitch.hpp>
#include <sglGeode.hpp>

#include <sglIndexedTriangleStripSet.hpp>
#include <sglIndexedTriangleSet.hpp>

#include <sglCone.hpp>
#include <sglCube.hpp>
#include <sglCylinder.hpp>
#include <sglSphere.hpp>
#include <sglTextSet.hpp>
#include <sglTexturedTextSet.hpp>
#include <sglTextureFont.hpp>

#include <sglStatelet.hpp>
#include <sglMaterial.hpp>
#include <sglTexture2D.hpp>
#include <sglTexEnv.hpp>

#include <sglBillboard.hpp>
#include <sglAxes.hpp>

#include <math.h>

#include "uniq.hpp"

//----------------------------------------------------------------------------

extern "C"
{
   SGLWRL_DLL_API
   void *loadModelWRL(const string &filepath,
                      unsigned int mode)
   {
      sglLoadFile_wrl ldr;
      sglNode *model = ldr.loadFile(filepath, mode);
      return model;
   }
}

//============================================================================
// class sglLoadFile_wrl
//============================================================================

map<string, sglTexture2D*> sglLoadFile_wrl::s_tex_map;

//----------------------------------------------------------------------------
sglLoadFile_wrl::sglLoadFile_wrl()
{
   //
   // tunable params  see sglLoadFile_wrl.hpp for details
   //
   m_compute_all_normals = false;
   m_compute_normals_if_absent = false;

   m_use_colormaterial = false;
   m_compress_textures = false;

   m_verbose = false;

   m_retain_comments = true;

   m_enable_lighting = true;
   m_enable_shading = true;

   m_enable_line_aa = false;

   m_use_alphafunc_transparency = true;
   m_alpha_threshold = 0.5f;

   m_free_texture_images = false; // don't turn this on yet.

   m_sort_unique = true;

   m_default_min_filter = sglTexture::eLINEAR_MIPMAP_LINEAR;
   m_default_mag_filter = sglTexture::eLINEAR;
   m_default_compression_level = sglTexture::eCOMPRESS_MAX;

   m_primitive_quality = 0.5;

   m_min_texture_dimension = 1;

   m_display_stats = true;
   m_num_points = 
   m_num_lines = 
   m_num_tris = 
   m_num_quads =
   m_num_polys = 
   m_num_strip_tris = 
   m_num_strip_quads =
   m_num_strip_polys = 
   m_avg_strip_len = 
   m_max_strip_len = 
   m_num_lines = 
   m_num_verts =
   m_num_strips =
   m_num_materials =
   m_num_textures =
   m_opaque_texels =
   m_num_indeptris =
   m_trans_texels = 0;

   m_min_strip_len = 99999;

   // get callback, if it exists
   m_callback = sglModelLoader::getCallback( "wrl", &m_callbackUserData );
   string options = sglModelLoader::getOptionString( "wrl" );

#if 0
   if( options.empty() )
      sglPrintInfo() << "Options string empty" << endl;
   else
      sglPrintInfo() << "Options: " << options << endl;
#endif
}

//----------------------------------------------------------------------------
sglLoadFile_wrl::~sglLoadFile_wrl()
{ 
   if( m_sceneGraph )
      delete m_sceneGraph;
}

//----------------------------------------------------------------------------
sglTexture2D *sglLoadFile_wrl::createPixelTexture( unsigned int num_s,
                                                   unsigned int num_t,
                                                   unsigned int depth,
                                                   void * image,
                                                   bool repeatS, 
                                                   bool repeatT )
{
   sglTexture::ExternalFormatEnum format;
   sglTexture::TypeEnum type;
   sglTexture::WrapEnum ws, wt;
   sglTexture2D * tex = NULL;

   switch( depth )
   {
      case 1:
         format = sglTexture::eLUMINANCE;
         break;
         
      case 2:
         format = sglTexture::eLUMINANCE_ALPHA;
         break;

      case 3:
         format = sglTexture::eRGB;
         break;

      case 4:
         format = sglTexture::eRGBA;
         break;

      default:
         sglPrintInfo() << "createPixelTexture: unknown depth format.\n";
         return 0;
   }

   // not sure if this is right?
   type = sglTexture::eUNSIGNED_BYTE;

   if (image && num_s && num_t)
   {
      tex = new sglTexture2D;

      tex->setAutoMipmapFlag(true);

      if( repeatS )
         ws = sglTexture::eREPEAT;
      else
         ws = sglTexture::eCLAMP;

      if( repeatT )
         wt = sglTexture::eREPEAT;
      else
         wt = sglTexture::eCLAMP;

      tex->setWrapMode( ws, wt );
      tex->setMinFilter( m_default_min_filter );
      tex->setMagFilter( m_default_mag_filter );

      // set max anisotropic filter
      //tex->setMaxAnisotropy( tex->getMaxSupportedMaxAnisotropy() );

      tex->setImage(image, format, type, num_s, num_t, false);

      if( m_compress_textures ) 
      {
         tex->setCompressionLevel( m_default_compression_level );
      }

      m_num_textures++;
      if( format == sglTexture::eRGB )
         m_opaque_texels += (num_s * num_t);
      else
         m_trans_texels += (num_s * num_t);
   }
   else
      return 0;

   return tex;
}

//----------------------------------------------------------------------------
sglTexture2D *sglLoadFile_wrl::createTexture(const string &filename,
                                          bool repeatS, bool repeatT )
{
   sglTexture::ExternalFormatEnum format;
   sglTexture::TypeEnum type;
   sglTexture::WrapEnum ws, wt;
   sglTexture2D * tex = NULL;
   unsigned int num_s, num_t;
   void * image = NULL;

   map<string, sglTexture2D*>::iterator elt = s_tex_map.find(filename);

   //cout << "Trying to load texture: " << filename << endl; 

   if (elt == s_tex_map.end())
   {
      string foundname;

      // try to find texture
      if (!sglDataLoader::findFile(filename, NULL, foundname))
         if (!sglDataLoader::findFile(filename, &m_alternate_path, foundname))
            return 0;

      image = sglImageLoader::loadImage(foundname, num_s, num_t, format, type);

      if (image && num_s && num_t)
      {
         // ignore "small" textures
         if( num_s <= (unsigned)m_min_texture_dimension ||
             num_t <= (unsigned)m_min_texture_dimension )
            return 0;

         tex = new sglTexture2D;

         tex->setAutoMipmapFlag(true);

         if( repeatS )
            ws = sglTexture::eREPEAT;
         else
            ws = sglTexture::eCLAMP;

         if( repeatT )
            wt = sglTexture::eREPEAT;
         else
            wt = sglTexture::eCLAMP;

         tex->setWrapMode( ws, wt );
         tex->setMinFilter( m_default_min_filter );
         tex->setMagFilter( m_default_mag_filter );

         // set max anisotropic filter
         //tex->setMaxAnisotropy( tex->getMaxSupportedMaxAnisotropy() );

         tex->setImage(image, format, type, num_s, num_t, false);

         if( m_compress_textures ) 
         {
            tex->setCompressionLevel( m_default_compression_level );
         }

         if( m_verbose )
            sglPrintInfo()<<"sglLoadFile_wrl::createTexture: Texture loaded: '"
                       << filename << "' (" << num_s << "x" << num_t << ")"
                       << endl;

         m_num_textures++;
         if( format == sglTexture::eRGB )
            m_opaque_texels += (num_s * num_t);
         else
            m_trans_texels += (num_s * num_t);
      }
      else
         return 0;

      s_tex_map[filename] = tex;

   }
   else
   {
      //sglPrintInfo() << "sglLoadFile_wrl::createTexture: Texture reused: '"
      //               << filename << "'" << endl;
      tex = elt->second;
   }
   return tex;
}

//----------------------------------------------------------------------------
bool
sglLoadFile_wrl::doCallback( sglNode * sNode, void * loaderNode, 
                                                      const string &func, 
                                                      const string &name )
{
   if( m_callback )
   {
      string arg = func + name;

      return m_callback( sNode, loaderNode, arg, m_callbackUserData );
   }

   return true;
}


//----------------------------------------------------------------------------
void
sglLoadFile_wrl::travShapeNode( sglGroup * parent, Node * cvNode )
{
   vector<sglStatelet *> state;

   bool hasTexture = false;

   ShapeNode      * shape      = dynamic_cast<ShapeNode *>( cvNode );

   AppearanceNode * appearance = shape->getAppearanceNodes();
   GeometryNode   * geometry   = shape->getGeometry();

   // no geometry??
   if( !geometry )
   {
      return;
   }

   if( appearance )
   {
      MaterialNode * cvMaterial = appearance->getMaterialNodes();

      if( cvMaterial )
      {
         sglMaterial * material = new sglMaterial();
         float tmpv[4];

         //cout << "Has Material..\n";

         // set material properties

         if( m_use_colormaterial )
         {
            material->setColorMaterial( sglMaterial::eFRONT,
                                        sglMaterial::eAMBIENT_AND_DIFFUSE );
         }
         else
         {
            float ai = cvMaterial->getAmbientIntensity();

            cvMaterial->getDiffuseColor( tmpv );
            material->setDiffuse( tmpv[0], tmpv[1], tmpv[2] );

            // ambient color is defined as ambienIntensity*(RGB)
            material->setAmbient( ai*tmpv[0], ai*tmpv[1], ai*tmpv[2] );
         }

         cvMaterial->getSpecularColor( tmpv );
         material->setSpecular( tmpv[0], tmpv[1], tmpv[2] );

         cvMaterial->getEmissiveColor( tmpv );
         material->setEmission(  tmpv[0], tmpv[1], tmpv[2] );

         material->setShininess( cvMaterial->getShininess() );
         material->setAlpha( 1.0f - cvMaterial->getTransparency() );

         state.push_back( material );

         m_num_materials++;
      }

      ImageTextureNode * cvImage  = appearance->getImageTextureNodes();
      PixelTextureNode * cvPixTex = appearance->getPixelTextureNodes();

      if( cvImage )
      {
         // only use the first texture for now
         sglTexture2D * tex = createTexture( cvImage->getUrl(0),
                                             cvImage->getRepeatS(),
                                             cvImage->getRepeatT() );

         //cout << "Has texture: " << cvImage->getUrl(0) << endl;

         if( tex )
         {
            if (tex->getInternalFormat() == sglTexture::eINT_RGBA)
            {
               if( m_use_alphafunc_transparency )
                  state.push_back(new 
                     sglAlphaTest(sglAlphaTest::eGREATER, m_alpha_threshold ) );
               else
                  state.push_back(new sglTransparency( true ) );
            }

            sglTexEnv  *tev = new sglTexEnv();

            // set all to modulate for now
            tev->setMode(sglTexEnv::eMODULATE);

            state.push_back( tex );
            state.push_back( tev );

            hasTexture = true;
         }
         else
            sglPrintInfo() << "Texture: " << cvImage->getUrl(0) << 
                              " not found.\n";
      }
      else if( cvPixTex )
      {
         //
         // first 3 values are width, height, depth (# components)
         //
         int width  = cvPixTex->getImage(0);
         int height = cvPixTex->getImage(1);
         int depth  = cvPixTex->getImage(2);
         int i;

         if( width <= 0 || height <= 0 || depth <= 0 ||
             cvPixTex->getNImages() != (width*height+3) )
         {
            sglPrintInfo() << 
               "width, height, or depth zero, or not enough " <<
               "data in PixelTexture\n";
         }
         else
         {
            unsigned char * ibuf = new unsigned char[ width * height * depth ];
            unsigned char * ptr  = ibuf;

#if 0
            cout << "width: " << width << " height: " << height
                  << " depth: " << depth << endl;
#endif

            //
            // Image is stored lower right to upper left, so reverse
            // the array when building it.
            //

            for(i=cvPixTex->getNImages()-1; i>2; i--)
            {
               union {
                  unsigned int value;
                  unsigned char b[4];
               } val; 

               // not clear if this is necessary, should be
#ifdef SGL_BIG_ENDIAN
#define _BB 3
#define _BG 2
#define _BR 1
#define _BA 0
#elif defined( SGL_LITTLE_ENDIAN )
#define _BB 0
#define _BG 1
#define _BR 2
#define _BA 3
#else
#error SGL_BIG_ENDIAN or SGL_LITTLE_ENDIAN must be defined!
#endif

               val.value = cvPixTex->getImage( i );

#if 0
               cout << "V: " << hex << val.value << " " 
                                                 << (int)val.b[_BA] << " " 
                                                 << (int)val.b[_BR] << " "
                                                 << (int)val.b[_BG] << " "
                                                 << (int)val.b[_BB] << endl;
#endif

               if( depth == 1 )
               {
                  *ptr++ = val.b[_BB];
               }
               else if( depth == 2 )
               {
                  *ptr++ = val.b[_BG];
                  *ptr++ = val.b[_BB];
               }
               else if( depth == 3 )
               {
                  *ptr++ = val.b[_BR];
                  *ptr++ = val.b[_BG];
                  *ptr++ = val.b[_BB];
               }
               else
               {
                  // the alpha pixel is actually "transparency", so alpha
                  // is 0xff - val.b[_BA]
                  //
                  *ptr++ = 0xff - val.b[_BA];
                  *ptr++ = val.b[_BR];
                  *ptr++ = val.b[_BG];
                  *ptr++ = val.b[_BB];
               }
            }

#undef _BB
#undef _BR
#undef _BG
#undef _BA

            sglTexture2D * tex = createPixelTexture( width, height, depth,
                                                (void *)ibuf,
                                                cvPixTex->getRepeatS(),
                                                cvPixTex->getRepeatT() );

            if( tex )
            {
               if (tex->getInternalFormat() == sglTexture::eINT_RGBA)
               {
                  if( m_use_alphafunc_transparency )
                     state.push_back(new 
                        sglAlphaTest(sglAlphaTest::eGREATER,m_alpha_threshold));
                  else
                     state.push_back(new sglTransparency( true ) );
               }

               sglTexEnv  *tev = new sglTexEnv();

               // set all to modulate for now
               tev->setMode(sglTexEnv::eMODULATE);

               state.push_back( tex );
               state.push_back( tev );

               hasTexture = true;
            }
            else
               sglPrintInfo() << "Unable to create pixel texture.\n";
         }
      }
   }

   sglGeode * geode = 0;

   if( geometry->isIndexedFaceSetNode() )
   {
      sglTriStripper<sglVertStructVNTC> triStripper;
      unsigned int i;
      float vec[4];

      IndexedFaceSetNode * ifs = static_cast<IndexedFaceSetNode *>(geometry);

      sglVec3f *coord = 0;
      sglVec4f *color = 0;
      sglVec3f *normal = 0;
      sglVec2f *texCoord = 0;

      // must be <int> since vrml ends geometries with '-1'
      vector<int> vcoordIndex;
      vector<int> vcolorIndex;
      vector<int> vnormalIndex;
      vector<int> vtexCoordIndex;

      geode = new sglGeode();

      if( shape->hasName() )
         geode->setName( shape->getName() );

      // check for 2 sided
      if( ifs->getSolid() == false )
      {
         state.push_back( new sglCullMode( sglCullMode::eOFF ) );
      }

      // check for Clockwise polys
      if( ifs->getCCW() == false )
      {
         state.push_back( new sglFrontFace( sglFrontFace::eCW ) );
      }

      //
      // first, the coords
      //
      CoordinateNode * cvcoord = ifs->getCoordinateNodes();

      if( !cvcoord )
      {
         sglPrintInfo() << "No Coordinate Nodes in IndexedFaceSet\n";
         return;
      }

      coord = new sglVec3f[ cvcoord->getNPoints() ];

      for( i=0; i< (unsigned)cvcoord->getNPoints(); i++ )
      {
         cvcoord->getPoint( i, vec );
         coord[i].set( vec[0], vec[1], vec[2] );
      }

      // now the indices
      for( i=0; i< (unsigned)ifs->getNCoordIndexes(); i++ )
         vcoordIndex.push_back( ifs->getCoordIndex(i) );

      //
      // Normals
      //
      bool hasNormals = false;
      bool normalPerVertex = ifs->getNormalPerVertex();

      NormalNode * cvnormal  = ifs->getNormalNodes();
      if( cvnormal && cvnormal->getNVectors() )
         hasNormals = true;

      if( hasNormals )
      {
         //cout << "Has Normals..\n";

         if( normalPerVertex )
         {
            if( ifs->getNNormalIndexes() <= 0 )
            {
               // convert to per-vertex using coord array
               vnormalIndex = vcoordIndex;
            }
            else
            {
               // use supplied per-vertex array
               for( i=0; i< (unsigned)ifs->getNNormalIndexes(); i++ )
                  vnormalIndex.push_back( ifs->getNormalIndex(i) );
            }
         }
         else
         {
            if( ifs->getNNormalIndexes() <= 0 )
            {
               // convert to per-prim index
               for( i=0; i < (unsigned)cvnormal->getNVectors(); i++ )
                  vnormalIndex.push_back( i );
            }
            else
            {
               // use supplied per-prim array
               for( i=0; i< (unsigned)ifs->getNNormalIndexes(); i++ )
                  vnormalIndex.push_back( ifs->getNormalIndex(i) );
            }
         }

         normal = new sglVec3f[ cvnormal->getNVectors() ];

         for( i=0; i < (unsigned)cvnormal->getNVectors(); i++ )
         {
            cvnormal->getVector( i, vec );
            normal[i].set( vec[0], vec[1], vec[2] );
         }
      }

      bool hasColors = false;
      bool colorPerVertex = ifs->getColorPerVertex();

      ColorNode  * cvcolor   = ifs->getColorNodes();
      if( cvcolor )
         hasColors = true;

      if( hasColors )
      {
         //cout << "Has Colors..\n";

         if( colorPerVertex )
         {
            if( ifs->getNColorIndexes() <= 0 )
            {
               // convert to per-vertex using coord array
               vcolorIndex = vcoordIndex;
            }
            else
            {
               // use supplied per-vertex array
               for( i=0; i< (unsigned)ifs->getNColorIndexes(); i++ )
                  vcolorIndex.push_back( ifs->getColorIndex(i) );
            }
         }
         else
         {
            if( ifs->getNColorIndexes() <= 0 )
            {
               // convert to per-prim index
               for( i=0; i < (unsigned)cvcolor->getNColors(); i++ )
                  vcolorIndex.push_back( i );
            }
            else
            {
               // use supplied per-prim array
               for( i=0; i< (unsigned)ifs->getNColorIndexes(); i++ )
                  vcolorIndex.push_back( ifs->getColorIndex(i) );
            }
         }

         color = new sglVec4f[ cvcolor->getNColors() ];

         for( i=0; i < (unsigned)cvcolor->getNColors(); i++ )
         {
            cvcolor->getColor( i, vec );
            color[i].set( vec[0], vec[1], vec[2], 1.0f );
         }
      }

      if( hasTexture )
      {
         TextureCoordinateNode * cvtex = ifs->getTextureCoordinateNodes();

         if( !cvtex )
         {
            hasTexture = false;
         }
         else
         {
            if( ifs->getNTexCoordIndexes() <= 0 )
            {
               vtexCoordIndex = vcoordIndex;
            }
            else
            {
               // fill tex index vec
               for( i=0; i< (unsigned)ifs->getNTexCoordIndexes(); i++ )
                  vtexCoordIndex.push_back( ifs->getTexCoordIndex(i) );
            }

            // fill in the texcoord vec
            texCoord = new sglVec2f[ cvtex->getNPoints() ];

            for( i=0; i< (unsigned)cvtex->getNPoints(); i++ )
            {
               cvtex->getPoint( i, vec );
               texCoord[i].set( vec[0], vec[1] );
            }
         }
      }


#if 0
   if( m_sort_unique )
   {
      if( hasColors )
      {
         Unique3 * unique3 = new Unique3( ifs->color, ifs->colorIndex );
         delete unique3;
      }

      if( hasNormals )
      {
         Unique3 * unique3 = new Unique3( ifs->normal, ifs->normalIndex );
         delete unique3;
      }

      if( tex && !ifs->texCoordIndex.empty() )
      {
         Unique2 * unique2 = new Unique2( ifs->texCoord, ifs->texCoordIndex );
         delete unique2;
      }
   }
#endif

      vector<sglVertStructVNTC> vertList;
      unsigned int primCounter = 0;

      // add all the verts/texcoords/norms
      for(i = 0; i < (unsigned)ifs->getNCoordIndexes(); i++ )
      {
         if( vcoordIndex[i] != -1 )
         {
            sglVertStructVNTC vert;

            vert.vi = vcoordIndex[i];

            if( hasTexture )
               vert.ti = vtexCoordIndex[i];
            //else
             //  vert.ti = UINT_MAX;

            //
            // NOTE: We convert per-prim colors and normals to per-vertex
            //
            if( hasNormals )
            {
               if( normalPerVertex )
                  vert.ni = vnormalIndex[i];
               else
                  vert.ni = vnormalIndex[ primCounter ];
            }
            //else
             //  vert.ni = UINT_MAX;

            if( hasColors )
            {
               if( colorPerVertex )
                  vert.ci = vcolorIndex[i];
               else
                  vert.ci = vcolorIndex[ primCounter ];
            }
            //else
             //  vert.ci = UINT_MAX;

            vertList.push_back( vert );
         }
         else
         {
            //unsigned int i;
            //cout << "addPolyFace( ";
            //for(i=0;i<vertList.size();i++)
             //  cout << vertList[i].vi << " ";
            //cout << ")" << endl;

            //checkCCW( &vertList, &ifs->coord );

            if( vertList.size() == 3 )
               m_num_strip_tris++;
            else if( vertList.size() == 4 )
               m_num_strip_quads++;
            else
               m_num_strip_polys++;

            triStripper.addPolyFace( vertList );
            vertList.clear();

            primCounter++;
         }
      }

      // stripify
      triStripper.triStripify( false );

      unsigned int numStrips = triStripper.getNumStrips();

      // do strips first
      if( numStrips )
      {
         unsigned int * primLengths = new unsigned int[ numStrips ];
         unsigned int * indexList = 0;
         unsigned int * uvIndexList = 0;
         unsigned int * normalIndexList = 0;
         unsigned int * colorIndexList = 0;
         unsigned int indexListSize = 0;
         unsigned int numIndices = 0, i, j;

         sglIndexedTriangleStripSet * strip = new sglIndexedTriangleStripSet();

         strip->setCoordList( coord );
         strip->setNumPrims( numStrips ); 

         if( hasNormals )
            strip->setNormalList( sglGeoSet::ePER_VERTEX, normal );

         if( hasColors )
            strip->setColorList( sglGeoSet::ePER_VERTEX, color );
#if 0
         else
         {
            // set unlit color from material
            MaterialNode * cvMaterial = appearance->getMaterialNodes();

            if( cvMaterial )
            {
               float tmpv[4];

               cvMaterial->getDiffuseColor( tmpv );
               strip->setColorList( sglGeoSet::eOVERALL, 
                          new sglVec4f( tmpv[0], tmpv[1], tmpv[2], 1.0f ) );
            }
         }
#endif

         // turn on display lists
         strip->enableDisplayList( true );

         // first, figure out how many indices there are total
         for(i=0; i<numStrips; i++) 
         {
            vector<sglVertStructVNTC> * v = triStripper.getStrip( i );

            primLengths[i] = v->size();
            numIndices += primLengths[i];
         }

         //
         // alloc indices
         //
         indexList = new unsigned int[ numIndices ];

         if( hasTexture )
            uvIndexList = new unsigned int[ numIndices ];

         if( hasNormals )
            normalIndexList = new unsigned int[ numIndices ];

         if( hasColors )
            colorIndexList = new unsigned int[ numIndices ];

         for(i=0; i<numStrips; i++)
         {
            vector<sglVertStructVNTC> * v = triStripper.getStrip( i );

            for( j=0; j<v->size(); j++ ) {
               // error check??
               indexList[indexListSize] = (*v)[j].vi;

               if( hasTexture )
                  uvIndexList[indexListSize] = (*v)[j].ti;

               if( hasNormals )
                  normalIndexList[indexListSize] = (*v)[j].ni;

               if( hasColors )
                  colorIndexList[indexListSize] = (*v)[j].ci;

               indexListSize++;
            }

            // stats
            m_num_strips++;

            if( v->size() < (unsigned)m_min_strip_len )
               m_min_strip_len = v->size();
            if( v->size() > (unsigned)m_max_strip_len )
               m_max_strip_len = v->size();

            m_avg_strip_len += v->size();
         }

         strip->setPrimLengths( primLengths );
         strip->setCoordIndexList( indexList );

         if( hasNormals )
            strip->setNormalIndexList( normalIndexList );

         if( hasColors )
            strip->setColorIndexList( colorIndexList );

         if( hasTexture )
         {
            strip->setTexCoordIndexList( uvIndexList );
            strip->setState( 0, state, texCoord );
         }
         else
            strip->setState(0, state, 0);

         // add geometry
         geode->addGeometry( strip );
      }

      // now extract isolated triangles
      unsigned int numTris = triStripper.getNumTriangles();

      if( numTris )
      {
         sglIndexedTriangleSet * tset = new sglIndexedTriangleSet(); 
         unsigned int * ilist = new unsigned int[ numTris * 3 ];
         unsigned int * tilist = new unsigned int[ numTris * 3 ];
         unsigned int * nilist = new unsigned int[ numTris * 3 ];
         unsigned int * cilist = new unsigned int[ numTris * 3 ];

         m_num_indeptris += numTris;

         tset->setCoordList( coord );

         tset->setNumPrims( numTris ); 

         triStripper.copyTriangleIndexLists( ilist, 
                                     nilist, 
                                     tilist, 
                                     cilist );

         tset->setCoordIndexList( ilist );

         if( hasColors )
         {
            tset->setColorList( sglGeoSet::ePER_VERTEX, color );
            tset->setColorIndexList( cilist );
         }
         else
         {
            delete [] cilist;
#if 0
            // set unlit color from material
            MaterialNode * cvMaterial = appearance->getMaterialNodes();

            if( cvMaterial )
            {
               float tmpv[4];

               cvMaterial->getDiffuseColor( tmpv );
               tset->setColorList( sglGeoSet::eOVERALL, 
                          new sglVec4f( tmpv[0], tmpv[1], tmpv[2], 1.0f ) );
            }
#endif
         }

         if( hasNormals )
         {
            tset->setNormalList( sglGeoSet::ePER_VERTEX, normal );
            tset->setNormalIndexList( nilist );
         }
         else
            delete [] nilist;

         // turn on display lists
         tset->enableDisplayList( true );

         // set state
         if( hasTexture )
         {
            tset->setTexCoordIndexList( tilist );
            tset->setState(0, state, texCoord);
         }
         else
         {
            delete [] tilist;
            tset->setState(0, state, 0);
         }

         geode->addGeometry( tset );
      }

   }

   else if( geometry->isElevationGridNode() )
   {
      sglPrintInfo() << "ElevationGridNodes not yet supported.\n";
   }

   else if( geometry->isExtrusionNode() )
   {
      sglPrintInfo() << "ExtrusionNodes not yet supported.\n";
   }

   else if( geometry->isIndexedLineSetNode() )
   {
      sglPrintInfo() << "IndexedLineSetNodes not yet supported.\n";
   }

   else if( geometry->isPointSetNode() )
   {
      sglPrintInfo() << "PointSetNodes not yet supported.\n";
   }

   else if( geometry->isTextNode() )
   {
      TextNode * cvtext = static_cast<TextNode *>(geometry);
      geode = new sglGeode();
#if 0
      sglTextureFont * font = new sglTextureFont( 
                           *sglBitmapFont::getDefaultFont(), true, '?' );
      sglTexturedTextSet * text = new sglTexturedTextSet( font );
      text->setCharHeight( font->getHeight() );
#else
      sglTextSet * text = new sglTextSet();
#endif

      // set unlit color from material
      MaterialNode * cvMaterial = appearance->getMaterialNodes();

      if( cvMaterial )
      {
         float tmpv[4];

         cvMaterial->getDiffuseColor( tmpv );
         text->setColor( tmpv[0], tmpv[1], tmpv[2], 0.3f );
      }

      // only draw string "0" for now

      text->setString( cvtext->getString(0) );
      text->enableDisplayList( true );

      text->setState(0, state, 0);

      geode->addGeometry( text );
   }

   else if( geometry->isBoxNode() )
   {
      BoxNode * cvbox = static_cast<BoxNode *>(geometry);
      float vec[3];
      geode = new sglGeode();

      cvbox->getSize( vec );

      sglCube * cube = new sglCube( vec[0], vec[1], vec[2] );

      // set unlit color from material
      MaterialNode * cvMaterial = appearance->getMaterialNodes();

      if( cvMaterial )
      {
         float tmpv[4];

         cvMaterial->getDiffuseColor( tmpv );
         cube->setColorList( sglGeoSet::eOVERALL, 
                             new sglVec4f( tmpv[0], tmpv[1], tmpv[2], 1.0f ) );
      }

      cube->setState(0, state, 0);

      geode->addGeometry( cube );
   }

   else if( geometry->isConeNode() )
   {
      ConeNode * cvcone = static_cast<ConeNode *>(geometry);
      geode = new sglGeode();

      sglCone * cone = new sglCone(
               (unsigned int)( m_primitive_quality * 15.0f ),
                                       cvcone->getHeight(),
                                       cvcone->getBottomRadius() );

      // set unlit color from material
      MaterialNode * cvMaterial = appearance->getMaterialNodes();

      if( cvMaterial )
      {
         float tmpv[4];

         cvMaterial->getDiffuseColor( tmpv );
         cone->setColorList( sglGeoSet::eOVERALL, 
                             new sglVec4f( tmpv[0], tmpv[1], tmpv[2], 1.0f ) );
      }

      cone->setState(0, state, 0);

      geode->addGeometry( cone );
   }

   else if( geometry->isCylinderNode() )
   {
      CylinderNode * cvcyl = static_cast<CylinderNode *>(geometry);
      geode = new sglGeode();

      sglCylinder * cyl = new sglCylinder( 
                              (unsigned int)( m_primitive_quality * 15.0f ),
                              (unsigned int)( m_primitive_quality * 5.0f ),
                                                cvcyl->getHeight(),
                                                cvcyl->getRadius(),
                                                cvcyl->getRadius() ); 

      // set unlit color from material
      MaterialNode * cvMaterial = appearance->getMaterialNodes();

      if( cvMaterial )
      {
         float tmpv[4];

         cvMaterial->getDiffuseColor( tmpv );
         cyl->setColorList( sglGeoSet::eOVERALL, 
                             new sglVec4f( tmpv[0], tmpv[1], tmpv[2], 1.0f ) );
      }

      cyl->setState(0, state, 0);

      geode->addGeometry( cyl );
   }

   else if( geometry->isSphereNode() )
   {
      SphereNode * cvsphere = static_cast<SphereNode *>(geometry);
      geode = new sglGeode();

      sglSphere * sphere = new sglSphere( 
                              (unsigned int)(m_primitive_quality * 15.0f), 
                              (unsigned int)(m_primitive_quality * 10.0f), 
                                          cvsphere->getRadius() ); 

      // set unlit color from material
      MaterialNode * cvMaterial = appearance->getMaterialNodes();

      if( cvMaterial )
      {
         float tmpv[4];

         cvMaterial->getDiffuseColor( tmpv );
         sphere->setColorList( sglGeoSet::eOVERALL, 
                             new sglVec4f( tmpv[0], tmpv[1], tmpv[2], 1.0f ) );
      }

      sphere->setState(0, state, 0);

      geode->addGeometry( sphere );
   }

   // 
   // Execute the callback for onCreate
   //

   if( geode )
   {
      if( !doCallback( geode, (void *)cvNode, "onCreate:", "sglGeode" ) )
      {
         delete geode;
         return;
      }

      parent->addChild( geode );
   }
}

void sglLoadFile_wrl::travNode( sglGroup * parent, Node * node )
{
   // 
   // Execute the callback for onParse
   //
   if( !doCallback( (sglNode *)0, (void *)node, "onParse:", node->getType() ))
   {
      return;
   }

   if( node->isGroupNode() )
   {
      sglGroup * group = new sglGroup();

      for(Node *cnode = node->getChildNodes(); cnode; cnode = cnode->next() )
      {
         travNode( group, cnode );
      }

      // set name, if available
      if( node->hasName() )
      {
         group->setName( node->getName() );
      }

      // 
      // Execute the callback for onCreate
      //
      if( !doCallback( group, (void *)node, "onCreate:", "sglGroup" ) )
      {
         delete group;
         return;
      }

      parent->addChild( group );

      return;
   }


   else if( node->isTransformNode() )
   {
      sglTransformf * trans = new sglTransformf();
      TransformNode * cvtrans = static_cast<TransformNode *>( node );

      for(Node *cnode = node->getChildNodes(); cnode; cnode = cnode->next() )
      {
         travNode( trans, cnode );
      }

#if 0
      // skip this for now
      // get cvMatrix, convert to sglMatrix
      SFMatrix mat;
      float v[4][4];

      cvtrans->getSFMatrix( &mat );
      mat.getValue( v );

      sglMat4f sglmat( v[0][0], v[1][0], v[2][0], v[3][0],
                       v[0][1], v[1][1], v[2][1], v[3][1],
                       v[0][2], v[1][2], v[2][2], v[3][2],
                       v[0][3], v[1][3], v[2][3], v[3][3] );

      trans->setMatrix( sglmat );
#endif

      float vec[4];

      cvtrans->getTranslation( vec );
      trans->setTranslation( vec[0], vec[1], vec[2] );

      cvtrans->getRotation( vec );
      trans->setRotation( vec[3], vec[0], vec[1], vec[2] );

      // set name, if available
      if( cvtrans->hasName() )
      {
         trans->setName( cvtrans->getName() );
      }

      // 
      // Execute the callback for onCreate
      //
      if( !doCallback( trans, (void *)node, "onCreate:", "sglTransform" ) )
      {
         delete trans;
         return;
      }

      parent->addChild( trans );

      return;
   }


   else if( node->isLodNode() )
   {
      sglLOD * lod = new sglLOD();
      LodNode * cvlod = static_cast<LodNode *>( node );
      int i;

      for(Node *cnode = node->getChildNodes(); cnode; cnode = cnode->next() )
      {
         travNode( lod, cnode );
      }

      // set parms
      //
      // VRML specifies that the lowest LOD is displayed if distance is <
      // first range
      lod->setRange( 0, 0.0f );

      for( i=0; i<cvlod->getNRanges(); i++ )
         lod->setRange( i+1, cvlod->getRange( i ) );

      // assume that last child is forever displayed if > last range  
      lod->setRange( i+1, 9999999.9f );

      // set center (doesn't always work)
      //float vec[3];
      //cvlod->getCenter( vec );
      //lod->setCenter( sglVec3f( vec[0], vec[1], vec[2] ) );

      // set name, if available
      if( cvlod->hasName() )
      {
         lod->setName( cvlod->getName() );
      }

      // 
      // Execute the callback for onCreate
      //
      if( !doCallback( lod, (void *)node, "onCreate:", "sglLOD" ) )
      {
         delete lod;
         return;
      }

      parent->addChild( lod );

      return;
   }


   else if( node->isSwitchNode() )
   {
      sglSwitch * sw = new sglSwitch();
      SwitchNode * cvswitch = static_cast<SwitchNode *>( node );

      for(Node *cnode = node->getChildNodes(); cnode; cnode = cnode->next() )
      {
         travNode( sw, cnode );
      }

      // set parms
      if( cvswitch->getWhichChoice() == -1 )
         sw->setVal( sglSwitch::eOFF );
      else
         sw->setVal( cvswitch->getWhichChoice() );

      // set name, if available
      if( cvswitch->hasName() )
      {
         sw->setName( cvswitch->getName() );
      }

      // 
      // Execute the callback for onCreate
      //
      if( !doCallback( sw, (void *)node, "onCreate:", "sglSwitch" ) )
      {
         delete sw;
         return;
      }

      parent->addChild( sw );

      return;
   }

   else if( node->isBillboardNode() )
   {
      sglBillboard * bill = new sglBillboard();
      BillboardNode * cvbill = static_cast<BillboardNode *>( node );

      for(Node *cnode = node->getChildNodes(); cnode; cnode = cnode->next() )
      {
         travNode( bill, cnode );
      }

      float vec[3];

      // set parms

      cvbill->getAxisOfRotation( vec );
      sglVec3f axis( vec[0], vec[1], vec[2] );

      bill->setAxis( axis );
      bill->setMode( sglBillboard::eAXIAL_ROT );

      // set name, if available
      if( cvbill->hasName() )
      {
         bill->setName( cvbill->getName() );
      }

      // 
      // Execute the callback for onCreate
      //
      if( !doCallback( bill, (void *)node, "onCreate:", "sglBillboard" ) )
      {
         delete bill;
         return;
      }

      parent->addChild( bill );

      return;
   }

   else if( node->isShapeNode() )
   {
      travShapeNode( parent, node );

      return;
   }

   // unknown node
   for(Node *cnode = node->getChildNodes(); cnode; cnode = cnode->next() )
   {
      //cout << "Unhandled Node..\n";
      travNode( parent, cnode );
   }
}

void sglLoadFile_wrl::buildScene( sglGroup * parent, Node * node )
{
   // traverse all "topLevel" nodes
   for(; node; node = node->next() )
   {
      travNode( parent, node );
   }
}

//----------------------------------------------------------------------------
sglNode *sglLoadFile_wrl::loadFile(const string &filename,
                                   unsigned int /* mode */ )
{
   sglGroup * topLevel = new sglGroup();
   bool result = false;

   m_sceneGraph = new SceneGraph();

   // generate normals and texture coords for shapes 
   //m_sceneGraph->setOption( SCENEGRAPH_NORMAL_GENERATION |
   //                        SCENEGRAPH_TEXTURE_GENERATION );

   // store the path for possible future use
   m_alternate_path = sglDataLoader::extractDirectoryFromPath(filename);

   // read the file 
   m_sceneGraph->load( const_cast<char *>(filename.c_str()) ); 
   result = m_sceneGraph->isOK();

   // try using sglDataLoader's search capabilities
   if ( !result )
   {
      string foundname;
      if (sglDataLoader::findFile(filename, NULL, foundname))
      {
         m_sceneGraph->load( const_cast<char *>(foundname.c_str()) ); 
         result = m_sceneGraph->isOK();
         if ( result )
         {
            m_alternate_path =
               sglDataLoader::extractDirectoryFromPath(foundname);
         }
      }
   }

   if ( result )
   {
      // build the scene
      buildScene( topLevel, m_sceneGraph->getNodes() );

      if( m_display_stats )
      {
         float tex_mb = m_trans_texels * 4 + m_opaque_texels * 3;

         tex_mb /= (1024.0f*1024.0f);

         sglPrintInfo() << "-- Statistics for: " << filename << " ---" << endl;
         sglPrintInfo() << "   Num S.Tris     : " << m_num_strip_tris << endl;
         sglPrintInfo() << "   Num S.Quads    : " << m_num_strip_quads << endl;
         sglPrintInfo() << "   Num S.Polygons : " << m_num_strip_polys << endl;
         sglPrintInfo() << "   ---------------- " << endl;
         sglPrintInfo() << "   Num Indep Tris : " << m_num_indeptris << endl;
         sglPrintInfo() << "   Num Strips     : " << m_num_strips << endl;
         if( m_num_strips )
            sglPrintInfo() << "    Length Stats  : " 
                                                << "Min: " << m_min_strip_len 
                                                << " Max: " << m_max_strip_len
                              << " Avg: " << (m_avg_strip_len/m_num_strips)
                                                << endl;
         sglPrintInfo() << "   Num Materials  : " << m_num_materials << endl;
         sglPrintInfo() << "   Num Textures   : " << m_num_textures << endl;
         sglPrintInfo() << "   Texture Size MB: " 
                        << tex_mb << " (32bpp) "
                        << tex_mb/2.0f << " (16bpp) " << endl;
         sglPrintInfo() << "    Mipped        : " 
                        << tex_mb * 1.333f << " (32bpp) "
                        << (tex_mb/2.0f) * 1.333f << " (16bpp) " << endl;

         tex_mb = (m_opaque_texels / 16) * 8;
         tex_mb += ( (m_trans_texels / 16) * 16 );
         tex_mb *= 1.3333f; // mips

         tex_mb /= (1024.0f*1024.0f);
         sglPrintInfo() << "   S3TC Cmp+Mipped: " << tex_mb << endl; 
      }

   }
   else
   {
      sglPrintInfo() << "sglLoadFile_wrl: Error loading: " << filename << endl;
      sglPrintInfo() << "\tLine " << m_sceneGraph->getErrorLineNumber() <<
                        ", at or about \"" << 
                        m_sceneGraph->getErrorToken() << "\""<<
                        endl <<
                        "\t>>" << m_sceneGraph->getErrorLineString() <<
                        endl;

      delete topLevel;
      topLevel = NULL;
   }

   int numChildren = 0;

   if ( topLevel && (numChildren = topLevel->getNumChildren()) == 0)
   {
      sglPrintInfo() << "sglLoadFile_wrl: file contained no scene\n";
      delete topLevel;
      topLevel = NULL;
   }

   return topLevel;
}
