/* vi:set ts=3 sw=3 expandtab: */
/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 * SGL's WRL loader Copyright (C) 2001  Michael M. Morrison  All Rights
 *   Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglLoadFile_wrl.hpp
 *  Created: June 2001
 *  Author : Michael M. Morrison
 *  Summary: VRML 2.0 file loader
 *****************************************************************************/

#ifndef __SGL_LOAD_FILE_WRL_HPP
#define __SGL_LOAD_FILE_WRL_HPP

#include <sglwrl.h>
#include <sglModelLoader.hpp>
#include <sglUserData.hpp>
#include <sglTexture.hpp>
#include <sglTriStripper.hpp>

#include <map>
#include <stack>

class sglTexture2D;
class sglNode;
class sglGroup;
class sglGeode;

#include <SceneGraph.h>

//============================================================================

class SGLWRL_DLL_API sglLoadFile_wrl
{
public:
   /// default constructor
   sglLoadFile_wrl();
   /// virtual destructor
   virtual ~sglLoadFile_wrl();

   /// load a file, build a scene graph, and get the sglGeode* pointing to it
   sglNode *loadFile(const string &filename, unsigned int mode);

private:
   /// Copy constructor forbidden
   sglLoadFile_wrl(const sglLoadFile_wrl &);
   /// Assignment operator forbidden
   sglLoadFile_wrl &operator=(const sglLoadFile_wrl &);

   sglTexture2D *createPixelTexture(   unsigned int num_s,
                                       unsigned int num_t,
                                       unsigned int depth,
                                       void * image,
                                       bool rS = true,
                                       bool rT = true );

   sglTexture2D *createTexture(const string &filename, bool rU = true,
                                                       bool rV = true );

   void travShapeNode( sglGroup *, Node * );
   void travNode( sglGroup *, Node * );
   void buildScene( sglGroup *, Node * );
   bool doCallback( sglNode * sNode, void * loaderNode, 
                                                const string &func, 
                                                const string &name );

private:
   static map<string, sglTexture2D*> s_tex_map;

   stack<sglGroup *>m_node_stack;

   string m_alternate_path;

   SceneGraph * m_sceneGraph;    // the VRML Scene graph

   bool   m_verbose;             // supply verbose output
   bool   m_display_stats;       // display statistics output
   bool   m_compute_all_normals; // compute all new normals from face
   bool   m_compute_normals_if_absent; // compute normals if they don't exist
   bool   m_use_colormaterial;   // use colormaterial for amb and diff.
   bool   m_enable_lighting;     // read and store materials and normals
   bool   m_enable_shading;      // read and store per-vertex colors
   bool   m_enable_line_aa;      // enable line anti-alias

   // default minification and magnification filters
   sglTexture::FilterEnum m_default_min_filter;
   sglTexture::FilterEnum m_default_mag_filter;

   bool   m_compress_textures;   // compress textures during load
   sglTexture::CompressionLevelEnum m_default_compression_level;

   int    m_min_texture_dimension; // textures with a dimension smaller:ignored

   bool   m_retain_comments;     // whether to retain comments in userdata

   bool   m_free_texture_images; // free texture images after bind

   bool   m_use_alphafunc_transparency; // use glAlphaFunc vs glBlendFunc
   float  m_alpha_threshold;     // alpha transparency threshold for a-test 

   bool   m_sort_unique;         // sort for unique colors and normals

   float  m_primitive_quality;   // quality/resolution of box,sphere,etc 0->1.0

   sglModelLoader::callbackFunction m_callback;   // the callback, if avail
   void  *m_callbackUserData;

   // statistics
   int    m_num_points;          // number of points
   int    m_num_lines;           // number of lines
   int    m_num_tris;            // number of triangles in db
   int    m_num_quads;           // number of quads in db
   int    m_num_polys;           // number of misc polygons in db
   int    m_num_verts;           // total verts in db
   int    m_num_strip_quads;     // number of stripped quads in db
   int    m_num_strip_polys;     // number of misc stripped polygons in db
   int    m_num_strip_tris;      // total tripped tris 
   int    m_num_strips;          // total number of strips created
   int    m_num_indeptris;       // total number of independent triangles
   int    m_num_materials;       // total materials
   int    m_num_textures;        // total number of textures
   int    m_opaque_texels;       // # opaque texels
   int    m_trans_texels;        // # translucent texels (+alpha)

   int    m_min_strip_len;       // strip stats
   int    m_max_strip_len;
   int    m_avg_strip_len;
};

#endif /* __SGL_LOAD_FILE_WRL_HPP */
