/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglfly.cpp
 *  Created: 20 March 1997
 *  Summary: GLUT-based sgl testbed
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglVector.hpp>
#include <sglMatrix.hpp>

#include <sglGroup.hpp>
#include <sglScene.hpp>
#include <sglTransform.hpp>
#include <sglSwitch.hpp>
#include <sglLOD.hpp>
#include <sglLODState.hpp>
#include <sglLODStateNode.hpp>
#include <sglBillboard.hpp>
#include <sglDiscriminator.hpp>
#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglIndexedTriangleStripSet.hpp>
#include <sglLineStripSet.hpp>
#include <sglLineSet.hpp>
#include <sglDefaultStateNode.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*>  override_statelets;

sglAntiAliasLine aa_line;
sglAntiAliasPoint aa_point;
sglAntiAliasPolygon aa_poly;

sglBillboard  *billboard_node;
sglTransformf *dcs_node;
sglSwitch     *switch_node;

sglLOD      *lod;
sglLODState *lod_state;

sglLODStateNode *lod_state_node;

unsigned int disc_mask = 0x0;

unsigned int win_x = 640, win_y = 480;
bool paused_flag = true;
float angle = 0.0;

//----------------------------------------------------------------------------
void displayFunc(void)
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;
   double fov_y = M_PI/(3.0*aspect);

   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 's':
         sglLOD::setScaleFactor(sglLOD::getScaleFactor()/1.2f);
         break;
      case 'S':
         sglLOD::setScaleFactor(sglLOD::getScaleFactor()*1.2f);
         break;

      case 'l':
         lod_state->setRangeScale(lod_state->getRangeScale()/1.2f);
         break;
      case 'L':
         lod_state->setRangeScale(lod_state->getRangeScale()*1.2f);
         break;

      case 'k':
         lod_state_node->setRangeScale(lod_state_node->getRangeScale()/1.2f);
         break;
      case 'K':
         lod_state_node->setRangeScale(lod_state_node->getRangeScale()*1.2f);
         break;

      case 'p':
         paused_flag = !paused_flag;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "a - toggle point anti-aliasing" << endl;
         cerr << "A - toggle line anti-aliasing" << endl;
         cerr << "z - toggle polygon anti-aliasing" << endl;

         cerr << "s/S - dec/inc static LOD scale factor" << endl;
         cerr << "k/K - dec/inc LOD state range scale" << endl;
         cerr << "l/L - dec/inc LOD state node range scale" << endl;

         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);

   displayFunc();

   if (!paused_flag)
   {
      angle += (float)M_PI/90.0f;
      if (angle > M_PI) angle -= (float)M_PI*2.0f;
      dcs_node->setRotation(angle, -angle, 2.0*angle);
   }
}

//----------------------------------------------------------------------------
inline void compute_face_normal(sglVec3f& v0, sglVec3f& v1, sglVec3f& v2,
                                sglVec3f& normal)
{
    sglVec3f a, b;
    a.sub(v1, v0);
    b.sub(v2, v0);

    normal.cross(a, b);
    normal.normalize();
}

//----------------------------------------------------------------------------
unsigned int vector_size = 0;

sglGeode *makeTile()
{
   int ix, iy;
   int xsize = 16;
   int ysize = 16;

   vector_size = (xsize+1)*(ysize+1);

   sglVec3f *vertex = new sglVec3f[vector_size];
   int index = 0;

   for (ix=0; ix<=xsize; ix++)
   {
      double lon = ix*M_PI/(3.0*xsize);
      for (iy=0; iy<=ysize; iy++)
      {
         double lat = iy*M_PI/(3.0*ysize);
         vertex[index].set((float)20.0*cos(lat)*cos(lon),
                           (float)20.0*cos(lat)*sin(lon),
                           (float)20.0*sin(lat));
         index++;
      }
   }

   // generate some reasonable normals
   sglVec3f *normal = new sglVec3f[vector_size];
   index = 0;

   for (ix=0; ix<=xsize; ix++)
   {
      for (iy=0; iy<=ysize; iy++)
      {
         if ((ix < xsize) && (iy < ysize))
         {
            compute_face_normal(vertex[index],
                                vertex[index + ysize + 1],
                                vertex[index + 1],
                                normal[index]);
         }
         else if ((ix == xsize) && (iy != ysize))
         {
            //cerr << "1: " << index << ' ' << ix << ',' << iy << ' '
            //     << index+1 << ',' << index-ysize << endl;
            compute_face_normal(vertex[index],
                                vertex[index + 1],
                                vertex[index - ysize -1],
                                normal[index]);
         }
         else if ((ix != xsize) && (iy == ysize))
         {
            //cerr << "2: " << index << ' ' << ix << ',' << iy << ' '
            //     << index-1 << ',' << index+ysize << endl;
            compute_face_normal(vertex[index],
                                vertex[index - 1],
                                vertex[index + ysize + 1],
                                normal[index]);
         }
         else
         {
            //cerr << "3: " << index << ' ' << ix << ',' << iy << ' '
            //     << index-ysize << ',' << index-1 << endl;
            compute_face_normal(vertex[index],
                                vertex[index - ysize -1],
                                vertex[index - 1],
                                normal[index]);
            //cerr << normal[index][0] << ' '
            //     << normal[index][1] << ' '
            //     << normal[index][2] << endl;

         }

         index++;
      }
   }

   //cerr << index << " == " << vector_size << endl;

   sglVec3f *new_vertex = new sglVec3f[vector_size*2];
   unsigned int *new_length = new unsigned int[vector_size];
   unsigned int new_prim_size = 0;
   index = 0;
   for (ix=0; ix<=xsize; ix++)
   {
      for (iy=0; iy<=ysize; iy++)
      {
         // compute vertices to render normals
         //cerr << index << ": " << 2*index << ", " << 2*index+1 << endl;
         new_length[index] = 2;
         new_prim_size += 2;
         new_vertex[2*index][0] = vertex[index][0];
         new_vertex[2*index][1] = vertex[index][1];
         new_vertex[2*index][2] = vertex[index][2];

         new_vertex[2*index+1][0] = vertex[index][0] + normal[index][0];
         new_vertex[2*index+1][1] = vertex[index][1] + normal[index][1];
         new_vertex[2*index+1][2] = vertex[index][2] + normal[index][2];
         //-----------------------------------

         index++;
      }
   }

   //cerr << new_prim_size << " == " << 2*vector_size << endl;

   // create the GeoSet
   int prim_size = 0;
   unsigned int *length = new unsigned int[xsize];
   for (ix=0; ix<xsize; ix++)
   {
      length[ix] = 2*(ysize+1);
      prim_size += length[ix];
   }

   index = 0;
   unsigned int *vindex = new unsigned int[prim_size];
   for (ix=0; ix<xsize; ix++)
   {
      for (iy=0; iy<=ysize; iy++)
      {
         vindex[index++] = ix*(ysize + 1) + iy;
         vindex[index++] = (ix + 1)*(ysize + 1) + iy;
      }
   }

   sglIndexedTriangleStripSet *gset = new sglIndexedTriangleStripSet();

   gset->setCoordList(vertex);
   gset->setNormalList(sglGeoSet::ePER_VERTEX, normal);
   gset->setNumPrims(xsize);
   gset->setPrimLengths(length);
   gset->setCoordIndexList(vindex);
   gset->setNormalIndexList(vindex);

   //cerr << "Tile set: " << gset << endl;

   sglVec4f *color2 = new sglVec4f[1];
   color2[0][0] = color2[0][3] = color2[0][1] = color2[0][2] = 1.0;
   unsigned int *cindex = new unsigned int[1];
   cindex[0] = 0;
   gset->setColorIndexList(cindex);
   gset->setColorList(sglGeoSet::eOVERALL, color2);

   // gset to render normals
   sglVec4f *color = new sglVec4f[1];
   color[0][0] = color[0][3] = 1.0; color[0][1] = color[0][2] = 0.0;

   sglLineStripSet *ngset = new sglLineStripSet();
   ngset->setCoordList(new_vertex);
   ngset->setColorList(sglGeoSet::eOVERALL, color);
   ngset->setNumPrims(vector_size);
   ngset->setPrimLengths(new_length);

   //cerr << "LineStrip Set: " << gset << endl;

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);
   geode->addGeometry(ngset);

   return geode;
}


//----------------------------------------------------------------------------
sglGeoSet *makeLineStripSet(sglGeoSet *source_gset)
{
   // create a display list to render normals
   //unsigned int num_verts = 289;

   sglVec3f *vertex = (sglVec3f*)source_gset->getCoordList()->getData();
   sglVec3f *normal = (sglVec3f*)source_gset->getNormalList()->getData();

   sglVec4f *color = new sglVec4f[1];
   color[0][0] = color[0][3] = 1.0; color[0][1] = color[0][2] = 0.0;
   sglVec3f *new_vertex = new sglVec3f[2*vector_size];
   unsigned int *length = new unsigned int[vector_size];
   unsigned int index = 0;
   for (unsigned int i=0; i<vector_size; i++)
   {
      length[i] = 2;
      new_vertex[index][0] = vertex[i][0];
      new_vertex[index][1] = vertex[i][1];
      new_vertex[index][2] = vertex[i][2];
      index++;
      new_vertex[index][0] = vertex[i][0] + normal[i][0];
      new_vertex[index][1] = vertex[i][1] + normal[i][1];
      new_vertex[index][2] = vertex[i][2] + normal[i][2];
      index++;
   }

   sglLineStripSet *gset = new sglLineStripSet();
   gset->setCoordList(new_vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(vector_size);
   gset->setPrimLengths(length);

   //cerr << "LineStrip Set: " << gset << " (source = "
   //     << source_gset << ")" << endl;
   return gset;
}

//----------------------------------------------------------------------------
#include <sglPointSet.hpp>
unsigned int num_verts = 8*8*8;

sglGeoSet *makePointSet()
{
   sglPointSet *gset = new sglPointSet();
   sglVec3f *vertex = new sglVec3f[num_verts];
   for (int i=0; i<8; i++)
      for (int j=0; j<8; j++)
         for (int k=0; k<8; k++)
         {
            int index = ((i*8) + j)*8 + k;
            vertex[index][0] = i;
            vertex[index][1] = j;
            vertex[index][2] = k;
         }

   gset->setCoordList(vertex);
   gset->setNumPrims(num_verts);
   //cerr << "Point Set: " << gset << endl;

   return gset;
}

//----------------------------------------------------------------------------
#include <sglIndexedPointSet.hpp>

sglGeoSet *makeIndexedPointSet()
{
   sglIndexedPointSet *gset = new sglIndexedPointSet();
   sglVec3f *vertex = new sglVec3f[num_verts];
   unsigned int *ilist = new unsigned int[num_verts];

   for (int i=0; i<8; i++)
      for (int j=0; j<8; j++)
         for (int k=0; k<8; k++)
         {
            int index = ((i*8) + j)*8 + k;
            vertex[index][0] = i;
            vertex[index][1] = j;
            vertex[index][2] = k;

            ilist[index] = index;
         }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(ilist);
   gset->setNumPrims(num_verts);

   //cerr << "Indexed Point Set: " << gset << endl;
   return gset;
}

//----------------------------------------------------------------------------
#include <sglIndexedLineSet.hpp>

sglGeoSet *makeIndexedLineSet(sglGeoSet *source_gset)
{
   sglVec3f *vertex = (sglVec3f*)source_gset->getCoordList()->getData();
   sglVec3f *normal = (sglVec3f*)source_gset->getNormalList()->getData();

   sglVec4f *color = new sglVec4f[1];
   unsigned int *clist = new unsigned int[1];
   clist[0] = 0;
   color[0][0] = color[0][3] = 1.0; color[0][1] = color[0][2] = 0.0;
   sglVec3f *new_vertex = new sglVec3f[2*vector_size];
   unsigned int *ilist = new unsigned int[2*vector_size];

   unsigned int index = 0;
   for (unsigned int i=0; i<vector_size; i++)
   {
      new_vertex[index][0] = vertex[i][0];
      new_vertex[index][1] = vertex[i][1];
      new_vertex[index][2] = vertex[i][2];
      ilist[index] = index;
      index++;
      new_vertex[index][0] = vertex[i][0] + normal[i][0];
      new_vertex[index][1] = vertex[i][1] + normal[i][1];
      new_vertex[index][2] = vertex[i][2] + normal[i][2];
      ilist[index] = index;
      index++;
   }

   sglIndexedLineSet *gset = new sglIndexedLineSet();
   gset->setNumPrims(vector_size);
   gset->setCoordList(new_vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setCoordIndexList(ilist);
   gset->setColorIndexList(clist);

   //cerr << "Indexed Line Set: " << gset << endl;
   return gset;
}

//----------------------------------------------------------------------------
void setupLODs(sglLOD *node)
{
   sglVec3f center(0.f, 0.f, 0.f);
   node->setCenter(center);

   for (unsigned int i=0; i<4; i++)
   {
      unsigned int num_points = 4*(4-i);
      sglVec3f *vertices = new sglVec3f[num_points+1];
      for (unsigned int j=0; j<=num_points; j++)
      {
         vertices[j][0] = 10.0*sin((2*M_PI*j)/num_points);
         vertices[j][1] = 10.0*cos((2*M_PI*j)/num_points);
         vertices[j][2] = 0.0;
      }

      unsigned int *length = new unsigned int[1];
      length[0] = num_points+1;

      sglLineStripSet *gset = new sglLineStripSet();
      gset->setCoordList(vertices);
      gset->setNumPrims(1);
      gset->setPrimLengths(length);

      sglGeode *geode = new sglGeode();
      geode->addGeometry(gset);
      node->addChild(geode);
      node->setRange(i, 1. + 75.0*i);  // 1, 26, 51, 76
   }
   node->setRange(4, 400.0);  // 1, 26, 51, 76, 150

   //node->printInfo("");
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   vector<sglStatelet*> default_statelets;
   default_statelets.push_back(new sglCullMode(sglCullMode::eOFF));

   sglDefaultStateNode *def = new sglDefaultStateNode();
   def->setStatelets(default_statelets);

   root = new sglScene;
   root->addChild(def);

   lod_state_node = new sglLODStateNode();
   def->addChild(lod_state_node);

   dcs_node = new sglTransformf();
   switch_node = new sglSwitch();
   lod = new sglLOD();
   lod_state = new sglLODState();
   lod->setLODState(lod_state);

   sglTransformf *scs_node = new sglTransformf();
   scs_node->setTranslation(0.0,0.0,25.0);
   lod_state_node->addChild(scs_node);

   billboard_node = new sglBillboard();
   scs_node->addChild(billboard_node);
   billboard_node->addChild(lod);
   billboard_node->setMode(sglBillboard::ePOINT_ROT_EYE);

   //sglVec3d axis;
   //axis[0] = 1.0; axis[1] = 0.0; axis[2] = 0.0;
   //billboard_node->setAxis(axis);
   //billboard_node->setMode(sglBillboard::eAXIAL_ROT);

   setupLODs(lod);

   sglDiscriminator *disc_node = new sglDiscriminator();
   disc_node->setMask(0x01);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes);
   disc_node->addChild(geode);
   billboard_node->addChild(disc_node);

   def->addChild(dcs_node);
   dcs_node->addChild(switch_node);

   sglGeode *model = makeTile();
   def->addChild(model);

   sglTransformf *txnode = new sglTransformf();
   txnode->setTranslation(35.0, 0., 0.);

   switch_node->addChild(txnode);

   for (unsigned int i=0; i<6; i++)
   {
      sglTransformf *txnode2 = new sglTransformf();
      txnode2->setRotation(i*M_PI/3.0, 1.0,0.0,0.0);
      txnode2->addChild(lod);
//    txnode2->addChild(model);

      txnode->addChild(txnode2);
   }

   sglGeode *test_models = new sglGeode();
   def->addChild(test_models);
   test_models->addGeometry(makeIndexedPointSet());
// test_models->addGeometry(makeLineStripSet(model->getGeometry(0)));


   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   double fov_y = (double)M_PI/(3.0f*aspect);

   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);
   //view_platform->setEnableZup(true);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(60.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(new sglPolyFrontStyle(sglPolyStyle::eLINE));
   override_statelets.push_back(new sglPolyBackStyle(sglPolyStyle::eLINE));
   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("sglfly");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
