/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: sglfly2.cpp
 *  Created:
 *  Summary: GLUT-based sgl testbed
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglVector.hpp>
#include <sglMatrix.hpp>

#include <sglGroup.hpp>
#include <sglScene.hpp>
#include <sglLOD.hpp>
#include <sglTransform.hpp>
#include <sglTranslate.hpp>
#include <sglTextSet.hpp>
#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglLineSet.hpp>
#include <sglLineStripSet.hpp>
#include <sglPointSet.hpp>
#include <sglTriangleStripSet.hpp>
#include <sglIndexedTriangleStripSet.hpp>
#include <sglDefaultStateNode.hpp>
#include <sglUnScale.hpp>
#include <sglBillboard.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*>  override_statelets;

sglAntiAliasLine aa_line;
sglAntiAliasPoint aa_point;
sglAntiAliasPolygon aa_poly;

unsigned int win_x = 640, win_y = 480;

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;
   double fov_y = M_PI/(4.0);

   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "a - toggle point anti-aliasing" << endl;
         cerr << "A - toggle line anti-aliasing" << endl;
         cerr << "z - toggle polygon anti-aliasing" << endl;

         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);

   displayFunc();
}

//----------------------------------------------------------------------------
sglGroup *makeTriStrips()
{
   const unsigned int LENGTH = 4;

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = LENGTH;

   unsigned int *ilist = new unsigned int[LENGTH];
   ilist[0] = 0; ilist[1] = 1; ilist[2] = 2; ilist[3] = 3;

   sglVec3f *vertex = new sglVec3f[LENGTH];
   vertex[0].set(3.0f, 0.0f, 3.0f);
   vertex[1].set(3.0f, 0.0f, 0.0f);
   vertex[2].set(3.0f, 3.0f, 3.0f);
   vertex[3].set(3.0f, 3.0f, 0.0f);

   sglIndexedTriangleStripSet *gset = new sglIndexedTriangleStripSet();

   gset->setCoordList(vertex);
   gset->setCoordIndexList(ilist);
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);

   sglGroup *group = new sglGroup();

   for (int i=0; i<4; i++)
      for (int j=0; j<4; j++)
         for (int k=0; k<4; k++)
         {
            sglTransformd *tx_node = new sglTransformd();
            tx_node->setTranslation(i*4.0f, j*4.0f, k*4.0f);
            tx_node->addChild(geode);

            group->addChild(tx_node);
         }

   return group;
}

//----------------------------------------------------------------------------
sglGroup *makeLines()
{
   sglGroup *group = new sglGroup();

   for (int i=0; i<4; i++)
      for (int j=0; j<4; j++)
         for (int k=0; k<4; k++)
         {
            sglGeode *geode = new sglGeode();
            sglLineSet *gset = new sglLineSet();

            sglVec3f *vertex = new sglVec3f[2];
            vertex[0].set(i*4.f, j*4.f, k*4.f);
            vertex[1].set(i*4.f+3.0f, j*4.f+3.0f, k*4.f+3.0f);

            gset->setCoordList(vertex);
            gset->setNumPrims(1);

            geode->addGeometry(gset);
            group->addChild(geode);
         }

   return group;
}

//----------------------------------------------------------------------------
sglGroup *makePoints()
{
   sglGroup *group = new sglGroup();

   for (int i=0; i<4; i++)
      for (int j=0; j<4; j++)
         for (int k=0; k<4; k++)
         {
            sglGeode *geode = new sglGeode();
            sglPointSet *gset = new sglPointSet();

            sglVec3f *vertex = new sglVec3f[1];
            vertex[0].set(i*4.f+1.0f, j*4.f+1.0f, k*4.f+3.0f);

            gset->setCoordList(vertex);
            gset->setNumPrims(1);

            geode->addGeometry(gset);
            group->addChild(geode);
         }

   return group;
}

//----------------------------------------------------------------------------
sglLOD *setupLOD()
{
   sglLOD *node = new sglLOD();
   sglVec3f center(0.f, 0.f, 0.f);
   node->setCenter(center);
   unsigned int i;
   for (i=0; i<25; i++)
   {
      unsigned int num_points = 26-i;
      sglVec3f *vertices = new sglVec3f[num_points+1];
      for (unsigned int j=0; j<=num_points; j++)
      {
         vertices[j][0] = center[0] + (i+1)*sin((2*M_PI*j)/num_points);
         vertices[j][1] = center[1] + (i+1)*cos((2*M_PI*j)/num_points);
         vertices[j][2] = center[2] + 0.0;
      }

      unsigned int *length = new unsigned int[1];
      length[0] = num_points+1;

      sglLineStripSet *gset = new sglLineStripSet();
      gset->setCoordList(vertices);
      gset->setNumPrims(1);
      gset->setPrimLengths(length);

      sglGeode *geode = new sglGeode();
      geode->addGeometry(gset);
      node->addChild(geode);
      node->setRange(i, 9.*(1. + i));  // 1, 26, 51, 76
   }
   node->setRange(i, i*30.0);  // 1, 26, 51, 76, 150

   //node->printInfo("");
   return node;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   root = new sglScene();

   sglCullMode *cull_mode = new sglCullMode();
   cull_mode->setCullMode(sglCullMode::eOFF);
   vector<sglStatelet*> default_statelets;
   default_statelets.push_back(cull_mode);
   sglDefaultStateNode *def = new sglDefaultStateNode();
   def->setStatelets(default_statelets);
   root->addChild(def);

   sglTranslate<double> *txnode = new sglTranslate<double>();
   txnode->setTranslation(-1., -1., -1.);
   def->addChild(txnode);
   sglTextSet *tsnode = new sglTextSet;
   tsnode->setPosition(0.f, 0.f, 10.f);
   tsnode->setString("example string 1 (sglTextSet)");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(tsnode);
   txnode->addChild(geode);

   tsnode = new sglTextSet;
   tsnode->setString("example string 2");
   sglUnScale *unscale = new sglUnScale(true);
   sglTranslate<double> *txnode2 = new sglTranslate<double>();
   txnode2->setTranslation(0, -15, 0);
   sglBillboard *billboard = new sglBillboard();
   billboard->addChild(txnode2);
   unscale->addChild(billboard);
   geode = new sglGeode;
   geode->addGeometry(tsnode);
   txnode2->addChild(geode);
   txnode->addChild(unscale);
   def->addChild(makeLines());
   def->addChild(makePoints());
   def->addChild(makeTriStrips());

   txnode->addChild(setupLOD());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   double fov_y = (double)M_PI/(3.0f*aspect);

   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);
   //view_platform->setEnableZup(true);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(60.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("sglfly2");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
