/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testBezierCurves.cpp
 *   Author: Tom Stimson
 *  Created: 28 February 2000
 *  Summary: test the bezier curve geometry classes
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglSwitch.hpp>
#include <sglDiscriminator.hpp>
#include <sglGeode.hpp>
#include <sglPointSet.hpp>
#include <sglBezierCurveSet.hpp>
#include <sglBitmapFont.hpp>
#include <sglTexture2D.hpp>
#include <sglTexEnv.hpp>

#include <string>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglOrthographicCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view_ul;
sglView                   *view_ur;
sglView                   *view_ll;  // the perspective view
sglView                   *view_lr;

sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;

sglOrthographicCamera     *view_camera_ul;
sglOrthographicCamera     *view_camera_ur;
sglPerspectiveCamera      *view_camera_ll;
sglOrthographicCamera     *view_camera_lr;

deque<const sglStatelet*> override_statelets;
sglAntiAliasLine    aa_line;
sglAntiAliasPoint   aa_point;
sglAntiAliasPolygon aa_poly;

unsigned int disc_mask = 0x0f;

unsigned int win_x = 640, win_y = 480;
int half1_x = win_x / 2;
int half1_y = win_y / 2;
int half2_x = win_x - half1_x;
int half2_y = win_y - half1_y;
double fov_y = M_PI/3.0;
double aspect = 1.;

sglSwitch   *switch_node;
int          switch_val = 0;

vector<sglStatelet*> tex_state;

vector<std::string> textList1;
vector<std::string> textList2;
vector<std::string> textList3;
vector<std::string> textList4;
vector<std::string> textList5;

sglIntersectf  segset;
sglSegmentf    segment;

vector<sglBezierCurveSet*> curves;
sglPointSet *drag_set = NULL;
sglVec3f orig_point;
int drag_start_x;
int drag_start_y;

#define CHECK_IMAGE_SIZE 64
static GLubyte check_image[CHECK_IMAGE_SIZE][CHECK_IMAGE_SIZE][4];

//----------------------------------------------------------------------------
void outputLabel()
{
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(0, win_x, 0, win_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glColor3f(1.0, 1.0, 1.0);

   glViewport(0, 0, win_x, win_y);
   glScissor(0, 0, win_x, win_y);

   glRasterPos2f(5, 45);
   sglBitmapFont::getDefaultFont()->rasterString(
      textList1[switch_node->getVal()].c_str());
   glRasterPos2f(5, 35);
   sglBitmapFont::getDefaultFont()->rasterString(
      textList2[switch_node->getVal()].c_str());
   glRasterPos2f(5, 25);
   sglBitmapFont::getDefaultFont()->rasterString(
      textList3[switch_node->getVal()].c_str());
   glRasterPos2f(5, 15);
   sglBitmapFont::getDefaultFont()->rasterString(
      textList4[switch_node->getVal()].c_str());
   glRasterPos2f(5, 5);
   sglBitmapFont::getDefaultFont()->rasterString(
      textList5[switch_node->getVal()].c_str());

   glBegin(GL_LINES);
   glVertex2f(half1_x, 0);
   glVertex2f(half1_x, win_y);
   glVertex2f(0, half1_y);
   glVertex2f(win_x, half1_y);
   glEnd();
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view_ll->cullAndDraw();
   view_ul->cullAndDraw();
   view_lr->cullAndDraw();
   view_ur->cullAndDraw();

   outputLabel();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   half1_x = win_x / 2;
   half1_y = win_y / 2;
   half2_x = win_x - half1_x;
   half2_y = win_y - half1_y;

   view_ul->setViewport(0, half1_y, half1_x, half2_y);
   view_ur->setViewport(half1_x, half1_y, half2_x, half2_y);
   view_ll->setViewport(0, 0, half1_x, half1_y);
   view_lr->setViewport(half1_x, 0, half2_x, half1_y);

   aspect = (double)w/(double)h;

   view_camera_ul->setOrtho(-15*aspect, 15*aspect, -15, 15, 0, 100);
   view_camera_ur->setOrtho(-15*aspect, 15*aspect, -15, 15, 0, 100);
   view_camera_lr->setOrtho(-15*aspect, 15*aspect, -15, 15, 0, 100);

   view_camera_ll->setFOV(fov_y, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         switch_val++;
         if ((unsigned int)switch_val >= switch_node->getNumChildren())
         {
            switch_val = 0;
         }
         switch_node->setVal(switch_val);
         break;

      case GLUT_KEY_DOWN:
         switch_val--;
         if (switch_val < 0)
         {
            switch_val = switch_node->getNumChildren() - 1;
         }
         switch_node->setVal(switch_val);
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   glutPostRedisplay();
}

//----------------------------------------------------------------------------
void movePoint(int x, int y)
{
   x -= drag_start_x;
   y -= drag_start_y;
   float xf = ((float)x / (float)half1_x) * 30*aspect;
   float yf = ((float)y / (float)half1_y) * 30;

   sglVec3f *drag_point = (sglVec3f*)drag_set->getCoordList()->getData();
   if (drag_start_x < half1_x && drag_start_y < half1_y)
   {
      (*drag_point)[0] = orig_point[0] + xf;
      (*drag_point)[2] = orig_point[2] - yf;
   }
   else if (drag_start_x > half1_x && drag_start_y < half1_y)
   {
      (*drag_point)[1] = orig_point[1] + xf;
      (*drag_point)[2] = orig_point[2] - yf;
   }
   else
   {
      (*drag_point)[0] = orig_point[0] + xf;
      (*drag_point)[1] = orig_point[1] - yf;
   }
   drag_set->setCoordList(drag_point);
   curves[switch_val]->setCoordList(curves[switch_val]->getCoordList());
}

//----------------------------------------------------------------------------
void handleMotion(int x, int y)
{
   if (x < half1_x && y > half1_y)
      sgluMouse::handleMotion(x*2, (y-half1_y)*2);

   if (drag_set)
   {
      movePoint(x, y);
   }
}

//----------------------------------------------------------------------------
void handleMouse(int button, int state, int x, int y)
{
   if (x < half1_x && y > half1_y)
   {
      sgluMouse::handleMouse(button, state, x*2, (y-half1_y)*2);
      return;
   }

   if (state == GLUT_DOWN && button == GLUT_LEFT_BUTTON)
   {
      if (x < half1_x && y < half1_y)
      {
         float xf = (((float)x / (float)half1_x) - 0.5f) * 30*aspect;
         float yf = (0.5f - ((float)y / (float)half1_y)) * 30;
         segment.setPos(sglVec3f(xf, 100, yf));
         segment.setDir(sglVec3f(0, -1, 0));
         segment.setLen(200);
      }
      else if (x > half1_x && y < half1_y)
      {
         float xf = ((((float)x-half1_x) / (float)half1_x) - 0.5f) * 30*aspect;
         float yf = (0.5f - ((float)y / (float)half1_y)) * 30;
         segment.setPos(sglVec3f(100, xf, yf));
         segment.setDir(sglVec3f(-1, 0, 0));
         segment.setLen(200);
      }
      else
      {
         float xf = ((((float)x-half1_x) / (float)half1_x) - 0.5f) * 30*aspect;
         float yf = (0.5f - (((float)y-half1_y) / (float)half1_y)) * 30;
         segment.setPos(sglVec3f(xf, yf, 100));
         segment.setDir(sglVec3f(0, 0, -1));
         segment.setLen(200);
      }
      segset.setSegment(segment);
      segset.setResultEnum(sglBound::eOUTSIDE);
      root->intersect(segset);
      if (segset.getResultEnum() == sglBound::eINTERSECT)
      {
         drag_set = const_cast<sglPointSet*>(
            dynamic_cast<const sglPointSet*>(segset.getGeometry()));
         if (drag_set)
         {
            drag_start_x = x;
            drag_start_y = y;
            sglVec3f *drag_point =
               (sglVec3f*)drag_set->getCoordList()->getData();
            orig_point = *drag_point;
         }
      }
   }
   else if (state == GLUT_UP && button == GLUT_LEFT_BUTTON)
   {
      if (drag_set)
      {
         sglVec3f *drag_point = (sglVec3f*)drag_set->getCoordList()->getData();
         movePoint(x, y);
         sglBoxBound *bound = new sglBoxBound();
         *bound = drag_set->getBound();
         bound->setMin(
            (*drag_point)[0]-.5, (*drag_point)[1]-.5, (*drag_point)[2]-.5);
         bound->setMax(
            (*drag_point)[0]+.5, (*drag_point)[1]+.5, (*drag_point)[2]+.5);
         drag_set->setBound(bound, sglBound::eSTATIC);

         drag_set = NULL;
      }
   }
}

//----------------------------------------------------------------------------
void makeCheckImage(void)
{
    int i,j;
    GLubyte c;
    for (i=0; i<CHECK_IMAGE_SIZE; i++)
    {
        for (j=0; j<CHECK_IMAGE_SIZE; j++)
        {
            c = 255;
            if ((i&0x8) != (j&0x8)) c = 100;
            check_image[i][j][0] = c;
            check_image[i][j][1] = c;
            check_image[i][j][2] = c;
            check_image[i][j][3] = 255;
        }
    }
}

//----------------------------------------------------------------------------
void makeStates()
{
    sglTexture2D *tex = new sglTexture2D;
    tex->setImage(&check_image[0][0][0], sglTexture::eRGBA, sglTexture::eUNSIGNED_BYTE,
                  CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);

    tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
    tex->setAutoMipmapFlag(true);
    tex->setWrapMode(sglTexture::eCLAMP_TO_EDGE, sglTexture::eCLAMP_TO_EDGE);
    tex->setBorderColor(sglVec4f(1.0, 0.0, 0.0, 1.0));

    tex->bind();

    sglTexEnv  *tev = new sglTexEnv();
    tev->setMode(sglTexEnv::eDECAL);

    tex_state.push_back(tex);
    tex_state.push_back(tev);
}

//----------------------------------------------------------------------------
sglGeode *createPoints(sglBezierCurveSet *curve)
{
   sglVec3f *vertex = (sglVec3f*)curve->getCoordList()->getData();

   sglGeode *geode = new sglGeode();
   for (int ii = 0;
        ii < curve->getCoordOrder() * (int)curve->getNumPrims();
        ++ii)
   {
       sglPointSet *points = new sglPointSet();
       points->setCoordList(&vertex[ii]);
       sglVec4f *color = new sglVec4f(1.0f, 1.0f, 1.0f, 1.0f);
       points->setColorList(sglGeoSet::eOVERALL, color);
       points->setNumPrims(1);

       sglBoxBound *bound = new sglBoxBound();
       bound->setMin(vertex[ii][0]-.5, vertex[ii][1]-.5, vertex[ii][2]-.5);
       bound->setMax(vertex[ii][0]+.5, vertex[ii][1]+.5, vertex[ii][2]+.5);
       points->setBound(bound, sglBound::eSTATIC);

       vector<sglStatelet*> state;
       state.push_back(new sglPointSize(2.0f));
       points->setState(0, state);

       geode->addGeometry(points);
   }

   return geode;
}

//----------------------------------------------------------------------------
sglGroup* createGroup(sglBezierCurveSet *gset)
{
   sglGroup *group = new sglGroup;
   sglGeode *geode = new sglGeode();

   geode->addGeometry(gset);
   sglDiscriminator *disc = new sglDiscriminator();
   disc->setMask(0x00000001);
   disc->addChild(geode);
   group->addChild(disc);

   group->addChild(createPoints(gset));

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeCurveSet1()
{
   textList1.push_back("1 primitive");
   textList2.push_back("coord order 4");
   textList3.push_back("color order 1 overall");
   textList4.push_back("not lit");
   textList5.push_back("not textured");

   sglBezierCurveSet *gset = new sglBezierCurveSet();
   curves.push_back(gset);

   sglVec3f *vertex = new sglVec3f[4];
   vertex[0].set(-10.0f, 0.0f, -10.0f);
   vertex[1].set(-08.0f, 7.0f,  10.0f);
   vertex[2].set( 08.0f, 0.0f, -10.0f);
   vertex[3].set( 10.0f, 0.0f,  10.0f);

   gset->setCoordList(vertex);
   gset->setCoordOrder(4);

   sglVec4f *color = new sglVec4f(1.0f, 0, 0, 1.0f);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorOrder(1);

   gset->setNumPrims(1);
   gset->setNumSegments(10);

   return createGroup(gset);
}


//----------------------------------------------------------------------------
sglGroup* makeCurveSet2()
{
   textList1.push_back("2 primitives");
   textList2.push_back("coord order 3");
   textList3.push_back("color order 2 overall");
   textList4.push_back("not lit");
   textList5.push_back("not textured");

   sglBezierCurveSet *gset = new sglBezierCurveSet();
   curves.push_back(gset);

   sglVec3f *vertex = new sglVec3f[6];
   vertex[0].set(-10.0f, 0.0f, -10.0f);
   vertex[1].set(-08.0f, 7.0f,  10.0f);
   vertex[2].set( 08.0f, 0.0f, -10.0f);
   vertex[3].set( 04.0f, 1.0f,  03.0f);
   vertex[4].set(-02.0f, 1.0f,  05.0f);
   vertex[5].set( 06.0f, 1.0f, -04.0f);

   gset->setCoordList(vertex);
   gset->setCoordOrder(3);

   sglVec4f *color = new sglVec4f[2];
   color[0] = sglVec4f(0, 0, 1.f, 1.f);
   color[1] = sglVec4f(1.f, 0, 0, 1.f);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorOrder(2);

   gset->setNumPrims(2);
   gset->setNumSegments(100);

   return createGroup(gset);
}


//----------------------------------------------------------------------------
sglGroup* makeCurveSet3()
{
   textList1.push_back("2 primitives");
   textList2.push_back("coord order 3");
   textList3.push_back("color order 2 per/prim");
   textList4.push_back("not lit");
   textList5.push_back("not textured");

   sglBezierCurveSet *gset = new sglBezierCurveSet();
   curves.push_back(gset);

   sglVec3f *vertex = new sglVec3f[6];
   vertex[0].set(-10.0f, 0.0f, -10.0f);
   vertex[1].set(-08.0f, 7.0f,  10.0f);
   vertex[2].set( 08.0f, 0.0f, -10.0f);
   vertex[3].set( 04.0f, 1.0f,  03.0f);
   vertex[4].set(-02.0f, 1.0f,  05.0f);
   vertex[5].set( 06.0f, 1.0f, -04.0f);

   gset->setCoordList(vertex);
   gset->setCoordOrder(3);

   sglVec4f *color = new sglVec4f[2];
   color[0] = sglVec4f(0, 0, 1.f, 1.f);
   color[1] = sglVec4f(1.f, 0, 0, 1.f);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setColorOrder(2);

   gset->setNumPrims(2);
   gset->setNumSegments(100);

   return createGroup(gset);
}


//----------------------------------------------------------------------------
sglGroup* makeCurveSet4()
{
   textList1.push_back("2 primitives");
   textList2.push_back("coord order 3");
   textList3.push_back("color order 1 per/vertex");
   textList4.push_back("not lit");
   textList5.push_back("not textured");

   sglBezierCurveSet *gset = new sglBezierCurveSet();
   curves.push_back(gset);

   sglVec3f *vertex = new sglVec3f[6];
   vertex[0].set(-10.0f, 0.0f, -10.0f);
   vertex[1].set(-08.0f, 7.0f,  10.0f);
   vertex[2].set( 08.0f, 0.0f, -10.0f);
   vertex[3].set( 04.0f, 1.0f,  03.0f);
   vertex[4].set(-02.0f, 1.0f,  05.0f);
   vertex[5].set( 06.0f, 1.0f, -04.0f);

   gset->setCoordList(vertex);
   gset->setCoordOrder(3);

   sglVec4f *color = new sglVec4f[2];
   color[0] = sglVec4f(0, 0, 1.f, 1.f);
   color[1] = sglVec4f(1.f, 0, 0, 1.f);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorOrder(1);

   gset->setNumPrims(2);
   gset->setNumSegments(100);

   return createGroup(gset);
}

//----------------------------------------------------------------------------
sglGroup* makeCurveSet5()
{
   textList1.push_back("2 primitives");
   textList2.push_back("coord order 3");
   textList3.push_back("color order 2 per/vertex");
   textList4.push_back("not lit");
   textList5.push_back("not textured");

   sglBezierCurveSet *gset = new sglBezierCurveSet();
   curves.push_back(gset);

   sglVec3f *vertex = new sglVec3f[6];
   vertex[0].set(-10.0f, 0.0f, -10.0f);
   vertex[1].set(-08.0f, 7.0f,  10.0f);
   vertex[2].set( 08.0f, 0.0f, -10.0f);
   vertex[3].set( 04.0f, 1.0f,  03.0f);
   vertex[4].set(-02.0f, 1.0f,  05.0f);
   vertex[5].set( 06.0f, 1.0f, -04.0f);

   gset->setCoordList(vertex);
   gset->setCoordOrder(3);

   sglVec4f *color = new sglVec4f[4];
   color[0] = sglVec4f(0, 0, 1.f, 1.f);
   color[1] = sglVec4f(1.f, 0, 0, 1.f);
   color[2] = sglVec4f(0, 1.f, 1.f, 1.f);
   color[3] = sglVec4f(0, 1.f, 0, 1.f);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorOrder(2);

   gset->setNumPrims(2);
   gset->setNumSegments(100);

   return createGroup(gset);
}

//----------------------------------------------------------------------------
sglGroup* makeCurveSet6()
{
   textList1.push_back("2 primitives");
   textList2.push_back("coord order 3");
   textList3.push_back("color order 1 overall");
   textList4.push_back("not lit");
   textList5.push_back("texture order 2 overall");

   sglBezierCurveSet *gset = new sglBezierCurveSet();
   curves.push_back(gset);

   sglVec3f *vertex = new sglVec3f[6];
   vertex[0].set(-10.0f, 0.0f, -10.0f);
   vertex[1].set(-08.0f, 7.0f,  10.0f);
   vertex[2].set( 08.0f, 0.0f, -10.0f);
   vertex[3].set( 04.0f, 1.0f,  03.0f);
   vertex[4].set(-02.0f, 1.0f,  05.0f);
   vertex[5].set( 06.0f, 1.0f, -04.0f);

   gset->setCoordList(vertex);
   gset->setCoordOrder(3);

   sglVec4f *color = new sglVec4f[1];
   color[0] = sglVec4f(1.f, 1.f, 1.f, 1.f);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorOrder(1);

   sglVec2f *tex = new sglVec2f[2];
   tex[0] = sglVec2f(0.f, 0.f);
   tex[1] = sglVec2f(0.f, 1.f);

   gset->setTexCoordBind(sglGeoSet::eOVERALL);
   gset->setTexCoordOrder(2);

   gset->setNumPrims(2);
   gset->setNumSegments(100);

   gset->setState(0, tex_state, tex);

   return createGroup(gset);
}


//----------------------------------------------------------------------------
void buildScenegraph()
{
   root = new sglScene;

   //====================================================================
   segset.setMode(sglProbe::eGEOMETRY);
   segset.setDiscriminatorMask(~0x000000001);  // mask out the curve
   //====================================================================

   makeCheckImage();
   makeStates();

   switch_node = new sglSwitch();
   switch_node->setVal(switch_val);

   switch_node->addChild(makeCurveSet1());
   switch_node->addChild(makeCurveSet2());
   switch_node->addChild(makeCurveSet3());
   switch_node->addChild(makeCurveSet4());
   switch_node->addChild(makeCurveSet5());
   switch_node->addChild(makeCurveSet6());

   //switch_node->addChild(makeCurveStripSet());

   root->addChild(switch_node);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);

   // ----------------------------------------------------------
   // the cameras (frusta), view platforms, and view transforms
   // ----------------------------------------------------------

   double aspect = (double)win_x/(double)win_y;

   // --------------------- lower left -------------------------
   view_camera_ll = new sglPerspectiveCamera;
   view_camera_ll->setFOV(fov_y, aspect, 0.1, 200.0);
   view_platform = new sglViewPlatform(*view_camera_ll);

   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   view_ll = new sglView;
   view_ll->setViewport(0, 0, half1_x, half1_y);
   view_ll->setOverrideStatelets(override_statelets);
   // ----------------------------------------------------------


   // --------------------- upper left -------------------------
   view_camera_ul = new sglOrthographicCamera;
   view_camera_ul->setOrtho(-15*aspect, 15*aspect, -15, 15, 0, 100);
   sglViewPlatform *vp_ul = new sglViewPlatform(*view_camera_ul);
   sglTransformf *vt_ul = new sglTransformf;
   vt_ul->setMatrix(sglMat4f(1, 0, 0, 0,
                             0, 0, 1, 0,
                             0,-1, 0, 0,
                             0, -50, 0, 1));
   vt_ul->addChild(vp_ul);
   root->addChild(vt_ul);

   view_ul = new sglView;
   view_ul->setViewport(0, half1_y, half1_x, half2_y);
   view_ul->setOverrideStatelets(override_statelets);
   // ----------------------------------------------------------


   // --------------------- upper right ------------------------
   view_camera_ur = new sglOrthographicCamera;
   view_camera_ur->setOrtho(-15*aspect, 15*aspect, -15, 15, 0, 100);
   sglViewPlatform *vp_ur = new sglViewPlatform(*view_camera_ur);
   sglTransformf *vt_ur = new sglTransformf;
   vt_ur->setMatrix(sglMat4f(0, 1, 0, 0,
                             0, 0, 1, 0,
                             1, 0, 0, 0,
                             50, 0, 0, 1));
   vt_ur->addChild(vp_ur);
   root->addChild(vt_ur);

   view_ur = new sglView;
   view_ur->setViewport(half1_x, half1_y, half2_x, half2_y);
   view_ur->setOverrideStatelets(override_statelets);
   // ----------------------------------------------------------


   // --------------------- lower right ------------------------
   view_camera_lr = new sglOrthographicCamera;
   view_camera_lr->setOrtho(-15*aspect, 15*aspect, -15, 15, 0, 100);
   sglViewPlatform *vp_lr = new sglViewPlatform(*view_camera_lr);
   sglTransformf *vt_lr = new sglTransformf;
   vt_lr->setMatrix(sglMat4f(1, 0, 0, 0,
                             0, 1, 0, 0,
                             0, 0, 1, 0,
                             0, 0, 50, 1));
   vt_lr->addChild(vp_lr);
   root->addChild(vt_lr);

   view_lr = new sglView;
   view_lr->setViewport(half1_x, 0, half2_x, half1_y);
   view_lr->setOverrideStatelets(override_statelets);
   // ----------------------------------------------------------

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);
   root->preDraw();

   view_ll->setScene(root, view_platform);
   view_ul->setScene(root, vp_ul);
   view_ur->setScene(root, vp_ur);
   view_lr->setScene(root, vp_lr);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testBezierCurves");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   // Override the mouse functions for editing
   glutMouseFunc(handleMouse);
   glutMotionFunc(handleMotion);
   glutPassiveMotionFunc(handleMotion);

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   idleFunc();

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
