/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testBounds.cpp
 *  Created: 1999
 *  Summary: test bounding boxes and spheres
 *****************************************************************************/

#include <sglBoxBound.hpp>
#include <sglSphereBound.hpp>
#include <sglOStream.hpp>

int main(int, char **)
{
   sglOStream::setStream(&cout);

   // =======================================================

   sglBoxBound box1;
   cout << "box1: " << box1 << endl;

   cout << "box1.isEmpty() = " << box1.isEmpty() << endl;

   if (box1.isEmpty() == false)
   {
       cerr << "sglBoxBound initialization test FAILED." << endl;
       exit(1);
   }
   cerr << "sglBoxBound initialization test PASSED." << endl;

   box1.setMin(-1.0, -3.0, -2.0);
   box1.setMax( 1.0,  3.0,  2.0);

   cout << "box1: " << box1 << endl;
   sglVec3f vmin = box1.getMin();
   sglVec3f vmax = box1.getMax();
   if (vmin[0] == -1.0 && vmin[1] == -3.0 && vmin[2] == -2.0 &&
       vmax[0] ==  1.0 && vmax[1] ==  3.0 && vmax[2] ==  2.0)
   {
       cerr << "setMin/setMax test PASSED." << endl;
   }
   else
   {
       cerr << "setMin/setMax test FAILED." << endl;
       exit(1);
   }

   sglBoxBound box2;

   box2.setMin(-2.0, 13.0, -12.0);
   box2.setMax( 2.0, 23.0,  -2.0);

   cout << "box2: " << box2 << endl;

   sglBoxBound box3 = box1;
   cout << "box3 = box1: " << box3 << endl;

   vmin = box3.getMin();
   vmax = box3.getMax();
   if (vmin[0] == -1.0 && vmin[1] == -3.0 && vmin[2] == -2.0 &&
       vmax[0] ==  1.0 && vmax[1] ==  3.0 && vmax[2] ==  2.0)
   {
       cerr << "assignment operator test PASSED." << endl;
   }
   else
   {
       cerr << "assignment operator test FAILED." << endl;
       exit(1);
   }

   box3.makeEmpty();
   cout << "box3:" << box3 << endl;

   if (box3.isEmpty() == false)
   {
       cerr << "makeEmpty() test FAILED." << endl;
       exit(1);
   }
   cerr << "makeEmpty() test PASSED." << endl;

   box3 = box1;
   box3.expandToInclude(box2);
   cout << "box3 = box1 + box2: " << box3 << endl;

   if (vmin[0] == -2.0 && vmin[1] == -3.0 && vmin[2] == -12.0 &&
       vmax[0] ==  2.0 && vmax[1] == 23.0 && vmax[2] ==  2.0)
   {
       cerr << "expandToInclude() test FAILED." << endl;
       exit(1);
   }
   cerr << "expandToInclude() test PASSED." << endl << endl;

   // =======================================================
   sglSphereBoundf sphere1;
   cout << "sphere1: " << sphere1 << endl;
   if (sphere1.isEmpty() == false)
   {
       cerr << "sglSphereBoundf initialization test FAILED." << endl;
       exit(1);
   }
   cerr << "sglSphereBoundf initialization test PASSED." << endl;


   sphere1.computeFromBox(box2);
   cout << "sphere1(box2): " << sphere1 << endl;

   vmin = box2.getMin();
   vmax = box2.getMax();
   sglVec3f center(0.5f*(vmin[0]+vmax[0]),
                   0.5f*(vmin[1]+vmax[1]),
                   0.5f*(vmin[2]+vmax[2]));
   vmax.set(vmin[0] - center[0], vmin[1] - center[1], vmin[2] - center[2]);
   float radius = vmax.length();

   if (radius != sphere1.getRadius() || !(center == sphere1.getCenter()))
   {
       cerr << "sglSphereBoundf::computeFromBox() test FAILED." << endl;
       exit(1);
   }
   cerr << "sglSphereBoundf::computeFromBox() test PASSED." << endl;

   sphere1.computeFromBox(box1);
   sglSphereBoundf sphere2;
   sphere2.computeFromBox(box2);
   sphere1.expandToInclude(sphere2);
   cout << "sphere(sphere(box1) + sphere(box2)): " << sphere1 << endl;

   sglSphereBoundf sphere3 = sphere1;

   sphere1.expandToInclude(sphere2);
   cout << "sphere(+sphere2): " << sphere1 << endl;

   if (sphere1 == sphere3)
   {
       cerr << "sglSphereBoundf::expandToInclude() test PASSED." << endl;
   }
   else
   {
       cerr << "sglSphereBoundf::expandToInclude() test FAILED." << endl;
       exit(1);
   }

   return 0;
}
