/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testBumpmap.cpp
 *  Created: 14 November 2001
 *  Summary: test the multi-texture state functionality
 *****************************************************************************/

#include <stdio.h>
#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglGeode.hpp>
#include <sglMonoIndexedTriangleStripSet.hpp>
#include <sglTexture2D.hpp>
#include <sglTexEnv.hpp>
#include <sglTexGen.hpp>
#include <sglBitmapFont.hpp>
#include <sglSwitch.hpp>
#include <sglOrderedGroup.hpp>
#include <sglPositionalLight.hpp>
#include <sglMaterial.hpp>
#include <sglTexCombinerParams.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

#include <sglImageLoader.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

bool help_flag = false;

unsigned int state_mask = 0;

unsigned int win_x = 720, win_y = 480;

sglPolyFrontStyle pfs;
sglPolyBackStyle pbs;

const unsigned int pgonsize = 10;
sglVec3f light_pos(0, -20, 10);
float bump_factor = 0.005f;
sglVec2f *tvert_bump = NULL;
sglVec2f *tvert = NULL;
sglVec3f *vertex = NULL;
sglPositionalLight *light = NULL;
sglSwitch *switch_node = NULL;
sglDrawable *gset1_1 = NULL;
sglDrawable *gset2_1 = NULL;
sglDrawable *gset2_2 = NULL;
sglDrawable *gset3_1 = NULL;
sglDrawable *gset3_2 = NULL;
sglDrawable *gset3_3 = NULL;
vector<sglStatelet*> statelets3_2;
vector<sglStatelet*> statelets2_1;
vector<sglStatelet*> statelets1_1;

bool light_sim = false;
double sim_time = 0;

string base_file;
string bump_file;


//----------------------------------------------------------------------------
void computeBumpCoords()
{
   unsigned int index = 0;
   for (unsigned int j=0; j<pgonsize; ++j)
      for (unsigned int i=0; i<pgonsize; ++i)
      {

         sglVec3f bump_offset(light_pos);
         bump_offset -= vertex[index];
         bump_offset.normalize();
         bump_offset *= bump_factor;

         tvert_bump[index] = tvert[index];
         tvert_bump[index][0] += bump_offset[0];
         tvert_bump[index][1] += bump_offset[2];
         ++index;
      }

   gset3_2->setState(0, statelets3_2, sglTexCoords((float*)tvert_bump, 2));

   vector<sglTexCoords> tcoord_list;
   tcoord_list.push_back(sglTexCoords((float*)tvert, 2));
   tcoord_list.push_back(sglTexCoords((float*)tvert_bump, 2));
   gset2_1->setState(0, statelets2_1, tcoord_list);
   gset1_1->setState(0, statelets1_1, tcoord_list);
}

//----------------------------------------------------------------------------
void drawOverlay()
{
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glDisable(GL_DEPTH_TEST);

   if (help_flag)
   {
      glColor4f(1.0f, 0.9f, 0.9f, 1.0f);
      glRasterPos3i(10,160,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "/? - toggle help");
      glRasterPos3i(10,145,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "@ - dump color buffer to dump.sgi");
      glRasterPos3i(10,130,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "o - ouput scene graph");
      glRasterPos3i(10,115,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "$ - toggle statistics reporting");
      glRasterPos3i(10,100,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "d - toggle SP/DP traversal");
      glRasterPos3i(10,85,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "f - toggle frame rate display");
      glRasterPos3i(10,70,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "p - cycle polygon style");
      glRasterPos3i(10,55,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "1..3 - use 1, 2, or 3 pass bump-mapping");
      glRasterPos3i(10,40,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "0 - render base texture only");
      glRasterPos3i(10,25,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "s - simulate moving light");
      glRasterPos3i(10,10,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "b/B - dec/inc bump factor");
   }

   glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
   glRasterPos3i(250,5,0);
   if (switch_node->getVal() == 0)
      sglBitmapFont::getDefaultFont()->rasterString("base texture only");
   else
   {
      char buf[256];
      sprintf(buf, "%d pass bump-mapping", switch_node->getVal());
      sglBitmapFont::getDefaultFont()->rasterString(buf);
   }

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   drawOverlay();

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case '0':
      case '1':
      case '2':
      case '3':
         switch_node->setVal(key-'0');
         break;

      case 'b':
         bump_factor *= 0.5f;
         computeBumpCoords();
         cout << "bump_factor = " << bump_factor << endl;
         break;

      case 'B':
         bump_factor *= 2.0f;
         computeBumpCoords();
         cout << "bump_factor = " << bump_factor << endl;
         break;

      case 's':
         light_sim = !light_sim;
         break;

      case 'p':
         if (pfs.getStyle() == sglPolyStyle::eFILLED)
         {
            pfs.setStyle(sglPolyStyle::ePOINT);
            pbs.setStyle(sglPolyStyle::ePOINT);
         }
         else if (pfs.getStyle() == sglPolyStyle::ePOINT)
         {
            pfs.setStyle(sglPolyStyle::eLINE);
            pbs.setStyle(sglPolyStyle::eLINE);
         }
         else if (pfs.getStyle() == sglPolyStyle::eLINE)
         {
            pfs.setStyle(sglPolyStyle::eFILLED);
            pbs.setStyle(sglPolyStyle::eFILLED);
         }

         cout << "polygon style = " << (int) pfs.getStyle() << endl;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '@':
      {
         glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
         glPixelStorei(GL_PACK_ALIGNMENT, 1);
         glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
         glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
         glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
         glViewport(0, 0, win_x, win_y);
         glRasterPos2f(0,0);
         void *image = malloc(win_x*win_y*3);
         glReadPixels(0, 0, win_x, win_y, GL_RGB, GL_UNSIGNED_BYTE, image);
         sglImageLoader::writeImage(
            "dump.sgi", image, win_x, win_y,
            sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE);
         free(image);
         break;
      }

      case '?':
      case '/':
         help_flag = !help_flag;
         cout << "help_flag = " << help_flag << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   if (light_sim)
   {
      sim_time += 0.1;
      light_pos.set(10.0f*(float)sin(sim_time), -20.0f,
                    10.0f*(float)cos(sim_time));
      computeBumpCoords();
      light->setPosition(light_pos);
   }

   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
sglDrawable* createDrawable(sglVec3f *vertex, sglVec3f *normal,
                            sglVec4f *color, unsigned int *indices,
                            unsigned int *lengths)
{
   sglMonoIndexedTriangleStripSet *gset = new sglMonoIndexedTriangleStripSet();
   gset->setCoordList(vertex);
   gset->setNormalList(sglGeoSet::ePER_VERTEX, normal);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setIndexList(indices);
   gset->setNumPrims((pgonsize-1));
   gset->setPrimLengths(lengths);
   return gset;
}

//----------------------------------------------------------------------------
sglGroup *makeTextureObj()
{
   unsigned int *lengths = new unsigned int[(pgonsize-1)];
   unsigned int i, j;
   for (i=0; i<(pgonsize-1); ++i)
      lengths[i] = pgonsize*2;

   vertex = new sglVec3f[pgonsize*pgonsize];
   sglVec3f *normal = new sglVec3f[pgonsize*pgonsize];
   tvert = new sglVec2f[pgonsize*pgonsize];
   tvert_bump = new sglVec2f[pgonsize*pgonsize];
   unsigned int index = 0;
   for (j=0; j<pgonsize; ++j)
      for (i=0; i<pgonsize; ++i)
      {
         vertex[index].set(16.0f*(float)i/(float)(pgonsize-1) - 8.0f,
                           0.0f,
                           16.0f*(float)j/(float)(pgonsize-1) - 8.0f);
         normal[index].set(0.0f, -1.0f, 0.0f);
         tvert[index].set((float)i/(float)(pgonsize-1),
                          (float)j/(float)(pgonsize-1));
         ++index;
      }

   unsigned int *indices = new unsigned int[(pgonsize-1)*pgonsize*2];
   index = 0;
   for (j=0; j<(pgonsize-1); ++j)
      for (i=0; i<pgonsize; ++i)
      {
         indices[index] = j + i*pgonsize;
         ++index;
         indices[index] = j+1 + i*pgonsize;
         ++index;
      }

   sglVec4f *color = new sglVec4f[1];
   color->set(1.0f, 1.0f, 1.0f, 1.0f);


   unsigned int base_width, base_height;
   sglTexture::ExternalFormatEnum base_format;
   sglTexture::TypeEnum base_type;
   void *base_image = sglImageLoader::loadImage(
      base_file, base_width, base_height, base_format, base_type);

   unsigned int bump_width, bump_height;
   sglTexture::ExternalFormatEnum bump_format;
   sglTexture::TypeEnum bump_type;
   void *bump_image = sglImageLoader::loadImage(
      bump_file, bump_width, bump_height, bump_format, bump_type);

   void *tmp_bump = sglImageLoader::changeImageType(
      bump_format, bump_type, bump_width, bump_height,
      sglTexture::eUNSIGNED_BYTE, bump_image);

   void *gray_bump = sglImageLoader::reformatImage(
      bump_format, sglTexture::eUNSIGNED_BYTE, bump_width, bump_height,
      sglTexture::eLUMINANCE, tmp_bump);

   void *inv_bump = malloc(bump_width * bump_height);
   for (index=0; index<bump_height*bump_height; ++index)
   {
      ((unsigned char*)inv_bump)[index] =
         255 - ((unsigned char*)gray_bump)[index];
   }

   sglImageLoader::scaleImageComponent(
      gray_bump, sglTexture::eLUMINANCE, sglTexture::eUNSIGNED_BYTE,
      bump_width, bump_height, 0.5f, 0);
   sglImageLoader::scaleImageComponent(
      inv_bump, sglTexture::eLUMINANCE, sglTexture::eUNSIGNED_BYTE,
      bump_width, bump_height, 0.5f, 0);

   void *base_bump = base_image;
   if (base_type != sglTexture::eUNSIGNED_BYTE)
   {
      base_bump = sglImageLoader::changeImageType(
         base_format, base_type, base_width, base_height,
         sglTexture::eUNSIGNED_BYTE, base_image);
   }
   if (base_format != sglTexture::eRGBA)
   {
      void *tmp = sglImageLoader::reformatImage(
         base_format, sglTexture::eUNSIGNED_BYTE, base_width, base_height,
         sglTexture::eRGBA, base_bump);
      if (base_bump != base_image)
         free(base_bump);
      base_bump = tmp;
   }

   unsigned int base_bump_width, base_bump_height;
   base_bump_width = (base_width > bump_width ? base_width : bump_width);
   base_bump_height = (base_height > bump_height ? base_height : bump_height);

   {
      void *tmp = NULL;
      sglTexture2D::resizeImage(sglTexture::eRGBA, sglTexture::eUNSIGNED_BYTE,
                                base_width, base_height, false,
                                base_bump_width, base_bump_height,
                                base_bump, tmp);
      if (base_bump != base_image)
         free(base_bump);
      base_bump = tmp;
   }

   void *resized_bump = NULL;
   sglTexture2D::resizeImage(sglTexture::eLUMINANCE,
                             sglTexture::eUNSIGNED_BYTE,
                             bump_width, bump_height, false,
                             base_bump_width, base_bump_height,
                             gray_bump, resized_bump);

   for (index=0; index<base_bump_width*base_bump_height; ++index)
   {
      ((unsigned char*)base_bump)[index*4+3] =
         ((unsigned char*)resized_bump)[index];
   }
   free(resized_bump);

   sglTexture2D *base_tex = new sglTexture2D;
   base_tex->setImage(base_image, base_format, base_type,
                      base_width, base_height, false);
   base_tex->setAutoMipmapFlag(true);
   base_tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
   base_tex->setMagFilter(sglTexture::eLINEAR);

   sglTexture2D *bump_tex = new sglTexture2D;
   bump_tex->setImage(gray_bump, sglTexture::eLUMINANCE,
                      sglTexture::eUNSIGNED_BYTE,
                      bump_width, bump_height, false);
   bump_tex->setAutoMipmapFlag(true);
   bump_tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
   bump_tex->setMagFilter(sglTexture::eLINEAR);

   sglTexture2D *inv_bump_tex = new sglTexture2D;
   inv_bump_tex->setImage(inv_bump, sglTexture::eLUMINANCE,
                          sglTexture::eUNSIGNED_BYTE,
                          bump_width, bump_height, false);
   inv_bump_tex->setAutoMipmapFlag(true);
   inv_bump_tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
   inv_bump_tex->setMagFilter(sglTexture::eLINEAR);

   sglTexture2D *base_bump_tex = new sglTexture2D;
   base_bump_tex->setImage(base_bump, sglTexture::eRGBA,
                           sglTexture::eUNSIGNED_BYTE,
                           base_bump_width, base_bump_height, false);
   base_bump_tex->setAutoMipmapFlag(true);
   base_bump_tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
   base_bump_tex->setMagFilter(sglTexture::eLINEAR);

   sglTexEnv *replace_env = new sglTexEnv();
   replace_env->setMode(sglTexEnv::eREPLACE);

   sglTexEnv *add_env = new sglTexEnv();
   add_env->setMode(sglTexEnv::eADD);

   sglTexEnv *combine_env1 = new sglTexEnv();
   combine_env1->setMode(sglTexEnv::eCOMBINE);
   sglTexCombinerParams *combiner_params = new sglTexCombinerParams();
   combiner_params->setRGBFunc(sglTexCombinerParams::RGB::eMODULATE);
   combiner_params->setAlphaFunc(sglTexCombinerParams::Alpha::eREPLACE);
   combine_env1->setCombinerParams(combiner_params);

   sglTexEnv *combine_env2 = new sglTexEnv();
   combine_env2->setMode(sglTexEnv::eCOMBINE);
   combiner_params = new sglTexCombinerParams();
   if (sgl::s_num_texture_units < 3)
      combiner_params->setRGBScale(2.0f);
   combiner_params->setRGBFunc(sglTexCombinerParams::RGB::eREPLACE);
   combiner_params->setRGBParameter(sglTexCombinerParams::eARG0,
                                    sglTexCombinerParams::ePREVIOUS,
                                    sglTexCombinerParams::RGB::eSRC_COLOR);
   combiner_params->setAlphaFunc(sglTexCombinerParams::Alpha::eADD_SIGNED);
   combiner_params->setAlphaParameter(
      sglTexCombinerParams::eARG0,
      sglTexCombinerParams::eTEXTURE,
      sglTexCombinerParams::Alpha::eONE_MINUS_SRC_ALPHA);
   combiner_params->setAlphaParameter(
      sglTexCombinerParams::eARG1,
      sglTexCombinerParams::ePREVIOUS,
      sglTexCombinerParams::Alpha::eSRC_ALPHA);
   combine_env2->setCombinerParams(combiner_params);

   sglTexEnv *combine_env3 = new sglTexEnv();
   combine_env3->setMode(sglTexEnv::eCOMBINE);
   combiner_params = new sglTexCombinerParams();
   combiner_params->setRGBScale(2.0f);
   combiner_params->setRGBFunc(sglTexCombinerParams::RGB::eMODULATE);
   combiner_params->setRGBParameter(sglTexCombinerParams::eARG0,
                                    sglTexCombinerParams::ePREVIOUS,
                                    sglTexCombinerParams::RGB::eSRC_COLOR);
   combiner_params->setRGBParameter(sglTexCombinerParams::eARG1,
                                    sglTexCombinerParams::ePREVIOUS,
                                    sglTexCombinerParams::RGB::eSRC_ALPHA);
   combiner_params->setAlphaFunc(sglTexCombinerParams::Alpha::eREPLACE);
   combiner_params->setAlphaParameter(
      sglTexCombinerParams::eARG0,
      sglTexCombinerParams::ePRIMARY_COLOR,
      sglTexCombinerParams::Alpha::eSRC_ALPHA);
   combine_env3->setCombinerParams(combiner_params);

   sglMaterial *mat = new sglMaterial();
   mat->setSpecular(1.0f, 1.0f, 1.0f);
   mat->setDiffuse(1.0f, 1.0f, 1.0f);
   mat->setAmbient(1.0f, 1.0f, 1.0f);
   mat->setShininess(20);

   switch_node = new sglSwitch();

   sglDrawable *gset = createDrawable(vertex, normal, color, indices, lengths);
   vector<sglStatelet*> statelets;
   statelets.push_back(base_tex);
   statelets.push_back(new sglLighting(true));
   statelets.push_back(mat);
   gset->setState(0, statelets, sglTexCoords((float*)tvert, 2));
   sglGeode *geode = new sglGeode;
   geode->addGeometry(gset);

   switch_node->addChild(geode);

   // *** Create one pass subtree ***
   gset1_1 = createDrawable(vertex, normal, color, indices, lengths);
   statelets1_1.push_back(base_bump_tex);
   statelets1_1.push_back(combine_env1);
   statelets1_1.push_back(new sglMultiTextureStatelet(base_bump_tex, 1));
   statelets1_1.push_back(new sglMultiTextureStatelet(combine_env2, 1));
   statelets1_1.push_back(new sglMultiTextureStatelet(base_bump_tex, 2));
   statelets1_1.push_back(new sglMultiTextureStatelet(combine_env3, 2));
   if (sgl::s_num_texture_units < 3)
      statelets1_1.push_back(new sglBlending(sglBlending::eSRC_ALPHA,
                                             sglBlending::eZERO, true));
   statelets1_1.push_back(new sglLighting(true));
   statelets1_1.push_back(mat);
   geode = new sglGeode;
   geode->addGeometry(gset1_1);

   switch_node->addChild(geode);

   // *** Create two pass subtree ***
   sglGroup *group = new sglOrderedGroup();

   gset2_1 =
      createDrawable(vertex, normal, color, indices, lengths);
   statelets2_1.push_back(bump_tex);
   statelets2_1.push_back(replace_env);
   statelets2_1.push_back(new sglMultiTextureStatelet(inv_bump_tex, 1));
   statelets2_1.push_back(new sglMultiTextureStatelet(add_env, 1));
   geode = new sglGeode;
   geode->addGeometry(gset2_1);
   group->addChild(geode);

   gset2_2 = createDrawable(vertex, normal, color, indices, lengths);
   vector<sglStatelet*> statelets2_2;
   statelets2_2.push_back(base_tex);
   statelets2_2.push_back(new sglBlending(sglBlending::eDST_COLOR,
                                          sglBlending::eSRC_COLOR, true));
   statelets2_2.push_back(new sglDepthTest(sglDepthTest::eLEQUAL));
   statelets2_2.push_back(new sglLighting(true));
   statelets2_2.push_back(new sglDepthRange(0., 1.0 - 1.0e-4));
   statelets2_2.push_back(mat);
   gset2_2->setState(0, statelets2_2, sglTexCoords((float*)tvert, 2));
   geode = new sglGeode;
   geode->addGeometry(gset2_2);
   group->addChild(geode);

   switch_node->addChild(group);

   // *** Create three pass subtree ***
   group = new sglOrderedGroup();

   gset3_1 =
      createDrawable(vertex, normal, color, indices, lengths);
   vector<sglStatelet*> statelets3_1;
   statelets3_1.push_back(bump_tex);
   statelets3_1.push_back(replace_env);
   gset3_1->setState(0, statelets3_1, sglTexCoords((float*)tvert, 2));
   geode = new sglGeode;
   geode->addGeometry(gset3_1);
   group->addChild(geode);

   gset3_2 = createDrawable(vertex, normal, color, indices, lengths);
   statelets3_2.push_back(inv_bump_tex);
   statelets3_2.push_back(replace_env);
   statelets3_2.push_back(new sglBlending(sglBlending::eONE,
                                       sglBlending::eONE, true));
   statelets3_2.push_back(new sglDepthTest(sglDepthTest::eLEQUAL));
   geode = new sglGeode;
   geode->addGeometry(gset3_2);
   group->addChild(geode);

   gset3_3 = createDrawable(vertex, normal, color, indices, lengths);
   vector<sglStatelet*> statelets3_3;
   statelets3_3.push_back(base_tex);
   statelets3_3.push_back(new sglBlending(sglBlending::eDST_COLOR,
                                       sglBlending::eSRC_COLOR, true));
   statelets3_3.push_back(new sglDepthTest(sglDepthTest::eLEQUAL));
   statelets3_3.push_back(new sglLighting(true));
   statelets3_3.push_back(new sglDepthRange(0., 1.0 - 1.0e-4));
   statelets3_3.push_back(mat);
   gset3_3->setState(0, statelets3_3, sglTexCoords((float*)tvert, 2));
   geode = new sglGeode;
   geode->addGeometry(gset3_3);
   group->addChild(geode);

   switch_node->addChild(group);

   computeBumpCoords();
   switch_node->setVal(3);

   return switch_node;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   sglVec4f no_light(0.0f, 0.0f, 0.0f, 1.0f);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, no_light.getPtr());
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);

   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   light = new sglPositionalLight();
   light->setPosition(light_pos);
   light->setAmbient(0, 0, 0, 1);
   light->setDiffuse(.4f, .4f, .4f, 1);
   light->setSpecular(.8f, .8f, .8f, 1);
   light->setBound(NULL, sglBound::eDYNAMIC);

   root->addChild(light);
   root->addChild(makeTextureObj());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setClearColor(0.4f, 0.4f, 0.4f, 1.0f);
   view->setViewport(0, 0, win_x, win_y);
   view->setStateMask(state_mask);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(&pfs);
   override_statelets.push_back(&pbs);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
void usage(char *name)
{
   cout << "Usage: " << name << " <base texture image> <bump texture image>"
        << endl;
   exit(1);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   char *base = "smile.bmp";
   char *bump = "bump.bmp";

   if (argc == 3)
   {
      base = argv[1];
      bump = argv[2];
   }
   else if (argc != 1)
   {
      usage(argv[0]);
   }

   if (!sglDataLoader::findFile(base, NULL, base_file))
      usage(argv[0]);
   if (!sglDataLoader::findFile(bump, NULL, bump_file))
      usage(argv[0]);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testBumpmap");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
