/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testBillboard.cpp
 *  Created: 6 March 2000
 *  Summary: GLUT-based sglBillboard testbed
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglGroup.hpp>
#include <sglScene.hpp>
#include <sglDefaultStateNode.hpp>
#include <sglDirectionalLight.hpp>
#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglTriangleStripSet.hpp>
#include <sglLineStripSet.hpp>

#include <sglFog.hpp>
#include <sglBitmapFont.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

sglFog *fog;

deque<const sglStatelet*>  override_statelets;

unsigned int win_x = 640, win_y = 480;

int test_index = 1;
char *test_desc[] = {"Fog: disabled",
                     "Fog: linear",
                     "Fog: exponential",
                     "Fog: exponential squared"};

//----------------------------------------------------------------------------
void outputDescription()
{
   glDisable(GL_DEPTH_TEST);
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, win_x, 0, win_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glEnable(GL_BLEND);
   glColor3f(1.0f, 1.0f, 0.5f);
   glRasterPos3i(1, win_y - 15, 0);
   sglBitmapFont::getDefaultFont()->rasterString(test_desc[test_index]);
   glDisable(GL_BLEND);

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   outputDescription();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;
   double fov_y = M_PI/(3.0*aspect);

   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'm':
      {
         sglFog::ModeEnum mode = fog->getMode();
         switch(mode)
         {
            case sglFog::eOFF:
               fog->setMode(sglFog::eLINEAR);
               test_index = 1;
               break;
            case sglFog::eLINEAR:
               fog->setMode(sglFog::eEXP);
               test_index = 2;
               break;
            case sglFog::eEXP:
               fog->setMode(sglFog::eEXP2);
               test_index = 3;
               break;
            case sglFog::eEXP2:
               fog->setMode(sglFog::eOFF);
               test_index = 0;
               break;
         }
      }
      break;

      case 'e':
      {
         sglFog::DistanceEqEnum deq = fog->getDistanceEquation();
         if( deq == sglFog::eEYE_PLANE_ABSOLUTE )
            fog->setDistanceEquation( sglFog::eEYE_RADIAL );
         else
            fog->setDistanceEquation( sglFog::eEYE_PLANE_ABSOLUTE );
      }
      break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "m - cycle through the fog modes" << endl;
         cerr << "e - modify fog equation (NV extension)" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
sglGeode *makeTriangleSet()
{
   sglVec3f *vertex = new sglVec3f[8];
   vertex[0].set(  0.f, 10.f,  0.f);
   vertex[1].set(  0.f,-10.f,  0.f);
   vertex[2].set( 20.f, 10.f,  0.f);
   vertex[3].set( 20.f,-10.f,  0.f);
   vertex[4].set( 40.f, 10.f,  0.f);
   vertex[5].set( 40.f,-10.f,  0.f);
   vertex[6].set( 60.f, 10.f,  0.f);
   vertex[7].set( 60.f,-10.f,  0.f);

   sglVec3f *normal = new sglVec3f[8];
   normal[0].set(0.f, 0.f, 1.0f);
   normal[1].set(0.f, 0.f, 1.0f);
   normal[2].set(0.f, 0.f, 1.0f);
   normal[3].set(0.f, 0.f, 1.0f);
   normal[4].set(0.f, 0.f, 1.0f);
   normal[5].set(0.f, 0.f, 1.0f);
   normal[6].set(0.f, 0.f, 1.0f);
   normal[7].set(0.f, 0.f, 1.0f);

   sglVec4f *color = new sglVec4f[8];
   color[0].set(1.0f, 0.0f, 0.0f, 1.0f);
   color[1].set(1.0f, 0.0f, 0.0f, 1.0f);
   color[2].set(1.0f, 1.0f, 0.0f, 1.0f);
   color[3].set(1.0f, 1.0f, 0.0f, 1.0f);
   color[4].set(0.0f, 1.0f, 1.0f, 1.0f);
   color[5].set(0.0f, 1.0f, 1.0f, 1.0f);
   color[6].set(0.0f, 0.0f, 1.0f, 1.0f);
   color[7].set(0.0f, 0.0f, 1.0f, 1.0f);

   unsigned int *length = new unsigned int[1];
   length[0] = 8;

   sglTriangleStripSet *gset = new sglTriangleStripSet;
   gset->setNumPrims(1);
   gset->setPrimLengths(length);
   gset->setCoordList(vertex);
   gset->setNormalList(sglGeoSet::ePER_VERTEX, normal);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);

   vector<sglStatelet*> state;
   state.push_back(new sglShadeModel(sglShadeModel::eSMOOTH));

   gset->setState(0, state);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);
   return geode;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   root = new sglScene;

   fog = new sglFog;
   fog->setMode(sglFog::eLINEAR);
   fog->setColor(0.f, 0.f, 0.f, 1.0f);
   fog->setExtent(0.f, 100.f);
   fog->setDensity(0.025f);

   vector<sglStatelet*> state;
   state.push_back(fog);

   sglDefaultStateNode *def = new sglDefaultStateNode;
   def->setStatelets(state);
   root->addChild(def);

   def->addChild(makeTriangleSet());

   // ================================================================

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   double fov_y = (double)M_PI/(3.0f*aspect);

   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);
   //view_platform->setEnableZup(true);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setAzimuth((float)(-M_PI*0.5));
   view_trans->setElevation(-0.5f);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   view_trans->addChild(new sglDirectionalLight());

   root->addChild(view_trans);

   sglAxes *axes = new sglAxes(2.0f);
   vector<sglStatelet*> state1;
   state1.push_back(new sglLineWidth(2.0f));
   axes->setState(0, state1);
   sglGeode *geode = new sglGeode;
   geode->addGeometry(axes);
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglPolyFrontStyle(sglPolyStyle::eFILLED));
   override_statelets.push_back(new sglPolyBackStyle(sglPolyStyle::eFILLED));
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(640, 480);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("sglfly");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
