/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testGeoSets.cpp
 *  Created: 2 January 2000
 *  Summary: test all of the geoset subtypes
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglBitmapFont.hpp>

#include <sglGroup.hpp>
#include <sglScene.hpp>
#include <sglSwitch.hpp>
#include <sglTextSet.hpp>
#include <sglDefaultStateNode.hpp>

#include <sglGeode.hpp>

#include<sglPointSet.hpp>
#include<sglLineSet.hpp>
#include<sglLineStripSet.hpp>
#include<sglTriangleFanSet.hpp>
#include<sglTriangleSet.hpp>
#include<sglTriangleStripSet.hpp>
#include<sglQuadSet.hpp>
#include<sglQuadStripSet.hpp>
#include<sglPolygonSet.hpp>

#include<sglIndexedPointSet.hpp>
#include<sglIndexedLineSet.hpp>
#include<sglIndexedLineStripSet.hpp>
#include<sglIndexedTriangleFanSet.hpp>
#include<sglIndexedTriangleSet.hpp>
#include<sglIndexedTriangleStripSet.hpp>
#include<sglIndexedQuadSet.hpp>
#include<sglIndexedQuadStripSet.hpp>
#include<sglIndexedPolygonSet.hpp>

#include<sglMonoIndexedPointSet.hpp>
#include<sglMonoIndexedLineSet.hpp>
#include<sglMonoIndexedLineStripSet.hpp>
#include<sglMonoIndexedTriangleFanSet.hpp>
#include<sglMonoIndexedTriangleSet.hpp>
#include<sglMonoIndexedTriangleStripSet.hpp>
#include<sglMonoIndexedQuadSet.hpp>
#include<sglMonoIndexedQuadStripSet.hpp>
#include<sglMonoIndexedPolygonSet.hpp>

#include<sglBezierCurveSet.hpp>
#include<sglBezierSurfaceSet.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;
sglAntiAliasLine aa_line;
sglAntiAliasPoint aa_point;
sglAntiAliasPolygon aa_poly;
sglPolyFrontStyle pfs;
sglPolyBackStyle pbs;

bool help_flag = false;

sglSwitch   *switch_node;
int          switch_val = 0;

unsigned int win_x = 640, win_y = 480;

float text_color[4] = {1.0f, 1.0f, 1.0f, 1.0f};

//----------------------------------------------------------------------------
void changeTextColor(sglNode *node,
                     float red, float green, float blue, float alpha)
{
   sglGroup *group = dynamic_cast<sglGroup*>(node);
   if (group)
   {
      for (unsigned int ii = 0; ii < group->getNumChildren(); ++ii)
      {
         changeTextColor(group->getChild(ii), red, green, blue, alpha);
      }
      return;
   }
   else
   {
      sglGeode *geode = dynamic_cast<sglGeode*>(node);
      if (geode)
      {
         for (unsigned int ii = 0; ii < geode->getNumGeometries(); ++ii)
         {
            sglTextSet *text =
               dynamic_cast<sglTextSet*>(geode->getGeometry(ii));
            if (text)
            {
               text->setColor(red, green, blue, alpha);
            }
         }
      }
   }
}

//----------------------------------------------------------------------------
void outputHelp()
{
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glEnable(GL_BLEND);

   glRasterPos3i(10,160,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "/? - toggle help");
   glRasterPos3i(10,145,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "f - enable frame rate output");
   glRasterPos3i(10,130,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "d - toggle between SP/DP traversal");
   glRasterPos3i(10,115,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "$ - toggle stats");
   glRasterPos3i(10,100,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "o - output scene graph");
   glRasterPos3i(10,85,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "up/down arrow - cycle through primitives");
   glRasterPos3i(10,70,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "a - switch between   point AA override modes");
   glRasterPos3i(10,55,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "A - switch between    line AA override modes");
   glRasterPos3i(10,40,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "z - switch between polygon AA override modes");
   glRasterPos3i(10,25,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "p - switch between point, line, fill polygon modes");
   glRasterPos3i(10,10,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "c - toggle text color (white vs red)");
   glDisable(GL_BLEND);
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   if (help_flag)
   {
      outputHelp();
   }

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'p':
         if (pfs.getStyle() == sglPolyStyle::eFILLED)
         {
            pfs.setStyle(sglPolyStyle::ePOINT);
            pbs.setStyle(sglPolyStyle::ePOINT);
         }
         else if (pfs.getStyle() == sglPolyStyle::ePOINT)
         {
            pfs.setStyle(sglPolyStyle::eLINE);
            pbs.setStyle(sglPolyStyle::eLINE);
         }
         else if (pfs.getStyle() == sglPolyStyle::eLINE)
         {
            pfs.setStyle(sglPolyStyle::eFILLED);
            pbs.setStyle(sglPolyStyle::eFILLED);
         }

         cout << "polygon style = " << (int) pfs.getStyle() << endl;
         break;

      case 'c':
         if (text_color[1] == 1.0f && text_color[3] == 1.0f)
         {
            text_color[1] = text_color[2] = 1.0f;
            text_color[3] = 0.5f;
         }
         else if (text_color[1] == 1.0f)
         {
            text_color[1] = text_color[2] = 0;
            text_color[3] = 1.0f;
         }
         else
         {
            text_color[1] = text_color[2] = 1.0f;
            text_color[3] = 1.0f;
         }
         changeTextColor(root, text_color[0], text_color[1],
                         text_color[2], text_color[3]);
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         cout << "traversal type: " << (view->getDPTraversalFlag() ?
                                        "double" : "single") << endl;
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '?':
      case '/':
         help_flag = !help_flag;
         cout << "help_flag = " << help_flag << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         switch_val++;
         if ((unsigned int)switch_val >= switch_node->getNumChildren())
         {
            switch_val = 0;
         }
         switch_node->setVal(switch_val);
         break;

      case GLUT_KEY_DOWN:
         switch_val--;
         if (switch_val < 0)
         {
            switch_val = switch_node->getNumChildren() - 1;
         }
         switch_node->setVal(switch_val);
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
sglGroup* makePointSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("PointSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   for (int i=0; i<4; i++)
   {
      sglPointSet *gset = new sglPointSet();

      sglVec3f *vertex = new sglVec3f[4];

      for (int k=0; k<4; k++)
      {
         vertex[k].set(i*4.f-6.0f, 0.0f, k*4.f-6.0f);
      }
      gset->setCoordList(vertex);
      gset->setNumPrims(4);

      vector<sglStatelet*> state;
      state.push_back(new sglPointSize(2.0f*i + 1.0f));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeLineSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("LineSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   short factor = 1;

   for (int i=0; i<4; i++)
   {
      sglLineSet *gset = new sglLineSet();

      factor *= 2;

      sglVec3f *vertex = new sglVec3f[2];
      vertex[0].set(i*4.f-6.0f, 0.0f, -6.0f);
      vertex[1].set(i*4.f-6.0f, 0.0f,  6.0f);

      gset->setCoordList(vertex);
      gset->setNumPrims(1);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(7.0f - 2.0f*i));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeLineStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("LineStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   short factor = 1;

   for (int i=0; i<4; i++)
   {
      sglLineStripSet *gset = new sglLineStripSet();

      factor *= 2;

      sglVec3f *vertex = new sglVec3f[4];
      unsigned int *length = new unsigned int[1];
      length[0] = 4;

      for (int j=0; j<4; j++)
      {
         vertex[j].set((float)(10.0*cos((double)j)),
                       i*4.0f, (float)(10.0*sin((double)j)));
      }

      gset->setCoordList(vertex);
      gset->setNumPrims(1);
      gset->setPrimLengths(length);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(2.0f*i + 1.0f));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeBezierCurveSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("BezierCurveSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   short factor = 1;

   for (int i=0; i<4; i++)
   {
      sglBezierCurveSet *gset = new sglBezierCurveSet();

      factor *= 2;

      sglVec3f *vertex = new sglVec3f[4];

      vertex[0].set(-10.0f, i*4.0f, -10.0f);
      vertex[1].set(-08.0f, i*4.0f+7,  10.0f);
      vertex[2].set( 08.0f, i*4.0f, -10.0f);
      vertex[3].set( 10.0f, i*4.0f,  10.0f);

      gset->setCoordList(vertex);
      gset->setCoordOrder(4);
      gset->setNumPrims(1);

      gset->setNumSegments(100);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(2.0f*i + 1.0f));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeBezierSurfaceSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("BezierSurfaceSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglBezierSurfaceSet *gset = new sglBezierSurfaceSet();

   sglVec3f *vertex = new sglVec3f[12];
   sglVec4f *color  = new sglVec4f[1];

   color[0].set(0.f, 1.f, 1.f, 1.f);

   for (int j=0; j<3; j++)
   {
      vertex[4*j  ].set(6.f*j - 8.f, 3.f, 4.0f);
      vertex[4*j+1].set(6.f*j - 8.f, 0.f, -4.0f);
      vertex[4*j+2].set(6.f*j - 4.f, 0.f, 4.0f);
      vertex[4*j+3].set(6.f*j - 4.f, 0.f, -4.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordOrder(2, 2);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorOrder(1, 1);
   gset->setNumPrims(3);

   gset->setNumSegments(20, 20);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleFanSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleFanSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglTriangleFanSet *gset = new sglTriangleFanSet();

   sglVec3f *vertex = new sglVec3f[5];
   sglVec4f *color  = new sglVec4f[5];
   unsigned int *length = new unsigned int[1];
   length[0] = 5;

   vertex[0].set(0.f, 0.f, 3.f);
   color[0].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (int j=0; j<4; j++)
   {
      vertex[j+1].set((float)(10.0*cos((double)j)),
                      0.f, (float)(10.0*sin((double)j)));
      color[j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglTriangleSet *gset = new sglTriangleSet();

   sglVec3f *vertex = new sglVec3f[9];
   sglVec4f *color  = new sglVec4f[3];

   for (int j=0; j<3; j++)
   {
      vertex[j*3 + 0].set(-6.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 1].set(-2.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 2].set(-4.0f + j*6.0f, 0.f, 6.0f);

      color[j].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglTriangleStripSet *gset = new sglTriangleStripSet();

   sglVec3f *vertex = new sglVec3f[8];
   sglVec4f *color  = new sglVec4f[8];
   unsigned int *length = new unsigned int[1];
   length[0] = 8;

   for (int j=0; j<4; j++)
   {
      vertex[2*j].set(3.f*j - 4.5f, 0.f, 4.0f);
      vertex[2*j+1].set(3.f*j - 4.5f, 0.f, -4.0f);
      color[2*j].set((float)j/4.0f, 0.8f, 1.0f - (float)j/4.0f, 1.0f);
      color[2*j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeQuadSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("QuadSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglQuadSet *gset = new sglQuadSet();

   sglVec3f *vertex = new sglVec3f[12];
   sglVec4f *color  = new sglVec4f[1];

   color[0].set(0.f, 1.f, 1.f, 1.f);

   for (int j=0; j<3; j++)
   {
      vertex[4*j  ].set(6.f*j - 8.f, 0.f, 4.0f);
      vertex[4*j+1].set(6.f*j - 8.f, 0.f, -4.0f);
      vertex[4*j+2].set(6.f*j - 4.f, 0.f, -4.0f);
      vertex[4*j+3].set(6.f*j - 4.f, 0.f, 4.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeQuadStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("QuadStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglQuadStripSet *gset = new sglQuadStripSet();

   sglVec3f *vertex = new sglVec3f[10];
   sglVec4f *color  = new sglVec4f[10];
   unsigned int *length = new unsigned int[1];
   length[0] = 10;

   for (int j=0; j<5; j++)
   {
      vertex[2*j].set(3.f*j - 6.f, 0.f, 4.0f);
      vertex[2*j+1].set(3.f*j - 6.f, 0.f, -4.0f);
      color[2*j].set((float)j/5.0f, 0.8f, 1.0f - (float)j/5.0f, 1.0f);
      color[2*j+1].set((float)j/5.0f, 1.0f - (float)j/5.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makePolygonSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("PolygonSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglPolygonSet *gset = new sglPolygonSet();

   sglVec3f *vertex = new sglVec3f[18];

   sglVec4f *color  = new sglVec4f[1];
   color[0].set(0.f, 1.f, 1.f, 1.f);

   unsigned int *length = new unsigned int[3];
   length[0] = 5;
   length[1] = 6;
   length[2] = 7;

   int index = 0;
   for (int j=0; j<3; j++)
   {
      for (unsigned int i=0; i<length[j]; i++)
      {
         vertex[index++].set(
            (8.f*j-8.f) + (float)(3.0*cos(2.0*M_PI*i/(double)length[j])),
            0.0f,
            (float)(3.0*sin(2.0*M_PI*i/(double)length[j])));
      }
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setPrimLengths(length);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}


//----------------------------------------------------------------------------
sglGroup* makeIndexedPointSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedPointSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   for (int i=0; i<4; i++)
   {
      sglIndexedPointSet *gset = new sglIndexedPointSet();

      sglVec3f *vertex = new sglVec3f[4];
      unsigned int *vindex = new unsigned int[4];

      for (int k=0; k<4; k++)
      {
         vindex[k] = k;
         vertex[k].set(i*4.f-6.0f, 0.0f, k*4.f-6.0f);
      }
      gset->setCoordIndexList(vindex);
      gset->setCoordList(vertex);
      gset->setNumPrims(4);

      vector<sglStatelet*> state;
      state.push_back(new sglPointSize(2.0f*i + 1.0f));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedLineSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedLineSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   short factor = 1;

   for (int i=0; i<4; i++)
   {
      sglIndexedLineSet *gset = new sglIndexedLineSet();

      factor *= 2;

      sglVec3f *vertex = new sglVec3f[2];
      vertex[0].set(i*4.f-6.0f, 0.0f, -6.0f);
      vertex[1].set(i*4.f-6.0f, 0.0f,  6.0f);

      unsigned int *vindex = new unsigned int[2];
      vindex[0] = 0;
      vindex[1] = 1;

      gset->setCoordList(vertex);
      gset->setCoordIndexList(vindex);
      gset->setNumPrims(1);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(2.0f*i + 1.0f));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedLineStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedLineStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   short factor = 1;

   for (int i=0; i<4; i++)
   {
      sglIndexedLineStripSet *gset = new sglIndexedLineStripSet();

      factor *= 2;

      sglVec3f *vertex = new sglVec3f[4];
      unsigned int *vindex = new unsigned int[4];
      unsigned int *length = new unsigned int[1];
      length[0] = 4;

      for (int j=0; j<4; j++)
      {
         vindex[j] = j;
         vertex[j].set((float)(10.0*cos((double)j)),
                       i*4.0f, (float)(10.0*sin((double)j)));
      }

      gset->setCoordList(vertex);
      gset->setCoordIndexList(vindex);
      gset->setNumPrims(1);
      gset->setPrimLengths(length);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(2.0f*i + 1.0f));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedTriangleFanSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedTriangleFanSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglIndexedTriangleFanSet *gset = new sglIndexedTriangleFanSet();

   sglVec3f *vertex = new sglVec3f[5];
   sglVec4f *color  = new sglVec4f[5];
   unsigned int *vindex = new unsigned int[5];
   unsigned int *length = new unsigned int[1];
   length[0] = 5;

   vindex[0] = 0;
   vertex[0].set(0.f, 0.f, 3.f);
   color[0].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (int j=0; j<4; j++)
   {
      vindex[j+1] = j+1;
      vertex[j+1].set((float)(10.0*cos((double)j)),
                      0.f, (float)(10.0*sin((double)j)));
      color[j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorIndexList(vindex);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedTriangleSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedTriangleSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglIndexedTriangleSet *gset = new sglIndexedTriangleSet();

   sglVec3f *vertex = new sglVec3f[9];
   unsigned int *vindex = new unsigned int[9];

   sglVec4f *color  = new sglVec4f[3];
   unsigned int *cindex = new unsigned int[3];

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      vertex[index++].set(-6.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      vertex[index++].set(-2.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      vertex[index++].set(-4.0f + j*6.0f, 0.f, 6.0f);

      cindex[j] = j;
      color[j].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setColorIndexList(cindex);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedTriangleStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedTriangleStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglIndexedTriangleStripSet *gset = new sglIndexedTriangleStripSet();

   sglVec3f *vertex = new sglVec3f[8];
   sglVec4f *color  = new sglVec4f[8];
   unsigned int *vindex = new unsigned int[8];
   unsigned int *length = new unsigned int[1];
   length[0] = 8;

   for (int j=0; j<4; j++)
   {
      vindex[2*j] = 2*j;
      vindex[2*j+1] = 2*j + 1;
      vertex[2*j].set(3.f*j - 4.5f, 0.f, 4.0f);
      vertex[2*j+1].set(3.f*j - 4.5f, 0.f, -4.0f);
      color[2*j].set((float)j/4.0f, 0.8f, 1.0f - (float)j/4.0f, 1.0f);
      color[2*j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorIndexList(vindex);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedQuadSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedQuadSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglIndexedQuadSet *gset = new sglIndexedQuadSet();

   sglVec3f *vertex = new sglVec3f[12];
   unsigned int *vindex = new unsigned int[12];
   sglVec4f *color  = new sglVec4f[1];
   unsigned int *cindex = new unsigned int[1];

   cindex[0] = 0;
   color[0].set(0.f, 1.f, 1.f, 1.f);

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      vertex[index++].set(6.f*j - 8.f, 0.f, 4.0f);
      vindex[index] = index;
      vertex[index++].set(6.f*j - 8.f, 0.f, -4.0f);
      vindex[index] = index;
      vertex[index++].set(6.f*j - 4.f, 0.f, -4.0f);
      vindex[index] = index;
      vertex[index++].set(6.f*j - 4.f, 0.f, 4.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorIndexList(cindex);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedQuadStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedQuadStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglIndexedQuadStripSet *gset = new sglIndexedQuadStripSet();

   sglVec3f *vertex = new sglVec3f[10];
   sglVec4f *color  = new sglVec4f[10];
   unsigned int *vindex = new unsigned int[10];
   unsigned int *length = new unsigned int[1];
   length[0] = 10;

   unsigned int index = 0;
   for (int j=0; j<5; j++)
   {
      vertex[index].set(3.f*j - 6.f, 0.f, 4.0f);
      color[index].set((float)j/5.0f, 0.8f, 1.0f - (float)j/5.0f, 1.0f);
      vindex[index] = index;
      index++;

      vertex[index].set(3.f*j - 6.f, 0.f, -4.0f);
      color[index].set((float)j/5.0f, 1.0f - (float)j/5.0f, 0.5f, 1.0f);
      vindex[index] = index;
      index++;
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorIndexList(vindex);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedPolygonSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedPolygonSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglIndexedPolygonSet *gset = new sglIndexedPolygonSet();

   sglVec3f *vertex = new sglVec3f[18];
   unsigned int *vindex = new unsigned int[18];

   sglVec4f *color  = new sglVec4f[1];
   unsigned int *cindex = new unsigned int[1];
   cindex[0] = 0;
   color[0].set(0.f, 1.f, 1.f, 1.f);

   unsigned int *length = new unsigned int[3];
   length[0] = 5;
   length[1] = 6;
   length[2] = 7;

   int index = 0;
   for (int j=0; j<3; j++)
   {
      for (unsigned int i=0; i<length[j]; i++)
      {
         vindex[index] = index;
         vertex[index++].set(
            (8.f*j-8.f) + (float)(3.0*cos(2.0*M_PI*i/(double)length[j])),
            0.0f,
            (float)(3.0*sin(2.0*M_PI*i/(double)length[j])));
      }
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorIndexList(cindex);
   gset->setPrimLengths(length);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}


//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedPointSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedPointSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   for (int i=0; i<4; i++)
   {
      sglMonoIndexedPointSet *gset = new sglMonoIndexedPointSet();

      sglVec3f *vertex = new sglVec3f[4];
      unsigned int *vindex = new unsigned int[4];

      for (int k=0; k<4; k++)
      {
         vindex[k] = k;
         vertex[k].set(i*4.f-6.0f, 0.0f, k*4.f-6.0f);
      }
      gset->setIndexList(vindex);
      gset->setCoordList(vertex);
      gset->setNumPrims(4);

      vector<sglStatelet*> state;
      state.push_back(new sglPointSize(2.0f*i + 1.0f));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedLineSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedLineSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   short factor = 1;

   for (int i=0; i<4; i++)
   {
      sglMonoIndexedLineSet *gset = new sglMonoIndexedLineSet();

      factor *= 2;

      sglVec3f *vertex = new sglVec3f[2];
      vertex[0].set(i*4.f-6.0f, 0.0f, -6.0f);
      vertex[1].set(i*4.f-6.0f, 0.0f,  6.0f);

      unsigned int *vindex = new unsigned int[2];
      vindex[0] = 0;
      vindex[1] = 1;

      gset->setCoordList(vertex);
      gset->setIndexList(vindex);
      gset->setNumPrims(1);
      gset->setDisablePrefetch(true);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(2.0f*i + 1.0f));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedLineStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedLineStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   short factor = 1;

   for (int i=0; i<4; i++)
   {
      sglMonoIndexedLineStripSet *gset = new sglMonoIndexedLineStripSet();

      factor *= 2;

      sglVec3f *vertex = new sglVec3f[4];
      unsigned int *vindex = new unsigned int[4];
      unsigned int *length = new unsigned int[1];
      length[0] = 4;

      for (int j=0; j<4; j++)
      {
         vindex[j] = j;
         vertex[j].set((float)(10.0*cos((double)j)),
                       i*4.0f, (float)(10.0*sin((double)j)));
      }

      gset->setCoordList(vertex);
      gset->setIndexList(vindex);
      gset->setNumPrims(1);
      gset->setPrimLengths(length);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(2.0f*i + 1.0f));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedTriangleFanSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedTriangleFanSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglMonoIndexedTriangleFanSet *gset = new sglMonoIndexedTriangleFanSet();

   sglVec3f *vertex = new sglVec3f[5];
   sglVec4f *color  = new sglVec4f[5];
   unsigned int *vindex = new unsigned int[5];
   unsigned int *length = new unsigned int[1];
   length[0] = 5;

   vindex[0] = 0;
   vertex[0].set(0.f, 0.f, 3.f);
   color[0].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (int j=0; j<4; j++)
   {
      vindex[j+1] = j+1;
      vertex[j+1].set((float)(10.0*cos((double)j)),
                      0.f, (float)(10.0*sin((double)j)));
      color[j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedTriangleSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedTriangleSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglMonoIndexedTriangleSet *gset = new sglMonoIndexedTriangleSet();

   sglVec3f *vertex = new sglVec3f[9];
   sglVec4f *color  = new sglVec4f[9];
   unsigned int *vindex = new unsigned int[9];

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      color[index].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
      vertex[index++].set(-6.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      color[index].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
      vertex[index++].set(-2.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      color[index].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
      vertex[index++].set(-4.0f + j*6.0f, 0.f, 6.0f);
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedTriangleStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedTriangleStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglMonoIndexedTriangleStripSet *gset = new sglMonoIndexedTriangleStripSet();

   sglVec3f *vertex = new sglVec3f[8];
   sglVec4f *color  = new sglVec4f[8];
   unsigned int *vindex = new unsigned int[8];
   unsigned int *length = new unsigned int[1];
   length[0] = 8;

   unsigned int index = 0;
   for (int j=0; j<4; j++)
   {
      vindex[index] = index;
      vertex[index].set(3.f*j - 4.5f, 0.f, 4.0f);
      color[index].set((float)j/4.0f, 0.8f, 1.0f - (float)j/4.0f, 1.0f);
      index++;

      vindex[index] = 2*j + 1;
      vertex[index].set(3.f*j - 4.5f, 0.f, -4.0f);
      color[index].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
      index++;
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedQuadSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedQuadSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglMonoIndexedQuadSet *gset = new sglMonoIndexedQuadSet();

   sglVec3f *vertex = new sglVec3f[12];
   unsigned int *vindex = new unsigned int[12];
   sglVec4f *color  = new sglVec4f[12];

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 8.f, 0.f, 4.0f);
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 8.f, 0.f, -4.0f);
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 4.f, 0.f, -4.0f);
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 4.f, 0.f, 4.0f);
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);  // bind enum should be ignored
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedQuadStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedQuadStripSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglMonoIndexedQuadStripSet *gset = new sglMonoIndexedQuadStripSet();

   sglVec3f *vertex = new sglVec3f[10];
   sglVec4f *color  = new sglVec4f[10];
   unsigned int *vindex = new unsigned int[10];
   unsigned int *length = new unsigned int[1];
   length[0] = 10;

   unsigned int index = 0;
   for (int j=0; j<5; j++)
   {
      vertex[index].set(3.f*j - 6.f, 0.f, 4.0f);
      color[index].set((float)j/5.0f, 0.8f, 1.0f - (float)j/5.0f, 1.0f);
      vindex[index] = index;
      index++;

      vertex[index].set(3.f*j - 6.f, 0.f, -4.0f);
      color[index].set((float)j/5.0f, 1.0f - (float)j/5.0f, 0.5f, 1.0f);
      vindex[index] = index;
      index++;
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedPolygonSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedPolygonSet");
   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);
   group->addChild(geode);

   sglMonoIndexedPolygonSet *gset = new sglMonoIndexedPolygonSet();

   sglVec3f *vertex = new sglVec3f[18];
   sglVec4f *color  = new sglVec4f[18];
   unsigned int *vindex = new unsigned int[18];

   unsigned int *length = new unsigned int[3];
   length[0] = 5;
   length[1] = 6;
   length[2] = 7;

   int index = 0;
   for (int j=0; j<3; j++)
   {
      for (unsigned int i=0; i<length[j]; i++)
      {
         vindex[index] = index;
         color[index].set(0.f, 1.f, 1.f, 1.f);
         vertex[index++].set(
            (8.f*j-8.f) + (float)(3.0*cos(2.0*M_PI*i/(double)length[j])),
            0.0f,
            (float)(3.0*sin(2.0*M_PI*i/(double)length[j])));
      }
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setPrimLengths(length);
   gset->setNumPrims(3);

   geode->addGeometry(gset);

   return group;
}


//----------------------------------------------------------------------------
void myinit (void)
{
   glDepthFunc(GL_LESS);
   glEnable(GL_DEPTH_TEST);

   glShadeModel(GL_FLAT);
   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);

   glDisable(GL_CULL_FACE);

   glDisable (GL_LIGHTING);
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   root = new sglScene();
   switch_node = new sglSwitch();
   switch_node->setVal(switch_val);

   switch_node->addChild(makePointSet());
   switch_node->addChild(makeIndexedPointSet());
   switch_node->addChild(makeMonoIndexedPointSet());

   switch_node->addChild(makeLineSet());
   switch_node->addChild(makeIndexedLineSet());
   switch_node->addChild(makeMonoIndexedLineSet());

   switch_node->addChild(makeLineStripSet());
   switch_node->addChild(makeIndexedLineStripSet());
   switch_node->addChild(makeMonoIndexedLineStripSet());

   switch_node->addChild(makeBezierCurveSet());
   switch_node->addChild(makeBezierSurfaceSet());

   switch_node->addChild(makeTriangleSet());
   switch_node->addChild(makeIndexedTriangleSet());
   switch_node->addChild(makeMonoIndexedTriangleSet());

   switch_node->addChild(makeTriangleStripSet());
   switch_node->addChild(makeIndexedTriangleStripSet());
   switch_node->addChild(makeMonoIndexedTriangleStripSet());

   switch_node->addChild(makeTriangleFanSet());
   switch_node->addChild(makeIndexedTriangleFanSet());
   switch_node->addChild(makeMonoIndexedTriangleFanSet());

   switch_node->addChild(makeQuadSet());
   switch_node->addChild(makeIndexedQuadSet());
   switch_node->addChild(makeMonoIndexedQuadSet());

   switch_node->addChild(makeQuadStripSet());
   switch_node->addChild(makeIndexedQuadStripSet());
   switch_node->addChild(makeMonoIndexedQuadStripSet());

   switch_node->addChild(makePolygonSet());
   switch_node->addChild(makeIndexedPolygonSet());
   switch_node->addChild(makeMonoIndexedPolygonSet());

   sglCullMode *cull_mode = new sglCullMode();
   cull_mode->setCullMode(sglCullMode::eOFF);
   vector<sglStatelet*> default_statelets;
   default_statelets.push_back(cull_mode);
   default_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   sglDefaultStateNode *def = new sglDefaultStateNode();
   def->setStatelets(default_statelets);
   root->addChild(def);

   def->addChild(switch_node);

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;

   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);
   //view_platform->setEnableZup(true);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0f);
   view_trans->setAzimuth(0.0f);
   view_trans->setElevation(0.0f);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(&pfs);
   override_statelets.push_back(&pbs);
   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testGeoSets");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
