/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testGroup.cpp
 *  Created: 1998
 *  Summary: test the sglGroup functionality
 *****************************************************************************/

#if defined(WIN32)
#include <windows.h>
#endif

#include <sgl.h>
#include <sglu.h>
#include <sglNode.hpp>
#include <sglGroup.hpp>
#include <sglBillboard.hpp>
#include <sglLOD.hpp>
#include <sglSwitch.hpp>
#include <sglTranslate.hpp>
#include <sglTransform.hpp>
#include <sglTextSet.hpp>
#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglOStream.hpp>

int main(void)
{
   sglOStream::setStream(&cout);

   int i;
   sglGroup root;
   sglNode *node;
   sglGroup *child_list[5];

   for (i=0; i<5; i++)
   {
      child_list[i] = new sglGroup();
      cout << "adding node: " << child_list[i] << endl;
      root.addChild(child_list[i]);
   }

   for (i=0; i<5; i++)
   {
      if (root.getChild(i) != child_list[i])
      {
          cout << "add test FAILED for child " << i << endl;
          exit(1);
      }
      if (child_list[i]->getNumParents() != 1 ||
          child_list[i]->getParent(0) != &root)
      {
          cout << "add test FAILED for child " << i << endl;
          exit(1);
      }
      if (child_list[i]->getRefCount() != 1)
      {
          cout << "add test FAILED for child " << i << endl;
          exit(1);
      }
   }
   cout << "add test PASSED." << endl;

   node = root.getChild(2);
   cout << "removing child 2: " << node << endl;
   root.removeChild(node);

   for (i=3; i<5; i++)
   {
      if (root.getChild(i-1) != child_list[i])
      {
          cout << "remove test FAILED" << endl;
          exit(1);
      }
   }
   if (child_list[2]->getRefCount() != 0 ||
       child_list[2]->getNumParents() != 0)
   {
      cout << "remove test FAILED" << endl;
      exit(1);
   }
   cout << "remove test PASSED." << endl;

   cout << "reinserting child 2" << endl;
   root.insertChild(2, child_list[2]);
   for (i=0; i<5; i++)
   {
      if (root.getChild(i) != child_list[i])
      {
          cout << "insert(2) test FAILED for child " << i << endl;
          exit(1);
      }
      if (child_list[i]->getNumParents() != 1 ||
          child_list[i]->getParent(0) != &root)
      {
          cout << "insert(2) test FAILED for child " << i << endl;
          exit(1);
      }
      if (child_list[i]->getRefCount() != 1)
      {
          cout << "insert(2) test FAILED for child " << i << endl;
          exit(1);
      }
   }
   cout << "add test PASSED." << endl;

   cout << "inserting child 2 as child 12: " << node << endl;
   bool success = root.insertChild(12, child_list[2]);
   if (success)
   {
       cout << "insert(12) test FAILED." << endl;
       exit(1);
   }
   cout << "insert(12) test PASSED." << endl;

   for (i=0; i<5; i++)
   {
      cout << "adding node: " << child_list[i] << endl;
      root.addChild(child_list[i]);
   }
   for (i=0; i<10; i++)
   {
      if (root.getChild(i) != child_list[i%5])
      {
          cout << "add test FAILED for child " << i << endl;
          exit(1);
      }
   }
   for (i=0; i<5; i++)
   {
      if (child_list[i]->getNumParents() != 1)
      {
         cout << "child[" << i << "]->getNumParents() FAILED" << endl;
         exit(1);
      }
   }
   cout << "add test PASSED." << endl;

   if (root.getNumChildren() != 10)
   {
      cout << "getNumChildren test FAILED" << endl;
      exit(1);
   }

   int indx = root.searchChild(root.getChild(2));
   if (indx != 2)
   {
      cout << "searchChild test FAILED" << endl;
      exit(1);
   }
   indx = root.searchChild(root.getChild(2), indx+1);
   if (indx != 7)
   {
      cout << "searchChild test2 FAILED" << endl;
      exit(1);
   }

   node = root.getChild(2);
   cout << "removing child: " << node << endl;
   root.removeChild(node);

   for (i=0; i<6; i++)
   {
      if (root.getChild(i) != child_list[i%5])
      {
          cout << "remove test FAILED" << endl;
          exit(1);
      }
   }
   for (i=8; i<10; i++)
   {
      if (root.getChild(i-1) != child_list[i%5])
      {
          cout << "remove test FAILED" << endl;
          exit(1);
      }
   }
   for (i=0; i<5; i++)
   {
      if (child_list[i]->getNumParents() != 1)
      {
         cout << "remove test FAILED" << endl;
         exit(1);
      }
   }
   cout << "remove test PASSED." << endl;

   cout << "reinserting child" << endl;
   root.insertChild(7, child_list[2]);
   for (i=0; i<10; i++)
   {
      if (root.getChild(i) != child_list[i%5])
      {
          cout << "add test FAILED for child " << i << endl;
          exit(1);
      }
   }
   for (i=0; i<5; i++)
   {
      if (child_list[i]->getNumParents() != 1)
      {
         cout << "add test FAILED for child " << i << endl;
         exit(1);
      }
   }
   cout << "add test PASSED." << endl;

   cout << "removing child 2: " << node << endl;
   if (root.removeChild(2) != node)
   {
      cout << "remove test FAILED" << endl;
      exit(1);
   }

   for (i=3; i<10; i++)
   {
      if (root.getChild(i-1) != child_list[i%5])
      {
          cout << "remove test FAILED" << endl;
          exit(1);
      }
   }
   cout << "remove test PASSED." << endl;

   cout << "reinserting child" << endl;
   root.insertChild(2, child_list[2]);
   for (i=0; i<10; i++)
   {
      if (root.getChild(i) != child_list[i%5])
      {
          cout << "add test FAILED for child " << i << endl;
          exit(1);
      }
   }
   cout << "add test PASSED." << endl;

   cout << "replacing child 2 with 3" << endl;
   node = root.replaceChild(2, child_list[3]);
   if (node != child_list[2])
   {
      cout << "replace test FAILED" << endl;
      exit(1);
   }
   if (root.getChild(2) != child_list[3])
   {
      cout << "replace test FAILED" << endl;
      exit(1);
   }
   for (i=0; i<5; i++)
   {
      if (child_list[i]->getNumParents() != 1 ||
          child_list[i]->getParent(0) != &root)
      {
          cout << "replace test FAILED" << endl;
          exit(1);
      }
      if (child_list[i]->getRefCount() != 1)
      {
          cout << "replace test FAILED" << endl;
          exit(1);
      }
   }
   cout << "replacing child 7 with 3" << endl;
   node = root.replaceChild(7, child_list[3]);
   if (node != child_list[2])
   {
      cout << "replace test FAILED" << endl;
      exit(1);
   }
   if (root.getChild(7) != child_list[3])
   {
      cout << "replace test FAILED" << endl;
      exit(1);
   }
   for (i=0; i<5; i++)
   {
      if (i==2)
      {
         if (child_list[i]->getNumParents() != 0 ||
             child_list[i]->getRefCount() != 0)
         {
            cout << "replace test FAILED" << endl;
            exit(1);
         }
      }
      else
      {
         if (child_list[i]->getNumParents() != 1 ||
             child_list[i]->getParent(0) != &root)
         {
            cout << "replace test FAILED" << endl;
            exit(1);
         }
         if (child_list[i]->getRefCount() != 1)
         {
            cout << "replace test FAILED" << endl;
            exit(1);
         }
      }
   }
   node = root.replaceChild(2, child_list[2]);
   node = root.replaceChild(7, child_list[2]);

   cout << "removing all children" << endl;
   for (i=9; i>=0; --i)
   {
      if (root.removeChild(i) != child_list[i%5])
      {
          cout << "remove test FAILED" << endl;
          exit(1);
      }
   }
   for (i=0; i<5; i++)
   {
      if (child_list[i]->getNumParents() != 0 ||
          child_list[i]->getRefCount() != 0)
      {
         cout << "remove test FAILED" << endl;
         exit(1);
      }
   }
   if (root.getNumChildren() != 0)
   {
      cout << "getNumChildren test FAILED" << endl;
      exit(1);
   }

   cout << "sizeof(sglObject):     " << sizeof(sglObject) << endl;
   cout << "sizeof(sglNode):       " << sizeof(sglNode) << endl;
   cout << "sizeof(sglGroup):      " << sizeof(sglGroup) << endl;
   cout << "sizeof(sglLOD):        " << sizeof(sglLOD) << endl;
   cout << "sizeof(sglSwitch):     " << sizeof(sglSwitch) << endl;
   cout << "sizeof(sglBillboard):  " << sizeof(sglBillboard) << endl;
   cout << "sizeof(sglTransformf): " << sizeof(sglTransformf) << endl;
   cout << "sizeof(sglTransformd): " << sizeof(sglTransformd) << endl;
   cout << "sizeof(sglTranslatef): " << sizeof(sglTranslatef) << endl;
   cout << "sizeof(sglTranslated): " << sizeof(sglTranslated) << endl;
   cout << "sizeof(sglGeode):      " << sizeof(sglGeode) << endl;
   cout << "sizeof(sglTextSet):    " << sizeof(sglTextSet) << endl;
   cout << "sizeof(sglGeoSet):     " << sizeof(sglGeoSet) << endl;

   cout << "testGroup.cpp: all tests PASSED." << endl;
   return 0;
}
