/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testTexture.cpp
 *  Created: 29 February 2000
 *  Summary: test the texture state functionality
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglGeode.hpp>
#include <sglTriangleStripSet.hpp>
#include <sglTexture2D.hpp>
#include <sglTexEnv.hpp>
#include <sglTexGen.hpp>
#include <sglBitmapFont.hpp>
#include <sglImageLoader.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

bool help_flag = false;

sglTexture2D *tex;
sglTriangleStripSet *gset;
sglVec4f *color;

unsigned int win_x = 640, win_y = 480;

const sglVec4f white_color(1.0f, 1.0f, 1.0f, 1.0f);
const sglVec4f gray_color(0.4f, 0.4f, 0.4f, 1.0f);
const sglVec4f black_color(0.0f, 0.0f, 0.0f, 1.0f);
const sglVec4f red_color(1.0f, 0.0f, 0.0f, 1.0f);

//----------------------------------------------------------------------------
void outputHelp()
{
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glDisable(GL_DEPTH_TEST);

   glColor4f(1.0f, 0.9f, 0.9f, 1.0f);
   glRasterPos3i(10,100,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "/? - toggle help");
   glRasterPos3i(10,85,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "o - ouput scene graph");
   glRasterPos3i(10,70,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "$ - toggle statistics reporting");
   glRasterPos3i(10,55,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "d - toggle SP/DP traversal");
   glRasterPos3i(10,40,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "f - toggle frame rate display");
   glRasterPos3i(10,25,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "m/M - cycle through min/mag filters");
   glRasterPos3i(10,10,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "c - cycle through background colors");

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   if (help_flag)
   {
      outputHelp();
   }

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'c':
         if (view->getClearColor() == gray_color)
            view->setClearColor(black_color);
         else if (view->getClearColor() == black_color)
            view->setClearColor(white_color);
         else if (view->getClearColor() == white_color)
            view->setClearColor(red_color);
         else if (view->getClearColor() == red_color)
            view->setClearColor(gray_color);
         break;

      case 'm':
         switch (tex->getMinFilter())
         {
            case sglTexture::eNEAREST:
               tex->setMinFilter(sglTexture::eLINEAR);
               cout << "sglTexture::eLINEAR" << endl;
               break;
            case sglTexture::eLINEAR:
               tex->setMinFilter(sglTexture::eNEAREST_MIPMAP_NEAREST);
               cout << "sglTexture::eNEAREST_MIPMAP_NEAREST" << endl;
               break;
            case sglTexture::eNEAREST_MIPMAP_NEAREST:
               tex->setMinFilter(sglTexture::eNEAREST_MIPMAP_LINEAR);
               cout << "sglTexture::eNEAREST_MIPMAP_LINEAR" << endl;
               break;
            case sglTexture::eNEAREST_MIPMAP_LINEAR:
               tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_NEAREST);
               cout << "sglTexture::eLINEAR_MIPMAP_NEAREST" << endl;
               break;
            case sglTexture::eLINEAR_MIPMAP_NEAREST:
               tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
               cout << "sglTexture::eLINEAR_MIPMAP_LINEAR" << endl;
               break;
            case sglTexture::eLINEAR_MIPMAP_LINEAR:
               tex->setMinFilter(sglTexture::eNEAREST);
               cout << "sglTexture::eNEAREST" << endl;
               break;
         }
         break;

      case 'M':
         if (tex->getMagFilter() == sglTexture::eNEAREST)
         {
            tex->setMagFilter(sglTexture::eLINEAR);
            cout << "sglTexture::eLINEAR" << endl;
         }
         else
         {
            tex->setMagFilter(sglTexture::eNEAREST);
            cout << "sglTexture::eNEAREST" << endl;
         }
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '?':
      case '/':
         help_flag = !help_flag;
         cout << "help_flag = " << help_flag << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}


//----------------------------------------------------------------------------
sglVec3f normal(0.0f, -1.0f, 0.0f);

sglGroup *makeTextureObj(const string &filename)
{
   const unsigned int LENGTH = 4;

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = LENGTH;

   sglTexture::ExternalFormatEnum format;
   sglTexture::TypeEnum type;
   unsigned int width, height;

   sglOStream::setThreshold(sglOStream::eDEBUG);
   void *image =
      sglImageLoader::loadImage(filename, width, height, format, type);
   sglOStream::setThreshold(sglOStream::eWARNING);

//    {
//       void *old_image = image;
//       image = sglImageLoader::changeImageType(
//          format, type, width, height, sglTexture::eUNSIGNED_INT, old_image);
//       type = sglTexture::eUNSIGNED_INT;
//       free(old_image);
//    }

//    {
//       void *old_image = image;
//       image = sglImageLoader::changeImageType(
//          format, type, width, height, sglTexture::eFLOAT, old_image);
//       type = sglTexture::eFLOAT;
//       free(old_image);
//    }

//    {
//       void *old_image = image;
//       image = sglImageLoader::reformatImage(
//          format, type, width, height, sglTexture::eLUMINANCE, old_image);
//       format = sglTexture::eLUMINANCE;
//       free(old_image);
//    }

//    {
//       sglTimespec ts1, ts2;
//       void *old_image = image;
//       image = NULL;

//       sglTimespec::getSysTime(ts1);
//       sglTexture2D::resizeImage(
//          format, type, width, height, false, 1024, 1024, old_image, image);
//       sglTimespec::getSysTime(ts2);
//       cout << "resizeImage took "
//            << sglTimespec::convertToDouble(ts2 - ts1)
//            << " seconds" << endl;

//       sglTimespec::getSysTime(ts1);
//       gluScaleImage(
//          format, width, height, type, old_image, 1024, 1024, type, image);
//       sglTimespec::getSysTime(ts2);
//       cout << "gluScaleImage took "
//            << sglTimespec::convertToDouble(ts2 - ts1)
//            << " seconds" << endl;

//       sglTimespec::getSysTime(ts1);
//       sglTexture2D::resizeImage(
//          format, type, width, height, false, 1024, 1024, old_image, image);
//       sglTimespec::getSysTime(ts2);
//       cout << "resizeImage took "
//            << sglTimespec::convertToDouble(ts2- ts1)
//            << " seconds" << endl;

//       free(old_image);
//       width = 1024;
//       height = 1024;
//    }

//    {
//       sglImageLoader::writeImage(
//          "dump.png", image, width, height, format, type);
//    }

   tex = new sglTexture2D;
   if (image)
      tex->setImage(image, format, type, width, height, false);
//tex->setInternalFormat(sglTexture::eINT_INTENSITY16);
   tex->setAutoMipmapFlag(true);
   tex->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
   tex->setMagFilter(sglTexture::eLINEAR); //sglTexture::eNEAREST);
   tex->setWrapMode(sglTexture::eCLAMP, sglTexture::eCLAMP);

   vector<sglStatelet*> tex_state;
   tex_state.push_back(new sglTransparency(true));
   tex_state.push_back(tex);

   sglVec3f *vertex = new sglVec3f[LENGTH];
   vertex[0].set(-8.0f, 0.0f,  8.0f);
   vertex[1].set(-8.0f, 0.0f, -8.0f);
   vertex[2].set( 8.0f, 0.0f,  8.0f);
   vertex[3].set( 8.0f, 0.0f, -8.0f);

   sglVec3f *normal = new sglVec3f[LENGTH];
   normal[0].set(1.0f, 0.0f, 0.0f);
   normal[1].set(1.0f, 0.0f, 0.0f);
   normal[2].set(1.0f, 0.0f, 0.0f);
   normal[3].set(1.0f, 0.0f, 0.0f);

   color = new sglVec4f[1];
   color->set(1.0f, 1.0f, 1.0f, 1.0f);

   gset = new sglTriangleStripSet();

   gset->setCoordList(vertex);
   gset->setNormalList(sglGeoSet::ePER_VERTEX, normal);
   gset->setColorList(sglGeoSet::eOVERALL,   color);
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);

   // This object is too small to want a display list.
   gset->enableDisplayList(false);

   sglVec2f *tvert = new sglVec2f[LENGTH];
   tvert[0].set(0.0f, 1.0f);
   tvert[1].set(0.0f, 0.0f);
   tvert[2].set(1.0f, 1.0f);
   tvert[3].set(1.0f, 0.0f);
   gset->setState(0, tex_state, tvert);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
void buildScenegraph(const string &filename)
{
   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);
   root->addChild(makeTextureObj(filename));

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setClearColor(gray_color);
   view->setViewport(0, 0, win_x, win_y);
   view->setStateMask(0);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   if (argc != 2)
   {
      cerr << "usage: " << argv[0] << " imagefile" << endl;
      exit(1);
   }

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testTexture");

   sgl::initialize();
   mouse = sgluMouse::initialize();

   string filename;
   if (sglDataLoader::findFile(argv[1], NULL, filename))
      buildScenegraph(filename);
   else
      buildScenegraph(argv[1]);

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
