/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testInterleavedGeometry.cpp
 *  Created: 21 March 2000
 *  Summary: test the interleaved vertex arrays.  This test program incomplete
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglBitmapFont.hpp>

#include <sglGroup.hpp>
#include <sglScene.hpp>
#include <sglLOD.hpp>
#include <sglSwitch.hpp>
#include <sglTransform.hpp>
#include <sglTranslate.hpp>
#include <sglGeode.hpp>
#include <sglInterleavedGeometry.hpp>
#include <sglTextSet.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;
sglAntiAliasPoint   aa_point;
sglAntiAliasLine    aa_line;
sglAntiAliasPolygon aa_poly;

sglSwitch *switch_node;
int switch_val = 0;

unsigned int win_x = 640, win_y = 480;

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "up/down arrow - cycle through modes" << endl;
         cerr << "a - toggle point anti-aliasing" << endl;
         cerr << "A - toggle line anti-aliasing" << endl;
         cerr << "z - toggle polygon anti-aliasing" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         switch_val++;
         if ((unsigned int)switch_val >= switch_node->getNumChildren())
         {
            switch_val = 0;
         }
         switch_node->setVal(switch_val);
         break;
      case GLUT_KEY_DOWN:
         switch_val--;
         if (switch_val < 0)
         {
            switch_val = switch_node->getNumChildren() - 1;
         }
         switch_node->setVal(switch_val);
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------

sglVec3f color[3];

sglGroup *makeInterleavedObject()
{
   color[0].set(1.0f, 0.6f, 0.6f);
   color[1].set(0.6f, 1.0f, 0.6f);
   color[2].set(0.6f, 0.6f, 1.0f);

   sglC3fV3fStruct *array = new sglC3fV3fStruct[6];

   sglGeode *geode = new sglGeode();

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = 6;

   unsigned int index = 0;
   for (unsigned int ix=0; ix<3; ix++)
   {
      array[index].vertex.set(ix*8.0f - 8.0f, 0.0f, 8.0f);
      array[index].color = color[ix];
      index++;

      array[index].vertex.set(ix*8.0f - 8.0f, 0.0f, -8.0f);
      array[index].color = color[ix];
      index++;
   }

   sglInterleavedGeometry *gset = new sglInterleavedGeometry();
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);
   gset->setGLPrimType(GL_QUAD_STRIP);
   gset->setInterleavedArray(GL_C3F_V3F, array, 0);

   // This object is too small to want a display list.
   gset->enableDisplayList(false);

   geode->addGeometry(gset);

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makePointSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("PointSet(C3F_V3F)");

   sglGeode *geode = new sglGeode;
   geode->addGeometry(text);

   for (int i=0; i<4; i++)
   {
      sglC3fV3fStruct *array = new sglC3fV3fStruct[4];

      unsigned int *lengths = new unsigned int[1];
      lengths[0] = 4;

      for (int k=0; k<4; k++)
      {
         array[k].color.set(0.33f*i, 0.0f, 1.0f-0.33f*k);
         array[k].vertex.set(i*4.f-6.0f, 0.0f, k*4.f-6.0f);
      }
      sglInterleavedGeometry *gset = new sglInterleavedGeometry();
      gset->setNumPrims(1);
      gset->setPrimLengths(lengths);
      gset->setGLPrimType(GL_POINTS);
      gset->setInterleavedArray(GL_C3F_V3F, array, 0);

      vector<sglStatelet*> state;
      state.push_back(new sglPointSize(2.0f*i + 1.0f));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeLineSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("LineSet(C3F_V3F)");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   short factor = 1;
   for (int i=0; i<4; i++)
   {
      factor *= 2;

      unsigned int *lengths = new unsigned int[1];
      lengths[0] = 2;

      sglC3fV3fStruct *array = new sglC3fV3fStruct[2];

      array[0].color.set(0.33f*i, 1.0f, 0.0f);
      array[0].vertex.set(i*4.f-6.0f, 0.0f, -6.0f);

      array[1].color.set(0.33f*i, 1.0f, 0.0f);
      array[1].vertex.set(i*4.f-6.0f, 0.0f,  6.0f);

      sglInterleavedGeometry *gset = new sglInterleavedGeometry;
      gset->setNumPrims(1);
      gset->setPrimLengths(lengths);
      gset->setGLPrimType(GL_LINES);
      gset->setInterleavedArray(GL_C3F_V3F, array, 0);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(7.0f - 2.0f*i));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeLineStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("LineStripSet(C3F_V3F)");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   short factor = 1;
   for (int i=0; i<4; i++)
   {
      factor *= 2;

      unsigned int *length = new unsigned int[1];
      length[0] = 4;

      sglC3fV3fStruct *array = new sglC3fV3fStruct[4];
      for (int j=0; j<4; j++)
      {
         array[j].color.set(0.0f, 0.33f*i, 1.0f);
         array[j].vertex.set((float)(10.0*cos((double)j)),
                             i*4.0f,
                             (float)(10.0*sin((double)j)));
      }

      sglInterleavedGeometry *gset = new sglInterleavedGeometry;
      gset->setNumPrims(1);
      gset->setPrimLengths(length);
      gset->setGLPrimType(GL_LINE_STRIP);
      gset->setInterleavedArray(GL_C3F_V3F, array, 0);

      vector<sglStatelet*> state;
      state.push_back(new sglLineWidth(2.0f*i + 1.0f));
      state.push_back(new sglLineStipple((factor - 1)*0x1111, factor));
      gset->setState(0, state);

      geode->addGeometry(gset);
   }
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleSet(C3F_V3F)");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = 9;

   sglC3fV3fStruct *array = new sglC3fV3fStruct[9];
   for (int j=0; j<3; j++)
   {
      array[j*3 + 0].vertex.set(-6.0f + j*6.0f, 0.f, -6.0f);
      array[j*3 + 1].vertex.set(-2.0f + j*6.0f, 0.f, -6.0f);
      array[j*3 + 2].vertex.set(-4.0f + j*6.0f, 0.f, 6.0f);

      array[j*3 + 0].color.set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f);
      array[j*3 + 1].color.set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f);
      array[j*3 + 2].color.set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f);
   }

   sglInterleavedGeometry *gset = new sglInterleavedGeometry;
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);
   gset->setGLPrimType(GL_TRIANGLES);
   gset->setInterleavedArray(GL_C3F_V3F, array, 0);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleStripSet(C3F_V3F)");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = 8;

   sglC3fV3fStruct *array = new sglC3fV3fStruct[8];
   for (int j=0; j<4; j++)
   {
      array[2*j].vertex.set(3.f*j - 4.5f, 0.f, 4.0f);
      array[2*j+1].vertex.set(3.f*j - 4.5f, 0.f, -4.0f);
      array[2*j].color.set((float)j/4.0f, 0.8f, 1.0f - (float)j/4.0f);
      array[2*j+1].color.set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f);
   }

   sglInterleavedGeometry *gset = new sglInterleavedGeometry;
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);
   gset->setGLPrimType(GL_TRIANGLE_STRIP);
   gset->setInterleavedArray(GL_C3F_V3F, array, 0);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleFanSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleFanSet(C3F_V3F)");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglC3fV3fStruct *array = new sglC3fV3fStruct[5];
   unsigned int *length = new unsigned int[1];
   length[0] = 5;

   array[0].vertex.set(0.f, 0.f, 3.f);
   array[0].color.set(1.0f, 1.0f, 1.0f);
   for (int j=0; j<4; j++)
   {
      array[j+1].vertex.set((float)(10.0*cos((double)j)),
                            0.f, (float)(10.0*sin((double)j)));
      array[j+1].color.set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f);
   }

   sglInterleavedGeometry *gset = new sglInterleavedGeometry;
   gset->setNumPrims(1);
   gset->setPrimLengths(length);
   gset->setGLPrimType(GL_TRIANGLE_FAN);
   gset->setInterleavedArray(GL_C3F_V3F, array, 0);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeQuadSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("QuadSet(C3F_V3F)");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = 12;

   sglC3fV3fStruct *array = new sglC3fV3fStruct[12];
   for (int j=0; j<3; j++)
   {
      array[4*j].color.set(0.f, 1.f, 1.f);
      array[4*j].vertex.set(6.f*j - 8.f, 0.f, 4.0f);

      array[4*j+1].color.set(0.f, 1.f, 1.f);
      array[4*j+1].vertex.set(6.f*j - 8.f, 0.f, -4.0f);

      array[4*j+2].color.set(0.f, 1.f, 1.f);
      array[4*j+2].vertex.set(6.f*j - 4.f, 0.f, -4.0f);

      array[4*j+3].color.set(0.f, 1.f, 1.f);
      array[4*j+3].vertex.set(6.f*j - 4.f, 0.f, 4.0f);
   }

   sglInterleavedGeometry *gset = new sglInterleavedGeometry;
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);
   gset->setGLPrimType(GL_QUADS);
   gset->setInterleavedArray(GL_C3F_V3F, array, 0);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeQuadStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("QuadStripSet(C3F_V3F)");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   unsigned int *length = new unsigned int[1];
   length[0] = 10;

   sglC3fV3fStruct *array = new sglC3fV3fStruct[10];
   for (int j=0; j<5; j++)
   {
      array[2*j].vertex.set(3.f*j - 6.f, 0.f, 4.0f);
      array[2*j+1].vertex.set(3.f*j - 6.f, 0.f, -4.0f);
      array[2*j].color.set((float)j/5.0f, 0.8f, 1.0f - (float)j/5.0f);
      array[2*j+1].color.set((float)j/5.0f, 1.0f - (float)j/5.0f, 0.5f);
   }

   sglInterleavedGeometry *gset = new sglInterleavedGeometry;
   gset->setNumPrims(1);
   gset->setPrimLengths(length);
   gset->setGLPrimType(GL_QUAD_STRIP);
   gset->setInterleavedArray(GL_C3F_V3F, array, 0);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makePolygonSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("PolygonSet(C3F_V3F)");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   unsigned int *length = new unsigned int[3];
   length[0] = 5;
   length[1] = 6;
   length[2] = 7;

   sglC3fV3fStruct *array = new sglC3fV3fStruct[18];

   int index = 0;
   for (int j=0; j<3; j++)
   {
      for (unsigned int i=0; i<length[j]; i++)
      {
         array[index].color.set(0.f, 1.f, 1.f);
         array[index].vertex.set(
            (8.f*j-8.f) + (float)(3.0*cos(2.0*M_PI*i/(double)length[j])),
            0.0f,
            (float)(3.0*sin(2.0*M_PI*i/(double)length[j])));
         index++;
      }
   }

   sglInterleavedGeometry *gset = new sglInterleavedGeometry;
   gset->setNumPrims(3);
   gset->setPrimLengths(length);
   gset->setGLPrimType(GL_POLYGON);
   gset->setInterleavedArray(GL_C3F_V3F, array, 0);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   // make scene and default state:

   root = new sglScene();

   switch_node = new sglSwitch;
   switch_node->setVal(switch_val);
   root->addChild(switch_node);

   switch_node->addChild(makePointSet());
   switch_node->addChild(makeLineSet());
   switch_node->addChild(makeLineStripSet());
   switch_node->addChild(makeTriangleSet());
   switch_node->addChild(makeTriangleStripSet());
   switch_node->addChild(makeTriangleFanSet());
   switch_node->addChild(makeQuadSet());
   switch_node->addChild(makeQuadStripSet());
   switch_node->addChild(makePolygonSet());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;

   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);
   //view_platform->setEnableZup(true);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(640, 480);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testInterleavedGeometry");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   cout << "sizeof(sglC4ubV2fStruct): " << sizeof(sglC4ubV2fStruct) << endl;
   cout << "sizeof(sglC4ubV3fStruct): " << sizeof(sglC4ubV3fStruct) << endl;
   cout << "sizeof(sglC3fV3fStruct): " << sizeof(sglC3fV3fStruct) << endl;
   cout << "sizeof(sglN3fV3fStruct): " << sizeof(sglN3fV3fStruct) << endl;
   cout << "sizeof(sglC4fN3fV3fStruct): " << sizeof(sglC4fN3fV3fStruct) <<endl;
   cout << "sizeof(sglT2fV3fStruct): " << sizeof(sglT2fV3fStruct) << endl;
   cout << "sizeof(sglT4fV4fStruct): " << sizeof(sglT4fV4fStruct) << endl;
   cout << "sizeof(sglT2fC4ubV3fStruct): " << sizeof(sglT2fC4ubV3fStruct)
        << endl;
   cout << "sizeof(sglT2fC3fV3fStruct): " << sizeof(sglT2fC3fV3fStruct)
        << endl;
   cout << "sizeof(sglT2fN3fV3fStruct): " << sizeof(sglT2fN3fV3fStruct)
        << endl;
   cout << "sizeof(sglT2fC4fN3fV3fStruct): " << sizeof(sglT2fC4fN3fV3fStruct)
        << endl;
   cout << "sizeof(sglT4fC4fN3fV4fStruct): " << sizeof(sglT4fC4fN3fV4fStruct)
        << endl;

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
