/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testIsect1.cpp
 *  Created: 1999
 *  Summary: textual test of intersection routines
 *****************************************************************************/

#include <sglSegment.hpp>
#include <sglBoxBound.hpp>
#include <sglSphereBound.hpp>
#include <sglOStream.hpp>

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum intersect(sglBoxBound &bbox,
                                        sglSegmentf &seg,
                                        sglVec3f &pos)
{
   cout << "Before: " << seg << endl;
   sglBound::IntersectResultEnum hit = bbox.intersect(seg);

   cout << " After: " << seg << endl;

   if (hit == sglBound::eINTERSECT)
   {
      pos  = seg.getDir();
      pos *= seg.getLen();
      pos += seg.getPos();

      cout << "Hit at: " << pos << endl;
   }
   else if (hit == sglBound::eINSIDE)
   {
      cout << "Segment starts inside." << endl;
   }
   else if (hit == sglBound::eOUTSIDE)
   {
      cout << "Segment is outside." << endl;
   }

   return hit;
}

//----------------------------------------------------------------------------
sglBound::IntersectResultEnum intersect(sglSphereBoundf &bsph,
                                        sglSegmentf &seg,
                                        sglVec3f &pos)
{
   cout << "Before: " << seg << endl;
   sglBound::IntersectResultEnum hit = bsph.intersect(seg);

   cout << " After: " << seg << endl;

   if (hit == sglBound::eINTERSECT)
   {
      pos  = seg.getDir();
      pos *= seg.getLen();
      pos += seg.getPos();

      cout << "Hit at: " << pos << endl;
   }
   else if (hit == sglBound::eINSIDE)
   {
      cout << "Segment starts inside." << endl;
   }
   else if (hit == sglBound::eOUTSIDE)
   {
      cout << "Segment is outside." << endl;
   }
   return hit;
}

//----------------------------------------------------------------------------
int main()
{
   sglOStream::setStream(&cout);

   // =======================================================
   sglVec3f    pos;
   sglSegmentf seg;

   sglSphereBoundf bsph;
   bsph.setCenter(0.0f, 0.0f, 0.0f);
   bsph.setRadius(2.0f);
   cout << "Testing sglSphereBound::intersect(sglSegment), " << endl;
   cout << "bsph = " << bsph << endl;

   // --------------------
   sglVec3f p1(-3.0f, 0.0f, 0.0f);
   sglVec3f p2( 0.0f, 0.0f, 0.0f);
   seg.makePts(p1, p2);

   if (intersect(bsph, seg, pos) == sglBound::eINTERSECT &&
       pos == sglVec3f(-2.0f, 0.f, 0.f))
   {
       cerr << "test1 PASSED" << endl;
   }
   else
   {
       cerr << "test1 FAILED" << endl;
       exit(1);
   }
   cout << endl;

   // --------------------
   p1.set(-3.0, 0.0, 0.0);
   p2.set(-2.5, 0.0, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bsph, seg, pos) != sglBound::eOUTSIDE)
   {
       cerr << "test2 FAILED" << endl;
       exit(1);
   }
   cerr << "test2 PASSED" << endl;
   cout << endl;

   // --------------------
   p1.set(-3.0, 0.5, 0.0);
   p2.set( 3.0, 0.5, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bsph, seg, pos) == sglBound::eINTERSECT &&
       pos.almostEqual(sglVec3f(-1.93649f, 0.5f, 0.f), 1.0e-5))
   {
       cerr << "test3 PASSED" << endl;
   }
   else
   {
       cerr << "test3 FAILED" << endl;
       exit(1);
   }
   cout << endl;

   // --------------------
   p1.set(-3.0, 2.1, 0.0);
   p2.set( 3.0, 2.1, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bsph, seg, pos) != sglBound::eOUTSIDE)
   {
       cerr << "test4 FAILED" << endl;
       exit(1);
   }
   cerr << "test4 PASSED" << endl;
   cout << endl;

   // --------------------
   p1.set(-1.0, 0.0, 0.0);
   p2.set( 1.0, 0.0, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bsph, seg, pos) != sglBound::eINSIDE)
   {
       cerr << "test5 FAILED" << endl;
       exit(1);
   }
   cerr << "test5 PASSED" << endl;
   cout << endl;

   // --------------------
   p1.set( 3.0, 0.0, 0.0);
   p2.set( 4.0, 0.0, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bsph, seg, pos) != sglBound::eOUTSIDE)
   {
       cerr << "test6 FAILED" << endl;
       exit(1);
   }
   cerr << "test6 PASSED" << endl;
   cout << endl;

   // ===============================================================

   sglBoxBound bbox;
   bbox.setMin(-2.0f, -2.0f, -2.0f);
   bbox.setMax( 2.0f,  2.0f,  2.0f);
   cout << "Testing sglBoxBound::intersect(sglSegment), " << endl;
   cout << "bbox = " << bbox << endl;
   cout << endl;

   // --------------------
   p1.set(-3.0f, 0.0f, 0.0f);
   p2.set( 0.0f, 0.0f, 0.0f);
   seg.makePts(p1, p2);

   if (intersect(bbox, seg, pos) == sglBound::eINTERSECT &&
       pos == sglVec3f(-2.0f, 0.f, 0.f))
   {
       cerr << "test1b PASSED" << endl;
   }
   else
   {
       cerr << "test1b FAILED" << endl;
       exit(1);
   }
   cout << endl;

   // --------------------
   p1.set(-3.0, 0.0, 0.0);
   p2.set(-2.5, 0.0, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bbox, seg, pos) != sglBound::eOUTSIDE)
   {
       cerr << "test2b FAILED" << endl;
       exit(1);
   }
   cerr << "test2b PASSED" << endl;
   cout << endl;

   // --------------------
   p1.set(-3.0, 0.5, 0.0);
   p2.set( 3.0, 0.5, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bbox, seg, pos) == sglBound::eINTERSECT &&
       pos.almostEqual(sglVec3f(-2.0f, 0.5f, 0.f),1.0e-5))
   {
       cerr << "test3b PASSED" << endl;
   }
   else
   {
       cerr << "test3b FAILED" << endl;
       exit(1);
   }
   cout << endl;

   // --------------------
   p1.set(-3.0, 2.1, 0.0);
   p2.set( 3.0, 2.1, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bbox, seg, pos) != sglBound::eOUTSIDE)
   {
       cerr << "test4b FAILED" << endl;
       exit(1);
   }
   cerr << "test4b PASSED" << endl;
   cout << endl;

   // --------------------
   p1.set(-1.0, 0.0, 0.0);
   p2.set( 1.0, 0.0, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bbox, seg, pos) != sglBound::eINSIDE)
   {
       cerr << "test5b FAILED" << endl;
       exit(1);
   }
   cerr << "test5b PASSED" << endl;
   cout << endl;

   // --------------------
   p1.set( 3.0, 0.0, 0.0);
   p2.set( 4.0, 0.0, 0.0);
   seg.makePts(p1, p2);

   if (intersect(bbox, seg, pos) != sglBound::eOUTSIDE)
   {
       cerr << "test6b FAILED" << endl;
       exit(1);
   }
   cerr << "test6b PASSED" << endl;
   cout << endl;

   cerr << "All tests PASSED" << endl;
   return 0;
}
