/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testIsectGeoSets.cpp
 *   Author: Scott Mc Millan
 *  Created: 14 April 2000
 *  Summary: test intersection with all of the geoset subtypes
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglTransform.hpp>
#include <sglTranslate.hpp>
#include <sglSwitch.hpp>

#include <sglSegment.hpp>
#include <sglIntersect.hpp>

#include <sglGeode.hpp>

#include <sglTextSet.hpp>
#include <sglPointSet.hpp>
#include <sglLineSet.hpp>
#include <sglLineStripSet.hpp>
#include <sglTriangleFanSet.hpp>
#include <sglTriangleSet.hpp>
#include <sglTriangleStripSet.hpp>
#include <sglQuadSet.hpp>
#include <sglQuadStripSet.hpp>
#include <sglPolygonSet.hpp>

#include <sglIndexedPointSet.hpp>
#include <sglIndexedLineSet.hpp>
#include <sglIndexedLineStripSet.hpp>
#include <sglIndexedTriangleFanSet.hpp>
#include <sglIndexedTriangleSet.hpp>
#include <sglIndexedTriangleStripSet.hpp>
#include <sglIndexedQuadSet.hpp>
#include <sglIndexedQuadStripSet.hpp>
#include <sglIndexedPolygonSet.hpp>

#include <sglMonoIndexedPointSet.hpp>
#include <sglMonoIndexedLineSet.hpp>
#include <sglMonoIndexedLineStripSet.hpp>
#include <sglMonoIndexedTriangleFanSet.hpp>
#include <sglMonoIndexedTriangleSet.hpp>
#include <sglMonoIndexedTriangleStripSet.hpp>
#include <sglMonoIndexedQuadSet.hpp>
#include <sglMonoIndexedQuadStripSet.hpp>
#include <sglMonoIndexedPolygonSet.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

bool help_flag = false;

sglSwitch   *switch_node;
int          switch_val = 0;

sglIntersectf  isector;
sglSegmentf    segment;

bool paused_flag = false;
sglVec3f pos(0.0, -10.0, 0.0);
double heading = 0.0, pitch = 0;
float len = 20.0;
float angle = 0.0;

unsigned int win_x = 640, win_y = 480;

//----------------------------------------------------------------------------
void outputHelp()
{
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glEnable(GL_BLEND);
   glColor3f(1.f, 1.f, 1.f);

   glRasterPos3i(10,85,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "/? - toggle help");
   glRasterPos3i(10,70,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "up/down arrow - cycle through primitives");
   glRasterPos3i(10,55,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "f - toggle culling front face");
   glRasterPos3i(10,40,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "b - toggle culling back face");
   glRasterPos3i(10,25,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "p - toggle (pause) segment motion");
   glDisable(GL_BLEND);
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------
void preDrawFunc(void *)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);

   if (!paused_flag)
   {
      heading += 0.005;
      if (heading > M_PI/4.0) heading -= M_PI/2.0;
   }

   segment.makePolar(pos, heading, pitch, len);

   isector.setSegment(segment);
   isector.setResultEnum(sglBound::eOUTSIDE);
   isector.getModelViewMatrix().buildIdentity();
   sglBound::IntersectResultEnum result = root->intersect(isector);
   glColor3f(0.0f, 1.0f, 0.0f);
   if (result == sglBound::eINTERSECT)
   {
      sglVec3f endpoint = isector.getSegment().getEndPoint();
      sglMat4f matrix = isector.getTransformMatrix();
      sglVec3f normal = isector.getNormal();
      sglVec4f n;

      mul(normal, 0.f, matrix, n);

      sglSegmentf nseg(endpoint, sglVec3f(n[0], n[1], n[2]), 4.0f);
      glColor3f(0.f, 0.f, 1.f);

      nseg.drawImmediate();

      glColor3f(1.0f, 0.4f, 0.0f);
   }
   (isector.getSegment()).drawImmediate();
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   if (help_flag)
   {
      outputHelp();
   }

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'f':
         if (isector.getMode() & sglProbe::eCULL_FRONT)
         {
            cout << "Intersecting with front facing polygons" << endl;
            isector.setMode(isector.getMode() & ~sglProbe::eCULL_FRONT);
         }
         else
         {
            cout << "Not intersecting with front facing polygons" << endl;
            isector.setMode(isector.getMode() | sglProbe::eCULL_FRONT);
         }
         break;

      case 'b':
         if (isector.getMode() & sglProbe::eCULL_BACK)
         {
            cout << "Intersecting with back facing polygons" << endl;
            isector.setMode(isector.getMode() & ~sglProbe::eCULL_BACK);
         }
         else
         {
            cout << "Not intersecting with back facing polygons" << endl;
            isector.setMode(isector.getMode() | sglProbe::eCULL_BACK);
         }
         break;

      case '?':
      case '/':
         help_flag = !help_flag;
         cout << "help_flag = " << help_flag << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         switch_val++;
         if ((unsigned int)switch_val >= switch_node->getNumChildren())
         {
            switch_val = 0;
         }
         switch_node->setVal(switch_val);
         break;

      case GLUT_KEY_DOWN:
         switch_val--;
         if (switch_val < 0)
         {
            switch_val = switch_node->getNumChildren() - 1;
         }
         switch_node->setVal(switch_val);
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglTriangleSet *gset = new sglTriangleSet();

   sglVec3f *vertex = new sglVec3f[9];
   sglVec4f *color  = new sglVec4f[3];

   for (int j=0; j<3; j++)
   {
      vertex[j*3 + 0].set(-6.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 1].set(-2.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 2].set(-4.0f + j*6.0f, 0.f, 6.0f);

      color[j].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleStripSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglTriangleStripSet *gset = new sglTriangleStripSet();

   sglVec3f *vertex = new sglVec3f[16];
   sglVec4f *color  = new sglVec4f[16];
   unsigned int *length = new unsigned int[2];
   length[0] = 8;
   length[1] = 8;

   int j;
   for (j=0; j<4; j++)
   {
      vertex[2*j].set(3.f*j - 4.5f, 0.f, 4.0f);
      vertex[2*j+1].set(3.f*j - 4.5f, 0.f, -4.0f);
      color[2*j].set((float)j/4.0f, 0.8f, 1.0f - (float)j/4.0f, 1.0f);
      color[2*j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   for (j=4; j<8; j++)
   {
      vertex[2*j].set(10.5f - 3.f*j, 4.f, 4.0f);
      vertex[2*j+1].set(10.5f - 3.f*j, 4.f, -4.0f);
      color[2*j].set((float)(j-4)/4.0f, 0.8f, 1.0f - (float)(j-4)/4.0f, 1.0f);
      color[2*j+1].set((float)(j-4)/4.0f, 1.0f -(float)(j-4)/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleFanSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleFanSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglTriangleFanSet *gset = new sglTriangleFanSet();

   sglVec3f *vertex = new sglVec3f[10];
   sglVec4f *color  = new sglVec4f[10];
   unsigned int *length = new unsigned int[2];
   length[0] = 5;
   length[1] = 5;

   int j;

   vertex[0].set(0.f, 0.f, -5.f);
   color[0].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (j=0; j<4; j++)
   {
      vertex[j+1].set((float)(10.0*cos(j/2.0)), 0.f, (float)(10.0*sin(j/2.0)));
      color[j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   vertex[5].set(-4.f, 4.f, -5.f);
   color[5].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (j=5; j<9; j++)
   {
      vertex[j+1].set((float)(10.0*cos((10-j)/2.0))-4.f, 4.f,
                      (float)(10.0*sin((10-j)/2.0)));
      color[j+1].set((float)(j-5)/4.0f, 1.0f - (float)(j-5)/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeQuadSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("QuadSet");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   sglQuadSet *gset = new sglQuadSet();

   sglVec3f *vertex = new sglVec3f[12];
   sglVec4f *color  = new sglVec4f[1];

   color[0].set(0.f, 1.f, 1.f, 1.f);

   for (int j=0; j<3; j++)
   {
      vertex[4*j  ].set(6.f*j - 8.f, 0.f, 4.0f);
      vertex[4*j+1].set(6.f*j - 8.f, 0.f, -4.0f);
      vertex[4*j+2].set(6.f*j - 4.f, 0.f, -4.0f);
      vertex[4*j+3].set(6.f*j - 4.f, 0.f, 4.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeQuadStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("QuadStripSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglQuadStripSet *gset = new sglQuadStripSet();

   sglVec3f *vertex = new sglVec3f[22];
   sglVec4f *color  = new sglVec4f[22];
   unsigned int *length = new unsigned int[2];
   length[0] = 11;
   length[1] = 11;

   int j;
   for (j=0; j<5; j++)
   {
      vertex[2*j].set(3.f*j - 12.f, 0.f, 4.0f);
      vertex[2*j+1].set(3.f*j - 12.f, 0.f, -4.0f);
      color[2*j].set((float)j/5.0f, 0.8f, 1.0f - (float)j/5.0f, 1.0f);
      color[2*j+1].set((float)j/5.0f, 1.0f - (float)j/5.0f, 0.5f, 1.0f);
   }
   vertex[10].set(0.f, 0.f, 4.0f);
   color[10].set(1.f, 1.f, 1.f, 1.f);

   for (j=6; j<11; j++)
   {
      vertex[2*j-1].set(29.f - 3.f*j, 3.f, 4.0f);
      vertex[2*j].set(29.f - 3.f*j, 3.f, -4.0f);
      color[2*j-1].set((float)(j-6)/5.0f, 0.8f, 1.0f -(float)(j-6)/5.0f, 1.0f);
      color[2*j].set((float)(j-6)/5.0f, 1.0f - (float)(j-6)/5.0f, 0.5f, 1.0f);
   }
   vertex[21].set(0.f, 0.f, -4.0f);
   color[21].set(1.f, 1.f, 1.f, 1.f);

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makePolygonSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("PolygonSet");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   sglPolygonSet *gset = new sglPolygonSet();

   sglVec3f *vertex = new sglVec3f[18];

   sglVec4f *color  = new sglVec4f[1];
   color[0].set(0.f, 1.f, 1.f, 1.f);

   unsigned int *length = new unsigned int[3];
   length[0] = 5;
   length[1] = 6;
   length[2] = 7;

   int index = 0;
   for (int j=0; j<3; j++)
   {
      for (unsigned int i=0; i<length[j]; i++)
      {
         vertex[index++].set(
            (8.f*j-8.f) + (float)(3.0*cos(2.0*M_PI*i/(double)length[j])),
            0.0f,
            (float)(3.0*sin(2.0*M_PI*i/(double)length[j])));
      }
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setPrimLengths(length);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}


//----------------------------------------------------------------------------
sglGroup* makeIndexedTriangleSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedTriangleSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglIndexedTriangleSet *gset = new sglIndexedTriangleSet();

   sglVec3f *vertex = new sglVec3f[9];
   unsigned int *vindex = new unsigned int[9];

   sglVec4f *color  = new sglVec4f[3];
   unsigned int *cindex = new unsigned int[3];

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      vertex[index++].set(-6.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      vertex[index++].set(-2.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      vertex[index++].set(-4.0f + j*6.0f, 0.f, 6.0f);

      cindex[j] = j;
      color[j].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setColorIndexList(cindex);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedTriangleStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedTriangleStripSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglIndexedTriangleStripSet *gset = new sglIndexedTriangleStripSet();

   sglVec3f *vertex = new sglVec3f[16];
   sglVec4f *color  = new sglVec4f[16];
   unsigned int *vindex = new unsigned int[16];
   unsigned int *length = new unsigned int[2];
   length[0] = 8;
   length[1] = 8;

   int j;
   for (j=0; j<4; j++)
   {
      vindex[2*j] = 2*j;
      vindex[2*j+1] = 2*j + 1;
      vertex[2*j].set(3.f*j - 4.5f, 0.f, 4.0f);
      vertex[2*j+1].set(3.f*j - 4.5f, 0.f, -4.0f);
      color[2*j].set((float)j/4.0f, 0.8f, 1.0f - (float)j/4.0f, 1.0f);
      color[2*j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   for (j=4; j<8; j++)
   {
      vindex[2*j] = 2*j;
      vindex[2*j+1] = 2*j + 1;
      vertex[2*j].set(10.5f - 3.f*j, 4.f, 4.0f);
      vertex[2*j+1].set(10.5f - 3.f*j, 4.f, -4.0f);
      color[2*j].set((float)(j-4)/4.0f, 0.8f, 1.0f - (float)(j-4)/4.0f, 1.0f);
      color[2*j+1].set((float)(j-4)/4.0f, 1.0f -(float)(j-4)/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorIndexList(vindex);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedTriangleFanSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedTriangleFanSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglIndexedTriangleFanSet *gset = new sglIndexedTriangleFanSet();

   sglVec3f *vertex = new sglVec3f[10];
   sglVec4f *color  = new sglVec4f[10];
   unsigned int *vindex = new unsigned int[10];
   unsigned int *length = new unsigned int[2];
   length[0] = 5;
   length[1] = 5;

   int j;

   vindex[0] = 0;
   vertex[0].set(0.f, 0.f, -5.f);
   color[0].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (j=0; j<4; j++)
   {
      vindex[j+1] = j+1;
      vertex[j+1].set((float)(10.0*cos(j/2.0)), 0.f, (float)(10.0*sin(j/2.0)));
      color[j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   vindex[5] = 5;
   vertex[5].set(-4.f, 4.f, -5.f);
   color[5].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (j=5; j<9; j++)
   {
      vindex[j+1] = j+1;
      vertex[j+1].set((float)(10.0*cos((10-j)/2.0))-4.f, 4.f,
                      (float)(10.0*sin((10-j)/2.0)));
      color[j+1].set((float)(j-5)/4.0f, 1.0f - (float)(j-5)/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorIndexList(vindex);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedQuadSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedQuadSet");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   sglIndexedQuadSet *gset = new sglIndexedQuadSet();

   sglVec3f *vertex = new sglVec3f[12];
   unsigned int *vindex = new unsigned int[12];
   sglVec4f *color  = new sglVec4f[1];
   unsigned int *cindex = new unsigned int[1];

   cindex[0] = 0;
   color[0].set(0.f, 1.f, 1.f, 1.f);

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      vertex[index++].set(6.f*j - 8.f, 0.f, 4.0f);
      vindex[index] = index;
      vertex[index++].set(6.f*j - 8.f, 0.f, -4.0f);
      vindex[index] = index;
      vertex[index++].set(6.f*j - 4.f, 0.f, -4.0f);
      vindex[index] = index;
      vertex[index++].set(6.f*j - 4.f, 0.f, 4.0f);
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorIndexList(cindex);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedQuadStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedQuadStripSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglIndexedQuadStripSet *gset = new sglIndexedQuadStripSet();

   sglVec3f *vertex = new sglVec3f[22];
   sglVec4f *color  = new sglVec4f[22];
   unsigned int *vindex = new unsigned int[22];
   unsigned int *length = new unsigned int[2];
   length[0] = 11;
   length[1] = 11;

   int j;
   unsigned int index = 0;
   for (j=0; j<5; j++)
   {
      vertex[index].set(3.f*j - 12.f, 0.f, 4.0f);
      color[index].set((float)j/5.0f, 0.8f, 1.0f - (float)j/5.0f, 1.0f);
      vindex[index] = index;
      index++;

      vertex[index].set(3.f*j - 12.f, 0.f, -4.0f);
      color[index].set((float)j/5.0f, 1.0f - (float)j/5.0f, 0.5f, 1.0f);
      vindex[index] = index;
      index++;
   }
   vertex[10].set(0.f, 0.f, 4.0f);
   color[10].set(1.f, 1.f, 1.f, 1.f);
   vindex[index] = index;
   index++;

   for (j=6; j<11; j++)
   {
      vertex[index].set(29.f - 3.f*j, 3.f, 4.0f);
      color[index].set((float)(j-6)/5.0f, 0.8f, 1.0f -(float)(j-6)/5.0f, 1.0f);
      vindex[index] = index;
      index++;

      vertex[index].set(29.f - 3.f*j, 3.f, -4.0f);
      color[index].set((float)(j-6)/5.0f, 1.0f -(float)(j-6)/5.0f, 0.5f, 1.0f);
      vindex[index] = index;
      index++;
   }
   vertex[21].set(0.f, 0.f, -4.0f);
   color[21].set(1.f, 1.f, 1.f, 1.f);
   vindex[21] = 21;

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setColorIndexList(vindex);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeIndexedPolygonSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("IndexedPolygonSet");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   sglIndexedPolygonSet *gset = new sglIndexedPolygonSet();

   sglVec3f *vertex = new sglVec3f[18];
   unsigned int *vindex = new unsigned int[18];

   sglVec4f *color  = new sglVec4f[1];
   unsigned int *cindex = new unsigned int[1];
   cindex[0] = 0;
   color[0].set(0.f, 1.f, 1.f, 1.f);

   unsigned int *length = new unsigned int[3];
   length[0] = 5;
   length[1] = 6;
   length[2] = 7;

   int index = 0;
   for (int j=0; j<3; j++)
   {
      for (unsigned int i=0; i<length[j]; i++)
      {
         vindex[index] = index;
         vertex[index++].set(
            (8.f*j-8.f) + (float)(3.0*cos(2.0*M_PI*i/(double)length[j])),
            0.0f,
            (float)(3.0*sin(2.0*M_PI*i/(double)length[j])));
      }
   }

   gset->setCoordList(vertex);
   gset->setCoordIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setColorIndexList(cindex);
   gset->setPrimLengths(length);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}


//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedTriangleSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedTriangleSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglMonoIndexedTriangleSet *gset = new sglMonoIndexedTriangleSet();

   sglVec3f *vertex = new sglVec3f[9];
   sglVec4f *color  = new sglVec4f[9];
   unsigned int *vindex = new unsigned int[9];

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      color[index].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
      vertex[index++].set(-6.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      color[index].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
      vertex[index++].set(-2.0f + j*6.0f, 0.f, -6.0f);
      vindex[index] = index;
      color[index].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
      vertex[index++].set(-4.0f + j*6.0f, 0.f, 6.0f);
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedTriangleStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedTriangleStripSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglMonoIndexedTriangleStripSet *gset = new sglMonoIndexedTriangleStripSet();

   sglVec3f *vertex = new sglVec3f[16];
   sglVec4f *color  = new sglVec4f[16];
   unsigned int *vindex = new unsigned int[16];
   unsigned int *length = new unsigned int[2];
   length[0] = 8;
   length[1] = 8;

   int j;
   unsigned int index = 0;
   for (j=0; j<4; j++)
   {
      vindex[index] = index;
      vertex[index].set(3.f*j - 4.5f, 0.f, 4.0f);
      color[index].set((float)j/4.0f, 0.8f, 1.0f - (float)j/4.0f, 1.0f);
      index++;

      vindex[index] = 2*j + 1;
      vertex[index].set(3.f*j - 4.5f, 0.f, -4.0f);
      color[index].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
      index++;
   }

   for (j=4; j<8; j++)
   {
      vindex[index] = index;
      vertex[index].set(10.5f - 3.f*j, 4.f, 4.0f);
      color[index].set((float)(j-4)/4.0f, 0.8f, 1.0f- (float)(j-4)/4.0f, 1.0f);
      index++;

      vindex[index] = 2*j + 1;
      vertex[index].set(10.5f - 3.f*j, 4.f, -4.0f);
      color[index].set((float)(j-4)/4.0f, 1.0f- (float)(j-4)/4.0f, 0.5f, 1.0f);
      index++;
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedTriangleFanSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedTriangleFanSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglMonoIndexedTriangleFanSet *gset = new sglMonoIndexedTriangleFanSet();

   sglVec3f *vertex = new sglVec3f[10];
   sglVec4f *color  = new sglVec4f[10];
   unsigned int *vindex = new unsigned int[10];
   unsigned int *length = new unsigned int[2];
   length[0] = 5;
   length[1] = 5;

   int j;

   vindex[0] = 0;
   vertex[0].set(0.f, 0.f, -5.f);
   color[0].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (j=0; j<4; j++)
   {
      vindex[j+1] = j+1;
      vertex[j+1].set((float)(10.0*cos(j/2.0)), 0.f, (float)(10.0*sin(j/2.0)));
      color[j+1].set((float)j/4.0f, 1.0f - (float)j/4.0f, 0.5f, 1.0f);
   }

   vindex[5] = 5;
   vertex[5].set(-4.f, 4.f, -5.f);
   color[5].set(1.0f, 1.0f, 1.0f, 1.0f);
   for (j=5; j<9; j++)
   {
      vindex[j+1] = j+1;
      vertex[j+1].set((float)(10.0*cos((10-j)/2.0))-4.f, 4.f,
                      (float)(10.0*sin((10-j)/2.0)));
      color[j+1].set((float)(j-5)/4.0f, 1.0f - (float)(j-5)/4.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedQuadSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedQuadSet");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   sglMonoIndexedQuadSet *gset = new sglMonoIndexedQuadSet();

   sglVec3f *vertex = new sglVec3f[12];
   unsigned int *vindex = new unsigned int[12];
   sglVec4f *color  = new sglVec4f[12];

   unsigned int index = 0;
   for (int j=0; j<3; j++)
   {
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 8.f, 0.f, 4.0f);
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 8.f, 0.f, -4.0f);
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 4.f, 0.f, -4.0f);
      vindex[index] = index;
      color[index].set(0.f, 1.0f, 1.0f, 1.0f);
      vertex[index++].set(6.f*j - 4.f, 0.f, 4.0f);
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);  // bind enum should be ignored
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedQuadStripSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedQuadStripSet");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglMonoIndexedQuadStripSet *gset = new sglMonoIndexedQuadStripSet();

   sglVec3f *vertex = new sglVec3f[22];
   sglVec4f *color  = new sglVec4f[22];
   unsigned int *vindex = new unsigned int[22];
   unsigned int *length = new unsigned int[2];
   length[0] = 11;
   length[1] = 11;

   int j;

   unsigned int index = 0;
   for (j=0; j<5; j++)
   {
      vertex[index].set(3.f*j - 12.f, 0.f, 4.0f);
      color[index].set((float)j/5.0f, 0.8f, 1.0f - (float)j/5.0f, 1.0f);
      vindex[index] = index;
      index++;

      vertex[index].set(3.f*j - 12.f, 0.f, -4.0f);
      color[index].set((float)j/5.0f, 1.0f - (float)j/5.0f, 0.5f, 1.0f);
      vindex[index] = index;
      index++;
   }
   vertex[10].set(0.f, 0.f, 4.0f);
   color[10].set(1.f, 1.f, 1.f, 1.f);
   vindex[index] = index;
   index++;

   for (j=6; j<11; j++)
   {
      vertex[index].set(29.f - 3.f*j, 3.f, 4.0f);
      color[index].set((float)(j-6)/5.0f, 0.8f, 1.0f -(float)(j-6)/5.0f, 1.0f);
      vindex[index] = index;
      index++;

      vertex[index].set(29.f - 3.f*j, 3.f, -4.0f);
      color[index].set((float)(j-6)/5.0f, 1.0f -(float)(j-6)/5.0f, 0.5f, 1.0f);
      vindex[index] = index;
      index++;
   }
   vertex[21].set(0.f, 0.f, -4.0f);
   color[21].set(1.f, 1.f, 1.f, 1.f);
   vindex[21] = 21;

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::ePER_VERTEX, color);
   gset->setNumPrims(2);
   gset->setPrimLengths(length);

   geode->addGeometry(gset);
   group->addChild(geode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeMonoIndexedPolygonSet()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("MonoIndexedPolygonSet");

   sglGeode  *geode = new sglGeode();
   geode->addGeometry(text);

   sglMonoIndexedPolygonSet *gset = new sglMonoIndexedPolygonSet();

   sglVec3f *vertex = new sglVec3f[18];
   sglVec4f *color  = new sglVec4f[18];
   unsigned int *vindex = new unsigned int[18];

   unsigned int *length = new unsigned int[3];
   length[0] = 5;
   length[1] = 6;
   length[2] = 7;

   int index = 0;
   for (int j=0; j<3; j++)
   {
      for (unsigned int i=0; i<length[j]; i++)
      {
         vindex[index] = index;
         color[index].set(0.f, 1.f, 1.f, 1.f);
         vertex[index++].set(
            (8.f*j-8.f) + (float)(3.0*cos(2.0*M_PI*i/(double)length[j])),
            0.0f,
            (float)(3.0*sin(2.0*M_PI*i/(double)length[j])));
      }
   }

   gset->setCoordList(vertex);
   gset->setIndexList(vindex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setPrimLengths(length);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}


//----------------------------------------------------------------------------
void buildScenegraph()
{
   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   switch_node = new sglSwitch();
   switch_node->setVal(switch_val);

   switch_node->addChild(makeTriangleSet());
   switch_node->addChild(makeIndexedTriangleSet());
   switch_node->addChild(makeMonoIndexedTriangleSet());

   switch_node->addChild(makeTriangleStripSet());
   switch_node->addChild(makeIndexedTriangleStripSet());
   switch_node->addChild(makeMonoIndexedTriangleStripSet());

   switch_node->addChild(makeTriangleFanSet());
   switch_node->addChild(makeIndexedTriangleFanSet());
   switch_node->addChild(makeMonoIndexedTriangleFanSet());

   switch_node->addChild(makeQuadSet());
   switch_node->addChild(makeIndexedQuadSet());
   switch_node->addChild(makeMonoIndexedQuadSet());

   switch_node->addChild(makeQuadStripSet());
   switch_node->addChild(makeIndexedQuadStripSet());
   switch_node->addChild(makeMonoIndexedQuadStripSet());

   switch_node->addChild(makePolygonSet());
   switch_node->addChild(makeIndexedPolygonSet());
   switch_node->addChild(makeMonoIndexedPolygonSet());

   root->addChild(switch_node);

   // ================================================================
   isector.setMode(sglProbe::eGEODE|sglProbe::eGEOMETRY|sglProbe::ePRIMITIVE|
                   sglProbe::eTRANSFORM|sglProbe::eNORMAL|
                   sglProbe::eCULL_BACK);
   isector.setDiscriminatorMask(~0xffffffff);
   // ================================================================

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);
   view->setDrawCallback(preDrawFunc, NULL);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglPolyFrontStyle(sglPolyStyle::eFILLED));
   override_statelets.push_back(new sglPolyBackStyle(sglPolyStyle::eLINE));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testIsectGeoSets");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
