/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testIsectLOD.cpp
 *   Author: Tom Stimson
 *  Created: 17 April 2000
 *  Summary: test intersection with a lod node
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglTransform.hpp>
#include <sglTranslate.hpp>
#include <sglScale.hpp>
#include <sglLOD.hpp>
#include <sglLODState.hpp>
#include <sglLODStateNode.hpp>
#include <sglTextSet.hpp>
#include <sglGeode.hpp>
#include <sglTriangleSet.hpp>

#include <sglSegment.hpp>
#include <sglIntersect.hpp>
#include <sglSphereBound.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

bool help_flag = false;

sglLOD      *lod_node;
sglLODState *lod_state;
sglLODStateNode *lod_state_node;

sglScalef   *scale_node;
sglTranslatef *translate_node;
sglTransformf *transform_node;
float rot = 0;

sglIntersectf  isector;
sglSegmentf    segment;

bool paused_flag = false;
sglVec3f pos(0.0, -10.0, 0.0);
double heading = 0.0, pitch = 0;
float len = 20.0;
float angle = 0.0;

unsigned int win_x = 640, win_y = 480;

//----------------------------------------------------------------------------
void outputHelp()
{
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glEnable(GL_BLEND);
   glColor3f(1.f, 1.f, 1.f);

   glRasterPos3i(10,175,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "/? - toggle help");
   glRasterPos3i(10,160,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "v - cycle through lod intersect modes");
   glRasterPos3i(10,145,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "g/G dec/inc global LOD scale factor");
   glRasterPos3i(10,130,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "l/L dec/inc LOD state scale factor");
   glRasterPos3i(10,115,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "k/K dec/inc LOD state-node scale factor");
   glRasterPos3i(10,100,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "s/S dec/inc scale node factor");
   glRasterPos3i(10,85,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "t/T dec/inc translation");
   glRasterPos3i(10,70,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "r/R dec/inc rotation");
   glRasterPos3i(10,55,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "f - toggle culling front face");
   glRasterPos3i(10,40,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "b - toggle culling back face");
   glDisable(GL_BLEND);
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------
void preDrawFunc(void *)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);

   if (!paused_flag)
   {
      heading += 0.005;
      if (heading > M_PI/4.0) heading -= M_PI/2.0;
   }

   segment.makePolar(pos, heading, pitch, len);

   isector.reset();
   isector.setSegment(segment);
   sglMat4f view_mat;
   view_platform->computeViewMatrix(view_mat);
   isector.setModelViewMatrix(view_mat);
   float fov_y = view_camera->getFOVY();
   float fov_x = view_camera->getFOVX();
   isector.setLODScale(sglScene::computeLODScale(fov_x, fov_y, win_x, win_y));
   sglBound::IntersectResultEnum result = root->intersect(isector);
   glColor3f(0.0f, 1.0f, 0.0f);
   if (result == sglBound::eINTERSECT)
   {
      sglVec3f endpoint = isector.getSegment().getEndPoint();
      sglMat4f matrix = isector.getTransformMatrix();
      sglVec3f normal = isector.getNormal();
      sglVec4f n;

      mul(normal, 0.f, matrix, n);

      // undo the affects of the view matrix
      sglVec4f n1;
      matrix.inverse(view_mat);
      mul(sglVec3f(n[0], n[1], n[2]), 0.f, matrix, n1);

      sglSegmentf nseg(endpoint, sglVec3f(n1[0], n1[1], n1[2]), 4.0f);
      glColor3f(0.f, 0.f, 1.f);

      nseg.drawImmediate();

      glColor3f(1.0f, 0.4f, 0.0f);
      deque<sglNode*> &path = isector.getPath();
      if (path.size() == 8)
      {
         if (path[0] != root || path[1] != translate_node ||
             path[2] != transform_node || path[3] != scale_node ||
             path[4] != lod_state_node || path[5] != lod_node ||
             dynamic_cast<sglGroup*>(path[6]) == NULL ||
             dynamic_cast<sglGeode*>(path[7]) == NULL)
            cerr << "ERROR1: path to intersect is incorrect!" << endl;
      }
      else if (path.size() == 9)
      {
         if (path[0] != root || path[1] != translate_node ||
             path[2] != transform_node || path[3] != scale_node ||
             path[4] != lod_state_node || path[5] != lod_node ||
             dynamic_cast<sglGroup*>(path[6]) == NULL ||
             dynamic_cast<sglTransformd*>(path[7]) == NULL ||
             dynamic_cast<sglGeode*>(path[8]) == NULL)
            cerr << "ERROR2: path to intersect is incorrect!" << endl;
      }
      else
         cerr << "ERROR3: path to intersect is incorrect!" << endl;

   }
   (isector.getSegment()).drawImmediate();
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   if (help_flag)
   {
      outputHelp();
   }

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'f':
         if (isector.getMode() & sglProbe::eCULL_FRONT)
         {
            cout << "Intersecting with front facing polygons" << endl;
            isector.setMode(isector.getMode() & ~sglProbe::eCULL_FRONT);
         }
         else
         {
            cout << "Not intersecting with front facing polygons" << endl;
            isector.setMode(isector.getMode() | sglProbe::eCULL_FRONT);
         }
         break;

      case 'b':
         if (isector.getMode() & sglProbe::eCULL_BACK)
         {
            cout << "Intersecting with back facing polygons" << endl;
            isector.setMode(isector.getMode() & ~sglProbe::eCULL_BACK);
         }
         else
         {
            cout << "Not intersecting with back facing polygons" << endl;
            isector.setMode(isector.getMode() | sglProbe::eCULL_BACK);
         }
         break;

      case 'v':
         if (isector.getMode() & sglProbe::eLOD_ALL)
         {
            cout << "Intersecting with no lod levels" << endl;
            isector.setMode(isector.getMode() & ~(sglProbe::eLOD_ALL|
                                                  sglProbe::eLOD_NONE|
                                                  sglProbe::eLOD_VIEW));
            isector.setMode(isector.getMode() | sglProbe::eLOD_NONE);
         }
         else if (isector.getMode() & sglProbe::eLOD_NONE)
         {
            cout << "Intersecting with the visible lod level" << endl;
            isector.setMode(isector.getMode() & ~(sglProbe::eLOD_ALL|
                                                  sglProbe::eLOD_NONE|
                                                  sglProbe::eLOD_VIEW));
            isector.setMode(isector.getMode() | sglProbe::eLOD_VIEW);
         }
         else if (isector.getMode() & sglProbe::eLOD_VIEW)
         {
            cout << "Intersecting with first lod level" << endl;
            isector.setMode(isector.getMode() & ~(sglProbe::eLOD_ALL|
                                                  sglProbe::eLOD_NONE|
                                                  sglProbe::eLOD_VIEW));
         }
         else
         {
            cout << "Intersecting with all lod levels" << endl;
            isector.setMode(isector.getMode() & ~(sglProbe::eLOD_ALL|
                                                  sglProbe::eLOD_NONE|
                                                  sglProbe::eLOD_VIEW));
            isector.setMode(isector.getMode() | sglProbe::eLOD_ALL);
         }
         break;

      case 'g':
         sglLOD::setScaleFactor(sglLOD::getScaleFactor()/1.2f);
         break;
      case 'G':
         sglLOD::setScaleFactor(sglLOD::getScaleFactor()*1.2f);
         break;

      case 'l':
         lod_state->setRangeScale(lod_state->getRangeScale()/1.2f);
         break;
      case 'L':
         lod_state->setRangeScale(lod_state->getRangeScale()*1.2f);
         break;

      case 'k':
         lod_state_node->setRangeScale(lod_state_node->getRangeScale()/1.2f);
         break;
      case 'K':
         lod_state_node->setRangeScale(lod_state_node->getRangeScale()*1.2f);
         break;

      case 's':
         scale_node->setScale(scale_node->getScale()/1.2f);
         break;
      case 'S':
         scale_node->setScale(scale_node->getScale()*1.2f);
         break;

      case 't':
      {
         float x, y, z;
         translate_node->getTranslation(x, y, z);
         x -= 1.0f;
         translate_node->setTranslation(x, y, z);
         break;
      }
      case 'T':
      {
         float x, y, z;
         translate_node->getTranslation(x, y, z);
         x += 1.0f;
         translate_node->setTranslation(x, y, z);
         break;
      }

      case 'r':
         rot -= (float)(M_PI/4.);
         transform_node->setRotation(rot, 0, 0);
         break;
      case 'R':
         rot += (float)(M_PI/4.);
         transform_node->setRotation(rot, 0, 0);
         break;

      case 'p':
         paused_flag = !paused_flag;
         break;

      case '?':
      case '/':
         help_flag = !help_flag;
         cout << "help_flag = " << help_flag << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleSet1()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleSet1");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglTriangleSet *gset = new sglTriangleSet();

   sglVec3f *vertex = new sglVec3f[3];
   sglVec4f *color  = new sglVec4f[1];

   vertex[0].set(0.0f, 0.f, -6.0f);
   vertex[1].set(4.0f, 0.f, -6.0f);
   vertex[2].set(-2.0f, 0.f, 6.0f);

   color[0].set(1.0f, 0.5f, 0.5f, 1.0f);

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setNumPrims(1);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleSet2()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleSet2");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglTriangleSet *gset = new sglTriangleSet();

   sglVec3f *vertex = new sglVec3f[6];
   sglVec4f *color  = new sglVec4f[2];

   for (int j=0; j<2; j++)
   {
      vertex[j*3 + 0].set(-3.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 1].set( 1.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 2].set(-1.0f + j*6.0f, 0.f, 6.0f);

      color[j].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setNumPrims(2);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
sglGroup* makeTriangleSet3()
{
   sglGroup *group = new sglGroup;
   sglTextSet  *text  = new sglTextSet;
   text->setString("TriangleSet3");

   sglGeode *geode = new sglGeode();
   geode->addGeometry(text);

   sglTriangleSet *gset = new sglTriangleSet();

   sglVec3f *vertex = new sglVec3f[9];
   sglVec4f *color  = new sglVec4f[3];

   for (int j=0; j<3; j++)
   {
      vertex[j*3 + 0].set(-6.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 1].set(-2.0f + j*6.0f, 0.f, -6.0f);
      vertex[j*3 + 2].set(-4.0f + j*6.0f, 0.f, 6.0f);

      color[j].set((float)j/3.0f, 1.0f - (float)j/3.0f, 0.5f, 1.0f);
   }

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::ePER_PRIM, color);
   gset->setNumPrims(3);

   geode->addGeometry(gset);
   group->addChild(geode);

   sglTransformd *txnode = new sglTransformd;
   txnode->setRotation(M_PI, 0., 0., 1.);
   txnode->setTranslation(0., 4., 0.);
   txnode->addChild(geode);
   group->addChild(txnode);

   return group;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   translate_node = new sglTranslatef();
   translate_node->setTranslation(0, 0, 0);
   root->addChild(translate_node);

   transform_node = new sglTransformf();
   transform_node->loadIdentity();
   translate_node->addChild(transform_node);

   scale_node = new sglScalef();
   scale_node->setScale(1.0f);
   transform_node->addChild(scale_node);

   lod_state_node = new sglLODStateNode();
   scale_node->addChild(lod_state_node);

   lod_node = new sglLOD();
   lod_node->addChild(makeTriangleSet3());
   lod_node->addChild(makeTriangleSet2());
   lod_node->addChild(makeTriangleSet1());
   lod_node->setRange(0, 0);
   lod_node->setRange(1, 50);
   lod_node->setRange(2, 75);
   lod_node->setRange(3, 100);

   lod_state = new sglLODState();
   lod_node->setLODState(lod_state);

   lod_state_node->addChild(lod_node);

   // ================================================================
   isector.setMode(sglProbe::eGEODE|sglProbe::eGEOMETRY|sglProbe::ePRIMITIVE|
                   sglProbe::eTRANSFORM|sglProbe::eNORMAL|
                   sglProbe::eCULL_BACK|sglProbe::eLOD_VIEW|sglProbe::ePATH);
   isector.setDiscriminatorMask(~0xffffffff);
   // ================================================================

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);
   view->setDrawCallback(preDrawFunc, NULL);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglPolyFrontStyle(sglPolyStyle::eFILLED));
   override_statelets.push_back(new sglPolyBackStyle(sglPolyStyle::eLINE));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testIsectLOD");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
