/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testIsectPrim.cpp
 *   Author: Scott McMillan
 *  Created: 1999
 *  Summary: textual test of intersection with geometry primitives
 *****************************************************************************/

#include <GL/glut.h>
#include <sglSegment.hpp>
#include <sglIntersect.hpp>
#include <sglBoxBound.hpp>
#include <sglSphereBound.hpp>
#include <sglTriangleSet.hpp>
#include <sglIndexedTriangleSet.hpp>

// -----------------------------------------------------------------------

void testTriangleSet()
{
   cerr << "============ Testing sglTriangleSet ===============" << endl;
   sglTriangleSet gset;
   gset.enableDisplayList(false);
   gset.setNumPrims(1);
   sglVec3f *vertices = new sglVec3f[3];
   vertices[0].set( 1.0f, -1.0f, 0.0f);
   vertices[1].set( 0.0f,  1.0f, 0.0f);
   vertices[2].set(-1.0f, -1.0f, 0.0f);
   gset.setCoordList(vertices);
   gset.preDraw();

   sglBound::IntersectResultEnum hit;
   sglSegmentf seg;
   sglIntersectf segset;
   segset.setMode(sglProbe::ePRIMITIVE);
   sglVec3f p1(0.0f, 0.0f,  3.0f);
   sglVec3f p2(0.0f, 0.0f, -3.0f);

   cerr << "---------------- testing front face -----------------" << endl;
   seg.makePts(p1, p2);
   segset.setSegment(seg);
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "Hit at: " << pos << endl;
   }
   else
   {
      cerr << "ERROR" << endl;
   }

   cerr << "---------------- testing back face -----------------" << endl;
   seg.makePts(p2, p1);
   segset.setSegment(seg);
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "Hit at: " << pos << endl;
   }
   else
   {
      cerr << "ERROR" << endl;
   }

   cerr << "---------------- testing back face (culled) -----------------"
        << endl;
   seg.makePts(p2, p1);
   segset.setSegment(seg);
   segset.setMode(segset.getMode()|sglProbe::eCULL_BACK);
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "ERROR: Hit at: " << pos << endl;
   }
   else
   {
      cerr << "No hit (correct)" << endl;
   }

   cerr << "---------------- testing parallel ray -----------------" << endl;
   p1.set(0.0f,  3.0f, 0.0f);
   p2.set(0.0f, -3.0f, 0.0f);
   seg.makePts(p1, p2);
   segset.setSegment(seg);
   segset.setMode(segset.getMode()&(~sglProbe::eCULL_BACK));
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "ERROR: Hit at: " << pos << endl;
   }
   else
   {
      cerr << "No hit (correct)" << endl;
   }

   cerr << "---------------- testing multiple triangles -----------------"
        << endl;
   sglTriangleSet gset2;
   gset2.enableDisplayList(false);
   gset2.setNumPrims(2);
   sglVec3f *vertices2 = new sglVec3f[6];
   vertices2[0].set( 1.0f, -1.0f, 0.0f);
   vertices2[1].set( 0.0f,  1.0f, 0.0f);
   vertices2[2].set(-1.0f, -1.0f, 0.0f);
   vertices2[3].set( 1.0f, -1.0f, 1.0f);
   vertices2[4].set( 0.0f,  1.0f, 1.0f);
   vertices2[5].set(-1.0f, -1.0f, 1.0f);
   gset2.setCoordList(vertices2);
   gset2.preDraw();

   p1.set(0.0f, 0.0f,  3.0f);
   p2.set(0.0f, 0.0f, -3.0f);
   seg.makePts(p1, p2);
   segset.setSegment(seg);

   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset2.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "Hit at: " << pos << endl;
   }
   else
   {
      cerr << "ERROR: No hit" << endl;
   }

}

// -----------------------------------------------------------------------

void testIndexedTriangleSet()
{
   cerr << "========= Testing sglIndexedTriangleSet ===========" << endl;
   sglIndexedTriangleSet gset;
   gset.enableDisplayList(false);
   gset.setNumPrims(1);
   sglVec3f *vertices = new sglVec3f[3];
   vertices[0].set( 1.0f, -1.0f, 0.0f);
   vertices[1].set( 0.0f,  1.0f, 0.0f);
   vertices[2].set(-1.0f, -1.0f, 0.0f);
   gset.setCoordList(vertices);
   unsigned int *vindices = new unsigned int[3];
   vindices[0] = 0;
   vindices[1] = 1;
   vindices[2] = 2;
   gset.setCoordIndexList(vindices);
   gset.preDraw();

   sglBound::IntersectResultEnum hit;
   sglSegmentf seg;
   sglIntersectf segset;
   segset.setMode(sglProbe::ePRIMITIVE);
   sglVec3f p1(0.0f, 0.0f,  3.0f);
   sglVec3f p2(0.0f, 0.0f, -3.0f);

   cerr << "---------------- testing front face -----------------" << endl;
   seg.makePts(p1, p2);
   segset.setSegment(seg);
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "Hit at: " << pos << endl;
   }
   else
   {
      cerr << "ERROR" << endl;
   }

   cerr << "---------------- testing back face -----------------" << endl;
   seg.makePts(p2, p1);
   segset.setSegment(seg);
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "Hit at: " << pos << endl;
   }
   else
   {
      cerr << "ERROR" << endl;
   }

   cerr << "---------------- testing back face (culled) -----------------"
        << endl;
   seg.makePts(p2, p1);
   segset.setSegment(seg);
   segset.setMode(segset.getMode()|sglProbe::eCULL_BACK);
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "ERROR: Hit at: " << pos << endl;
   }
   else
   {
      cerr << "No hit (correct)" << endl;
   }

   cerr << "---------------- testing parallel ray -----------------" << endl;
   p1.set(0.0f,  3.0f, 0.0f);
   p2.set(0.0f, -3.0f, 0.0f);
   seg.makePts(p1, p2);
   segset.setSegment(seg);
   segset.setMode(segset.getMode()&(~sglProbe::eCULL_BACK));
   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "ERROR: Hit at: " << pos << endl;
   }
   else
   {
      cerr << "No hit (correct)" << endl;
   }

   cerr << "---------------- testing multiple triangles -----------------"
        << endl;
   sglIndexedTriangleSet gset2;
   gset2.enableDisplayList(false);
   gset2.setNumPrims(2);
   sglVec3f *vertices2 = new sglVec3f[6];
   vertices2[0].set( 1.0f, -1.0f, 0.0f);
   vertices2[1].set( 0.0f,  1.0f, 0.0f);
   vertices2[2].set(-1.0f, -1.0f, 0.0f);
   vertices2[3].set( 1.0f, -1.0f, 1.0f);
   vertices2[4].set( 0.0f,  1.0f, 1.0f);
   vertices2[5].set(-1.0f, -1.0f, 1.0f);
   gset2.setCoordList(vertices2);
   unsigned int *vindices2 = new unsigned int[6];
   vindices2[0] = 0;
   vindices2[1] = 1;
   vindices2[2] = 2;
   vindices2[3] = 3;
   vindices2[4] = 4;
   vindices2[5] = 5;
   gset2.setCoordIndexList(vindices2);
   gset2.preDraw();

   p1.set(0.0f, 0.0f,  3.0f);
   p2.set(0.0f, 0.0f, -3.0f);
   seg.makePts(p1, p2);
   segset.setSegment(seg);

   cerr << "Initial segment: " << segset.getSegment() << endl;

   hit = gset2.intersectPrim(segset);
   cerr << "  Final segment: " << segset.getSegment() << endl;

   if (hit == sglBound::eINTERSECT)
   {
      sglVec3f pos = segset.getSegment().getDir();
      pos *= segset.getSegment().getLen();
      pos += segset.getSegment().getPos();

      cerr << "Hit at: " << pos << endl;
   }
   else
   {
      cerr << "ERROR: No hit" << endl;
   }

}

// -----------------------------------------------------------------------

int main(int argc, char **argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testIsectPrim");

   sgl::initialize();

   testTriangleSet();
   testIndexedTriangleSet();
   return 0;
}
