/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testUnScale.cpp
 *  Created: Fall 2000
 *  Summary: GLUT-based sgl testbed
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglDefaultStateNode.hpp>
#include <sglTranslate.hpp>
#include <sglBillboard.hpp>
#include <sglUnScale.hpp>
#include <sglGeode.hpp>
#include <sglMonoIndexedQuadSet.hpp>
#include <sglQuadSet.hpp>

#include <sglSegment.hpp>
#include <sglIntersect.hpp>
#include <sglSphereBound.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglOrthographicCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *perspective_camera;
sglOrthographicCamera     *orthographic_camera;

deque<const sglStatelet*> override_statelets;
sglAntiAliasPoint   aa_point;
sglAntiAliasLine    aa_line;
sglAntiAliasPolygon aa_poly;

sglIntersectf  isector;
sglSegmentf    segment;
sglVec3f pos(0.0, 0.0, 0.0);
double heading = 0.0, pitch = 0;
float len = 30.0;

bool paused_flag = false;
bool ortho = false;
bool draw_grid = 0;
double fov_y = M_PI/3.0;
unsigned int win_x = 640, win_y = 480;
double aspect = 640./480.;


//----------------------------------------------------------------------------
void preDrawFunc(void *)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);

   if (draw_grid)
   {
      glPushAttrib(GL_ALL_ATTRIB_BITS); // VERY COSTLY?
      glPushClientAttrib(~0x0u); //GL_ALL_CLIENT_ATTRIB_BITS);

      glMatrixMode (GL_PROJECTION);
      glPushMatrix();
      glLoadIdentity();
      gluOrtho2D(0, win_x, 0, win_y);

      glMatrixMode (GL_MODELVIEW);
      glPushMatrix();
      glLoadIdentity();

      glDepthMask(GL_FALSE);
      glDisable(GL_DEPTH_TEST);

      unsigned int i;
      glColor3f(0.3f, 0.3f, 0.3f);
      for (i=5; i<win_x; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(i, 0);
         glVertex2i(i, win_y);
         glEnd();
      }
      for (i=5; i<win_y; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(0, i);
         glVertex2i(win_x, i);
         glEnd();
      }
      glColor3f(0.5f, 0.5f, 0.5f);
      for (i=0; i<win_x; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(i, 0);
         glVertex2i(i, win_y);
         glEnd();
      }
      for (i=0; i<win_y; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(0, i);
         glVertex2i(win_x, i);
         glEnd();
      }

      glMatrixMode(GL_PROJECTION);
      glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();

      glPopAttrib();
      glPopClientAttrib();
   }

   if (!paused_flag)
   {
      heading += 0.005;
      if (heading > M_PI) heading -= M_PI;
   }

   sglFrustumf frustum;
   if (ortho)
   {
      frustum = orthographic_camera->getFrustum();
   }
   else
   {
      frustum = perspective_camera->getFrustum();
   }

   segment.makePolar(pos, heading, pitch, len);

   isector.reset();
   //isector.setResultEnum(sglBound::eOUTSIDE);
   //isector.getModelViewMatrix().buildIdentity();
   isector.setSegment(segment);
   isector.setViewFrustum(frustum);
   isector.setViewportSize(win_x, win_y);

   sglMat4f view_mat;
   view_platform->computeViewMatrix(view_mat);
   isector.setModelViewMatrix(view_mat);

   sglBound::IntersectResultEnum result = root->intersect(isector);
   glColor3f(0.0f, 1.0f, 0.0f);
   if (isector.getResultEnum() == sglBound::eINTERSECT ||
       result == sglBound::eINTERSECT) // huh?
   {
      sglVec3f endpoint = isector.getSegment().getEndPoint();
      sglMat4f matrix = isector.getTransformMatrix();
      sglVec3f normal = isector.getNormal();
      sglVec4f n;

      mul(normal, 0.f, matrix, n);

      sglSegmentf nseg(endpoint, sglVec3f(n[0], n[1], n[2]), 4.0f);
      glColor3f(0.f, 0.f, 1.f);

      nseg.drawImmediate();

      glColor3f(1.0f, 0.0f, 0.0f);
   }
   (isector.getSegment()).drawImmediate();
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   aspect = (double)w/(double)h;

   if (ortho)
   {
#if 1
      orthographic_camera->setOrtho(-fov_y*aspect, fov_y*aspect,
                                    -fov_y, fov_y, 1.0, 200.);
#else
      orthographic_camera->setOrtho(1*aspect, fov_y*aspect,
                                    1, fov_y, 1.0, 200.);
#endif
   }
   else
   {
      perspective_camera->setFOV(fov_y, aspect, 1.0, 200.0);
   }
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'g':
         draw_grid = !draw_grid;
         break;

      case 'p':
         ortho = false;
         fov_y = M_PI/6.;
         view_platform->setCamera(*perspective_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'P':
         ortho = false;
         fov_y = M_PI/3.;
         view_platform->setCamera(*perspective_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'o':
         ortho = true;
         fov_y = 10.;
         view_platform->setCamera(*orthographic_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'O':
         ortho = true;
         fov_y = 20.;
         view_platform->setCamera(*orthographic_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'F':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 's':
         sgluOutputTree(root, cout, 3);
         break;

      case 'f':
         if (isector.getMode() & sglProbe::eCULL_FRONT)
         {
            cout << "Intersecting with front facing polygons" << endl;
            isector.setMode(isector.getMode() & ~sglProbe::eCULL_FRONT);
         }
         else
         {
            cout << "Not intersecting with front facing polygons" << endl;
            isector.setMode(isector.getMode() | sglProbe::eCULL_FRONT);
         }
         break;

      case 'b':
         if (isector.getMode() & sglProbe::eCULL_BACK)
         {
            cout << "Intersecting with back facing polygons" << endl;
            isector.setMode(isector.getMode() & ~sglProbe::eCULL_BACK);
         }
         else
         {
            cout << "Not intersecting with back facing polygons" << endl;
            isector.setMode(isector.getMode() | sglProbe::eCULL_BACK);
         }
         break;

      case 'm':
         paused_flag = !paused_flag;
         break;

      case '/':
      case '?':
         cout << "f - toggle intersection with front faces" << endl;
         cout << "b - toggle intersection with back faces" << endl;
         cout << "m - toggle (pause) segment motion" << endl;
         cerr << "a - toggle point anti-aliasing" << endl;
         cerr << "A - toggle line anti-aliasing" << endl;
         cerr << "z - toggle polygon anti-aliasing" << endl;
         cerr << "g - toggle grid" << endl;
         cerr << "p - 30-degree perspective view" << endl;
         cerr << "P - 60-degree perspective view" << endl;
         cerr << "o - 10-unit orthographic view" << endl;
         cerr << "O - 20-unit orthographic view" << endl;
         cerr << "F - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "s - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
sglGeoSet *makeBox()
{
   sglVec3f *vertex = new sglVec3f[8];
   sglVec4f *color = new sglVec4f[1];
   unsigned int *indices = new unsigned int[24];

   color[0][0] = 1.0; color[0][1] = 0.5; color[0][2] = 0.5; color[0][3] = 1.0;

   unsigned int index = 0;
   for (int x=-1; x<2; x+=2)
      for (int y=-1; y<2; y+=2)
         for (int z=-1; z<2; z+=2)
         {
            vertex[index][0] = x*0.05;
            vertex[index][1] = y*0.05;
            vertex[index][2] = z*0.05;
            index++;
         }

   indices[0] = 0; indices[1] = 1; indices[2] = 3; indices[3] = 2;
   indices[4] = 0; indices[5] = 2; indices[6] = 6; indices[7] = 4;
   indices[8] = 4; indices[9] = 5; indices[10] = 7; indices[11] = 6;
   indices[12] = 1; indices[13] = 3; indices[14] = 7; indices[15] = 5;
   indices[16] = 2; indices[17] = 6; indices[18] = 7; indices[19] = 3;
   indices[20] = 0; indices[21] = 1; indices[22] = 5; indices[23] = 4;

   sglMonoIndexedQuadSet *gset = new sglMonoIndexedQuadSet();
   gset->setCoordList(vertex);
   gset->setIndexList(indices);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(6);

   vector<sglStatelet*> statelets;
//   statelets.push_back(new sglTransparency(true));
   gset->setState(0, statelets);

   return gset;
}


//----------------------------------------------------------------------------
sglGeoSet *makeQuad()
{
   sglVec3f *vertex = new sglVec3f[5];
   sglVec4f *color = new sglVec4f[1];

   color[0][0] = 1.0; color[0][1] = 0.0; color[0][2] = 0.0; color[0][3] = 0.3f;

   float size = 20.0f;
   vertex[0][0] =  0.0;
   vertex[0][1] =  size;
   vertex[0][2] =  size;

   vertex[1][0] =  0.0;
   vertex[1][1] =  size;
   vertex[1][2] = -size;

   vertex[2][0] =  0.0;
   vertex[2][1] = -size;
   vertex[2][2] = -size;

   vertex[3][0] =  0.0;
   vertex[3][1] = -size;
   vertex[3][2] =  size;

   vertex[4][0] =  0.0;
   vertex[4][1] =  size;
   vertex[4][2] =  size;

   sglQuadSet *gset = new sglQuadSet();
   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(1);

   vector<sglStatelet*> statelets;
   statelets.push_back(new sglTransparency(true));
   gset->setState(0, statelets);

   return gset;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   root = new sglScene;

   vector<sglStatelet*> default_statelets;
   default_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   sglDefaultStateNode *def = new sglDefaultStateNode();
   def->setStatelets(default_statelets);
   root->addChild(def);

   sglUnScale *unscale = new sglUnScale();

   sglGeode *geode1 = new sglGeode;
   geode1->addGeometry(new sglAxes);
   unscale->addChild(geode1);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(makeBox());
   sglTranslatef *translate = new sglTranslatef();
   translate->setTranslation(20, 0, 0);
   translate->addChild(unscale);
   def->addChild(translate);

   unscale = new sglUnScale(true);
   geode = new sglGeode();
   unscale->addChild(geode);
   geode->addGeometry(makeQuad());
   translate = new sglTranslatef();
   translate->setTranslation(-20, 0, 0);
   //sglBillboard *billboard = new sglBillboard();
   //translate->addChild(billboard);
   //billboard->addChild(unscale);
   translate->addChild(unscale);
   def->addChild(translate);

   // ================================================================
   isector.setMode(sglProbe::eGEODE|sglProbe::eGEOMETRY|
                   sglProbe::ePRIMITIVE|
                   sglProbe::eTRANSFORM|sglProbe::eNORMAL|
                   sglProbe::eUNSCALE_VIEW);
   isector.setDiscriminatorMask(~0x000000001);  // mask out the tris
   // ================================================================

   // the cameras (frusta)
   perspective_camera = new sglPerspectiveCamera;
   perspective_camera->setFOV(fov_y, aspect, 1.0, 200.0);

   orthographic_camera = new sglOrthographicCamera;
   orthographic_camera->setOrtho(-20.0*aspect, 20.0*aspect,
                                 -20.0, 20.0, 1.0, 200.);

   // the view platform
   view_platform = new sglViewPlatform(*perspective_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(60.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode2 = new sglGeode;
   geode2->addGeometry(new sglAxes(2.0f));
   root->addChild(geode2);
   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);
   view->setDrawCallback(preDrawFunc, NULL);

   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testIsectUnScale");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
