/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testAnimation.cpp
 *  Created: 30 May 2001
 *  Summary: test sglKeyFrameAnimation and sglTimedAnimation nodes
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglVector.hpp>
#include <sglMatrix.hpp>

#include <sglGroup.hpp>
#include <sglScene.hpp>
#include <sglTranslate.hpp>
#include <sglKeyFrameAnimation.hpp>
#include <sglBitMaskSwitch.hpp>
#include <sglBitmapFont.hpp>
#include <sglGeode.hpp>
#include <sglQuadSet.hpp>
#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

#include <stdio.h>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*>  override_statelets;

sglAntiAliasLine aa_line;
sglAntiAliasPoint aa_point;
sglAntiAliasPolygon aa_poly;
sglPolyFrontStyle pfs;
sglPolyBackStyle pbs;

sglKeyFrameAnimation *animation = NULL;
sglAnimation::ModeEnum mode = sglAnimation::eCYCLE;
sglAnimation::StateEnum state = sglAnimation::eSTOP;
bool paused_flag = true;
char label[12];

unsigned int switch_vals[] = {0xfe8282fe,
                              0xfe000000,
                              0x62928a46,
                              0x6c929244,
                              0xfe1010f0,
                              0x9e9292f2,
                              0x9e9292fe,
                              0xfe808080,
                              0xfe9292fe,
                              0xfe9292f2};

unsigned int win_x = 640, win_y = 480;

float translationScaleFactor = 4.0e3;

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/3.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'F':
         if (pfs.getStyle() == sglPolyStyle::eFILLED)
            pfs.setStyle(sglPolyStyle::ePOINT);
         else if (pfs.getStyle() == sglPolyStyle::ePOINT)
            pfs.setStyle(sglPolyStyle::eLINE);
         else if (pfs.getStyle() == sglPolyStyle::eLINE)
            pfs.setStyle(sglPolyStyle::eFILLED);
         break;

      case 'B':
         if (pbs.getStyle() == sglPolyStyle::eFILLED)
            pbs.setStyle(sglPolyStyle::ePOINT);
         else if (pbs.getStyle() == sglPolyStyle::ePOINT)
            pbs.setStyle(sglPolyStyle::eLINE);
         else if (pbs.getStyle() == sglPolyStyle::eLINE)
            pbs.setStyle(sglPolyStyle::eFILLED);
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case 'O':
         animation->printInfo(cerr, "   ");
         break;

      case 't':
         animation->setOffsetFrame(40);
         cerr << "frame offset: 40" << endl;
         break;

      case 'T':
         animation->setOffsetFrame(0);
         cerr << "frame offset: 0" << endl;
         break;

      case 'P':
         if (animation->setState(sglAnimation::eSTART))
         {
            cerr << "state: START" << endl;
            state = sglAnimation::eSTART;
         }
         break;

      case 'p':
         switch (state)
         {
            case sglAnimation::eSTOP:
               state = sglAnimation::eSTART;
               cerr << "state: START" << endl;
               break;
            case sglAnimation::eSTART:
               state = sglAnimation::ePAUSE;
               cerr << "state: PAUSE" << endl;
               break;
            case sglAnimation::ePAUSE:
               state = sglAnimation::eRESUME;
               cerr << "state: RESUME" << endl;
               break;
            case sglAnimation::eRESUME:
               state = sglAnimation::eSTOP;
               cerr << "state: STOP" << endl;
               break;
         }
         animation->setState(state);
         break;

      case 'm':
         if (mode == sglAnimation::eCYCLE)
         {
            mode = sglAnimation::eSWING;
            cerr << "mode: SWING" << endl;
         }
         else
         {
            mode = sglAnimation::eCYCLE;
            cerr << "mode: CYCLE" << endl;
         }
         animation->setMode(mode);
         break;

      case '0':
         animation->setNumCycles(0);
         cerr << "cycles: continuous" << endl;
         break;
      case '1':
         animation->setNumCycles(1);
         cerr << "cycles: one-shot" << endl;
         break;
      case '2':
         animation->setNumCycles(2);
         cerr << "cycles: 2" << endl;
         break;

      case '3':
         animation->setSpeed(0.5f);
         cerr << "speed = 0.5" << endl;
         break;
      case '4':
         animation->setSpeed(1.f);
         cerr << "speed = 1.0" << endl;
         break;
      case '5':
         animation->setSpeed(2.f);
         cerr << "speed = 2.0" << endl;
         break;
      case '6':
         animation->setSpeed(-2.f);
         cerr << "speed = -2.0" << endl;
         break;

      case '7':
         animation->setRange(6, 7);
         cerr << "range = 6..7" << endl;
         break;
      case '8':
         animation->setRange(3, 8);
         cerr << "range = 3..8" << endl;
         break;
      case '9':
         animation->setRange();
         cerr << "range = all" << endl;
         break;

      case '/':
      case '?':
         cerr << "a - toggle point anti-aliasing" << endl;
         cerr << "A - toggle line anti-aliasing" << endl;
         cerr << "z - toggle polygon anti-aliasing" << endl;
         cerr << "F - cycle through front face polygon styles" << endl;
         cerr << "B - cycle through back face polygon styles" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "O - output sglKeyFrameAnimation node" << endl;
         cerr << "p - cycle through animation states" << endl;
         cerr << "P - set animation state to eSTART directly" << endl;
         cerr << "t/T - set frame offset to 3/0" << endl;
         cerr << "m - toggle mode" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "0/1/2   - set number of cycles (0 = continuous)" << endl;
         cerr << "3/4/5/6 - choose a speed: 3=0.5, 4=1, 5=2, 6=-2" << endl;
         cerr << "7/8/9   - choose a range: 7=6..7, 8=3..8, 9=all" << endl;
         cerr << "ESC - exit" << endl;
         break;
   }
}

//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;

      case GLUT_KEY_RIGHT:
         break;

      case GLUT_KEY_UP:
         break;

      case GLUT_KEY_DOWN:
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
sglNode *makeTile()
{
   animation = new sglKeyFrameAnimation;

   for (unsigned int i=0; i<10; ++i)
   {
      sglBitMaskSwitch *switch_node = new sglBitMaskSwitch;
      switch_node->setMask(switch_vals[i]);

      sglVec3f *vertex = new sglVec3f[4];
      vertex[0].set(0.f, 0.f, 0.f);
      vertex[1].set(2.f, 0.f, 0.f);
      vertex[2].set(2.f, 2.f, 0.f);
      vertex[3].set(0.f, 2.f, 0.f);

      sglQuadSet *gset = new sglQuadSet;
      gset->setNumPrims(1);
      gset->setCoordList(vertex);

      sglGeode *geode = new sglGeode;
      geode->addGeometry(gset);

      float x = 0.f;
      float y = 0.f;
      unsigned int ix, iy;

      y = -11.0f;
      for (iy = 0; iy < 4; ++iy)
      {
         x = 3.0f;
         for (ix = 0; ix < 8; ++ix)
         {
            sglTranslatef *trans_node = new sglTranslatef;
            trans_node->setTranslation(x, y, 0.f);
            trans_node->addChild(geode);
            switch_node->addChild(trans_node);

            x -= 3.f;
         }
         y += 3.f;
      }
      animation->addChild(switch_node);
      animation->setDuration(i, 10);
   }

   return animation;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   root = new sglScene();
   root->addChild(makeTile());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   double fov_y = (double)M_PI/(3.0f*aspect);

   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(fov_y, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);
   //view_platform->setEnableZup(true);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(50.0f);
   view_trans->setAzimuth((float)M_PI*0.5f);
   view_trans->setElevation((float)-M_PI*0.5f);
   view_trans->setCOI(0.0, 0.0, 0.0);
   translationScaleFactor = 0.2f;
   view_trans->setTranslationScale(translationScaleFactor);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(4.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root);
   view->setViewPlatform(view_platform);
   view->setViewport(0, 0, win_x, win_y);
   view->setClearColor(0.5f, 0.5f, 0.5f, 1.0f);

   pfs.setStyle(sglPolyStyle::eFILLED);
   override_statelets.push_back(&pfs);

   pbs.setStyle(sglPolyStyle::eLINE);
   override_statelets.push_back(&pbs);

   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}


//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testAnimation");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
