/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testLayer.cpp
 *  Created: January 2001
 *  Summary: test the sglLayer functionality
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglLayer.hpp>
#include <sglGeode.hpp>
#include <sglTriangleStripSet.hpp>

#include <sglStatelet.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

sglLayer *layer;

unsigned int win_x = 640, win_y = 480;

//----------------------------------------------------------------------------
// needs to be called just before the culled geometry is rendered
void preDrawFunc(void *)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);
   glMatrixMode (GL_MODELVIEW);

   glPushMatrix();
   glLoadIdentity();
   glTranslatef(0.0, 0.0, -30.0);
   glColor3f(0.5, 0.5, 0.5);
   glBegin(GL_TRIANGLE_FAN);
   glVertex3f( 0.0,  0.0, 2.0);
   glVertex3f( 1.0,  0.0, 0.0);
   glVertex3f( 0.0,  1.0, 0.0);
   glVertex3f(-1.0,  0.0, 0.0);
   glVertex3f( 0.0, -1.0, 0.0);
   glVertex3f( 1.0,  0.0, 0.0);
   glEnd();
   glPopMatrix();
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'l':
         if (layer->getMethod() == sglLayer::eSTENCIL)
            layer->setMethod(sglLayer::eMULTIPASS);
         else
            layer->setMethod(sglLayer::eSTENCIL);

         if (layer->getMethod() == sglLayer::eSTENCIL)
            cout << "stencil method" << endl;
         else
            cout << "multipass method" << endl;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "l - toggle between multipass and stencil layer modes"
              << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}

//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------

sglVec4f ncolor[5];

sglGroup *makeCoplanarObj()
{
   layer = new sglLayer(sglLayer::eSTENCIL);

   ncolor[0].set(1.0, 0.0, 0.0, 1.0);
   ncolor[1].set(0.0, 1.0, 0.0, 1.0);
   ncolor[2].set(0.0, 0.0, 1.0, 1.0);
   ncolor[3].set(1.0, 0.0, 1.0, 1.0);
   ncolor[4].set(1.0, 1.0, 0.0, 1.0);

   for (int ix=4; ix>=0; --ix)
   {
      sglGeode *geode = new sglGeode();

      const unsigned int LENGTH = 4;

      unsigned int *lengths = new unsigned int[1];
      lengths[0] = LENGTH;

      float val = 4.0f+1.0f*ix;
      sglVec3f *vertex = new sglVec3f[LENGTH];
      vertex[0].set(0.0f, -val,  val);
      vertex[1].set(0.0f, -val, -val);
      vertex[2].set(0.0f,  val,  val);
      vertex[3].set(0.0f,  val, -val);

      sglTriangleStripSet *gset = new sglTriangleStripSet();

      gset->setCoordList(vertex);
      gset->setColorList(sglGeoSet::eOVERALL, &ncolor[ix]);
      gset->setNumPrims(1);
      gset->setPrimLengths(lengths);

      vector<sglStatelet*> state;
      sglCullMode *cull_mode = new sglCullMode();
      cull_mode->setCullMode(sglCullMode::eOFF);
      state.push_back(cull_mode);
      gset->setState(0, state);

      // This object is too small to want a display list.
      gset->enableDisplayList(false);

      geode->addGeometry(gset);
      layer->addChild(geode);
   }

   return layer;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   root->addChild(makeCoplanarObj());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setAzimuth((float)(-M_PI*0.5));
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setDrawCallback(preDrawFunc, NULL);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(640, 480);

   if (argc > 1 && strcmp(argv[1], "-nostencil") == 0)
      glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   else
      glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);

   glutCreateWindow("testLayers");

   sglOStream::setThreshold(sglOStream::eINFO);
   sgl::initialize();
   sglOStream::setThreshold(sglOStream::eWARNING);

   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
