/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testLightProcessor.cpp
 *  Created: 20 April 2002
 *  Summary: test sglLightProcessor class
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>
#include <stdio.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglSwitch.hpp>
#include <sglTransform.hpp>
#include <sglDirectionalLight.hpp>
#include <sglPositionalLight.hpp>
#include <sglSpotLight.hpp>
#include <sglBitmapFont.hpp>
#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglPointSet.hpp>
#include <sglLineSet.hpp>
#include <sglTriangleStripSet.hpp>
#include <sglTriangleFanSet.hpp>
#include <sglMonoIndexedTriangleStripSet.hpp>
#include <sglMaterial.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>
#include <sglSphere.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;
sglPolyFrontStyle pfs;
sglPolyBackStyle pbs;

sglTransformf  *transformf;
sglSwitch *switch_node;

double angle[3] = {0.0, 0.0, 0.0};

unsigned int win_x = 640, win_y = 480;

bool paused_flag = false;

sglVec4f amb_light(0.75f, 0.75f, 0.75f, 1.0f);
sglVec4f no_light(0.30f, 0.30f, 0.30f, 1.0f);

int sort_index = 0;
char *sort_desc[] = {"sort mode: default O(n)",
                     "sort mode: O(n*log(n))",
                     "sort mode: O(n+k)"};

int light_index = 0;
char *light_desc[] = {"1 directional, 101 positional",
                      "1 directional, 10001 positional",
                      "1 directional, 1 positional",
                      "0 directional, 101 positional",
                      "100 directional, 101 positional",
                      "100 directional, 1 positional",
                      "100 directional, 0 positional",
                      "0 directional, 1 positional",
                      "0 directional, 0 positional"};

bool global_ambient_flag = false;
bool local_light_flag = false;
bool light_color_flag = true;

float floor_size = 200;

//----------------------------------------------------------------------------
void outputDescription()
{
   glDisable(GL_DEPTH_TEST);
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glEnable(GL_BLEND);
   glColor3f(1.0f, 1.0f, 0.5f);
   glRasterPos2i(1, 15);
   sglBitmapFont::getDefaultFont()->rasterString(sort_desc[sort_index]);
   glDisable(GL_BLEND);

   glEnable(GL_BLEND);
   glColor3f(1.0f, 1.0f, 0.5f);
   glRasterPos2i(1, 30);
   sglBitmapFont::getDefaultFont()->rasterString(light_desc[light_index]);
   glDisable(GL_BLEND);

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   outputDescription();

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 2000.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'F':
         if (pfs.getStyle() == sglPolyStyle::eFILLED)
         {
            pfs.setStyle(sglPolyStyle::ePOINT);
            pbs.setStyle(sglPolyStyle::ePOINT);
         }
         else if (pfs.getStyle() == sglPolyStyle::ePOINT)
         {
            pfs.setStyle(sglPolyStyle::eLINE);
            pbs.setStyle(sglPolyStyle::eLINE);
         }
         else if (pfs.getStyle() == sglPolyStyle::eLINE)
         {
            pfs.setStyle(sglPolyStyle::eFILLED);
            pbs.setStyle(sglPolyStyle::eFILLED);
         }

         cout << "polygon style = " << (int) pfs.getStyle() << endl;
         break;

      case 'a':
         global_ambient_flag = !global_ambient_flag;
         if (global_ambient_flag)
         {
            glLightModelfv(GL_LIGHT_MODEL_AMBIENT, amb_light.getPtr());
            cerr << "global ambient: enabled" << endl;
         }
         else
         {
            glLightModelfv(GL_LIGHT_MODEL_AMBIENT, no_light.getPtr());
            cerr << "global ambient: disabled" << endl;
         }
         break;

      case 'm':
         local_light_flag = !local_light_flag;
         if (local_light_flag)
         {
            glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
            cerr << "local light model: enabled" << endl;
         }
         else
         {
            glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
            cerr << "local light model: disabled" << endl;
         }
         break;

      case 's':
         sort_index = (sort_index+1)%3;
         switch (sort_index)
         {
            case 0:
               view->setLightProcessor(NULL);
               break;
            case 1:
               view->setLightProcessor(new sglSortedLightProcessor());
               break;
            case 2:
               view->setLightProcessor(new sglCountSortedLightProcessor());
               break;
         }
         break;

      case 'l':
         light_index = (light_index+1)%switch_node->getNumChildren();
         switch_node->setVal(light_index);
         break;

      case 'p':
         paused_flag = !paused_flag;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "F - cycle through polygon modes" << endl;
         cerr << "a - toggle global ambient" << endl;
         cerr << "m - toggle local light model" << endl;
         cerr << "s - cycle through sorted modes" << endl;
         cerr << "l - cycle through light sets" << endl;
         cerr << "p - toggle light animation" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);

   if (!paused_flag)
   {
      angle[0] += 0.02;
      angle[1] += 0.043;
      angle[2] += 0.11;

      transformf->setRotation(angle[0], angle[1], angle[2]);
   }

   displayFunc();
}

//----------------------------------------------------------------------------

sglGroup *makeRoom()
{
   // Make one wall and transform it to the other walls
   sglVec4f *color  = new sglVec4f[1]; // overall
   color[0].set(1.0, 1.0, 1.0, 1.0);

   sglVec3f *normal = new sglVec3f[1]; // per prim
   normal[0].set(0.0f, 0.0f, 1.0f);

   const unsigned int size = 24;
   sglVec3f *vertex = new sglVec3f[size*(2*size + 2)];
   unsigned int *length = new unsigned int[size];

   unsigned int index = 0;
   for (unsigned int ix = 0; ix < size; ix++)
   {
      float x1 = floor_size*((float)ix/(float)size - 0.5f);
      float x2 = floor_size*((float)(ix + 1)/(float)size - 0.5f);

      length[ix] = 2*size + 2;

      for (unsigned int iy = 0; iy <= size; ++iy)
      {
         float y = floor_size*((float)iy/(float)size - 0.5f);

         vertex[index].set(x1, y, -3.0f);
         index++;
         vertex[index].set(x2, y, -3.0f);
         index++;
      }
   }

   sglTriangleStripSet *gset_floor = new sglTriangleStripSet;
   gset_floor->setNumPrims(size);
   gset_floor->setPrimLengths(length);
   gset_floor->setCoordList(vertex);
   gset_floor->setNormalList(sglGeoSet::eOVERALL, normal);
   gset_floor->setColorList(sglGeoSet::eOVERALL, color);

   sglMaterial *mat = new sglMaterial;
   mat->setAmbient(color[0][0], color[0][1], color[0][2]);
   mat->setDiffuse(color[0][0], color[0][1], color[0][2]);

   vector<sglStatelet*> tex_state;
   tex_state.push_back(new sglLighting(true));
   tex_state.push_back(mat);

   gset_floor->setState(0, tex_state);

   // -----------------------------------------------------------------
   sglGroup *group = new sglGroup;
   sglGeode *geode = new sglGeode;
   geode->addGeometry(gset_floor);
   group->addChild(geode);          // floor

   return group;
}


//----------------------------------------------------------------------------
sglGroup *makeDirectionalLights(unsigned int num_lights)
{
   sglGroup *lights = new sglGroup();

   //-------------------------------------------------------------------------
   // make the directional light with a vector pointing in the right direction
   for (unsigned int i = 0; i < num_lights; ++i)
   {
      sglDirectionalLight *dlite = new sglDirectionalLight;
      float val = 0.3f;
      if (i != 0)
         val = 0.1f * (float)i/(float)num_lights;
      dlite->setDiffuse(val, val, val, 1.0f);
      dlite->setDirection(sglVec3f(0., 0., 1.));

      sglVec3f *coord = new sglVec3f[2];
      coord[0].set(0.f, 0.f, 0.f);
      coord[1].set(0.f, 0.f, 5.f);

      sglLineSet *gset = new sglLineSet;
      gset->setNumPrims(1);
      gset->setCoordList(coord);

      vector<sglStatelet*> state;
      state.push_back(new sglLighting(false));
      gset->setState(0, state);

      sglGeode *geode = new sglGeode;
      geode->addGeometry(gset);

      lights->addChild(dlite);
      lights->addChild(geode);
   }
   return lights;
}


//----------------------------------------------------------------------------
sglGroup *makeSpotLight()
{
   //----------------------------------------------------------------------
   // make a spot light with a vector pointing in the correct direction
   sglSpotLight *spot = new sglSpotLight;
   spot->setDiffuse(0.f, 1.f, 0.f, 1.f);
   spot->setPosition(sglVec3f(0., 0., 7.0f));
   spot->setDirection(sglVec3f(0., 0., -1.0f));
   spot->setCutoff((float)(M_PI*0.25));
   spot->setExponent(3.0f);

   sglVec3f *coord = new sglVec3f[2];
   coord[0].set(0.f, 0.f, 7.f);
   coord[1].set(0.f, 0.f, 4.f);

   sglLineSet *gset = new sglLineSet;
   gset->setNumPrims(1);
   gset->setCoordList(coord);

   vector<sglStatelet*> state2;
   state2.push_back(new sglLighting(false));
   gset->setState(0, state2);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(gset);

   transformf = new sglTransformf;
   transformf->addChild(spot);
   transformf->addChild(geode);

   return transformf;
}

//----------------------------------------------------------------------------
sglGroup *makePositionalLights(unsigned int num_lights)
{
   sglGroup *lights = new sglGroup();

   //---------------------------------------------------------------------
   // make a set of positional lights with a point at the correct position
   int num_lights_sqrt = (int)sqrt((double)num_lights);
   sglVec3f *coord = new sglVec3f[num_lights];

   int max_num = num_lights_sqrt-1;
   for (int y = max_num; y >= 0; --y)
   {
      float posy = floor_size*((float)y/(float)max_num - 0.5f);

      for (int x = max_num; x >= 0; --x)
      {
         float posx = floor_size*((float)x/(float)max_num - 0.5f);

         sglPositionalLight *plite = new sglPositionalLight;
         plite->setPosition(sglVec3f(posx, posy, 3.0f));
         if ((x == 0 || x == max_num) && (y == 0 || y == max_num))
         {
            plite->setDiffuse(1.f, 0.f, 0.f, 1.f);
            plite->setAttenuation(1.0f, 0.0f, 0.01f);
         }
         else
         {
            plite->setDiffuse(0.f, 0.f, 1.f, 1.f);
            plite->setAttenuation(1.0f, 0.0f, 0.1f);
         }
         lights->addChild(plite);

         coord[y*num_lights_sqrt+x].set(posx, posy, 3.f);
      }
   }

   sglPointSet *gset1 = new sglPointSet;
   gset1->setNumPrims(num_lights);
   gset1->setCoordList(coord);

   vector<sglStatelet*> state1;
   state1.push_back(new sglLighting(false));
   state1.push_back(new sglPointSize(4.0f));
   gset1->setState(0, state1);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(gset1);
   lights->addChild(geode);

   return lights;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, no_light.getPtr());

   // make scene
   root = new sglScene;

   // add your stuff here.
   root->addChild(makeRoom());

   sglGroup *spot = makeSpotLight();
   sglGroup *dir1 = makeDirectionalLights(1);
   sglGroup *dir100 = makeDirectionalLights(100);
   sglGroup *pos100 = makePositionalLights(100);
   sglGroup *pos10000 = makePositionalLights(10000);

   switch_node = new sglSwitch();
   sglGroup *grp = new sglGroup();
   grp->addChild(dir1);
   grp->addChild(pos100);
   grp->addChild(spot);
   switch_node->addChild(grp);

   grp = new sglGroup();
   grp->addChild(dir1);
   grp->addChild(pos10000);
   grp->addChild(spot);
   switch_node->addChild(grp);

   grp = new sglGroup();
   grp->addChild(dir1);
   grp->addChild(spot);
   switch_node->addChild(grp);

   grp = new sglGroup();
   grp->addChild(pos100);
   grp->addChild(spot);
   switch_node->addChild(grp);

   grp = new sglGroup();
   grp->addChild(dir100);
   grp->addChild(pos100);
   grp->addChild(spot);
   switch_node->addChild(grp);

   grp = new sglGroup();
   grp->addChild(dir100);
   grp->addChild(spot);
   switch_node->addChild(grp);

   grp = new sglGroup();
   grp->addChild(dir100);
   switch_node->addChild(grp);

   grp = new sglGroup();
   grp->addChild(spot);
   switch_node->addChild(grp);

   grp = new sglGroup();
   switch_node->addChild(grp);

   switch_node->setVal(0);
   root->addChild(switch_node);

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(33.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(&pfs);
   override_statelets.push_back(&pbs);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInitDisplayString("stencil>=1 rgb double depth samples");
   glutInit(&argc, argv);
   glutInitWindowSize(700, 512);

   glutCreateWindow("sglLight/sglMaterial Test Program");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
