/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * Copyright 1999, Scott McMillan
 *****************************************************************************
 *     File: testMat.cpp
 *  Created: 1999
 *  Summary: test templated matrix classes
 *****************************************************************************/

#include <sglVec2.hpp>
#include <sglVec3.hpp>
#include <sglVec4.hpp>
#include <sglVector.hpp>

#include <sglMat2.hpp>
#include <sglMat3.hpp>
#include <sglMat4.hpp>
#include <sglMatrix.hpp>

//----------------------------------------------------------------------------
void testMat2f()
{
   cout << "******************** sglMat2<float> ********************" << endl;
   sglMat2f m0;
   cout << "def ctor: " << m0 << endl;

   sglMat2f m1(1.2, 2.3, 3.4, 4.5);
   cout << "ctor(f,f,f,f): " << m1 << endl;

   double m00=1.0, m01=2.0, m10=3.0, m11=4.0;
   sglMat2f m2(m00, m01, m10, m11);
   cout << "ctor(d,d,d,d): " << m2 << endl;

   double md[2][2] = {{1.0, 2.0}, {3.0, 4.0}};
   sglMat2f m3(md);
   cout << "ctor(d m[2][2]): " << m3 << endl;

   float mf[2][2] = {{1.0, 2.0}, {3.0, 4.0}};
   sglMat2f m4(mf);
   cout << "ctor(f m[2][2]): " << m4 << endl;

   sglMat2f m5(m4);
   cout << "ctor(mat2f): " << m5 << endl;

   m0.set(md[0][0], md[0][1], md[1][0], md[1][1]);
   cout << "set(d,d,d,d): " << m0 << endl;

   m0.set(mf[0][0], mf[0][1], mf[1][0], mf[1][1]);
   cout << "set(f,f,f,f): " << m0 << endl;

   sglVec2d v1(9.8, 7.6);
   m0.setRow(v1, 1);
   cout << "setRow(Vecd, 1): " << m0 << endl;

   m0.getCol(v1, 1);
   cout << "getCol(Vecd, 1): " << v1 << endl;

   m5 = m0;
   cout << "assignment Mat2f: " << m5 << endl;

   cout << "m5 == m1: " << (m5 == m1) << endl;
   cout << "m5 != m1: " << (m5 != m1) << endl;

   sglMat2d mdd(m1);
   cout << mdd << " + " << m5 << " = " << endl;
   m0 = mdd;
   m0 += m5;
   cout << " += " << m0 << endl;
   m0 = mdd + m5;
   cout << "  + " << m0 << endl;
   m0.add(mdd,m5);
   cout << "add " << m0 << endl;

   cout << mdd << " - " << m5 << " = " << endl;
   m0 = mdd;
   m0 -= m5;
   cout << " -= " << m0 << endl;
   m0 = mdd - m5;
   cout << "  - " << m0 << endl;
   m0.sub(mdd,m5);
   cout << "sub " << m0 << endl;

   m0 = mdd;
   m0 *= 2.0;
   cout << mdd << " *= 2.0  = " << m0 << endl;
   m0 = mdd * 2.0;
   cout << mdd << " * 2.0 = " << m0 << endl;
   m0.scale(mdd, 2.0, 2.0);
   cout << mdd << " scale by 2.0: " << m0 << endl;

   cout << m1 << " * " << m2 << " = " << endl;
   m0 = m1*m2;
   cout << "m * m = " << m0 << endl;

   m0 = m1;
   m0 *=m2;
   cout << "m *= " <<  m0 << endl;

   mdd = m1;
   m0.mul(mdd,m2);
   cout << "mul(m,m)" << m0 << endl;

   m0.buildIdentity();
   cout << "buildIdentity: " << m0 << endl;
   m0.buildScale(2.0, -3.0);
   cout << "buildScale(2,-3): " << m0 << endl;
   m0.buildRotation(45.0*M_PI/180.0);
   cout << "buildRotation(45.0): " << m0 << endl;
   m1.transpose(m0);
   cout << "transpose: " << m1 << endl;

   cout << "det(" << m2 << ") = " << m2.det() << endl;
   cout << "inv(" << m2 << ") = ";
   m0.inverse(m2);
   cout << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;
}

//----------------------------------------------------------------------------
void testMat2d()
{
   cout << "******************** sglMat2<double> ********************" << endl;

   sglMat2d m0;
   cout << "def ctor: " << m0 << endl;

   sglMat2d m1(1.2, 2.3, 3.4, 4.5);
   cout << "ctor(f,f,f,f): " << m1 << endl;

   double m00=1.0, m01=2.0, m10=3.0, m11=4.0;
   sglMat2d m2(m00, m01, m10, m11);
   cout << "ctor(d,d,d,d): " << m2 << endl;

   double md[2][2] = {{1.0, 2.0}, {3.0, 4.0}};
   sglMat2d m3(md);
   cout << "ctor(d m[2][2]): " << m3 << endl;

   float mf[2][2] = {{1.0, 2.0}, {3.0, 4.0}};
   sglMat2d m4(mf);
   cout << "ctor(f m[2][2]): " << m4 << endl;

   sglMat2d m5(m4);
   cout << "ctor(mat2f): " << m5 << endl;

   m0.set(md[0][0], md[0][1], md[1][0], md[1][1]);
   cout << "set(d,d,d,d): " << m0 << endl;

   m0.set(mf[0][0], mf[0][1], mf[1][0], mf[1][1]);
   cout << "set(f,f,f,f): " << m0 << endl;

   sglVec2f v1(9.8, 7.6);
   m0.setRow(v1, 1);
   cout << "setRow(Vecd, 1): " << m0 << endl;

   m0.getCol(v1, 1);
   cout << "getCol(Vecd, 1): " << v1 << endl;

   m5 = m0;
   cout << "assignment Mat2f: " << m5 << endl;

   cout << "m5 == m1: " << (m5 == m1) << endl;
   cout << "m5 != m1: " << (m5 != m1) << endl;

   sglMat2f mdd(m1);
   cout << mdd << " + " << m5 << " = " << endl;
   m0 = mdd;
   m0 += m5;
   cout << " += " << m0 << endl;
   m0 = mdd + m5;
   cout << "  + " << m0 << endl;
   m0.add(mdd,m5);
   cout << "add " << m0 << endl;

   cout << mdd << " - " << m5 << " = " << endl;
   m0 = mdd;
   m0 -= m5;
   cout << " -= " << m0 << endl;
   m0 = mdd - m5;
   cout << "  - " << m0 << endl;
   m0.sub(mdd,m5);
   cout << "sub " << m0 << endl;

   m0 = mdd;
   m0 *= 2.0;
   cout << mdd << " *= 2.0  = " << m0 << endl;
   m0 = mdd * 2.0;
   cout << mdd << " * 2.0 = " << m0 << endl;
   m0.scale(mdd, 2.0, 2.0);
   cout << mdd << " scale by 2.0: " << m0 << endl;

   cout << m1 << " * " << m2 << " = " << endl;
   m0 = m1*m2;
   cout << "m * m = " << m0 << endl;

   m0 = m1;
   m0 *=m2;
   cout << "m *= " <<  m0 << endl;

   mdd = m1;
   m0.mul(mdd,m2);
   cout << "mul(m,m)" << m0 << endl;

   m0.buildIdentity();
   cout << "buildIdentity: " << m0 << endl;
   m0.buildScale(2.0, -3.0);
   cout << "buildScale(2,-3): " << m0 << endl;
   m0.buildRotation(45.0*M_PI/180.0);
   cout << "buildRotation(45.0): " << m0 << endl;
   m1.transpose(m0);
   cout << "transpose: " << m1 << endl;

   cout << "det(" << m2 << ") = " << m2.det() << endl;
   m0.inverse(m2);
   cout << "inv(" << m2 << ") = " << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;
}

//----------------------------------------------------------------------------
void testMat3f()
{
   cout << "******************** sglMat3<float> ********************" << endl;
   sglMat3f m0;
   cout << "def ctor: " << m0 << endl;

   sglMat3f m1(1.2, 2.3, 3.4, 4.5, 5.6, 6.7, 7.8, 8.9, 9.0);
   cout << "ctor(f,f,f,f...): " << m1 << endl;

   double m00=1.0, m01=2.0, m02=3.0;
   double m10=4.0, m11=5.0, m12=6.0;
   double m20=7.0, m21=8.0, m22=9.0;
   sglMat3f m2(m00, m01, m02, m10, m11, m12, m20, m21, m22);
   cout << "ctor(d,d,d,d...): " << m2 << endl;

   double md[3][3] = {{1.0, 2.0, 3.0}, {4.0, 5.0, 6.0}, {7.0, 8.0, 9.0}};
   sglMat3f m3(md);
   cout << "ctor(d m[3][3]): " << m3 << endl;

   float mf[3][3] = {{1.0, 2.0, 3.0}, {4.0, 5.0, 6.0}, {7.0, 8.0, 9.0}};
   sglMat3f m4(mf);
   cout << "ctor(f m[3][3]): " << m4 << endl;

   sglMat3f m5(m4);
   cout << "ctor(Mat3f): " << m5 << endl;

   m0.set(md[0][0], md[0][1], md[0][2],
          md[1][0], md[1][1], md[1][2],
          md[2][0], md[2][1], md[2][2]);
   cout << "set(d,d,d,d): " << m0 << endl;

   m0.set(mf[0][0], mf[0][1], mf[0][2],
          mf[1][0], mf[1][1], mf[1][2],
          mf[2][0], mf[2][1], mf[2][2]);
   cout << "set(f,f,f,f): " << m0 << endl;

   sglVec3d v1(9.8, 7.6, 5.4);
   m0.setRow(v1, 1);
   cout << "setRow(Vecd, 1): " << m0 << endl;

   m0.getCol(v1, 1);
   cout << "getCol(Vecd, 1): " << v1 << endl;

   m5 = m0;
   cout << "assignment Mat3f: " << m5 << endl;

   cout << "m5 == m1: " << (m5 == m1) << endl;
   cout << "m5 != m1: " << (m5 != m1) << endl;

   sglMat3d mdd(m1);
   cout << mdd << " + " << m5 << " = " << endl;
   m0 = mdd;
   m0 += m5;
   cout << " += " << m0 << endl;
   m0 = mdd + m5;
   cout << "  + " << m0 << endl;
   m0.add(mdd,m5);
   cout << "add " << m0 << endl;

   cout << mdd << " - " << m5 << " = " << endl;
   m0 = mdd;
   m0 -= m5;
   cout << " -= " << m0 << endl;
   m0 = mdd - m5;
   cout << "  - " << m0 << endl;
   m0.sub(mdd,m5);
   cout << "sub " << m0 << endl;

   m0 = mdd;
   m0 *= 2.0;
   cout << mdd << " *= 2.0  = " << m0 << endl;
   m0 = mdd * 2.0;
   cout << mdd << " * 2.0 = " << m0 << endl;
   m0.scale(mdd, 2.0, 2.0, 2.0);
   cout << mdd << " scale by 2.0: " << m0 << endl;

   cout << m1 << " * " << m2 << " = " << endl;
   m0 = m1*m2;
   cout << "m * m = " << m0 << endl;

   m0 = m1;
   m0 *=m2;
   cout << "m *= " <<  m0 << endl;

   mdd = m1;
   m0.mul(mdd,m2);
   cout << "mul(m,m)" << m0 << endl;

   m0.buildIdentity();
   cout << "buildIdentity: " << m0 << endl;
   m0.buildScale(2.0, -3.0, 4.0);
   cout << "buildScale(2,-3): " << m0 << endl;
   m2.buildRotation(1.0, 0.0, 0.0, 45.0*M_PI/180.0);
   cout << "buildRotation(45.0): " << m2 << endl;
   m1.transpose(m2);
   cout << "transpose: " << m1 << endl;

   cout << "det(" << m2 << ") = " << m2.det() << endl;
   cout << "inv(" << m2 << ") = ";
   m0.inverse(m2);
   cout << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;
}

//----------------------------------------------------------------------------
void testMat3d()
{
   cout << "******************** sglMat3<double> ********************" << endl;

   sglMat3d m0;
   cout << "def ctor: " << m0 << endl;

   sglMat3d m1(1.2, 2.3, 3.4, 4.5, 5.6, 6.7, 7.8, 8.9, 9.0);
   cout << "ctor(f,f,f,f...): " << m1 << endl;

   double m00=1.0, m01=2.0, m02=3.0;
   double m10=4.0, m11=5.0, m12=6.0;
   double m20=4.0, m21=5.0, m22=6.0;
   sglMat3d m2(m00, m01, m02, m10, m11, m12, m20, m21, m22);
   cout << "ctor(d,d,d,d...): " << m2 << endl;

   double md[3][3] = {{1.0, 2.0, 3.0}, {4.0, 5.0, 6.0}, {7.0, 8.0, 9.0}};
   sglMat3d m3(md);
   cout << "ctor(d m[3][3]): " << m3 << endl;

   float mf[3][3] = {{1.0, 2.0, 3.0}, {4.0, 5.0, 6.0}, {7.0, 8.0, 9.0}};
   sglMat3d m4(mf);
   cout << "ctor(f m[3][3]): " << m4 << endl;

   sglMat3d m5(m4);
   cout << "ctor(Mat3f): " << m5 << endl;

   m0.set(md[0][0], md[0][1], md[0][2],
          md[1][0], md[1][1], md[1][2],
          md[2][0], md[2][1], md[2][2]);
   cout << "set(d,d,d,d): " << m0 << endl;

   m0.set(mf[0][0], mf[0][1], mf[0][2],
          mf[1][0], mf[1][1], mf[1][2],
          mf[2][0], mf[2][1], mf[2][2]);
   cout << "set(f,f,f,f): " << m0 << endl;

   sglVec3f v1(9.8, 7.6, 5.4);
   m0.setRow(v1, 1);
   cout << "setRow(Vecd, 1): " << m0 << endl;

   m0.getCol(v1, 1);
   cout << "getCol(Vecd, 1): " << v1 << endl;

   m5 = m0;
   cout << "assignment Mat3f: " << m5 << endl;

   cout << "m5 == m1: " << (m5 == m1) << endl;
   cout << "m5 != m1: " << (m5 != m1) << endl;

   sglMat3f mdd(m1);
   cout << mdd << " + " << m5 << " = " << endl;
   m0 = mdd;
   m0 += m5;
   cout << " += " << m0 << endl;
   m0 = mdd + m5;
   cout << "  + " << m0 << endl;
   m0.add(mdd,m5);
   cout << "add " << m0 << endl;

   cout << mdd << " - " << m5 << " = " << endl;
   m0 = mdd;
   m0 -= m5;
   cout << " -= " << m0 << endl;
   m0 = mdd - m5;
   cout << "  - " << m0 << endl;
   m0.sub(mdd,m5);
   cout << "sub " << m0 << endl;

   m0 = mdd;
   m0 *= 2.0;
   cout << mdd << " *= 2.0  = " << m0 << endl;
   m0 = mdd * 2.0;
   cout << mdd << " * 2.0 = " << m0 << endl;
   m0.scale(mdd, 2.0, 2.0, 2.0);
   cout << mdd << " scale by 2.0: " << m0 << endl;

   cout << m1 << " * " << m2 << " = " << endl;
   m0 = m1*m2;
   cout << "m * m = " << m0 << endl;

   m0 = m1;
   m0 *=m2;
   cout << "m *= " <<  m0 << endl;

   mdd = m1;
   m0.mul(mdd,m2);
   cout << "mul(m,m)" << m0 << endl;

   m0.buildIdentity();
   cout << "buildIdentity: " << m0 << endl;
   m0.buildScale(2.0, -3.0, 4.0);
   cout << "buildScale(2,-3): " << m0 << endl;
   m2.buildRotation(1.0, 0.0, 0.0, 45.0*M_PI/180.0);
   cout << "buildRotation(45.0): " << m2 << endl;
   m1.transpose(m2);
   cout << "transpose: " << m1 << endl;

   cout << "det(" << m2 << ") = " << m2.det() << endl;
   m0.inverse(m2);
   cout << "inv(" << m2 << ") = " << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;
}

//----------------------------------------------------------------------------
void testMat4f()
{
   cout << "******************** sglMat4<float> ********************" << endl;
   sglMat4f m0;
   cout << "def ctor: " << m0 << endl;

   sglMat4f m1(1.2, 2.3, 3.4, 4.0,   4.5, 5.6, 6.7, 7.0,
               7.8, 8.9, 9.0, 10.0,  1.0, 2.0, 3.0, 4.0);
   cout << "ctor(f,f,f,f...): " << m1 << endl;

   double m00=1.0, m01=2.0, m02=3.0, m03=4.0;
   double m10=4.0, m11=5.0, m12=6.0, m13=6.0;
   double m20=7.0, m21=8.0, m22=9.0, m23=9.0;
   double m30=1.5, m31=2.5, m32=3.5, m33=4.5;
   sglMat4f m2(m00, m01, m02, m03,
               m10, m11, m12, m13,
               m20, m21, m22, m23,
               m30, m31, m32, m33);
   cout << "ctor(d,d,d,d...): " << m2 << endl;

   double md[4][4] = {{1.0, 2.0, 3.0, 4.0}, {4.0, 5.0, 6.0, 6.0},
                      {7.0, 8.0, 9.0, 9.0}, {1.5, 2.5, 3.5, 4.5}};
   sglMat4f m3(md);
   cout << "ctor(d m[4][4]): " << m3 << endl;

   float mf[4][4] = {{1.0, 2.0, 3.0, 4.0}, {4.0, 5.0, 6.0, 6.0},
                     {7.0, 8.0, 9.0, 9.0}, {1.5, 2.5, 3.5, 4.5}};
   sglMat4f m4(mf);
   cout << "ctor(f m[4][4]): " << m4 << endl;

   sglMat4f m5(m4);
   cout << "ctor(Mat4f): " << m5 << endl;

   m0.set(md[0][0], md[0][1], md[0][2], md[0][3],
          md[1][0], md[1][1], md[1][2], md[1][3],
          md[2][0], md[2][1], md[2][2], md[2][3],
          md[3][0], md[3][1], md[3][2], md[3][3]);
   cout << "set(d,d,d,d): " << m0 << endl;

   m0.set(mf[0][0], mf[0][1], mf[0][2], mf[0][3],
          mf[1][0], mf[1][1], mf[1][2], mf[1][3],
          mf[2][0], mf[2][1], mf[2][2], mf[2][3],
          mf[3][0], mf[3][1], mf[3][2], mf[3][3]);
   cout << "set(f,f,f,f): " << m0 << endl;

   sglVec4d v1(9.8, 7.6, 5.4, 2.0);
   m0.setRow(v1, 1);
   cout << "setRow(Vecd, 1): " << m0 << endl;

   m0.getCol(v1, 1);
   cout << "getCol(Vecd, 1): " << v1 << endl;

   m5 = m0;
   cout << "assignment Mat4f: " << m5 << endl;

   cout << "m5 == m1: " << (m5 == m1) << endl;
   cout << "m5 != m1: " << (m5 != m1) << endl;

   sglMat4d mdd(m1);
   cout << mdd << " + " << m5 << " = " << endl;
   m0 = mdd;
   m0 += m5;
   cout << " += " << m0 << endl;
   m0 = mdd + m5;
   cout << "  + " << m0 << endl;
   m0.add(mdd,m5);
   cout << "add " << m0 << endl;

   cout << mdd << " - " << m5 << " = " << endl;
   m0 = mdd;
   m0 -= m5;
   cout << " -= " << m0 << endl;
   m0 = mdd - m5;
   cout << "  - " << m0 << endl;
   m0.sub(mdd,m5);
   cout << "sub " << m0 << endl;

   m0 = mdd;
   m0 *= 2.0;
   cout << mdd << " *= 2.0  = " << m0 << endl;
   m0 = mdd * 2.0;
   cout << mdd << " * 2.0 = " << m0 << endl;
   m0.scale(mdd, 2.0, 2.0, 2.0, 2.0);
   cout << mdd << " scale by 2.0: " << m0 << endl;

   cout << m1 << " * " << m2 << " = " << endl;
   m0 = m1*m2;
   cout << "m * m = " << m0 << endl;

   m0 = m1;
   m0 *=m2;
   cout << "m *= " <<  m0 << endl;

   mdd = m1;
   m0.mul(mdd,m2);
   cout << "mul(m,m)" << m0 << endl;

   m0.buildIdentity();
   cout << "buildIdentity: " << m0 << endl;
   m0.buildScale(2.0, -3.0, 4.0, -5.0);
   cout << "buildScale(2,-3,4,-5): " << m0 << endl;
   m0.buildTranslation(2.0, -3.0, 4.0);
   cout << "buildTranslation(2,-3,4): " << m0 << endl;
   m2.buildRotation(1.0, 0.0, 0.0, 45.0*M_PI/180.0);
   cout << "buildRotation(45.0): " << m2 << endl;
   m1.transpose(m2);
   cout << "transpose: " << m1 << endl;

   cout << "det(" << m2 << ") = " << m2.det() << endl;
   m0.inverse(m2);
   cout << "inv(" << m2 << ") = " << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;
   m0.inverseAffine(m2);
   cout << "invAffine(" << m2 << ") = " << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;
}

//----------------------------------------------------------------------------
void testMat4d()
{
   cout << "******************** sglMat4<double> ********************" << endl;

   sglMat4d m0;
   cout << "def ctor: " << m0 << endl;

   sglMat4d m1(1.2, 2.3, 3.4, 4.0,   4.5, 5.6, 6.7, 7.0,
               7.8, 8.9, 9.0, 10.0,  1.0, 2.0, 3.0, 4.0);
   cout << "ctor(f,f,f,f...): " << m1 << endl;

   double m00=1.0, m01=2.0, m02=3.0, m03=4.0;
   double m10=4.0, m11=5.0, m12=6.0, m13=6.0;
   double m20=7.0, m21=8.0, m22=9.0, m23=9.0;
   double m30=1.5, m31=2.5, m32=3.5, m33=4.5;
   sglMat4d m2(m00, m01, m02, m03,
               m10, m11, m12, m13,
               m20, m21, m22, m23,
               m30, m31, m32, m33);
   cout << "ctor(d,d,d,d...): " << m2 << endl;

   double md[4][4] = {{1.0, 2.0, 3.0, 4.0}, {4.0, 5.0, 6.0, 6.0},
                      {7.0, 8.0, 9.0, 9.0}, {1.5, 2.5, 3.5, 4.5}};
   sglMat4d m3(md);
   cout << "ctor(d m[4][4]): " << m3 << endl;

   float mf[4][4] = {{1.0, 2.0, 3.0, 4.0}, {4.0, 5.0, 6.0, 6.0},
                     {7.0, 8.0, 9.0, 9.0}, {1.5, 2.5, 3.5, 4.5}};
   sglMat4d m4(mf);
   cout << "ctor(f m[4][4]): " << m4 << endl;

   sglMat4d m5(m4);
   cout << "ctor(Mat4f): " << m5 << endl;

   m0.set(md[0][0], md[0][1], md[0][2], md[0][3],
          md[1][0], md[1][1], md[1][2], md[1][3],
          md[2][0], md[2][1], md[2][2], md[2][3],
          md[3][0], md[3][1], md[3][2], md[3][3]);
   cout << "set(d,d,d,d): " << m0 << endl;

   m0.set(mf[0][0], mf[0][1], mf[0][2], mf[0][3],
          mf[1][0], mf[1][1], mf[1][2], mf[1][3],
          mf[2][0], mf[2][1], mf[2][2], mf[2][3],
          mf[3][0], mf[3][1], mf[3][2], mf[3][3]);
   cout << "set(f,f,f,f): " << m0 << endl;

   sglVec4f v1(9.8, 7.6, 5.4, 2.0);
   m0.setRow(v1, 1);
   cout << "setRow(Vecd, 1): " << m0 << endl;

   m0.getCol(v1, 1);
   cout << "getCol(Vecd, 1): " << v1 << endl;

   m5 = m0;
   cout << "assignment Mat4f: " << m5 << endl;

   cout << "m5 == m1: " << (m5 == m1) << endl;
   cout << "m5 != m1: " << (m5 != m1) << endl;

   sglMat4f mdd(m1);
   cout << mdd << " + " << m5 << " = " << endl;
   m0 = mdd;
   m0 += m5;
   cout << " += " << m0 << endl;
   m0 = mdd + m5;
   cout << "  + " << m0 << endl;
   m0.add(mdd,m5);
   cout << "add " << m0 << endl;

   cout << mdd << " - " << m5 << " = " << endl;
   m0 = mdd;
   m0 -= m5;
   cout << " -= " << m0 << endl;
   m0 = mdd - m5;
   cout << "  - " << m0 << endl;
   m0.sub(mdd,m5);
   cout << "sub " << m0 << endl;

   m0 = mdd;
   m0 *= 2.0;
   cout << mdd << " *= 2.0  = " << m0 << endl;
   m0 = mdd * 2.0;
   cout << mdd << " * 2.0 = " << m0 << endl;
   m0.scale(mdd, 2.0, 2.0, 2.0);
   cout << mdd << " scale by 2.0: " << m0 << endl;

   cout << m1 << " * " << m2 << " = " << endl;
   m0 = m1*m2;
   cout << "m * m = " << m0 << endl;

   m0 = m1;
   m0 *=m2;
   cout << "m *= " <<  m0 << endl;

   mdd = m1;
   m0.mul(mdd,m2);
   cout << "mul(m,m)" << m0 << endl;

   m0.buildIdentity();
   cout << "buildIdentity: " << m0 << endl;
   m0.buildScale(2.0, -3.0, 4.0);
   cout << "buildScale(2,-3,4): " << m0 << endl;
   m0.buildTranslation(2.0, -3.0, 4.0);
   cout << "buildTranslation(2,-3,4): " << m0 << endl;
   m2.buildRotation(1.0, 0.0, 0.0, 45.0*M_PI/180.0);
   cout << "buildRotation(45.0): " << m2 << endl;
   m1.transpose(m2);
   cout << "transpose: " << m1 << endl;

   cout << "det(" << m2 << ") = " << m2.det() << endl;
   m0.inverse(m2);
   cout << "inv(" << m2 << ") = " << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;

   m0.inverseAffine(m2);
   cout << "invAffine(" << m2 << ") = " << m0 << endl;
   m0*=m2;
   cout << "inv check: " << m0 << endl;
}
