/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testMultiTexture.cpp
 *  Created: 11 November 2001
 *  Summary: test the multi-texture state functionality
 *****************************************************************************/

#include <stdio.h>
#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglGeode.hpp>
#include <sglMonoIndexedTriangleStripSet.hpp>
#include <sglTexture2D.hpp>
#include <sglTexEnv.hpp>
#include <sglTexGen.hpp>
#include <sglBitmapFont.hpp>
#include <sglTextureFont.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

#include <sglImageLoader.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

bool help_flag = false;

unsigned int state_mask = 0;
unsigned int unit = 0;

#define NUM_MULTITEXTURES 10
vector<sglStatelet*> tex_state;
sglTexture2D *tex[NUM_MULTITEXTURES+1];
sglTexEnv *tev[NUM_MULTITEXTURES+1];
sglTexEnv::ModeEnum texenv_mode[NUM_MULTITEXTURES+1];
sglTextureMatrix *tem[NUM_MULTITEXTURES+1];
bool tem_flag[NUM_MULTITEXTURES+1];
double sim_time = 0.0;

sglTextureFont *texture_font = NULL;

sglMonoIndexedTriangleStripSet *gset;
sglVec4f *color;

unsigned int win_x = 720, win_y = 480;

sglPolyFrontStyle pfs;
sglPolyBackStyle pbs;

//----------------------------------------------------------------------------
#define CHECK_IMAGE_SIZE 64
#define SUB_IMAGE_SIZE 16
static GLubyte check_image[CHECK_IMAGE_SIZE][CHECK_IMAGE_SIZE][3];
static GLubyte sub_image[SUB_IMAGE_SIZE][SUB_IMAGE_SIZE][3];

//----------------------------------------------------------------------------
void makeCheckImage(void)
{
   int i,j;
   GLubyte c;
   for (i=0; i<CHECK_IMAGE_SIZE; i++)
   {
      for (j=0; j<CHECK_IMAGE_SIZE; j++)
      {
         c = 255;
         if ((i&0x8) != (j&0x8)) c = 64;
         check_image[i][j][0] = c;
         check_image[i][j][1] = c;
         check_image[i][j][2] = c;
      }
   }
   for (i=0; i<SUB_IMAGE_SIZE; i++)
   {
      for (j=0; j<SUB_IMAGE_SIZE; j++)
      {
         c = 255;
         if ((i&0x4) != (j&0x4)) c = 64;
         sub_image[i][j][0] = c;
         sub_image[i][j][1] = 0;
         sub_image[i][j][2] = 0;
      }
   }
}

//----------------------------------------------------------------------------
void outputHelp()
{
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glDisable(GL_DEPTH_TEST);

   glColor4f(1.0f, 0.9f, 0.9f, 1.0f);
   glRasterPos3i(10,205,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "/? - toggle help");
   glRasterPos3i(10,190,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "o - ouput scene graph");
   glRasterPos3i(10,175,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "$ - toggle statistics reporting");
   glRasterPos3i(10,160,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "d - toggle SP/DP traversal");
   glRasterPos3i(10,145,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "f - toggle frame rate display");
   glRasterPos3i(10,130,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "p - cycle polygon style");
   glRasterPos3i(10,115,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "0..9 - set current texture to num");
   glRasterPos3i(10,100,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "- - set current texture to base");
   glRasterPos3i(10,85,0);
   if( sgl::s_texture_env_add_capable )
   {
      sglBitmapFont::getDefaultFont()->rasterString(
         "b - toggle blend/modulate/add on current texture");
   }
   else
   {
      sglBitmapFont::getDefaultFont()->rasterString(
         "b - toggle blend/modulate on current texture");
   }
   glRasterPos3i(10,70,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "g - toggle gset color");
   glRasterPos3i(10,55,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "r - reset current texture");
   glRasterPos3i(10,40,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "R - reset ALL textures");
   glRasterPos3i(10,25,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "s - subtexload into base texture");
   glRasterPos3i(10,10,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "m - toggle tex matrix animation for current texture");

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   if (help_flag)
   {
      outputHelp();
   }

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
         unit = key - '0' + 1;
         break;

      case '-':
         unit = 0;
         break;

      case 'b':
         if( texenv_mode[unit] == sglTexEnv::eBLEND )
         {
            texenv_mode[unit] = sglTexEnv::eMODULATE;
            cout << "texenv mode: eMODULATE" << endl;
         }
         else if( texenv_mode[unit] == sglTexEnv::eMODULATE )
         {
            if( sgl::s_texture_env_add_capable )
            {
               texenv_mode[unit] = sglTexEnv::eADD;
               cout << "texenv mode: eADD" << endl;
            }
            else
            {
               texenv_mode[unit] = sglTexEnv::eBLEND;
               cout << "texenv mode: eBLEND" << endl;
            }

         }
         else
         {
            texenv_mode[unit] = sglTexEnv::eBLEND;
            cout << "texenv mode: eBLEND" << endl;
         }

         tev[unit]->setMode( texenv_mode[unit] );
         break;

      case 'g':
         if (color[0][0] == 1.0f)
            color->set(0.5f, 1.0f, 1.0f, 1.0f);
         else
            color->set(1.0f, 1.0f, 1.0f, 1.0f);
         gset->setColorList(sglGeoSet::eOVERALL, color);
         break;

      case 'r':
         tex[unit]->reset();
         break;

      case 'R':
      {
         for (unsigned int i=0; i<NUM_MULTITEXTURES+1; ++i)
            tex[i]->reset();
         break;
      }

      case 'm':
         tem_flag[unit] = !tem_flag[unit];
         if (!tem_flag[unit])
            tem[unit]->setTextureMatrix(NULL);
         break;

      case 's':
         tex[0]->subloadImage(&sub_image[0][0][0], 0, 12, 44,
                              SUB_IMAGE_SIZE, SUB_IMAGE_SIZE);
         break;

      case 'p':
         if (pfs.getStyle() == sglPolyStyle::eFILLED)
         {
            pfs.setStyle(sglPolyStyle::ePOINT);
            pbs.setStyle(sglPolyStyle::ePOINT);
         }
         else if (pfs.getStyle() == sglPolyStyle::ePOINT)
         {
            pfs.setStyle(sglPolyStyle::eLINE);
            pbs.setStyle(sglPolyStyle::eLINE);
         }
         else if (pfs.getStyle() == sglPolyStyle::eLINE)
         {
            pfs.setStyle(sglPolyStyle::eFILLED);
            pbs.setStyle(sglPolyStyle::eFILLED);
         }

         cout << "polygon style = " << (int) pfs.getStyle() << endl;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '?':
      case '/':
         help_flag = !help_flag;
         cout << "help_flag = " << help_flag << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   sim_time += 0.1f;

   if (tem_flag[0])
   {
      sglMat4f mat;
      mat.buildTranslation(0.2f*cos(sim_time), 0.2f*sin(sim_time), 0.f);
      mat[0][0] = 1.1f + cos(sim_time);
      mat[1][1] = 1.1f + sin(sim_time);
      tem[0]->setTextureMatrix(&mat);
   }

   sglMat4f mat2;
   mat2.buildTranslation(0.05f*cos(sim_time), 0.05f*sin(sim_time), 0.f);

   for (unsigned int i=0; i<NUM_MULTITEXTURES; ++i)
      if (tem_flag[i])
         tem[i]->setTextureMatrix(&mat2);

   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------

void makeStates()
{
   makeCheckImage();

   tex[0] = new sglTexture2D;
   tex[0]->setImage(&check_image[0][0][0], sglTexture::eRGB,
                  sglTexture::eUNSIGNED_BYTE,
                  CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);

   tex[0]->setAutoMipmapFlag(true);
   tex[0]->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
   tex[0]->setMagFilter(sglTexture::eLINEAR); //sglTexture::eNEAREST);
   tex[0]->setWrapMode(sglTexture::eCLAMP, sglTexture::eCLAMP);
   tex[0]->setBorderColor(sglVec4f(1.0, 0.0, 0.0, 1.0));

   tev[0] = new sglTexEnv();
   tev[0]->setBlendColor(1.0f, 0.7f, 0.0f, 1.0f);
   texenv_mode[0] = sglTexEnv::eBLEND;
   tev[0]->setMode(texenv_mode[0]);
   tem[0] = new sglTextureMatrix(NULL);
   tem_flag[0] = false;

   tex_state.push_back(tex[0]);
   tex_state.push_back(tev[0]);
   tex_state.push_back(tem[0]);

   vector<char> valid_chars;
   for (char c = '0'; c <= '9'; ++c)
      valid_chars.push_back(c);
   valid_chars.push_back('.');
   cerr << "default font = " << sglBitmapFont::getDefaultFont() << endl;
   texture_font = new sglTextureFont(*sglBitmapFont::getDefaultFont(),
                                     valid_chars, true, '.');

   void *old_image;
   sglTexture::ExternalFormatEnum old_format;
   sglTexture::TypeEnum old_type;
   unsigned int num_s, num_t;
   bool border;
   texture_font->getTexture()->getImage(
      old_image, old_format, old_type, num_s, num_t, border);
   void* new_image = sglImageLoader::reformatImage(
      old_format, old_type, num_s, num_t, sglTexture::eRGBA, old_image);

   for (unsigned int i=1; i <=NUM_MULTITEXTURES; ++ i)
   {
      tex[i] = new sglTexture2D;
      tex[i]->setImage(new_image, sglTexture::eRGBA, old_type,
                     num_s, num_t, border);
      tex[i]->setAutoMipmapFlag(true);
      tex[i]->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
      tex[i]->setMagFilter(sglTexture::eLINEAR); //sglTexture::eNEAREST);
      tex[i]->setWrapMode(sglTexture::eCLAMP, sglTexture::eCLAMP);

      tev[i] = new sglTexEnv();
      tev[i]->setBlendColor(1.0f, 0.7f, 0.0f, 1.0f);
      texenv_mode[i] = sglTexEnv::eDECAL;
      tev[i]->setMode(texenv_mode[i]);

      tem[i] = new sglTextureMatrix(NULL);
      tem_flag[i] = false;

      tex_state.push_back(new sglMultiTextureStatelet(tex[i], i));
      tex_state.push_back(new sglMultiTextureStatelet(tev[i], i));
      tex_state.push_back(new sglMultiTextureStatelet(tem[i], i));
   }
}

//----------------------------------------------------------------------------
sglGroup *makeTextureObj()
{
   const unsigned int SIZE = NUM_MULTITEXTURES + 2;

   unsigned int *lengths = new unsigned int[(SIZE-1)];
   unsigned int i, j;
   for (i=0; i<(SIZE-1); ++i)
      lengths[i] = SIZE*2;

   sglVec3f *vertex = new sglVec3f[SIZE*SIZE];
   sglVec3f *normal = new sglVec3f[SIZE*SIZE];
   sglVec2f *tvert = new sglVec2f[SIZE*SIZE];
   unsigned int index = 0;
   for (j=0; j<SIZE; ++j)
      for (i=0; i<SIZE; ++i)
      {
         vertex[index].set(16.0f*(float)i/(float)(SIZE-1) - 8.0f,
                           0.0f,
                           16.0f*(float)j/(float)(SIZE-1) - 8.0f);
         normal[index].set(1.0f, 0.0f, 0.0f);
         tvert[index].set(2.0f*(float)i/(float)(SIZE-1),
                          2.0f*(float)j/(float)(SIZE-1));
         ++index;
      }

   unsigned int *indices = new unsigned int[(SIZE-1)*SIZE*2];
   index = 0;
   for (j=0; j<(SIZE-1); ++j)
      for (i=0; i<SIZE; ++i)
      {
         indices[index] = j + i*SIZE;
         ++index;
         indices[index] = j+1 + i*SIZE;
         ++index;
      }

   color = new sglVec4f[1];
   color->set(0.5f, 1.0f, 1.0f, 1.0f);

   gset = new sglMonoIndexedTriangleStripSet();

   gset->setCoordList(vertex);
   gset->setNormalList(sglGeoSet::ePER_VERTEX, normal);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setIndexList(indices);
   gset->setNumPrims((SIZE-1));
   gset->setPrimLengths(lengths);

   vector<sglTexCoords> tex_coords;
   tex_coords.push_back(sglTexCoords((float*)tvert, 2));

   sglVec3f tmp_coord[4];
   sglVec2f tmp_tvert[4];
   sglVec3f offset(0,0,0);

   for (unsigned int tex=0; tex<NUM_MULTITEXTURES; ++tex)
   {
      char str[20];
      sprintf(str, "%d", tex);

      tvert = new sglVec2f[SIZE*SIZE];

      int num = texture_font->getStringInfo(str, tmp_coord, tmp_tvert,
                                            10, offset);
      if (num)
      {
         float sdiff = tmp_tvert[2][0] - tmp_tvert[0][0];
         float tdiff = tmp_tvert[2][1] - tmp_tvert[0][1];

         index = 0;
         for (j=0; j<SIZE; ++j)
            for (i=0; i<SIZE; ++i)
            {

               float s, t;
               if (i <= tex)
                  s = tmp_tvert[0][0];
               else if (i == tex+1)
                  s = tmp_tvert[0][0] + sdiff/3;
               else if (i == tex+2)
                  s = tmp_tvert[0][0] + 2*sdiff/3;
               else
                  s = tmp_tvert[2][0];

               if (j <= tex)
                  t = tmp_tvert[0][1];
               else if (j == tex+1)
                  t = tmp_tvert[0][1] + tdiff/2;
               else if (j == tex+2)
                  t = tmp_tvert[0][1] + 2*tdiff/3;
               else
                  t = tmp_tvert[2][1];

               tvert[index].set(s, t);
               ++index;
            }

         tex_coords.push_back(sglTexCoords((float*)tvert, 2));
      }
      else
      {
         cerr << "error num=0" << endl;
         tex_coords.push_back(sglTexCoords(NULL, 2));
      }
   }

   gset->setState(0, tex_state, tex_coords);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   makeStates();

   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   sglGeode *geode = new sglGeode;
//   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);
   root->addChild(makeTextureObj());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setClearColor(0.4f, 0.4f, 0.4f, 1.0f);
   view->setViewport(0, 0, win_x, win_y);
   view->setStateMask(state_mask);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   override_statelets.push_back(&pfs);
   override_statelets.push_back(&pbs);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testMultiTexture");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
