/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testPlane.cpp
 *  Created: 1998
 *  Summary: 
 *****************************************************************************/

#include <sglVector.hpp>
#include <sglPlane.hpp>
#include <sglPolytope.hpp>
#include <sglSphereBound.hpp>

int main(int, char**)
{
   sglPlanef p1;
   cout << "default ctor: " << p1 << endl;

   sglVec3f dir(1.0f, 1.0f, 0.0f);
   sglPlanef p2(dir, 2.0);
   cout << "vec3f ctor: " << p2 << endl;

   sglPlanef pa;
   pa.scale(p2, 3.0);
   cout << "p2.scale(3.0): " << pa << endl;

   sglPlanef p3(p2);
   cout << "copy ctor: " << p3 << endl;

   sglPlanef p4(1.0f, 1.0f, 0.0f, 2.0f);
   cout << "float ctor: " << p4 << endl;

   p1 = p4;
   cout << "p1 = p4 -> " << p1 << endl;
   p1.set(dir, -1.f);
   cout << "p1.set{Normal,Distance} -> " << p1 << endl;

   sglVec3f v1(1.f, 0.f, 0.f);
   sglVec3f v2(0.f, 1.f, 0.f);
   sglVec3f v3(0.f, 0.f, 0.f);
   p1.build(v1, v2, v3);
   cout << "p1.build(x, y, z) -> " << p1 << endl;

   p1.getVertices(v1, v2, v3);
   cout << "p1.getVertices(): " << endl
        << v1 << endl << v2 << endl << v3 << endl;

   cout << "sd(v1) = " << p1.signedDistance(v1) << endl;
   v1[2] = 1.0;
   cout << "sd(v1) = " << p1.signedDistance(v1) << endl;
   v1[2] = -1.0;
   cout << "sd(v1) = " << p1.signedDistance(v1) << endl;

   // test the basic polytope
   sglPolytopef ptope;

   sglVec3f nbl(0.,0.,0.); // near, bottom, left...
   sglVec3f nbr(1.,0.,0.);
   sglVec3f ntr(1.,1.,0.);
   sglVec3f ntl(0.,1.,0.);
   sglVec3f fbl(0.,0.,1.);
   sglVec3f fbr(1.,0.,1.);
   sglVec3f ftr(1.,1.,1.);
   sglVec3f ftl(0.,1.,1.);

   sglPlanef  near_clip;  near_clip.build(nbr, nbl, ntr);
   sglPlanef   far_clip;   far_clip.build(fbl, fbr, ftl);
   sglPlanef right; right.build(fbr, nbr, ftr);
   sglPlanef  left;  left.build(nbl, fbl, ntl);
   sglPlanef   top;   top.build(ntr, ntl, ftr);
   sglPlanef   bot;   bot.build(nbl, nbr, fbl);

   ptope.addPlane(near_clip);
   ptope.addPlane(far_clip);
   ptope.addPlane(right);
   ptope.addPlane(left);
   ptope.addPlane(top);
   ptope.addPlane(bot);

   sglVec3f center(0.5f, 0.5f, 0.5f);
   sglSphereBoundf bsph; bsph.setRadius(0.1f); bsph.setCenter(center);
   cout << "bsph.intersect(ptope) = " << (int)bsph.intersect(ptope) << endl;

   center[2] = 0.9f;
   bsph.setCenter(center);
   cout << "bsph.intersect(ptope) = " << (int)bsph.intersect(ptope) << endl;

   center[2] = 0.95f;
   bsph.setCenter(center);
   cout << "bsph.intersect(ptope) = " << (int)bsph.intersect(ptope) << endl;

   center[2] = 1.09999f;
   bsph.setCenter(center);
   cout << "bsph.intersect(ptope) = " << (int)bsph.intersect(ptope) << endl;

   center[2] = 1.101f;
   bsph.setCenter(center);
   cout << "bsph.intersect(ptope) = " << (int)bsph.intersect(ptope) << endl;

   // test translate (as opposed to mul)
   sglMat4f mat;
   mat.buildTranslation(4.f, -1.f, 5.f);
   sglPlanef test_mul;
   sglPlanef test_trans;

   test_mul.mul(near_clip, mat);
   test_trans.translate(near_clip, 4.f, -1.f, 5.f);
   cout << "Testing mul vs. translate: " << endl;
   cout << "translating(4,-1,5): " << near_clip << endl;
   cout << "  mul: " << setprecision(16) << test_mul << endl;
   cout << "trans: " << setprecision(16) << test_trans << endl;

   return 0;
}
