/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File:
 *  Created:
 *  Summary: template for graphical test programs
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>
#include <math.h>
#include <stdlib.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglGeode.hpp>
#include <sglScene.hpp>
#include <sglStatelet.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

#include <sglTransform.hpp>
#include <sglPointSet.hpp>

struct {
   float min_size, max_size, fade_threshold, A, B, C;
} pp;

sglTransformf * cubeTrans;
sglPointParameters * pparm;
bool useRandom = false;

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;
sglAntiAliasPoint   aa_point;
sglAntiAliasLine    aa_line;
sglAntiAliasPolygon aa_poly;

unsigned int win_x = 640, win_y = 480;

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'q':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'a':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.A -= 0.01f;
         if( pp.A < 0.0f )
            pp.A = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point parm A = " << pp.A << endl;
         break;

      case 'A':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.A += 0.01f;
         if( pp.A < 0.0f )
            pp.A = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point parm A = " << pp.A << endl;
         break;

      case 'b':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.B -= 0.01f;
         if( pp.B < 0.0f )
            pp.B = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point parm B = " << pp.B << endl;
         break;

      case 'B':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.B += 0.01f;
         if( pp.B < 0.0f )
            pp.B = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point parm B = " << pp.B << endl;
         break;

      case 'c':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.C -= 0.01f;
         if( pp.C < 0.0f )
            pp.C = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point parm C = " << pp.C << endl;
         break;

      case 'C':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.C += 0.01f;
         if( pp.C < 0.0f )
            pp.C = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point parm C = " << pp.C << endl;
         break;

      case 't':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.fade_threshold -= 0.01f;
         if( pp.fade_threshold < 0.0f )
            pp.fade_threshold = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point fade threshold = " << pp.fade_threshold << endl;
         break;

      case 'T':
         pparm->getPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );
         pp.fade_threshold += 0.01f;
         if( pp.fade_threshold < 0.0f )
            pp.fade_threshold = 0.0f;
         pparm->setPointParameters( pp.A, pp.B, pp.C, pp.fade_threshold,
                                    pp.min_size, pp.max_size );

         cout << "point fade threshold = " << pp.fade_threshold << endl;
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "q - toggle point anti-aliasing" << endl;
         cerr << "                         1" << endl;
         cerr << "atten(dist) = -----------------------" << endl;
         cerr << "               A + B*dist + C*dist^2" << endl;
         cerr << endl;
         cerr << "derivedSize = pointSize * sqrt(atten(dist))" << endl;
         cerr << endl;
         cerr << "if ( derivedSize >= threshold )" << endl;
         cerr << "   pointDiameter = derivedSize" << endl;
         cerr << "else" << endl;
         cerr << "   pointDiameter = threshold" << endl;
         cerr << endl;
         cerr << "if ( derivedSize >= threshold )" << endl;
         cerr << "   pointAlpha = 1" << endl;
         cerr << "else" << endl;
         cerr << "   pointAlpha = (derivedSize/threshold)^2" << endl;
         cerr << endl;
         cerr << "a/A - dec/inc 'A' parameter" << endl;
         cerr << "b/B - dec/inc 'B' parameter" << endl;
         cerr << "c/C - dec/inc 'C' parameter" << endl;
         cerr << "t/T - dec/inc 'threshold' parameter" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}

//----------------------------------------------------------------------------
static float
xrand48( void )
{
   return (float)rand()/((float)RAND_MAX+1.0f);
}

//----------------------------------------------------------------------------
void
moveCube( sglTransformf * trans )
{
   static float x = 0.0f, y = 0.0f, z = 0.0f;

//   x += ((xrand48()-0.5) / 100.0);
//   y += ((xrand48()-0.5) / 100.0);
   z -= (xrand48() / 500.0);

   if( x <= -1.0 || (x >= 1.0) )
      x = 0.0;

   if( y <= -1.0 || (y >= 1.0) )
      y = 0.0;

   if( z <= -1.0 )
      z = 0.0;

   trans->setTranslation( x, y, z );
}


//----------------------------------------------------------------------------
void idleFunc()
{

   moveCube( cubeTrans );
   view_trans->update(mouse);

   glutPostRedisplay();
}


//----------------------------------------------------------------------------
sglPointSet *
createPointCube( int dim )
{
   sglPointSet * ps = new sglPointSet();
   sglVec3f * verts = new sglVec3f[ dim * dim * dim ];
   sglVec4f * color;
   int dim_2 = dim/2;

   int num=0;
   for( int i = -dim_2; i < dim_2; i ++ )
      for( int j= -dim_2;j<dim_2;j++)
         for(int k= -dim_2;k<dim_2;k++)
            if( useRandom )
            {
               float radius = (float)dim_2*xrand48()*xrand48();
               double elevation = M_PI*(xrand48() - 0.5f);
               double azimuth = 2.*M_PI*(xrand48() - 0.5f);

               verts[ num++ ].set(radius*cos(elevation)*cos(azimuth),
                                  radius*sin(elevation)*cos(azimuth),
                                  radius*sin(azimuth));
            }
            else
               verts[ num++ ].set( (float)i, (float)j, (float)k );

   color = new sglVec4f( 1.f, 1.f, 1.f, 1.f );

   ps->setNumPrims( dim*dim*dim );
   ps->setCoordList( verts );
   ps->setColorList( sglGeoSet::eOVERALL, color );

   pp.A = 1.0f;
   pp.B = 0.34f;
   pp.C = 0.8f;

   vector<sglStatelet *>cubeState;
   cubeState.push_back(new sglPointSize(10.0f));
   pparm = new sglPointParameters( pp.A, pp.B, pp.C );
   cubeState.push_back( pparm );
   ps->setState( 0, cubeState, 0 );

   return ps;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   // make scene and default state:
   root = new sglScene();

   // add your stuff here


   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry( new sglAxes(2.0f) );

   cubeTrans = new sglTransformf();
   sglGeode *geodeCube = new sglGeode;
   geodeCube->addGeometry( createPointCube( 50 ) );
   cubeTrans->addChild( geodeCube );
   root->addChild(cubeTrans);
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(640, 480);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("template");

   if( argc > 1 && argv[1][0] == '-' && argv[1][1] == 'r' )
      useRandom = true;

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
