/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testQuaternion.cpp
 *   Author: William DeVore
 *  Created: 18 July 2000
 *  Summary: test the sglQuaternion class
 *****************************************************************************/

#include <sglQuaternion.hpp>
#include <sglVector.hpp>
#include <sglMatrix.hpp>

//----------------------------------------------------------------------------
void testQuaternion()
{
   cout << "******************* testQuaternion **********************" << endl;
   //-------------------------------------------------------------------------
   // 1) Check the mechcanics of template.
   //-------------------------------------------------------------------------
   // Test conjugate
   cout << endl << "Calculate the conjugate of a quaternion." << endl;

   sglQuaterniond qtn( 1.0 , 2.0 , 3.0 , 4.0 );
   cout << "generic quaternion(gc) = " << qtn << endl;
   cout << "conjugate of gc = " << qtn.conjugate() << endl << endl;

   sglQuaterniond qUnit( 1.0 , 1.0 , 0.0 , 0.0 );
   cout << "a non-unit quaternion = " << qUnit << endl;
   // result should be ( 0.707, 0.707, 0, 0)
   qUnit.normalize();
   cout << "in unit form = " << qUnit << endl;

   //-------------------------------------------------------------------------
   // 2) Check the mathematical validty of the Quaternions template
   //-------------------------------------------------------------------------
   sglQuaterniond q0( sglVec3d( 0.0 , 1.0 , 0.0), 45.0*M_PI/180.0 );
   // rotate 45.0 degrees about Y-axis.
   cout << "a quaternion generated from an axis(0,1,0) angle(45.0) = "
        << q0 << endl;

   // pull it back out
   sglVec3d axis;
   double angle;
   q0.getAxisAngle(axis, angle);
   // vAa be ( 0 , 1 , 0 , 0.7853) 0.7853 radians is 45 degrees.
   cout << "axis angle extracted back out = " << axis << " : " << angle
        << endl << endl;

   sglQuaterniond qPoint( 2.0 , 0.0 , 0.0 , 0.0 );
   // point to be rotated in the X,Z plane. Y is up.
   cout << "point to be rotated = " << qPoint << endl;

   // For a right-handed system this should rotate the point CCW to
   // (1.414 , 0 , -1.414)
   sglQuaterniond q1 = q0 * qPoint * q0.conjugate();
   cout << "point rotated about Y axis in non unit form = " << q1 << endl;

   sglVec3d point(2.0, 0.0, 0.0);
   point = q0.rotate(point);
   cout << "point rotated about Y axis using rotate function: "
        << point << endl;

   // Test rotate about +Z
   q0.set(sglVec3d(0.0 , 0.0 , 1.0), 45.0*M_PI/180.0);
   // rotate 45.0 degrees about Z-axis.
   cout << "point to be rotated = " << qPoint << endl;
   qPoint = q0 * qPoint * q0.conjugate();
   cout << "point rotated about Z axis in non unit form = " << qPoint
        << endl;  // Should be (1.414, 1.414, 0, 0)

   point.set(2.0, 0.0, 0.0);
   point = q0.rotate(point);
   cout << "point rotated about Z axis using rotate function: "
        << point << endl;

   // Test rotate about +X
   q0.set(sglVec3d(1.0 , 0.0 , 0.0), 45.0*M_PI/180.0);
   qPoint = sglQuaterniond( 0.0 , 0.0 , 2.0 , 0.0 );
   // point to be rotated in the Z,Y plane. Y is up.
   cout << "point to be rotated = " << qPoint << endl;
   qPoint = q0 * qPoint * q0.conjugate();
   cout << "point rotated about X axis in non unit form = " << qPoint
        << endl;  // Should be (0, -1.414, 1.414, 0)

   point.set(0.0 , 0.0 , 2.0);
   point = q0.rotate(point);
   cout << "point rotated about X axis using rotate function: "
        << point << endl;


   sglMat4d m = q0.getTransformation();
   cout << "rotation matrix from axis angle = " << m << endl;

   q1.buildFromTransformation(m);
   cout << "quaternion from rotation matrix = " << q1 << endl;
   cout << "original quaternion = " << q0 << endl;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
int main()
{
   testQuaternion();

   return 0;
}
