/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testRefCount.cpp
 *  Created: 9 June 1999
 *  Summary: test the reference counting functionality
 *****************************************************************************/

#if defined(WIN32)
#include <windows.h>
#endif

#include <sgl.h>
#include <sglu.h>

#include <sglObject.hpp>
#include <sglNode.hpp>
#include <sglGroup.hpp>

#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglLineSet.hpp>

int main(int, char**)
{
    sglGroup *g1 = new sglGroup();
    g1->setName("g1");
    sglGeode *g2 = new sglGeode();
    g2->setName("g2");

    cerr << "adding g2 to g1" << endl;
    g1->addChild(g2);

    sgluOutputTree(g1, cerr, 4);

    cerr << endl << "removing g2 from g1" << endl;
    g1->removeChild(g2);
    cerr << "    ----------------------------------" << endl;
    g1->printInfo(cerr,"    ");
    cerr << "    ----------------------------------" << endl;
    g2->printInfo(cerr,"    ");
    cerr << "    ----------------------------------" << endl;

    cerr << endl << "adding g2 to g1, and deleting g2" << endl;
    g1->addChild(g2);
    delete g2;
    cerr << "    ----------------------------------" << endl;
    g1->printInfo(cerr,"    ");
    cerr << "    ----------------------------------" << endl;

    cerr << endl << "adding g2 to g1, and deleting g1" << endl;
    g2 = new sglGeode();
    g2->setName("g2");
    g1->addChild(g2);
    delete g1;

    cerr << endl << "adding g2 to g1, adding gs1 to g2" << endl;
    g1 = new sglGroup();
    g1->setName("g1");
    g2 = new sglGeode();
    g2->setName("g2");
    sglGeoSet *gs1 = new sglLineSet();
    gs1->setName("gs1");

    g1->addChild(g2);
    g2->addGeometry(gs1);

    sgluOutputTree(g1, cerr, 4);

    cerr << endl << "Removing g2 from g1" << endl;
    g1->removeChild(g2);
    cerr << "    ----------------------------------" << endl;
    g1->printInfo(cerr, "    ");
    cerr << "    ----------------------------------" << endl;
    g2->printInfo(cerr, "    ");
    cerr << "    ----------------------------------" << endl;

    cerr << endl << "Removing gs1 from g2" << endl;
    g2->removeGeometry(gs1);
    cerr << "    ----------------------------------" << endl;
    g2->printInfo(cerr, "    ");
    cerr << "    ----------------------------------" << endl;
    gs1->printInfo(cerr, "    ");
    cerr << "    ----------------------------------" << endl;

    cerr << endl << "Adding g2 to g1, adding gs1 to g2, deleting g1" << endl;
    g1->addChild(g2);
    g2->addGeometry(gs1);
    delete g1;


    cerr << endl << "Creating the following tree and deleting the top node: "
         << endl;
    cerr << "         group" << endl;
    cerr << "         /   \\" << endl;
    cerr << "     group   group" << endl;
    cerr << "         \\   /" << endl;
    cerr << "         geode" << endl;
    cerr << "         /   \\" << endl;
    cerr << "    geoset   geoset" << endl;

    sglGroup *root = new sglGroup();
    sglGroup *gp1 = new sglGroup();
    gp1->setName("g1");
    sglGroup *gp2 = new sglGroup();
    sglGeode *geode = new sglGeode();
    geode->setName("gd1");
    sglGeoSet *gset1 = new sglLineSet();
    gset1->setName("gs1");
    sglGeoSet *gset2 = new sglLineSet();

    root->addChild(gp1);
    root->addChild(gp2);
    gp1->addChild(geode);
    gp2->addChild(geode);
    geode->addGeometry(gset1);
    geode->addGeometry(gset2);

    sgluOutputTree(root, cerr, 4);

    sglObject *obj = sgluFindObject("g1", root);
    cerr << "sgluFindObject(g1) = " << hex << obj << dec << endl;
    obj = sgluFindObject("gd1", root);
    cerr << "sgluFindObject(gd1) = " << hex << obj << dec << endl;
    obj = sgluFindObject("gs1", root);
    cerr << "sgluFindObject(gs1) = " << hex << obj << dec << endl;
    obj = sgluFindObject("geoset1", root);
    cerr << "sgluFindObject(geoset1) = " << hex << obj << dec << endl;

    delete root;

    return 0;
}
