/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testStatelet.cpp
 *  Created: 20 March 1999 (originally called testGeoState.cpp)
 *  Summary: test the sglStatelets functionality (sans textures)
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>

#include <sgluSphericalTransform.hpp>

#include <sglGroup.hpp>
#include <sglScene.hpp>
#include <sglDefaultStateNode.hpp>
#include <sglSwitch.hpp>
#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglTriangleStripSet.hpp>
#include <sglLineStripSet.hpp>

#include <sglBitmapFont.hpp>
#include <sglDirectionalLight.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

#include <sglStatelet.hpp>

sglView                     *view;

sglScene                    *root;
sglSwitch                   *switch_node;
deque<const sglStatelet*>    override_statelets;

sgluSphericalTransform<float>   *view_trans;
sglViewPlatform             *view_platform;
sglPerspectiveCamera        *view_camera;

float depth_offset = 0.0001f;
sglDepthRange *surface_depth_range, *line_depth_range;

unsigned int                 state_mask = 0;

sgluMouse                   *mouse;

unsigned int                 win_x = 640;
unsigned int                 win_y = 480;

int test_index = 0;
char *test_desc[] = {"test transparency",
                     "test polygon stipple",
                     "test polygon offset",
                     "test depth range"};

//----------------------------------------------------------------------------
void outputDescription()
{
   glDisable(GL_DEPTH_TEST);
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, win_x, 0, win_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glEnable(GL_BLEND);
   glColor3f(1.0f, 1.0f, 0.5f);
   glRasterPos3i(1, win_y - 15, 0);
   sglBitmapFont::getDefaultFont()->rasterString(test_desc[test_index]);
   glDisable(GL_BLEND);

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glEnable(GL_DEPTH_TEST);
}

//----------------------------------------------------------------------------
void displayFunc (void)
{
   view->cullAndDraw();
   outputDescription();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;
   double fov_y = M_PI/(4.0*aspect);

   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'r':
         depth_offset *= 0.95f;
         cerr << "depth_offset = " << depth_offset << endl;
         surface_depth_range->setDepthRange(depth_offset, 1.0f);
         line_depth_range->setDepthRange(0.0f, 1.0f - depth_offset);
         break;

      case 'R':
         depth_offset *= 1.05f;
         if (depth_offset > 0.5f)
            depth_offset = 0.5f;

         cerr << "depth_offset = " << depth_offset << endl;
         surface_depth_range->setDepthRange(depth_offset, 1.0f);
         line_depth_range->setDepthRange(0.0f, 1.0f - depth_offset);
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "up/down arrow - cycle through tests" << endl;
         cerr << "r/R - dec/inc depth offset for depth range test" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         ++test_index;
         if (test_index > 3) test_index = 0;
         switch_node->setVal(test_index);
         break;
      case GLUT_KEY_DOWN:
         --test_index;
         if (test_index < 0)
            test_index = 3;
         switch_node->setVal(test_index);
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);

   displayFunc();
}

//----------------------------------------------------------------------------

sglVec4f color[3];

sglGroup *makeTransparentObj()
{
   color[0].set(1.0, 0.6, 0.6, 0.45);
   color[1].set(0.6, 1.0, 0.6, 0.45);
   color[2].set(0.6, 0.6, 1.0, 0.45);

   sglGeode *geode = new sglGeode();

   // make the proper state
   vector<sglStatelet*> transparent_state;
   transparent_state.push_back(new sglCullMode(sglCullMode::eOFF));
   transparent_state.push_back(new sglTransparency(true));

   for (unsigned int ix=0; ix<3; ix++)
   {
      const unsigned int LENGTH = 4;

      unsigned int *lengths = new unsigned int[1];
      lengths[0] = LENGTH;

      sglVec3f *vertex = new sglVec3f[LENGTH];
      vertex[0].set(ix*8.0f-8.0f,-4.0f, 4.0f);
      vertex[1].set(ix*8.0f-8.0f,-4.0f,-4.0f);
      vertex[2].set(ix*8.0f-8.0f, 4.0f, 4.0f);
      vertex[3].set(ix*8.0f-8.0f, 4.0f,-4.0f);

      sglTriangleStripSet *gset = new sglTriangleStripSet();

      gset->setCoordList(vertex);
      gset->setColorList(sglGeoSet::eOVERALL, &color[ix]);
      gset->setNumPrims(1);
      gset->setPrimLengths(lengths);

      gset->setState(0, transparent_state);

      // This object is too small to want a display list.
      gset->enableDisplayList(false);

      geode->addGeometry(gset);
   }

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
GLubyte fly[] = {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x03, 0x80, 0x01, 0xc0, 0x06, 0xc0, 0x03, 0x60,
    0x04, 0x60, 0x06, 0x20, 0x04, 0x30, 0x0c, 0x20,
    0x04, 0x18, 0x18, 0x20, 0x04, 0x0c, 0x30, 0x20,
    0x04, 0x06, 0x60, 0x20, 0x44, 0x03, 0xc0, 0x22,
    0x44, 0x01, 0x80, 0x22, 0x44, 0x01, 0x80, 0x22,
    0x44, 0x01, 0x80, 0x22, 0x44, 0x01, 0x80, 0x22,
    0x44, 0x01, 0x80, 0x22, 0x44, 0x01, 0x80, 0x22,
    0x66, 0x01, 0x80, 0x66, 0x33, 0x01, 0x80, 0xcc,
    0x19, 0x81, 0x81, 0x98, 0x0c, 0xc1, 0x83, 0x30,
    0x07, 0xe1, 0x87, 0xe0, 0x03, 0x3f, 0xfc, 0xc0,
    0x03, 0x31, 0x8c, 0xc0, 0x03, 0x33, 0xcc, 0xc0,
    0x06, 0x64, 0x26, 0x60, 0x0c, 0xcc, 0x33, 0x30,
    0x18, 0xcc, 0x33, 0x18, 0x10, 0xc4, 0x23, 0x08,
    0x10, 0x63, 0xc6, 0x08, 0x10, 0x30, 0x0c, 0x08,
    0x10, 0x18, 0x18, 0x08, 0x10, 0x00, 0x00, 0x08};

sglGroup *makeStippleObj()
{
   const unsigned int LENGTH = 4;

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = LENGTH;

   sglVec4f *color = new sglVec4f[1];
   color[0].set(1.0f, 0.0f, 1.0f, 1.0f);

   sglVec3f *vertex = new sglVec3f[LENGTH];
   vertex[0].set(0.0f,-4.0f, 4.0f);
   vertex[1].set(0.0f,-4.0f,-4.0f);
   vertex[2].set(0.0f, 4.0f, 4.0f);
   vertex[3].set(0.0f, 4.0f,-4.0f);

   sglTriangleStripSet *gset = new sglTriangleStripSet();

   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);

   vector<sglStatelet*> state;
   sglCullMode *cull_mode = new sglCullMode();
   cull_mode->setCullMode(sglCullMode::eOFF);
   state.push_back(cull_mode);
   state.push_back(new sglPolygonStipple(fly));
   gset->setState(0, state);

   // This object is too small to want a display list.
   gset->enableDisplayList(false);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
sglVec4f ncolor[2];

sglGroup *makeCoplanarObj()
{
   sglGeode *geode = new sglGeode();

   ncolor[0].set(1.0, 1.0, 0.3, 1.0);
   ncolor[1].set(0.3, 1.0, 1.0, 1.0);

   for (unsigned int ix=0; ix<2; ix++)
   {
      const unsigned int LENGTH = 4;

      unsigned int *lengths = new unsigned int[1];
      lengths[0] = LENGTH;

      sglVec3f *vertex = new sglVec3f[LENGTH];
      vertex[0].set(0.0f,-4.0f-2.0f*ix, 4.0f+2.0f*ix);
      vertex[1].set(0.0f,-4.0f-2.0f*ix,-4.0f-2.0f*ix);
      vertex[2].set(0.0f, 4.0f+2.0f*ix, 4.0f+2.0f*ix);
      vertex[3].set(0.0f, 4.0f+2.0f*ix,-4.0f-2.0f*ix);

      sglTriangleStripSet *gset = new sglTriangleStripSet();

      gset->setCoordList(vertex);
      gset->setColorList(sglGeoSet::eOVERALL, &ncolor[ix]);
      gset->setNumPrims(1);
      gset->setPrimLengths(lengths);

      vector<sglStatelet*> state;
      sglCullMode *cull_mode = new sglCullMode();
      cull_mode->setCullMode(sglCullMode::eOFF);
      state.push_back(cull_mode);
      if (ix == 0)
      {
         state.push_back(new sglPolygonOffset(-1.0, -1.0));
      }
      gset->setState(0, state);

      // This object is too small to want a display list.
      gset->enableDisplayList(false);

      geode->addGeometry(gset);
   }

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
sglGeode *makeTriangleSet()
{
   sglVec3f *vertex = new sglVec3f[8];
   vertex[0].set(  0.f, 10.f,  0.f);
   vertex[1].set(  0.f,  0.f,  0.f);
   vertex[2].set( 10.f, 10.f,  5.f);
   vertex[3].set( 10.f,  0.f,  5.f);
   vertex[4].set( 20.f, 10.f,  0.f);
   vertex[5].set( 20.f,  0.f,  0.f);
   vertex[6].set( 30.f, 10.f,  5.f);
   vertex[7].set( 30.f,  0.f,  5.f);

   sglVec3f *normal = new sglVec3f[8];
   normal[0].set(-0.44721f, 0.f, 0.894427f);
   normal[1].set(-0.44721f, 0.f, 0.894427f);
   normal[2].set(-0.44721f, 0.f, 0.894427f);
   normal[3].set(-0.44721f, 0.f, 0.894427f);
   normal[4].set( 0.44721f, 0.f, 0.894427f);
   normal[5].set( 0.44721f, 0.f, 0.894427f);
   normal[6].set(-0.44721f, 0.f, 0.894427f);
   normal[7].set(-0.44721f, 0.f, 0.894427f);

   sglVec4f *color = new sglVec4f[1];
   color[0].set(1.0f, 1.0f, 1.0f, 1.0f);

   unsigned int *length = new unsigned int[1];
   length[0] = 8;

   sglTriangleStripSet *gset = new sglTriangleStripSet;
   gset->setNumPrims(1);
   gset->setPrimLengths(length);
   gset->setCoordList(vertex);
   gset->setNormalList(sglGeoSet::ePER_VERTEX, normal);

   vector<sglStatelet*> state;
   surface_depth_range = new sglDepthRange(0.015f, 1.0f);
   state.push_back(surface_depth_range);
   state.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   state.push_back(new sglCullMode(sglCullMode::eOFF));
   state.push_back(new sglLighting(true));

   gset->setState(0, state);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);
   return geode;
}

//----------------------------------------------------------------------------
sglGeode *makeLineSet()
{
   sglVec3f *vertex = new sglVec3f[4];
   vertex[0].set( 0.f, 5.f, 0.f);
   vertex[1].set(10.f, 5.f, 5.f);
   vertex[2].set(20.f, 5.f, 0.f);
   vertex[3].set(30.f, 5.f, 5.f);

   sglVec4f *color = new sglVec4f[1];
   color[0].set(1.0f, 0.0f, 0.0f, 1.0f);

   unsigned int *length = new unsigned int[1];
   length[0] = 4;

   sglLineStripSet *gset = new sglLineStripSet();
   gset->setNumPrims(1);
   gset->setPrimLengths(length);
   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);

   vector<sglStatelet*> state;
   line_depth_range = new sglDepthRange(0.0f, 1.0f - depth_offset);
   state.push_back(line_depth_range);

   gset->setState(0, state);

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);
   return geode;
}

//----------------------------------------------------------------------------
sglGroup *makeDepthRangeObj()
{
   sglGroup *group = new sglGroup;
   group->addChild(makeLineSet());
   group->addChild(makeTriangleSet());

   return group;
}


//----------------------------------------------------------------------------
void buildScenegraph(void)
{
   // the geometry
   root = new sglScene;

   switch_node = new sglSwitch;
   switch_node->setVal(test_index);
   root->addChild(switch_node);

   switch_node->addChild(makeTransparentObj());
   switch_node->addChild(makeStippleObj());
   switch_node->addChild(makeCoplanarObj());
   switch_node->addChild(makeDepthRangeObj());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   double fov_y = (double)M_PI/(4.0f*aspect);

   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(fov_y, aspect, 1.0, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);
   //view_platform->setEnableZup(true);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setAzimuth((float)(-M_PI*0.5));
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   //view_trans->setFlyPos(0.0f, 0.0f, 0.0f);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   view_trans->addChild(new sglDirectionalLight);

   sglAxes *axes = new sglAxes(2.0f);
   vector<sglStatelet*> state1;
   state1.push_back(new sglLineWidth(2.0f));
   axes->setState(0, state1);
   sglGeode *geode = new sglGeode;
   geode->addGeometry(axes);
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(win_x, win_y);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testStatelets");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
