/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testTexture.cpp
 *  Created: 29 February 2000
 *  Summary: test the texture state functionality
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglGeode.hpp>
#include <sglTriangleStripSet.hpp>
#include <sglTexture2D.hpp>
#include <sglTexEnv.hpp>
#include <sglTexGen.hpp>
#include <sglBitmapFont.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

#include <sglImageLoader.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

bool help_flag = false;

unsigned int state_mask = 1;
unsigned int replace_index = 0;

vector<sglStatelet*> reflection_state;
vector<sglStatelet*> tex_state;
vector<sglStatelet*> tex3_state;
vector<sglStatelet*> texscale_state;
vector<sglStatelet*> tex4_state;
vector<sglStatelet*> tex5_state;

vector<sglStatelet*> tex6_state;
vector<sglStatelet*> tex7_state;

sglTexture2D *tex = NULL;
sglTexture2D *tex1 = NULL;
sglTexture2D *tex2 = NULL;
sglTexture2D *tex3 = NULL;
sglTexture2D *tex4 = NULL;
sglTexture2D *tex5 = NULL;

sglTexture2D *tex6 = NULL;
sglTexture2D *tex7 = NULL;

double sim_time = 0.0;
sglTextureMatrix *tmat;
sglTexEnv *tev1, *tev2;
sglTexEnv::ModeEnum texenv_mode = sglTexEnv::eBLEND;
int image_num = 0;

sglTriangleStripSet *gset;
sglVec4f *color;

sglTexture::ExternalFormatEnum external_format = sglTexture::eRED;
sglTexture::InternalFormatEnum internal_format = sglTexture::eINT_RGB;

unsigned int win_x = 720, win_y = 480;

bool lodBiasSup = false, lodStateSup = false, hwMipSup = false;

//----------------------------------------------------------------------------
sglTexture2D *buildMipmapTexture()
{
   unsigned char colors[][4] = {{0xff, 0x00, 0x00, 0xff},  // 32x32
                                {0x00, 0xff, 0x00, 0xff},  // 16x16
                                {0x00, 0x00, 0xff, 0xff},  // 8x8
                                {0xff, 0xff, 0xff, 0xff},  // 4x4
                                {0xff, 0xff, 0x00, 0xff},  // 2x2
                                {0xff, 0x00, 0xff, 0xff}}; // 1x1
   unsigned char *base_image = new unsigned char[32*32*4];

   for (unsigned int ix=0; ix<32*32*4; ix+=4)
   {
      base_image[ix  ] = colors[0][0];
      base_image[ix+1] = colors[0][1];
      base_image[ix+2] = colors[0][2];
      base_image[ix+3] = colors[0][3];
   }

   vector<void *> mipmap_images;
   unsigned int size = 32;
   for (unsigned int lvl=1; lvl<6; ++lvl)
   {
      size /= 2;
      unsigned char *mipmap_image = new unsigned char[size*size*4];
      for (unsigned int ix=0; ix<size*size*4; ix+=4)
      {
         mipmap_image[ix  ] = colors[lvl][0];
         mipmap_image[ix+1] = colors[lvl][1];
         mipmap_image[ix+2] = colors[lvl][2];
         mipmap_image[ix+3] = colors[lvl][3];
      }
      mipmap_images.push_back(mipmap_image);
   }

   sglTexture2D *tex = new sglTexture2D;
   tex->setImage(base_image, sglTexture::eRGBA,
                 sglTexture::eUNSIGNED_BYTE, 32, 32, false);
   tex->setAutoMipmapFlag(false);
   tex->setMipmapLevels(mipmap_images);
   tex->setMagFilter(sglTexture::eNEAREST);
   tex->setMinFilter(sglTexture::eNEAREST_MIPMAP_NEAREST);
   tex->setWrapMode(sglTexture::eREPEAT, sglTexture::eREPEAT);

   return tex;
}

//----------------------------------------------------------------------------
#define CHECK_IMAGE_SIZE 64
#define SUB_IMAGE_SIZE 16
static GLubyte check_image[2][CHECK_IMAGE_SIZE][CHECK_IMAGE_SIZE][3];
static GLubyte check_image2[2][CHECK_IMAGE_SIZE][CHECK_IMAGE_SIZE][1];
static GLubyte check_image3[CHECK_IMAGE_SIZE][CHECK_IMAGE_SIZE][4];
static GLubyte check_image4[CHECK_IMAGE_SIZE+2][CHECK_IMAGE_SIZE+2][3];
static GLubyte check_image5[SUB_IMAGE_SIZE][CHECK_IMAGE_SIZE][3];
static GLubyte check_image6[SUB_IMAGE_SIZE+2][CHECK_IMAGE_SIZE+2][3];
static GLubyte sub_image[SUB_IMAGE_SIZE][SUB_IMAGE_SIZE][3];
static vector<void*> mipmaps;
static void *resize_image = NULL;

//----------------------------------------------------------------------------
void makeCheckImageWithBorder(
   GLubyte image[][CHECK_IMAGE_SIZE+2][3], int height, int width)
{
   int i,j;
   GLubyte c;

   image[0][0][0] = 255;
   image[0][0][1] = 0;
   image[0][0][2] = 0;
   image[0][width-1][0] = 255;
   image[0][width-1][1] = 0;
   image[0][width-1][2] = 0;
   image[height-1][0][0] = 255;
   image[height-1][0][1] = 0;
   image[height-1][0][2] = 0;
   image[height-1][width-1][0] = 255;
   image[height-1][width-1][1] = 0;
   image[height-1][width-1][2] = 0;
   for (i=1; i<height-1; ++i)
   {
      image[i][0][0] = 0;
      image[i][0][1] = 255;
      image[i][0][2] = 0;
      image[i][width-1][0] = 0;
      image[i][width-1][1] = 255;
      image[i][width-1][2] = 0;
   }
   for (j=1; j<width-1; ++j)
   {
      image[0][j][0] = 0;
      image[0][j][1] = 0;
      image[0][j][2] = 255;
      image[height-1][j][0] = 0;
      image[height-1][j][1] = 0;
      image[height-1][j][2] = 255;
   }
   for (i=1; i<height-1; i++)
   {
      for (j=1; j<width-1; j++)
      {
         c = 255;
         if ((i&0x8) != (j&0x8)) c = 0;
         image[i][j][0] = c;
         image[i][j][1] = c;
         image[i][j][2] = c;
      }
   }
}

//----------------------------------------------------------------------------
void makeCheckImage(void)
{
   int i,j;
   GLubyte c, d;
   for (i=0; i<CHECK_IMAGE_SIZE; i++)
   {
      for (j=0; j<CHECK_IMAGE_SIZE; j++)
      {
         d = c = 255;
         if ((i&0x8) != (j&0x8)) c = 64;
         check_image[0][i][j][0] = c;
         check_image[0][i][j][1] = c;
         check_image[0][i][j][2] = c;

         if ((i&0x4) != (j&0x4)) d = 128;
         check_image[1][i][j][0] = 255;
         check_image[1][i][j][1] = d;
         check_image[1][i][j][2] = 0;
      }
   }

   makeCheckImageWithBorder(
      check_image4, CHECK_IMAGE_SIZE+2, CHECK_IMAGE_SIZE+2);
   makeCheckImageWithBorder(
      check_image6, SUB_IMAGE_SIZE+2, CHECK_IMAGE_SIZE+2);

   for (i=0; i<SUB_IMAGE_SIZE; i++)
   {
      for (j=0; j<CHECK_IMAGE_SIZE; j++)
      {
         c = 255;
         if ((i&0x8) != (j&0x8)) c = 0;
         check_image5[i][j][0] = c;
         check_image5[i][j][1] = c;
         check_image5[i][j][2] = c;
      }
   }

   for (i=0; i<CHECK_IMAGE_SIZE; i++)
   {
      for (j=0; j<CHECK_IMAGE_SIZE; j++)
      {
         c = 255;
         if ((i&0x8) != (j&0x8)) c = 0;
         check_image2[0][i][j][0] = c;

         check_image3[i][j][0] = c;
         check_image3[i][j][1] = c;
         check_image3[i][j][2] = c;
         check_image3[i][j][3] = c;

         check_image2[1][i][j][0] = 255;
      }
   }
   for (i=0; i<SUB_IMAGE_SIZE; i++)
   {
      for (j=0; j<SUB_IMAGE_SIZE; j++)
      {
         c = 255;
         if ((i&0x4) != (j&0x4)) c = 64;
         sub_image[i][j][0] = c;
         sub_image[i][j][1] = 0;
         sub_image[i][j][2] = 0;
      }
   }
   char a = 1;
   for (int k = CHECK_IMAGE_SIZE >> 1; k > 0; k >>= 1)
   {
      GLubyte r = ((a&0x01) ? 255 : 0);
      GLubyte g = ((a&0x02) ? 255 : 0);
      GLubyte b = ((a&0x04) ? 255 : 0);
      //cout << (int)r << " " << (int)g << " " << (int)b << endl;
      ++a;
      GLubyte *mipmap = new GLubyte[k*k*3];
      for (i=0; i<k; ++i)
      {
         for (j=0; j<k; ++j)
         {
            mipmap[(k*i*3)+(j*3)+0] = r;
            mipmap[(k*i*3)+(j*3)+1] = g;
            mipmap[(k*i*3)+(j*3)+2] = b;
         }
      }
      mipmaps.push_back(mipmap);
   }
}

//----------------------------------------------------------------------------
void outputHelp()
{
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, mouse->win_size_x, 0, mouse->win_size_y, -1, 1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glDisable(GL_DEPTH_TEST);

   glColor4f(1.0f, 0.9f, 0.9f, 1.0f);
   glRasterPos3i(10,370,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "/? - toggle help");
   glRasterPos3i(10,355,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "{ - resize current texture x/2 y*2");
   glRasterPos3i(10,340,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "} - resize current texture x*2 y/2");
   glRasterPos3i(10,325,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "[ - resize current texture x/2 y/2");
   glRasterPos3i(10,310,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "] - resize current texture x*2 y*2");
   glRasterPos3i(10,295,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "o - ouput scene graph");
   glRasterPos3i(10,280,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "$ - toggle statistics reporting");
   glRasterPos3i(10,265,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "d - toggle SP/DP traversal");
   glRasterPos3i(10,250,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "f - toggle frame rate display");
   glRasterPos3i(10,235,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "up/down arrow - cycle through texture tests");
   glRasterPos3i(10,220,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "z - do a timing test of setImage vs replaceImage");
   glRasterPos3i(10,205,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "l - toggle auto mipmaps on/off");
   glRasterPos3i(10,190,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "L - set manual mipmap levels");
   glRasterPos3i(10,175,0);
   if( sgl::s_texture_env_add_capable )
   {
      sglBitmapFont::getDefaultFont()->rasterString(
         "b - toggle blend/modulate/add");
   }
   else
   {
      sglBitmapFont::getDefaultFont()->rasterString(
         "b - toggle blend/modulate");
   }
   glRasterPos3i(10,160,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "g - toggle gset color");
   glRasterPos3i(10,145,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "r - replace texture");
   glRasterPos3i(10,130,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "R - reset texture");
   glRasterPos3i(10,115,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "s - subtexload");
   glRasterPos3i(10,100,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "t - toggle image with border on/off");
   glRasterPos3i(10,85,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "m/M - cycle through min/mag filters");
   glRasterPos3i(10,70,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "w - cycle through wrap mode");
   glRasterPos3i(10,55,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "c - copy framebuffer image");
   glRasterPos3i(10,40,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "a/A - turn anisotropic filtering on/off");
   glRasterPos3i(10,25,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "i - cycle through internal formats");
   glRasterPos3i(10,10,0);
   sglBitmapFont::getDefaultFont()->rasterString(
      "e - cycle through external formats");

   // second column
   if( lodBiasSup )
   {
      glRasterPos3i(500,370,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "</> - dec/inc lod bias");
   }

   if( lodStateSup )
   {
      glRasterPos3i(500,355,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "7/4 - inc/dec min lod (coarse)");

      glRasterPos3i(500,340,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "8/5 - inc/dec max lod (coarse)");

      glRasterPos3i(500,325,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "9/6 - inc/dec lod base level");

      glRasterPos3i(500,310,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "2/1 - inc/dec lod max level");
   }

   if( hwMipSup )
   {
      glRasterPos3i(500,295,0);
      sglBitmapFont::getDefaultFont()->rasterString(
         "h - toggle hardware mip");
   }

   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();

   if (help_flag)
   {
      outputHelp();
   }

   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'l':
         if (tex1->getAutoMipmapFlag())
         {
            tex1->setAutoMipmapFlag(false);
            cout << "auto mipmap off" << endl;
         }
         else
         {
            tex1->setAutoMipmapFlag(true);
            cout << "auto mipmap on" << endl;
         }
         break;

      case 'L':
         tex1->setMipmapLevels(mipmaps);
         break;

      case 'b':
         if( texenv_mode == sglTexEnv::eBLEND )
         {
            texenv_mode = sglTexEnv::eMODULATE;
            cout << "texenv mode: eMODULATE" << endl;
         }
         else if( texenv_mode == sglTexEnv::eMODULATE )
         {
            if( sgl::s_texture_env_add_capable )
            {
               texenv_mode = sglTexEnv::eADD;
               cout << "texenv mode: eADD" << endl;
            }
            else
            {
               texenv_mode = sglTexEnv::eBLEND;
               cout << "texenv mode: eBLEND" << endl;
            }

         }
         else
         {
            texenv_mode = sglTexEnv::eBLEND;
            cout << "texenv mode: eBLEND" << endl;
         }

         tev1->setMode( texenv_mode );
         break;

      case 'h':
         // only change option if supported
         if( hwMipSup )
         {
            if( sgl::s_auto_mipmap_capable )
            {
               sgl::s_auto_mipmap_capable = false;
               cout << "using software mip" << endl;
            }
            else
            {
               sgl::s_auto_mipmap_capable = true;
               cout << "using hardware mip" << endl;
            }
         }
         break;

      case '>':
         tev1->setLODBias( tev1->getLODBias() + 0.01 );
         tev2->setLODBias( tev2->getLODBias() + 0.01 );
         cout << "LOD Bias: " << tev1->getLODBias() << endl;
         break;

      case '<':
         tev1->setLODBias( tev1->getLODBias() - 0.01 );
         tev2->setLODBias( tev2->getLODBias() - 0.01 );
         cout << "LOD Bias: " << tev1->getLODBias() << endl;
         break;

      case '7':
      {
         float mlod, Mlod;

         tex1->getLODMinMax( mlod, Mlod );
         mlod += 10;
         tex1->setLODMinMax( mlod, Mlod );
         cout << "MIN LOD: " << mlod << endl;
         break;
      }
      case '4':
      {
         float mlod, Mlod;

         tex1->getLODMinMax( mlod, Mlod );
         mlod -= 10;
         tex1->setLODMinMax( mlod, Mlod );
         cout << "MIN LOD: " << mlod << endl;
         break;
      }
      case '8':
      {
         float mlod, Mlod;

         tex1->getLODMinMax( mlod, Mlod );
         Mlod += 10;
         tex1->setLODMinMax( mlod, Mlod );
         cout << "MAX LOD: " << Mlod << endl;
         break;
      }
      case '5':
      {
         float mlod, Mlod;

         tex1->getLODMinMax( mlod, Mlod );
         Mlod -= 10;
         tex1->setLODMinMax( mlod, Mlod );
         cout << "MAX LOD: " << Mlod << endl;
         break;
      }

      case '9':
         tex1->setLODBaseLevel( tex1->getLODBaseLevel() + 1 );
         tex5->setLODBaseLevel( tex5->getLODBaseLevel() + 1 );
         cout << "LOD Base: " << tex1->getLODBaseLevel() << endl;
         break;

      case '6':
         if( tex1->getLODBaseLevel() > 0 )
         {
            tex1->setLODBaseLevel( tex1->getLODBaseLevel() - 1 );
            tex5->setLODBaseLevel( tex5->getLODBaseLevel() - 1 );
         }
         cout << "LOD Base: " << tex1->getLODBaseLevel() << endl;
         break;

      case '2':
         if( tex1->getLODMaxLevel() > 19 )
         {
            tex1->setLODMaxLevel( 19 );
            tex5->setLODMaxLevel( 19 );
         }
         tex1->setLODMaxLevel( tex1->getLODMaxLevel() + 1 );
         tex5->setLODMaxLevel( tex5->getLODMaxLevel() + 1 );
         cout << "LOD Max: " << tex1->getLODMaxLevel() << endl;
         break;

      case '1':
         if( tex1->getLODMaxLevel() > 0 )
         {
            tex1->setLODMaxLevel( tex1->getLODMaxLevel() - 1 );
            tex5->setLODMaxLevel( tex5->getLODMaxLevel() - 1 );
         }
         cout << "LOD Max: " << tex1->getLODMaxLevel() << endl;
         break;

      case 'g':
         if (color[0][0] == 1.0f)
            color->set(0.5f, 1.0f, 1.0f, 1.0f);
         else
            color->set(1.0f, 1.0f, 1.0f, 1.0f);
         gset->setColorList(sglGeoSet::eOVERALL, color);
         break;

      case 'r':
         replace_index = 1 - replace_index;
         tex1->replaceImage(&check_image[replace_index][0][0][0]);
         break;

      case 'R':
         tex1->reset();
         break;

      case 's':
         tex1->subloadImage(&sub_image[0][0][0], 0, 12, 44,
                           SUB_IMAGE_SIZE, SUB_IMAGE_SIZE);
         tex->subloadImage(&sub_image[0][0][0], 0, 12, 44,
                           SUB_IMAGE_SIZE, SUB_IMAGE_SIZE);
         break;

      case 't':
         image_num = (image_num+1)%4;
         if (image_num == 0)
            tex1->setImage(&check_image[replace_index][0][0][0],
                           sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                           CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);
         else if (image_num == 1)
            tex1->setImage(&check_image4[0][0][0],
                           sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                           CHECK_IMAGE_SIZE+2, CHECK_IMAGE_SIZE+2, true);
         else if (image_num == 2)
            tex1->setImage(&check_image5[0][0][0],
                           sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                           CHECK_IMAGE_SIZE, SUB_IMAGE_SIZE, false);
         else
            tex1->setImage(&check_image6[0][0][0],
                           sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                           CHECK_IMAGE_SIZE+2, SUB_IMAGE_SIZE+2, true);
         break;

      case '}':
      case '{':
      case ']':
      case '[':
      {
         void *old_image;
         sglTexture2D::ExternalFormatEnum format;
         sglTexture2D::TypeEnum type;
         unsigned int width, height, new_width, new_height;
         bool border;
         tex1->getImage(old_image, format, type, width, height, border);
         cerr << "old width = " << width << ", height = " << height << endl;
         int b = (border ? 2 : 0);
         switch (key)
         {
            case '}':
               new_width = ((width-b) * 2) + b;
               new_height = ((height-b) / 2) + b;
               break;
            case '{':
               new_width = ((width-b) / 2) + b;
               new_height = ((height-b) * 2) + b;
               break;
            case ']':
               new_width = ((width-b) * 2) + b;
               new_height = ((height-b) * 2) + b;
               break;
            default:
               new_width = ((width-b) / 2) + b;
               new_height = ((height-b) / 2) + b;
               break;
         }
         cerr << "new width = " << new_width << ", height = "
              << new_height << endl;
         void *tmp_image = NULL;
         sglTexture2D::resizeImage(
            format, type, width, height, border,
            new_width, new_height, old_image, tmp_image);
         tex1->setImage(tmp_image, format, type,
                        new_width, new_height, border);
         if (resize_image) free(resize_image);
         resize_image = tmp_image;
         break;
      }

      case 'm':
         switch (tex1->getMinFilter())
         {
            case sglTexture::eNEAREST:
               tex1->setMinFilter(sglTexture::eLINEAR);
               cout << "sglTexture::eLINEAR" << endl;
               break;
            case sglTexture::eLINEAR:
               tex1->setMinFilter(sglTexture::eNEAREST_MIPMAP_NEAREST);
               cout << "sglTexture::eNEAREST_MIPMAP_NEAREST" << endl;
               break;
            case sglTexture::eNEAREST_MIPMAP_NEAREST:
               tex1->setMinFilter(sglTexture::eNEAREST_MIPMAP_LINEAR);
               cout << "sglTexture::eNEAREST_MIPMAP_LINEAR" << endl;
               break;
            case sglTexture::eNEAREST_MIPMAP_LINEAR:
               tex1->setMinFilter(sglTexture::eLINEAR_MIPMAP_NEAREST);
               cout << "sglTexture::eLINEAR_MIPMAP_NEAREST" << endl;
               break;
            case sglTexture::eLINEAR_MIPMAP_NEAREST:
               tex1->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
               cout << "sglTexture::eLINEAR_MIPMAP_LINEAR" << endl;
               break;
            case sglTexture::eLINEAR_MIPMAP_LINEAR:
               tex1->setMinFilter(sglTexture::eNEAREST);
               cout << "sglTexture::eNEAREST" << endl;
               break;
         }
         break;

      case 'M':
         if (tex1->getMagFilter() == sglTexture::eNEAREST)
         {
            tex1->setMagFilter(sglTexture::eLINEAR);
            cout << "sglTexture::eLINEAR" << endl;
         }
         else
         {
            tex1->setMagFilter(sglTexture::eNEAREST);
            cout << "sglTexture::eNEAREST" << endl;
         }
         break;

      case 'w':
         sglTexture::WrapEnum wrap_s, wrap_t;
         tex->getWrapMode(wrap_s, wrap_t);
         switch (wrap_t)
         {
            case sglTexture::eREPEAT:
               tex->setWrapMode(sglTexture::eREPEAT, sglTexture::eCLAMP);
               tex1->setWrapMode(sglTexture::eCLAMP, sglTexture::eCLAMP);
               cout << "CLAMP" << endl;
               break;
            case sglTexture::eCLAMP:
               tex->setWrapMode(
                  sglTexture::eREPEAT, sglTexture::eCLAMP_TO_EDGE);
               tex1->setWrapMode(
                  sglTexture::eCLAMP_TO_EDGE, sglTexture::eCLAMP_TO_EDGE);
               cout << "eCLAMP_TO_EDGE" << endl;
               break;
            case sglTexture::eCLAMP_TO_EDGE:
               tex->setWrapMode(sglTexture::eREPEAT, sglTexture::eREPEAT);
               tex1->setWrapMode(sglTexture::eREPEAT, sglTexture::eREPEAT);
               cout << "REPEAT" << endl;
               break;
         }
         break;

      case 'C':
         if( tex6->getCompressionLevel() == sglTexture::eCOMPRESS_NONE )
         {
            cout << "eCOMPRESS_MAX" << endl;
            tex6->setCompressionLevel( sglTexture::eCOMPRESS_MAX );
         }
         else
         {
            cout << "eCOMPRESS_NONE" << endl;
            tex6->setCompressionLevel( sglTexture::eCOMPRESS_NONE );
         }
         break;

      case 'c':
         tex1->copyTexSubImage(0, 0, 0,
                               (win_x-CHECK_IMAGE_SIZE)/2,
                               (win_y-CHECK_IMAGE_SIZE)/2,
                               CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE);
         break;

      case 'a':
         tex1->setMaxAnisotropy(tex1->getMaxSupportedMaxAnisotropy());
         cout << "max anisotropy = " << tex1->getMaxAnisotropy() << endl;
         break;

      case 'A':
         tex1->setMaxAnisotropy(1.0f);
         cout << "max anisotropy = " << tex1->getMaxAnisotropy() << endl;
         break;

      case 'i':
         if (internal_format == sglTexture::eINT_LUMINANCE)
         {
            internal_format = sglTexture::eINT_LUMINANCE_ALPHA;
            cout << "internal format = sglTexture::eINT_LUMINANCE_ALPHA"
                 << endl;
         }
         else if (internal_format == sglTexture::eINT_LUMINANCE_ALPHA)
         {
            internal_format = sglTexture::eINT_RGB;
            cout << "internal format = sglTexture::eINT_RGB" << endl;
         }
         else if (internal_format == sglTexture::eINT_RGB)
         {
            internal_format = sglTexture::eINT_RGBA;
            cout << "internal format = sglTexture::eINT_RGBA" << endl;
         }
         else
         {
            internal_format = sglTexture::eINT_LUMINANCE;
            cout << "internal format = sglTexture::eINT_LUMINANCE" << endl;
         }

         //tex3->setImage(&check_image2[0][0][0][0], external_format,
         //               sglTexture::eUNSIGNED_BYTE,
         //               CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE);
         tex3->setInternalFormat(internal_format);
         break;

      case 'e':
         if (external_format == sglTexture::eRED)
         {
            external_format = sglTexture::eGREEN;
            cout << "external format = sglTexture::eGREEN" << endl;
         }
         else if (external_format == sglTexture::eGREEN)
         {
            external_format = sglTexture::eBLUE;
            cout << "external format = sglTexture::eBLUE" << endl;
         }
         else if (external_format == sglTexture::eBLUE)
         {
            external_format = sglTexture::eALPHA;
            cout << "external format = sglTexture::eALPHA" << endl;
         }
         else if (external_format == sglTexture::eALPHA)
         {
            external_format = sglTexture::eLUMINANCE;
            cout << "external format = sglTexture::eLUMINANCE" << endl;
         }
         else
         {
            external_format = sglTexture::eRED;
            cout << "external format = sglTexture::eRED" << endl;
         }

         tex3->setImage(&check_image2[0][0][0][0], external_format,
                        sglTexture::eUNSIGNED_BYTE,
                        CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE,
                        false);
         break;

      case 'z':
      {
         unsigned int i;
         sglTimespec t, last_t;
         double e[8];

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->setImage(&check_image[0][0][0][0],
                          sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                          CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[0] = sglTimespec::convertToDouble(t - last_t);

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->replaceImage(&check_image[0][0][0][0]);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[1] = sglTimespec::convertToDouble(t - last_t);

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->replaceImage(&check_image[0][0][0][0]);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[2] = sglTimespec::convertToDouble(t - last_t);

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->setImage(&check_image[0][0][0][0],
                          sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                          CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[3] = sglTimespec::convertToDouble(t - last_t);

         tex->setAutoMipmapFlag(true);

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->setImage(&check_image[0][0][0][0],
                          sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                          CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[4] = sglTimespec::convertToDouble(t - last_t);

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->replaceImage(&check_image[0][0][0][0]);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[5] = sglTimespec::convertToDouble(t - last_t);

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->replaceImage(&check_image[0][0][0][0]);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[6] = sglTimespec::convertToDouble(t - last_t);

         sglTimespec::getSysTime(last_t);
         for (i=0; i<1000; ++i)
         {
            tex->setImage(&check_image[0][0][0][0],
                          sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                          CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);
            tex->bind();
         }
         sglTimespec::getSysTime(t);
         e[7] = sglTimespec::convertToDouble(t - last_t);

         cout << "setImage     no-mipmap = " << e[0] << endl;
         cout << "replaceImage no-mipmap = " << e[1] << endl;
         cout << "replaceImage no-mipmap = " << e[2] << endl;
         cout << "setImage     no-mipmap = " << e[3] << endl;
         cout << "setImage     mipmap    = " << e[4] << endl;
         cout << "replaceImage mipmap    = " << e[5] << endl;
         cout << "replaceImage mipmap    = " << e[6] << endl;
         cout << "setImage     mipmap    = " << e[7] << endl;

         tex->setAutoMipmapFlag(false);
         tex->reset();
         tex->bind();
         break;
      }

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '?':
      case '/':
         help_flag = !help_flag;
         cout << "help_flag = " << help_flag << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         if (state_mask == 0x40)
            state_mask = 1;
         else
            state_mask <<= 1;
         view->setStateMask(state_mask);
         break;
      case GLUT_KEY_DOWN:
         if (state_mask == 1)
            state_mask = 0x40;
         else
            state_mask >>= 1;
         view->setStateMask(state_mask);
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   sim_time += 0.1f;
   sglMat4f mat;
   mat.buildTranslation(0.2f*cos(sim_time), 0.2f*sin(sim_time), 0.f);
   mat[0][0] = 1.1f + cos(sim_time);
   mat[1][1] = 1.1f + sin(sim_time);
   tmat->setTextureMatrix(&mat);

   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------

void preTextureBindCallback(sglTexture *tex)
{
   cout << "prebind " << tex << endl;
}

//----------------------------------------------------------------------------

void postTextureBindCallback(sglTexture *tex)
{
   cout << "postbind " << tex << endl;
}

//----------------------------------------------------------------------------

void makeStates()
{
   makeCheckImage();

   {
      tex1 = new sglTexture2D;
      tex1->setImage(&check_image[0][0][0][0], sglTexture::eRGB,
                     sglTexture::eUNSIGNED_BYTE,
                     CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);

      tex1->setAutoMipmapFlag(true);
      tex1->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
      tex1->setMagFilter(sglTexture::eLINEAR); //sglTexture::eNEAREST);
      tex1->setWrapMode(sglTexture::eCLAMP, sglTexture::eCLAMP);
      tex1->setBorderColor(sglVec4f(1.0, 0.0, 0.0, 1.0));
      tex1->setPreBindCallback(preTextureBindCallback);
      tex1->setPostBindCallback(postTextureBindCallback);

      tev1 = new sglTexEnv();
      tev1->setBlendColor(1.0f, 0.7f, 0.0f, 1.0f);
      tev1->setMode(sglTexEnv::eBLEND);

      tex_state.push_back(tex1);
      tex_state.push_back(tev1);
   }

   {
      tex = new sglTexture2D;
      tex->setImage(&check_image[0][0][0][0], sglTexture::eRGB,
                    sglTexture::eUNSIGNED_BYTE,
                    CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);

      tex->setWrapMode(sglTexture::eREPEAT, sglTexture::eCLAMP);
      tex->setBorderColor(sglVec4f(1.0, 0.0, 0.0, 1.0));

      sglTexEnv  *tev = new sglTexEnv();
      tev->setMode(sglTexEnv::eDECAL);

      texscale_state.push_back(tex);
      texscale_state.push_back(tev);

      sglMat4f mat;
      mat.buildTranslation(-0.2f, -0.2f, 0.0f);
      tmat = new sglTextureMatrix();
      tmat->setTextureMatrix(&mat);
      texscale_state.push_back(tmat);
   }

   // test reflection mapping sglTexGen
   {
      tex2 = new sglTexture2D;
      tex2->setImage(&check_image[0][0][0][0], sglTexture::eRGB,
                     sglTexture::eUNSIGNED_BYTE,
                     CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);

      tex2->setMagFilter(sglTexture::eNEAREST);
      tex2->setMinFilter(sglTexture::eNEAREST);

      sglTexEnv  *tev = new sglTexEnv();
      tev->setMode(sglTexEnv::eMODULATE);

      sglTexGen *tgen = new sglTexGen();
      tgen->setMode(sglTexGen::eS, sglTexGen::eSPHERE_MAP);
      tgen->setMode(sglTexGen::eT, sglTexGen::eSPHERE_MAP);

      reflection_state.push_back(tex2);
      reflection_state.push_back(tev);
      reflection_state.push_back(tgen);
   }

   // test the various internal and external texture modes and
   // sglTexEnv blend colors
   {
      tex3 = new sglTexture2D;
      tex3->setImage(&check_image2[0][0][0][0], external_format,
                     sglTexture::eUNSIGNED_BYTE,
                     CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE,
                     false);
      tex3->setInternalFormat(internal_format);

      tex3->setAutoMipmapFlag(true);
      tex3->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
      //tex3->setMinFilter(sglTexture::eNEAREST);
      tex3->setMagFilter(sglTexture::eNEAREST);
      tex3->setWrapMode(sglTexture::eREPEAT, sglTexture::eCLAMP_TO_EDGE);
      tex3->setBorderColor(sglVec4f(1.0, 0.0, 0.0, 1.0));

      sglTexEnv  *tev = new sglTexEnv();
      tev->setBlendColor(1.0f, 0.7f, 0.0f, 1.0f);
      tev->setMode(sglTexEnv::eMODULATE);

      tex3_state.push_back(tex3);
      tex3_state.push_back(tev);
      tex3_state.push_back(new sglTransparency(true));
   }

   // test the REPLACE mode in sglTexEnv when using ALPHA internal format
   {
      tex4 = new sglTexture2D;
      tex4->setImage(&check_image2[0][0][0][0],
                     sglTexture::eALPHA, sglTexture::eUNSIGNED_BYTE,
                     CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);
      tex4->setInternalFormat(sglTexture::eINT_ALPHA);

      tex4->setMagFilter(sglTexture::eNEAREST);
      tex4->setMinFilter(sglTexture::eNEAREST);

      sglTexEnv  *tev = new sglTexEnv();
      tev->setMode(sglTexEnv::eMODULATE); //REPLACE);

      tex4_state.push_back(tex4);
      tex4_state.push_back(tev);
      tex4_state.push_back(new sglAlphaTest(sglAlphaTest::eGREATER,0.0f));
   }

   // test manual mipmapping
   {
      //sglTexEnv *tev = new sglTexEnv();
      tev2 = new sglTexEnv();
      tev2->setMode(sglTexEnv::eDECAL);
      tex5 = buildMipmapTexture();

      tex5_state.push_back(tev2);
      tex5_state.push_back( tex5 );
   }

   // test images for compression
   {
      sglTexture::ExternalFormatEnum format;
      sglTexture::TypeEnum type;
      unsigned int num_s, num_t;
      void * image = NULL;

      image = sglImageLoader::loadImage("../../models/sp01047.rgb", num_s,
                                        num_t, format, type );
      if( image )
      {
         tex6 = new sglTexture2D();
         tex6->setAutoMipmapFlag(true);
         tex6->setWrapMode(sglTexture::eREPEAT, sglTexture::eREPEAT);
         tex6->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
         tex6->setMagFilter(sglTexture::eLINEAR);

         tex6->setImage( image, format, type, num_s, num_t, false );
         tex6->setCompressionLevel( sglTexture::eCOMPRESS_NONE );

         sglTexEnv *tev = new sglTexEnv();
         tev = new sglTexEnv();
         tev->setMode(sglTexEnv::eREPLACE);

         tex6_state.push_back(tev);
         tex6_state.push_back( tex6 );
      }
      else
      {
         cout << "Unable to find ../models/wall.rgb.." << endl;
      }
   }
}

//----------------------------------------------------------------------------
sglVec3f normal(0.0f, -1.0f, 0.0f);

sglGroup *makeTextureObj()
{
   const unsigned int LENGTH = 4;

   unsigned int *lengths = new unsigned int[1];
   lengths[0] = LENGTH;

   sglVec3f *vertex = new sglVec3f[LENGTH];
   vertex[0].set(-8.0f, 0.0f,  8.0f);
   vertex[1].set(-8.0f, 0.0f, -8.0f);
   vertex[2].set( 8.0f, 0.0f,  8.0f);
   vertex[3].set( 8.0f, 0.0f, -8.0f);

   sglVec3f *normal = new sglVec3f[LENGTH];
   normal[0].set(1.0f, 0.0f, 0.0f);
   normal[1].set(1.0f, 0.0f, 0.0f);
   normal[2].set(1.0f, 0.0f, 0.0f);
   normal[3].set(1.0f, 0.0f, 0.0f);

   color = new sglVec4f[1];
   color->set(0.5f, 1.0f, 1.0f, 1.0f);

   gset = new sglTriangleStripSet();

   gset->setCoordList(vertex);
   gset->setNormalList(sglGeoSet::ePER_VERTEX, normal);
   gset->setColorList(sglGeoSet::eOVERALL,   color);
   gset->setNumPrims(1);
   gset->setPrimLengths(lengths);

   // This object is too small to want a display list.
   gset->enableDisplayList(false);

   {
      sglVec2f *tvert = new sglVec2f[LENGTH];
      tvert[0].set(0.0f, 2.0f);
      tvert[1].set(0.0f, 0.0f);
      tvert[2].set(2.0f, 2.0f);
      tvert[3].set(2.0f, 0.0f);
      gset->setState(0x01, tex_state, tvert);
   }

   {
      sglVec2f *tvert = new sglVec2f[LENGTH];
      tvert[0].set(0.0f, 1.0f);
      tvert[1].set(0.0f, 0.0f);
      tvert[2].set(1.0f, 1.0f);
      tvert[3].set(1.0f, 0.0f);
      gset->setState(0x02, texscale_state, tvert);
   }

   gset->setState(0x04, reflection_state);

   {
      sglVec2f *tvert = new sglVec2f[LENGTH];
      tvert[0].set(0.0f, 1.0f);
      tvert[1].set(0.0f, 0.0f);
      tvert[2].set(1.0f, 1.0f);
      tvert[3].set(1.0f, 0.0f);
      gset->setState(0x08, tex3_state, tvert);
   }


   {
      sglVec2f *tvert = new sglVec2f[LENGTH];
      tvert[0].set(0.0f, 1.0f);
      tvert[1].set(0.0f, 0.0f);
      tvert[2].set(1.0f, 1.0f);
      tvert[3].set(1.0f, 0.0f);
      gset->setState(0x10, tex4_state, tvert);
   }

   {
      sglVec2f *tvert = new sglVec2f[LENGTH];
      tvert[0].set(0.0f, 10.f);
      tvert[1].set(0.0f, 0.0f);
      tvert[2].set(10.f, 10.f);
      tvert[3].set(10.f, 0.0f);
      gset->setState(0x20, tex5_state, tvert);
   }

   {
      sglVec2f *tvert = new sglVec2f[LENGTH];
      tvert[0].set(0.0f, 1.f);
      tvert[1].set(0.0f, 0.0f);
      tvert[2].set(1.f, 1.f);
      tvert[3].set(1.f, 0.0f);
      gset->setState(0x40, tex6_state, tvert);
   }

   sglGeode *geode = new sglGeode();
   geode->addGeometry(gset);

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   return group;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   makeStates();

   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);
   root->addChild(makeTextureObj());

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setClearColor(0.4f, 0.4f, 0.4f, 1.0f);
   view->setViewport(0, 0, win_x, win_y);
   view->setStateMask(state_mask);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   view->setOverrideStatelets(override_statelets);

#ifdef DONT_AUTOBIND_TEXTURES
   tex->bind();
   tex1->bind();
   tex2->bind();
   tex3->bind();
   tex4->bind();
#endif
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testTexture");

   sgl::initialize();
#ifdef DONT_AUTOBIND_TEXTURES
   sglTexture::setAutoBind(false);
#endif
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   if( sgl::s_auto_mipmap_capable )
      hwMipSup = true;

   if( sgl::s_texture_lod_bias_capable )
      lodBiasSup = true;

   if( sgl::s_texture_lod_capable )
      lodStateSup = true;

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
