/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testTextureFont.cpp
 *  Created: January 2001
 *  Summary: test the sglTextureFont functionality
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglTextureFont.hpp>
#include <sglTexturedTextSet.hpp>
#include <sglBitmapFont.hpp>
#include <sglTexture2D.hpp>
#include <sglGeode.hpp>
#include <sglQuadSet.hpp>
#include <sglTranslate.hpp>
#include <sglStatelet.hpp>
#include <sglSwitch.hpp>
#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

#ifdef WIN32
# include <sglBitmapFontWGL.hpp>
#else
# include <GL/glx.h>
#endif


sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *view_camera;

deque<const sglStatelet*> override_statelets;

vector<sglTextureFont*> textureFonts;

sglSwitch   *switch_node;
int          switch_val = 0;

unsigned int win_x = 640, win_y = 480;

//----------------------------------------------------------------------------
// needs to be called just before the culled geometry is rendered
void preDrawFunc(void *)
{
   glClearColor(.5, .5, .5, 0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);
   glMatrixMode (GL_MODELVIEW);
}

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 10000.0);
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'm':
      {
         sglTexture::FilterEnum min_filter =
            textureFonts[0]->getTexture()->getMinFilter();
         switch (min_filter)
         {
            case sglTexture::eNEAREST:
               min_filter = sglTexture::eLINEAR;
               cout << "sglTexture::eLINEAR" << endl;
               break;
            case sglTexture::eLINEAR:
               min_filter = sglTexture::eNEAREST_MIPMAP_NEAREST;
               cout << "sglTexture::eNEAREST_MIPMAP_NEAREST" << endl;
               break;
            case sglTexture::eNEAREST_MIPMAP_NEAREST:
               min_filter = sglTexture::eNEAREST_MIPMAP_LINEAR;
               cout << "sglTexture::eNEAREST_MIPMAP_LINEAR" << endl;
               break;
            case sglTexture::eNEAREST_MIPMAP_LINEAR:
               min_filter = sglTexture::eLINEAR_MIPMAP_NEAREST;
               cout << "sglTexture::eLINEAR_MIPMAP_NEAREST" << endl;
               break;
            case sglTexture::eLINEAR_MIPMAP_NEAREST:
               min_filter = sglTexture::eLINEAR_MIPMAP_LINEAR;
               cout << "sglTexture::eLINEAR_MIPMAP_LINEAR" << endl;
               break;
            case sglTexture::eLINEAR_MIPMAP_LINEAR:
               min_filter = sglTexture::eNEAREST;
               cout << "sglTexture::eNEAREST" << endl;
               break;
         }

         for (vector<sglTextureFont*>::iterator it = textureFonts.begin();
              it != textureFonts.end(); ++it)
            (*it)->getTexture()->setMinFilter(min_filter);
         break;
      }
      case 'M':
      {
         sglTexture::FilterEnum mag_filter =
            textureFonts[0]->getTexture()->getMagFilter();
         if (mag_filter == sglTexture::eNEAREST)
         {
            mag_filter = sglTexture::eLINEAR;
            cout << "sglTexture::eLINEAR" << endl;
         }
         else
         {
            mag_filter = sglTexture::eNEAREST;
            cout << "sglTexture::eNEAREST" << endl;
         }

         for (vector<sglTextureFont*>::iterator it = textureFonts.begin();
              it != textureFonts.end(); ++it)
            (*it)->getTexture()->setMagFilter(mag_filter);
         break;
      }
      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 'o':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "m - cycle through texture min filters" << endl;
         cerr << "M - cycle through texture mag filters" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "o - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}

//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         switch_val++;
         if ((unsigned int)switch_val >= switch_node->getNumChildren())
         {
            switch_val = 0;
         }
         switch_node->setVal(switch_val);
         break;

      case GLUT_KEY_DOWN:
         switch_val--;
         if (switch_val < 0)
         {
            switch_val = switch_node->getNumChildren() - 1;
         }
         switch_node->setVal(switch_val);
         break;
   }
}

//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------

sglNode *makeQuadObj(sglTextureFont *font)
{
   sglTexture2D *tex = font->getTexture();
   tex->setAutoMipmapFlag(true);
   unsigned int width, height;
   tex->getSize(width, height);
   cerr << "texsize = " << width << ", " << height << endl;
   float w = (float)width / 2.0f;
   float h = (float)height / 2.0f;

   sglQuadSet *quad = new sglQuadSet();

   sglVec3f *vertex = new sglVec3f[4];
   vertex[0].set(-w, -h, 0.0f);
   vertex[1].set( w, -h, 0.0f);
   vertex[2].set( w,  h, 0.0f);
   vertex[3].set(-w,  h, 0.0f);

   sglVec4f *color = new sglVec4f[1];
   color[0].set(1.0f, 0.0f, 0.0f, 1.0f);

   quad->setCoordList(vertex);
   quad->setColorList(sglGeoSet::eOVERALL, color);
   quad->setNumPrims(1);

   sglVec2f *tex_coords = new sglVec2f[4];
   tex_coords[0].set(0.0f, 0.0f);
   tex_coords[1].set(1.0f, 0.0f);
   tex_coords[2].set(1.0f, 1.0f);
   tex_coords[3].set(0.0f, 1.0f);

   vector<sglStatelet*> state;
   state.push_back(new sglTransparency(true));
   state.push_back(tex);
   quad->setState(0, state, tex_coords);

   // This object is too small to want a display list.
   quad->enableDisplayList(false);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(quad);

   sglTexturedTextSet *ttext1 = new sglTexturedTextSet(font);
   ttext1->setVertical(true);
   ttext1->setCharHeight(font->getHeight());
   ttext1->setColor(sglVec4f(0, 0, 1.0f, 1.0f));
   ttext1->setPosition(0.f, 0.f, 20.0f);
   ttext1->setString("VerticaL Text");

   sglTexturedTextSet *ttext2 = new sglTexturedTextSet(font);
   ttext2->setCharHeight(font->getHeight());
   ttext2->setString("HorizontaL Text");

   sglTranslatef *trans = new sglTranslatef();
   sglGeode *geode2 = new sglGeode;
   geode2->addGeometry(ttext1);
   geode2->addGeometry(ttext2);
   trans->addChild(geode2);

   if (w < h)
      trans->setTranslation(-(w + 2*ttext1->getWidth()),
                            ttext1->getHeight()/2, 0);
   else
      trans->setTranslation(-ttext2->getWidth()/2,
                            h + 2*font->getHeight(), 0);

   sglGroup *group = new sglGroup();
   group->addChild(geode);
   group->addChild(trans);

   return group;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   // make scene and default state:
   root = new sglScene();

   // add your stuff here
   switch_node = new sglSwitch();
   switch_node->setVal(switch_val);
   root->addChild(switch_node);

   textureFonts.push_back(
      new sglTextureFont(*sglBitmapFont::getDefaultFont(), true, '.'));
   switch_node->addChild(makeQuadObj(textureFonts.back()));

   vector<char> valid_chars;
   for (char c = 'a'; c <= 'z'; ++c)
      valid_chars.push_back(c);
   valid_chars.push_back('.');
   textureFonts.push_back(
      new sglTextureFont(*sglBitmapFont::getDefaultFont(),
                         valid_chars, true, '.'));
   switch_node->addChild(makeQuadObj(textureFonts.back()));

   textureFonts.push_back(
      new sglTextureFont(*sglBitmapFont::getDefaultFont(),
                         valid_chars, false, '.'));
   switch_node->addChild(makeQuadObj(textureFonts.back()));

#ifdef WIN32
   LOGFONT lfont;
   memset(&lfont, 0, sizeof(LOGFONT));
   lfont.lfHeight = 24;
   lfont.lfWeight = FW_NORMAL;
   lfont.lfItalic = FALSE;
   lfont.lfCharSet = ANSI_CHARSET;
   lfont.lfPitchAndFamily = FIXED_PITCH|FF_DONTCARE;
   strcpy(lfont.lfFaceName, "*");

   HFONT hfont = CreateFontIndirect(&lfont);
   if (!hfont)
   {
       sglPrint() << "CreateFontIndirect() failed" << endl;
   }
   else
   {
       sglBitmapFontWGL fixedFont(hfont);
       textureFonts.push_back(
           new sglTextureFont(fixedFont, true, '.'));
       switch_node->addChild(makeQuadObj(textureFonts.back()));
       DeleteObject(hfont);
   }

   lfont.lfPitchAndFamily = VARIABLE_PITCH|FF_DONTCARE;
   hfont = CreateFontIndirect(&lfont);
   if (!hfont)
   {
       sglPrint() << "CreateFontIndirect() failed" << endl;
   }
   else
   {
       sglBitmapFontWGL proportionalFont(hfont);
       textureFonts.push_back(
           new sglTextureFont(proportionalFont, true, '.'));
       switch_node->addChild(makeQuadObj(textureFonts.back()));
       DeleteObject(hfont);
   }

   lfont.lfPitchAndFamily = FIXED_PITCH|FF_DONTCARE;
   strcpy(lfont.lfFaceName, "System");

   hfont = CreateFontIndirect(&lfont);
   if (!hfont)
   {
       sglPrint() << "CreateFontIndirect() failed" << endl;
   }
   else
   {
       sglBitmapFontWGL fixedFont(hfont);
       textureFonts.push_back(
           new sglTextureFont(fixedFont, true, '.'));
       switch_node->addChild(makeQuadObj(textureFonts.back()));
       DeleteObject(hfont);
   }

#else
   Font font1 = XLoadFont(glXGetCurrentDisplay(),
                          "*-medium-r-normal--24-*-*-*-m-*-iso8859-1");
//                          "-bitstream-courier-medium-r-normal--50-*-*-*-m-*-iso8859-1");
   sglBitmapFontGLX fixedFont(font1);
   textureFonts.push_back(
      new sglTextureFont(fixedFont, true, '.'));
   switch_node->addChild(makeQuadObj(textureFonts.back()));

   Font font2 = XLoadFont(glXGetCurrentDisplay(),
                          "*-medium-r-normal--24-*-*-*-p-*-iso8859-1");
//                          "-bitstream-charter-medium-r-normal--50-*-*-*-p-*-iso8859-1");
   sglBitmapFontGLX proportionalFont(font2);
   textureFonts.push_back(
      new sglTextureFont(proportionalFont, true, '.'));
   switch_node->addChild(makeQuadObj(textureFonts.back()));
#endif

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/4.0, aspect, 0.1, 10000.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eY_UP);
   view_trans->setAzimuth(0.0f);
   view_trans->setRadius(300.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(1.0f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   sglGeode *geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);

   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setDrawCallback(preDrawFunc, NULL);
   view->setViewport(0, 0, win_x, win_y);

   override_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   override_statelets.push_back(new sglShadeModel(sglShadeModel::eFLAT));
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(640, 480);

   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

   glutCreateWindow("testTextureFont");

   sglOStream::setThreshold(sglOStream::eINFO);
   sgl::initialize();
   sglOStream::setThreshold(sglOStream::eWARNING);

   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
