/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testTransform.cpp
 *   Author: Scott McMillan
 *  Created: 1998
 *  Summary: visually test matrices and sglTransform nodes (add scale and
 *           translate tests)
 *****************************************************************************/

#if defined(WIN32)
#include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glut.h>

#include <sgl.h>

#include <sglMatrix.hpp>
#include <sglTransform.hpp>

//----------------------------------------------------------------------------
void outputMatrix(float mat[16])
{
   int index =0;
   for (int i=0; i<4; i++)
   {
      for (int j=0; j<4; j++)
      {
         cout << mat[index++] << ' ';
      }
      cout << endl;
   }
   cout << endl;
}


//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   sglMat4f mat;

   glutInit(&argc, argv);
   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testTransform");
   sgl::initialize();

   glMatrixMode(GL_MODELVIEW);

   glLoadIdentity();
   glPushMatrix();
   glGetFloatv(GL_MODELVIEW_MATRIX, mat.getPtr());
   outputMatrix(mat.getPtr());

   glRotatef(33.0, 0.750, 2.0, -1.0);
   glGetFloatv(GL_MODELVIEW_MATRIX, mat.getPtr());
   outputMatrix(mat.getPtr());

   sglTransformf mat2;
   mat2.printInfo(cout, "");

   mat2.setRotation(5.0, 1.0, 0.0, 0.0);
   mat2.printInfo(cout, "");

   mat2.setRotation(5.0, 0.0, 1.0, 0.0);
   mat2.printInfo(cout, "");

   mat2.setRotation(5.0, 0.0, 0.0, 1.0);
   mat2.printInfo(cout, "");

   mat2.setRotation(5.0, 0.0, 0.0);
   mat2.printInfo(cout, "");

   mat2.setRotation(0.0, 5.0, 0.0);
   mat2.printInfo(cout, "");

   mat2.setRotation(0.0, 0.0, 5.0);
   mat2.printInfo(cout, "");

   return 0;
}
