/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testTraversal.cpp
 *   Author: Tom Stimson
 *  Created: 1 April, 2000
 *  Summary: Test program for the sglCull classes.
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sglTimespec.hpp>
#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>
#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>

#include <sglScene.hpp>
#include <sglGeode.hpp>
#include <sglGeoSet.hpp>
#include <sglTriangleSet.hpp>
#include <sglMaterial.hpp>
#include <sglTransform.hpp>
#include <sglTexture2D.hpp>
#include <sglTexEnv.hpp>

sgluMouse                     *mouse;
sglScene                      *root;

sgluSphericalTransform<float> *view_trans;
sglViewPlatform               *view_platform;
sglPerspectiveCamera          *view_camera;

sglMat4f                       view_matrix;
sglFrustumf                    frustum;

sglCullf                      *trav_state;
sglStatsCullf                 *stats_trav_state;

deque<const sglStatelet*>      override_statelets;
sglAntiAliasLine               aa_line;
sglAntiAliasPoint              aa_point;
sglAntiAliasPolygon            aa_poly;

bool print_stats = false;

unsigned int disc_mask = 0x0;

unsigned int win_x = 640, win_y = 480;

sglTimespec tv, last_tv;

int render_count = 0;

const unsigned int tris_per_node = 20;
const unsigned int branches_per_level = 5;
const unsigned int num_levels = 3;
unsigned int num_tris = 0;

double total_time = -1;
double total_tris = 0;
double cull_time = 0;
double draw_time = 0;

bool simple = false;
vector<sglStatelet*> state[9];
sglOcclusionCull *occlusion = NULL;
sglTransparency *transparency = NULL;

map<vector<unsigned int>,double> sort_order;
map<vector<unsigned int>,double>::iterator sort_it;
map<vector<unsigned int>,double>::iterator sort_end;
int sort_num = 0;

//----------------------------------------------------------------------------
void printPerms()
{
   map<double,vector<unsigned int> > tmp;

   for (map<vector<unsigned int>,double>::iterator it =
           sort_order.begin(); it != sort_order.end(); ++it)
   {
      tmp[it->second] = it->first;
      it->second = 0.0;
   }

   cout << "num perms = " << sort_order.size() << endl;
   for (map<double,vector<unsigned int> >::iterator it2
           = tmp.begin(); it2 != tmp.end(); ++it2)
   {
      for (vector<unsigned int>::const_iterator it3 =
              it2->second.begin();
           it3 != it2->second.end(); ++it3)
      {
         cout << (int)*it3 << " ";
      }
      cout << " -> " << it2->first << endl;
   }

   trav_state->setSortOrder(tmp.begin()->second);
   stats_trav_state->setSortOrder(tmp.begin()->second);
}

//----------------------------------------------------------------------------
void buildSortOrders()
{
   unsigned int types[4] = {sglStatelet::eLIGHTING,
                            sglStatelet::eMATERIAL,
                            sglStatelet::eTEXTURE,
                            sglStatelet::eTRANSFORM_MATRIX};

   for (unsigned int i=0; i<5; ++i)
   {
      for (unsigned int j=0; j<5; ++j)
      {
         for (unsigned int k=0; k<5; ++k)
         {
            for (unsigned int l=0; l<5; ++l)
            {
               vector<unsigned int> tmp;
               if (i != 4)
                  tmp.push_back(types[i]);
               if (j != 4 && j != i)
                  tmp.push_back(types[j]);
               if (k != 4 && k != i && k != j)
                  tmp.push_back(types[k]);
               if (l != 4 && l != i && l != j && l != k)
                  tmp.push_back(types[l]);
               sort_order[tmp] = 0.0;
            }
         }
      }
   }
   sort_it = sort_order.end();
   sort_end = sort_order.end();

#if 1
   cout << "num perms = " << sort_order.size() << endl;
   for (map<vector<unsigned int>,double>::iterator it =
           sort_order.begin(); it != sort_order.end(); ++it)
   {
      for (vector<unsigned int>::const_iterator it2 =
              it->first.begin();
           it2 != it->first.end(); ++it2)
      {
         cout << (int)*it2 << " ";
      }
      cout << endl;
   }
#endif
}


//----------------------------------------------------------------------------
#define CHECK_IMAGE_SIZE 64
static GLubyte check_image[2][CHECK_IMAGE_SIZE][CHECK_IMAGE_SIZE][3];
static GLubyte check_image2[2][CHECK_IMAGE_SIZE][CHECK_IMAGE_SIZE][1];

void makeCheckImage(void)
{
    int i,j;
    GLubyte c, d;
    for (i=0; i<CHECK_IMAGE_SIZE; i++)
    {
        for (j=0; j<CHECK_IMAGE_SIZE; j++)
        {
            d = c = 255;
            if ((i&0x8) != (j&0x8)) c = 64;
            check_image[0][i][j][0] = c;
            check_image[0][i][j][1] = c;
            check_image[0][i][j][2] = c;

            if ((i&0x4) != (j&0x4)) d = 128;
            check_image[1][i][j][0] = 255;
            check_image[1][i][j][1] = d;
            check_image[1][i][j][2] = 0;
        }
    }
    for (i=0; i<CHECK_IMAGE_SIZE; i++)
    {
        for (j=0; j<CHECK_IMAGE_SIZE; j++)
        {
           c = 255;
           if ((i&0x8) != (j&0x8)) c = 0;
           check_image2[0][i][j][0] = c;

           check_image2[1][i][j][0] = 255;
        }
    }
}

//----------------------------------------------------------------------------
void makeStates()
{
   if (simple)
   {
      state[0].push_back(new sglLighting(true));

      sglMaterial *mat = new sglMaterial();
      mat->setColorMaterial(sglMaterial::eFRONT_AND_BACK,
                            sglMaterial::eDIFFUSE);
      state[0].push_back(mat);

      occlusion = new sglOcclusionCull();
      occlusion->enable(false);
      state[0].push_back(occlusion);

      transparency = new sglTransparency();
      transparency->enable(false);
      state[0].push_back(transparency);
   }
   else
   {
      // tex1 mat1
      // tex2 mat1
      // notex mat1
      // tex1 mat2
      // tex2 mat2
      // notex mat2
      // tex1 nomat nolit
      // tex2 nomat nolit
      // notex nomat nolit

      unsigned int i;
      for (i=0; i<6; ++i)
         state[i].push_back(new sglLighting(true));
      for (i=6; i<9; ++i)
         state[i].push_back(new sglLighting(false));

      sglMaterial *mat = new sglMaterial();
      mat->setColorMaterial(sglMaterial::eFRONT_AND_BACK,
                            sglMaterial::eDIFFUSE);
      for (i=0; i<3; ++i)
         state[i].push_back(mat);
      mat = new sglMaterial();
      mat->setColorMaterial(sglMaterial::eFRONT_AND_BACK,
                            sglMaterial::eSPECULAR);
      for (i=3; i<6; ++i)
         state[i].push_back(mat);

      makeCheckImage();
      sglTexture2D *tex1 = new sglTexture2D;
      tex1->setImage(&check_image[0][0][0][0],
                     sglTexture::eRGB, sglTexture::eUNSIGNED_BYTE,
                     CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);

      tex1->setAutoMipmapFlag(true);
      tex1->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
      //tex1->setMinFilter(sglTexture::eNEAREST);
      tex1->setMagFilter(sglTexture::eLINEAR);
      tex1->setWrapMode(sglTexture::eREPEAT, sglTexture::eCLAMP_TO_EDGE);
      tex1->setBorderColor(sglVec4f(1.0, 0.0, 0.0, 1.0));

      sglTexEnv  *tev = new sglTexEnv();
      tev->setBlendColor(1.0f, 0.7f, 0.0f, 1.0f);
      tev->setMode(sglTexEnv::eBLEND);

      sglTexture2D *tex2 = new sglTexture2D;
      tex2->setImage(&check_image2[0][0][0][0],
                     sglTexture::eLUMINANCE, sglTexture::eUNSIGNED_BYTE,
                     CHECK_IMAGE_SIZE, CHECK_IMAGE_SIZE, false);

      tex2->setAutoMipmapFlag(true);
      tex2->setMinFilter(sglTexture::eLINEAR_MIPMAP_LINEAR);
      //tex2->setMinFilter(sglTexture::eNEAREST);
      tex2->setMagFilter(sglTexture::eLINEAR);
      tex2->setWrapMode(sglTexture::eREPEAT, sglTexture::eCLAMP_TO_EDGE);
      tex2->setBorderColor(sglVec4f(1.0, 0.0, 0.0, 1.0));

      for (i=0; i<9; i+=3)
      {
         state[i].push_back(tex1);
         state[i].push_back(tev);
      }
      for (i=1; i<9; i+=3)
      {
         state[i].push_back(tex2);
         state[i].push_back(tev);
      }
      tex1->bind();
      tex2->bind();

      occlusion = new sglOcclusionCull();
      occlusion->enable(false);

      transparency = new sglTransparency();
      transparency->enable(false);

      for (i=0; i<9; ++i)
      {
         state[i].push_back(occlusion);
         state[i].push_back(transparency);
      }
   }
}


//----------------------------------------------------------------------------
sglNode* makeTris()
{
   if (simple)
   {
      unsigned int num_verts = 3 * tris_per_node;

      sglVec3f* verts = new sglVec3f[num_verts];
      sglVec3f* norms = new sglVec3f[num_verts];
      sglVec4f* color = new sglVec4f[num_verts];
      for (unsigned int i=0; i<num_verts; ++i)
      {
         for (unsigned int j=0; j<3; ++j)
         {
            verts[i][j] = rand();
            verts[i][j] /= (float)RAND_MAX;
            color[i][j] = verts[i][j];
            verts[i][j] *= 20.0f;
            verts[i][j] -= 10.0f;
         }
         color[i][3] = 1.0f;
         norms[i] = verts[i];
         norms[i].normalize();
      }

      sglGroup* group = new sglGroup();

      for (unsigned int k=0; k<tris_per_node; ++k)
      {
         sglGeode* geode = new sglGeode();
         sglTriangleSet* gset = new sglTriangleSet();
         gset->setCoordList(&verts[k*3]);
         gset->setNormalList(sglGeoSet::ePER_VERTEX, &norms[k*3]);
         gset->setColorList(sglGeoSet::ePER_VERTEX, &color[k*3]);
         gset->setNumPrims(1);

         gset->setState(0, state[0]);
         geode->addGeometry(gset);
         group->addChild(geode);
      }

      num_tris += tris_per_node;

      return group;
   }
   else
   {
      sglVec3f* verts = new sglVec3f[9*3];
      sglVec4f* color = new sglVec4f[9*3];
      sglVec3f* norms = new sglVec3f[6*3];
      for (unsigned int i=0; i<9*3; ++i)
      {
         for (unsigned int j=0; j<3; ++j)
         {
            verts[i][j] = rand();
            verts[i][j] /= (float)RAND_MAX;
            color[i][j] = verts[i][j];
            verts[i][j] *= 20.0f;
            verts[i][j] -= 10.0f;
         }
         color[i][3] = 1.0f;
         if (i < 6*3)
         {
            norms[i] = verts[i];
            norms[i].normalize();
         }
      }

      sglVec2f* tex = new sglVec2f[3];
      tex[0][0] = 0; tex[0][1] = 0;
      tex[1][0] = 1; tex[1][1] = .5;
      tex[2][0] = 0; tex[2][1] = 1;

      sglGroup* group = new sglGroup();

      for (unsigned int k=0; k<9; ++k)
      {
         sglGeode* geode = new sglGeode();
         sglTriangleSet* gset = new sglTriangleSet();
         gset->setNumPrims(1);
         gset->setCoordList(&verts[k*3]);
         gset->setColorList(sglGeoSet::ePER_VERTEX, &color[k*3]);
         if (k<6)
            gset->setNormalList(sglGeoSet::ePER_VERTEX, &norms[k*3]);

         gset->setState(0, state[k], tex);
         geode->addGeometry(gset);
         group->addChild(geode);
      }

      num_tris += 9;

      return group;
   }
}

//----------------------------------------------------------------------------
sglNode* makeTree(unsigned int level)
{
   if (level == num_levels)
   {
      return makeTris();
   }
   else
   {
      sglTransformd* tform = new sglTransformd();
      // this should be identity every 3 levels
      tform->setRotation(level*120, 0, 0);
      for (unsigned int i=0; i<branches_per_level; ++i)
      {
         tform->addChild(makeTree(level+1));
      }
      return tform;
   }
}


//----------------------------------------------------------------------------
int num_times = 0;

void display (void)
{
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   (view_platform->getCamera()).applyProjection();

   glMatrixMode (GL_MODELVIEW);
   glPushMatrix ();

   root->preDraw();

   float fov_y = (float)view_camera->getFOVY();
   float fov_x = (float)view_camera->getFOVX();
   view_platform->computeViewMatrix(view_matrix);

   frustum = view_camera->getFrustum();

   // ===============================================================
   if (print_stats && sort_it == sort_end)
   {
      sglTimespec ts1, ts2, ts3;

      sglTimespec::getSysTime(ts1);
      root->cull(*stats_trav_state, fov_x, fov_y, win_x, win_y,
                frustum, view_matrix, override_statelets, disc_mask);
      sglTimespec::getSysTime(ts2);
      stats_trav_state->draw();
      sglTimespec::getSysTime(ts3);

      cull_time += sglTimespec::convertToDouble(ts2 - ts1);
      draw_time += sglTimespec::convertToDouble(ts3 - ts2);
   }
   else if (sort_it != sort_end)
   {
      sglTimespec ts4, ts5;

      sglTimespec::getSysTime(ts4);
      root->cull(*trav_state, fov_x, fov_y, win_x, win_y,
                frustum, view_matrix, override_statelets, disc_mask);
      trav_state->draw();
      sglTimespec::getSysTime(ts5);

      sort_it->second += sglTimespec::convertToDouble(ts5 - ts4);

      ++sort_num;
      if (sort_num == 100)
      {
         sort_num = 0;
         ++sort_it;
         if (sort_it == sort_order.end())
         {
            printPerms();
         }
         else
         {
            trav_state->setSortOrder(sort_it->first);
            stats_trav_state->setSortOrder(sort_it->first);
         }
      }
   }
   else
   {
      root->cull(*trav_state, fov_x, fov_y, win_x, win_y,
                frustum, view_matrix, override_statelets, disc_mask);
      trav_state->draw();
   }

   ++num_times;
   // ===============================================================

   glBegin(GL_LINES);
   glColor3f(1.0, 0.0, 0.0);
   glVertex3f(2.0, 0.0, 0.0);
   glVertex3f(0.0, 0.0, 0.0);

   glColor3f(0.0, 1.0, 0.0);
   glVertex3f(0.0, 2.0, 0.0);
   glVertex3f(0.0, 0.0, 0.0);

   glColor3f(0.0, 0.0, 1.0);
   glVertex3f(0.0, 0.0, 2.0);
   glVertex3f(0.0, 0.0, 0.0);
   glEnd();

   glColor3f(1.0, 1.0, 1.0);

   glPopMatrix ();

   glFlush ();
   glutSwapBuffers();

   GLenum glerror = glGetError();
   if (glerror)
   {
      cerr << "glGetError() = " << (int)glerror << endl;
   }

   render_count++;
}

//----------------------------------------------------------------------------
void myReshape(int w, int h)
{
   glViewport (0, 0, w, h);
   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   double aspect = (double)w/(double)h;

   view_camera->setFOV(M_PI/3.0, aspect, 1.0, 200.0);
}

//----------------------------------------------------------------------------
void processKeyboard(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         cout << "avg. triangles per second = "
              << (total_tris/total_time) << endl;
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'o':
         occlusion->enable(true);
         cout << "occlusion culling on" << endl;
         break;
      case 'O':
         occlusion->enable(false);
         cout << "occlusion culling off" << endl;
         break;

      case 't':
         transparency->enable(true);
         cout << "transparency on" << endl;
         break;
      case 'T':
         transparency->enable(false);
         cout << "transparency off" << endl;
         break;

      case 's':
         sort_it = sort_order.begin();
         sort_num = 0;
         break;

      case '$':
         print_stats = !print_stats;
         break;
   }
}


//----------------------------------------------------------------------------
void processSpecialKeys(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}


//----------------------------------------------------------------------------
void updateSim()
{
   view_trans->update(mouse);

   glutPostRedisplay();

   sglTimespec::getSysTime(tv);
   double elapsed_time = sglTimespec::convertToDouble(tv - last_tv);

   if (elapsed_time > 2.5)
   {
      cerr << "frame_rate: " << (double) render_count/elapsed_time << endl;

      if (print_stats && sort_it == sort_end &&
          stats_trav_state->getStats().getFrames() > 0)
      {
         cull_time /= (double)stats_trav_state->getStats().getFrames();
         draw_time /= (double)stats_trav_state->getStats().getFrames();
         cerr << "avg. cull time: " << cull_time << endl;
         cerr << "avg. draw time: " << draw_time << endl;
         stats_trav_state->getStats().average();
         stats_trav_state->getStats().printInfo(cerr, "");
         stats_trav_state->getStats().reset();
         cull_time = 0;
         draw_time = 0;
      }

      // Ignore the first time, as it is usually an outlyer.
      if (total_time == -1)
      {
         total_time = 0;
         num_times = 0;
      }
      else
      {
         total_time += elapsed_time;
         total_tris += num_tris*num_times;
         num_times = 0;
      }

      render_count = 0;
      last_tv = tv;
   }
}


//----------------------------------------------------------------------------
//    Summary: Initialize material property and light source.
//----------------------------------------------------------------------------
void buildScenegraph()
{
   GLfloat light_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
   GLfloat light_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
//     light_position is NOT default value
   GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

   glLightfv (GL_LIGHT0, GL_AMBIENT, light_ambient);
   glLightfv (GL_LIGHT0, GL_DIFFUSE, light_diffuse);
   glLightfv (GL_LIGHT0, GL_SPECULAR, light_specular);
   glLightfv (GL_LIGHT0, GL_POSITION, light_position);

   glEnable (GL_LIGHTING);
   glEnable (GL_LIGHT0);

   makeStates();

   root = new sglScene;
   root->addChild(makeTree(0));
   cout << "num_tris = " << num_tris << endl;

   // the camera (frustum)
   double aspect = (double)win_x/(double)win_y;
   view_camera = new sglPerspectiveCamera;
   view_camera->setFOV(M_PI/3.0, aspect, 1.0, 200.0);

   // the view platform
   view_platform = new sglViewPlatform(*view_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(30.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);

   trav_state = new sglCullf;
   stats_trav_state = new sglStatsCullf;
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   if (argc != 2)
   {
      cerr << "usage: " << argv[0] << " {-simple | -complex}" << endl;
      exit(1);
   }
   if (string("-simple") == argv[1])
      simple = true;
   else if (string("-complex") == argv[1])
      simple = false;
   else
   {
      cerr << "usage: " << argv[0] << " {-simple | -complex}" << endl;
      exit(1);
   }

   glutInitWindowSize(10, 10);
//   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testTraversal");

   sgl::initialize();
   if (sgl::s_occlusion_capable)
      cout << "Occlusion culling supported" << endl;
   else
      cout << "Occlusion culling NOT supported" << endl;

   mouse = sgluMouse::initialize();

   buildScenegraph();
   buildSortOrders();

   glutReshapeFunc(myReshape);
   glutKeyboardFunc(processKeyboard);
   glutSpecialFunc(processSpecialKeys);
   glutDisplayFunc(display);
   glutIdleFunc(updateSim);

   sglTimespec::getSysTime(last_tv);

   updateSim();

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
