/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testUnProject.cpp
 *  Created: Fall 2000
 *  Summary: GLUT-based sgl testbed
 *****************************************************************************/

#include <GL/glut.h>
#include <sgl.h>
#include <sglu.h>

#include <sgluMouse.hpp>
#include <sgluSphericalTransform.hpp>

#include <sglScene.hpp>
#include <sglDefaultStateNode.hpp>
#include <sglTranslate.hpp>
#include <sglScale.hpp>
#include <sglUnProject.hpp>
#include <sglGeode.hpp>
#include <sglLineStripSet.hpp>
#include <sglQuadSet.hpp>

#include <sglViewPlatform.hpp>
#include <sglPerspectiveCamera.hpp>
#include <sglOrthographicCamera.hpp>
#include <sglView.hpp>
#include <sglAxes.hpp>

sglView                   *view;
sglScene                  *root;

sgluMouse                 *mouse;
sgluSphericalTransform<float> *view_trans;
sglViewPlatform           *view_platform;
sglPerspectiveCamera      *perspective_camera;
sglOrthographicCamera     *orthographic_camera;

deque<const sglStatelet*> override_statelets;
sglAntiAliasPoint   aa_point;
sglAntiAliasLine    aa_line;
sglAntiAliasPolygon aa_poly;

bool ortho = false;
bool draw_grid = 0;
double fov_y = M_PI/3.0;
unsigned int win_x = 640, win_y = 480;
double aspect = 640./480.;

//----------------------------------------------------------------------------
void displayFunc()
{
   view->cullAndDraw();
   glutSwapBuffers();
}

//----------------------------------------------------------------------------
void reshapeFunc(int w, int h)
{
   view->setViewport(0, 0, w, h);

   mouse->win_size_x = win_x = w;
   mouse->win_size_y = win_y = h;

   aspect = (double)w/(double)h;

   if (ortho)
   {
#if 1
      orthographic_camera->setOrtho(-fov_y*aspect, fov_y*aspect,
                                    -fov_y, fov_y, 1.0, 200.);
#else
      orthographic_camera->setOrtho(1*aspect, fov_y*aspect,
                                    1, fov_y, 1.0, 200.);
#endif
   }
   else
   {
      perspective_camera->setFOV(fov_y, aspect, 1.0, 200.0);
   }
}

//----------------------------------------------------------------------------
void keyboardFunc(unsigned char key, int, int)
{
   switch (key)
   {
      case 27:
         glutDestroyWindow(glutGetWindow());
         exit(1);
         break;

      case 'a':
         aa_point.enable(!aa_point.isEnabled());
         cout << "point aa override = " << aa_point.isEnabled() << endl;
         break;

      case 'A':
         aa_line.enable(!aa_line.isEnabled());
         cout << "line aa override = " << aa_line.isEnabled() << endl;
         break;

      case 'z':
         aa_poly.enable(!aa_poly.isEnabled());
         cout << "polygon aa override = " << aa_poly.isEnabled() << endl;
         break;

      case 'g':
         draw_grid = !draw_grid;
         break;

      case 'p':
         ortho = false;
         fov_y = M_PI/6.;
         view_platform->setCamera(*perspective_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'P':
         ortho = false;
         fov_y = M_PI/3.;
         view_platform->setCamera(*perspective_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'o':
         ortho = true;
         fov_y = 10.;
         view_platform->setCamera(*orthographic_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'O':
         ortho = true;
         fov_y = 20.;
         view_platform->setCamera(*orthographic_camera);
         reshapeFunc(win_x, win_y);
         break;

      case 'f':
         view->enableFrameRateOutput(!(view->getFrameRateOutputFlag()));
         break;

      case 'd':
         view->setTraversalType(!(view->getDPTraversalFlag()),
                               view->getStatsFlag());
         break;

      case '$':
         view->setTraversalType(view->getDPTraversalFlag(),
                               !(view->getStatsFlag()));
         break;

      case 's':
         sgluOutputTree(root, cout, 3);
         break;

      case '/':
      case '?':
         cerr << "a - toggle point anti-aliasing" << endl;
         cerr << "A - toggle line anti-aliasing" << endl;
         cerr << "z - toggle polygon anti-aliasing" << endl;
         cerr << "g - toggle grid" << endl;
         cerr << "p - 30-degree perspective view" << endl;
         cerr << "P - 60-degree perspective view" << endl;
         cerr << "o - 10-unit orthographic view" << endl;
         cerr << "O - 20-unit orthographic view" << endl;
         cerr << "f - toggle frame rate statistics" << endl;
         cerr << "d - toggle between SP and DP traversal" << endl;
         cerr << "$ - toggle scene graph statistics reporting" << endl;
         cerr << "s - output scenegraph" << endl;
         cerr << "? - print help (this list)" << endl;
         cerr << "ECSAPE - exit" << endl;
         break;
   }
}


//----------------------------------------------------------------------------
void specialFunc(int key, int, int)
{
   switch (key)
   {
      case GLUT_KEY_LEFT:
         break;
      case GLUT_KEY_RIGHT:
         break;
      case GLUT_KEY_UP:
         break;
      case GLUT_KEY_DOWN:
         break;
   }
}


//----------------------------------------------------------------------------
void idleFunc()
{
   view_trans->update(mouse);
   displayFunc();
}

//----------------------------------------------------------------------------
void preDrawFunc(void *)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glEnable(GL_DEPTH_TEST);

   if (draw_grid)
   {
      glPushAttrib(GL_ALL_ATTRIB_BITS); // VERY COSTLY?
      glPushClientAttrib(~0x0u); //GL_ALL_CLIENT_ATTRIB_BITS);

      glMatrixMode (GL_PROJECTION);
      glPushMatrix();
      glLoadIdentity();
      gluOrtho2D(0, win_x, 0, win_y);

      glMatrixMode (GL_MODELVIEW);
      glPushMatrix();
      glLoadIdentity();

      glDepthMask(GL_FALSE);
      glDisable(GL_DEPTH_TEST);

      unsigned int i;
      glColor3f(0.3f, 0.3f, 0.3f);
      for (i=5; i<win_x; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(i, 0);
         glVertex2i(i, win_y);
         glEnd();
      }
      for (i=5; i<win_y; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(0, i);
         glVertex2i(win_x, i);
         glEnd();
      }
      glColor3f(0.5f, 0.5f, 0.5f);
      for (i=0; i<win_x; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(i, 0);
         glVertex2i(i, win_y);
         glEnd();
      }
      for (i=0; i<win_y; i+=10)
      {
         glBegin(GL_LINES);
         glVertex2i(0, i);
         glVertex2i(win_x, i);
         glEnd();
      }

      glMatrixMode(GL_PROJECTION);
      glPopMatrix();
      glMatrixMode(GL_MODELVIEW);
      glPopMatrix();

      glPopAttrib();
      glPopClientAttrib();
   }
}

//----------------------------------------------------------------------------
sglGeoSet *makeSquare()
{
   sglVec3f *vertex = new sglVec3f[5];
   sglVec4f *color = new sglVec4f[1];
   unsigned int *length = new unsigned int[1];
   length[0] = 5;

   color[0][0] = 1.0; color[0][1] = 1.0; color[0][2] = 1.0; color[0][3] = 1.0;

   float size = 20.0f;
   vertex[0][0] =  size;
   vertex[0][1] =  size;
   vertex[0][2] =  0.0;

   vertex[1][0] =  size;
   vertex[1][1] = -size;
   vertex[1][2] =  0.0;

   vertex[2][0] = -size;
   vertex[2][1] = -size;
   vertex[2][2] =  0.0;

   vertex[3][0] = -size;
   vertex[3][1] =  size;
   vertex[3][2] =  0.0;

   vertex[4][0] =  size;
   vertex[4][1] =  size;
   vertex[4][2] =  0.0;

   sglLineStripSet *gset = new sglLineStripSet();
   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(1);
   gset->setPrimLengths(length);

   //cerr << "Coord Sys Set: " << gset << endl;
   return gset;
}


//----------------------------------------------------------------------------
sglGeoSet *makeQuad()
{
   sglVec3f *vertex = new sglVec3f[5];
   sglVec4f *color = new sglVec4f[1];

   color[0][0] = 1.0; color[0][1] = 0.0; color[0][2] = 0.0; color[0][3] = 0.3f;

   vertex[0][0] =  0.05f;
   vertex[0][1] =  0.05f;
   vertex[0][2] =  0.0;

   vertex[1][0] =  0.05f;
   vertex[1][1] = -0.05f;
   vertex[1][2] =  0.0;

   vertex[2][0] = -0.05f;
   vertex[2][1] = -0.05f;
   vertex[2][2] =  0.0;

   vertex[3][0] = -0.05f;
   vertex[3][1] =  0.05f;
   vertex[3][2] =  0.0;

   vertex[4][0] =  0.05f;
   vertex[4][1] =  0.05f;
   vertex[4][2] =  0.0;

   sglQuadSet *gset = new sglQuadSet();
   gset->setCoordList(vertex);
   gset->setColorList(sglGeoSet::eOVERALL, color);
   gset->setNumPrims(1);

   vector<sglStatelet*> statelets;
   statelets.push_back(new sglTransparency(true));
   gset->setState(0, statelets);

   return gset;
}

//----------------------------------------------------------------------------
void buildScenegraph()
{
   root = new sglScene;

   vector<sglStatelet*> default_statelets;
   default_statelets.push_back(new sglCullMode(sglCullMode::eOFF));
   sglDefaultStateNode *def = new sglDefaultStateNode();
   def->setStatelets(default_statelets);
   root->addChild(def);

   sglTranslatef *translate = new sglTranslatef();
   translate->setTranslation(20, 0, 0);

   sglAxes *axes = new sglAxes;
   sglGeode *geode = new sglGeode;
   geode->addGeometry(axes);
   translate->addChild(geode);

   sglScalef *scale = new sglScalef();
   scale->setScale(2);
   def->addChild(scale);
   scale->addChild(translate);

   geode = new sglGeode();
   sglUnProject *unproject = new sglUnProject(true);
   unproject->addChild(geode);
   def->addChild(unproject);
   geode->addGeometry(makeSquare());

   geode = new sglGeode();
   unproject = new sglUnProject();
   unproject->addChild(geode);
   translate->addChild(unproject);
   sglGeoSet *quad = makeQuad();
   axes->setBound(NULL, sglBound::eSTATIC);
   quad->setBound(&(axes->getBound()), sglBound::eSTATIC);
   geode->addGeometry(quad);

   // the cameras (frusta)
   perspective_camera = new sglPerspectiveCamera;
   perspective_camera->setFOV(fov_y, aspect, 1.0, 200.0);

   orthographic_camera = new sglOrthographicCamera;
   orthographic_camera->setOrtho(-20.0*aspect, 20.0*aspect,
                                 -20.0, 20.0, 1.0, 200.);

   // the view platform
   view_platform = new sglViewPlatform(*perspective_camera);

   // the view transform
   view_trans = new sgluSphericalTransform<float>();
   view_trans->setMode(sgluSphericalTransform<float>::eZ_UP);
   view_trans->setRadius(60.0);
   view_trans->setCOI(0.0, 0.0, 0.0);
   view_trans->setTranslationScale(0.02f);
   view_trans->computeMatrix();
   view_trans->addChild(view_platform);
   root->addChild(view_trans);

   geode = new sglGeode;
   geode->addGeometry(new sglAxes(2.0f));
   root->addChild(geode);
   root->preDraw();

   view = new sglView;
   view->setScene(root, view_platform);
   view->setViewport(0, 0, win_x, win_y);
   view->setDrawCallback(preDrawFunc, NULL);

   override_statelets.push_back(&aa_point);
   override_statelets.push_back(&aa_line);
   override_statelets.push_back(&aa_poly);
   view->setOverrideStatelets(override_statelets);
}

//----------------------------------------------------------------------------
int main(int argc, char** argv)
{
   glutInit(&argc, argv);

   glutInitWindowSize(700, 512);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutCreateWindow("testUnProject");

   sgl::initialize();
   mouse = sgluMouse::initialize();
   buildScenegraph();

   glutReshapeFunc(reshapeFunc);
   glutKeyboardFunc(keyboardFunc);
   glutSpecialFunc(specialFunc);
   glutDisplayFunc(displayFunc);
   glutIdleFunc(idleFunc);

   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
}
