/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testVec.cpp
 *  Created: 1999
 *  Summary: test the templated vector classes
 *****************************************************************************/

#include <sglVec2.hpp>
#include <sglVec3.hpp>
#include <sglVec4.hpp>
#include <sglVector.hpp>

#include <sglMat2.hpp>
#include <sglMat3.hpp>
#include <sglMat4.hpp>
#include <sglMatrix.hpp>

//----------------------------------------------------------------------------
void testVec2f()
{
   // SP tests
   cout << "**** sglVec2<float> ****" << endl;
   sglVec2<float>  fv2;
   cout << "def ctor: " << fv2 << endl;
   sglVec2f fv2a(1.2, 2.3);
   cout << "ctor(f,f): " << fv2a << endl;

   double z = -1.2, x = -3.4;
   sglVec2f fv2b(z, x);
   cout << "ctor(d,d): " << fv2b << endl;

   double v[2] = {-5.4, 3.2123123123};
   sglVec2f fv2c(v);
   cout << "ctor(d v[2]): " << fv2c << endl;
   
   int v1[2] = {1,5};
   sglVec2f fv2d(v1);
   cout << "ctor(i v[2]): " << fv2d << endl;
   
   unsigned int v2[2] = {1,5};
   sglVec2f fv2e(v2);
   cout << "ctor(ui v[2]): " << fv2e << endl;

   fv2.set((int) 1, (int)-3);
   cout << "set(i,i): " << fv2 << endl;
   fv2.set((unsigned int) 1, (unsigned int)-3);
   cout << "set(ui,ui): " << fv2 << endl;
   fv2.set(-3.0, 1.14159);
   cout << "set(f,f): " << fv2 << endl;
   fv2.set(x,z);
   cout << "set(d,d): " << fv2 << endl;

   sglVec2d dv2a(3.141888888, -7.8777777777);
   sglVec2f fv2f = dv2a;
   cout << "v2f = v2d: " << fv2f << endl;
   sglVec2f fv2g = fv2f;
   cout << "v2f = v2f: " << fv2g << endl;
   
   fv2 = fv2g;
   cout << "v2f = v2f: " << fv2 << endl;
   fv2 = fv2;
   cout << "self assignment: " << fv2 << endl;

   unsigned int ix=0, iy=1;
   cout << "indexing: " << fv2[ix] << ' ' << fv2[iy] << endl;
   float fx=0.6, fy=1;
   cout << "indexing: " << fv2[fx] << ' ' << fv2[fy] << endl;

   cout << "fv2 == fv2 -> " << (fv2 == fv2) << endl;
   cout << "fv2 == fv2a -> " << (fv2 == fv2a) << endl;
   cout << "fv2 != fv2a -> " << (fv2 != fv2a) << endl;
   cout << "fv2 == dv2 -> " << (fv2 == dv2a) << endl;
   cout << "fv2 < fv2a -> " << (fv2 < fv2a) << endl;
   cout << "fv2 > fv2a -> " << (fv2 > fv2a) << endl;

   fv2b = fv2 + fv2a;
   cout << fv2 << " + " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b += fv2a;
   cout << fv2 << " += " << fv2a << " -> " << fv2b << endl;
   fv2b.add(fv2, fv2a);
   cout << fv2 << " add " << fv2a << " = " << fv2b << endl;

   fv2b = fv2 - fv2a;
   cout << fv2 << " - " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b -= fv2a;
   cout << fv2 << " -= " << fv2a << " -> " << fv2b << endl;
   fv2b.sub(fv2, fv2a);
   cout << fv2 << " sub " << fv2a << " = " << fv2b << endl;

   fv2b = fv2*2.0;
   cout << fv2 << " * 2.0 = " << fv2b << endl;
   fv2b = fv2;
   fv2b *= 2.0;
   cout << fv2 << " *= 2.0 -> " << fv2b << endl;
   fv2b.scale(fv2, 2.0);
   cout << fv2 << " scale by 2.0 = " << fv2b << endl;

   cout << fv2 << " - " << fv2a << " = ";
   fv2.sub(fv2, fv2a);
   cout << fv2 << endl;
   cout << fv2 << " - " << fv2 << " = ";
   fv2.sub(fv2, fv2);
   cout << fv2 << endl;

   fv2 = fv2a;
   cout << fv2 << " dot " << fv2 << " = " << fv2.dot(fv2) << endl;
   cout << fv2 << " dot " << fv2a << " = " << fv2.dot(fv2a) << endl;
   cout << "length" << fv2 << " = " << fv2.length() << endl;
   cout << "distance " << fv2 << "," << fv2a+fv2
        << " = " << fv2.distance(fv2a+fv2) << endl;
   cout << "distance " << fv2 << "," << fv2a
        << " = " << fv2.distance(fv2a) << endl;
   cout << "normalize(" << fv2 << ") = ";
   cout << fv2.normalize() << ", " << fv2 << endl;
}

//----------------------------------------------------------------------------
void testVec2d()
{
   // DP tests
   cout << "**** sglVec2<double> ****" << endl;
   sglVec2d  fv2;
   cout << "def ctor: " << fv2 << endl;
   sglVec2d fv2a(1.2, 2.3);
   cout << "ctor(f,f): " << fv2a << endl;

   double z = -1.2, x = -3.4;
   sglVec2d fv2b(z, x);
   cout << "ctor(d,d): " << fv2b << endl;

   double v[2] = {-5.4, 3.2123123123};
   sglVec2d fv2c(v);
   cout << "ctor(d v[2]): " << fv2c << endl;
   
   int v1[2] = {1,5};
   sglVec2d fv2d(v1);
   cout << "ctor(i v[2]): " << fv2d << endl;
   
   unsigned int v2[2] = {1,5};
   sglVec2d fv2e(v2);
   cout << "ctor(ui v[2]): " << fv2e << endl;

   fv2.set((int) 1, (int)-3);
   cout << "set(i,i): " << fv2 << endl;
   fv2.set((unsigned int) 1, (unsigned int)-3);
   cout << "set(ui,ui): " << fv2 << endl;
   fv2.set(-3.0, 1.14159);
   cout << "set(f,f): " << fv2 << endl;
   fv2.set(x,z);
   cout << "set(d,d): " << fv2 << endl;

   sglVec2d dv2a(3.141888888, -7.8777777777);
   sglVec2d fv2f = dv2a;
   cout << "v2f = v2d: " << fv2f << endl;
   sglVec2d fv2g = fv2f;
   cout << "v2f = v2f: " << fv2g << endl;
   
   fv2 = fv2g;
   cout << "v2f = v2f: " << fv2 << endl;
   fv2 = fv2;
   cout << "self assignment: " << fv2 << endl;

   unsigned int ix=0, iy=1;
   cout << "indexing: " << fv2[ix] << ' ' << fv2[iy] << endl;
   float fx=0.6, fy=1;
   cout << "indexing: " << fv2[fx] << ' ' << fv2[fy] << endl;

   cout << "fv2 == fv2 -> " << (fv2 == fv2) << endl;
   cout << "fv2 == fv2a -> " << (fv2 == fv2a) << endl;
   cout << "fv2 != fv2a -> " << (fv2 != fv2a) << endl;
   cout << "fv2 == dv2 -> " << (fv2 == dv2a) << endl;
   cout << "fv2 < fv2a -> " << (fv2 < fv2a) << endl;
   cout << "fv2 > fv2a -> " << (fv2 > fv2a) << endl;

   fv2b = fv2 + fv2a;
   cout << fv2 << " + " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b += fv2a;
   cout << fv2 << " += " << fv2a << " -> " << fv2b << endl;
   fv2b.add(fv2, fv2a);
   cout << fv2 << " add " << fv2a << " = " << fv2b << endl;

   fv2b = fv2 - fv2a;
   cout << fv2 << " - " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b -= fv2a;
   cout << fv2 << " -= " << fv2a << " -> " << fv2b << endl;
   fv2b.sub(fv2, fv2a);
   cout << fv2 << " sub " << fv2a << " = " << fv2b << endl;

   fv2b = fv2*2.0;
   cout << fv2 << " * 2.0 = " << fv2b << endl;
   fv2b = fv2;
   fv2b *= 2.0;
   cout << fv2 << " *= 2.0 -> " << fv2b << endl;
   fv2b.scale(fv2, 2.0);
   cout << fv2 << " scale by 2.0 = " << fv2b << endl;

   cout << fv2 << " - " << fv2a << " = ";
   fv2.sub(fv2, fv2a);
   cout << fv2 << endl;
   cout << fv2 << " - " << fv2 << " = ";
   fv2.sub(fv2, fv2);
   cout << fv2 << endl;

   fv2 = fv2a;
   cout << fv2 << " dot " << fv2 << " = " << fv2.dot(fv2) << endl;
   cout << fv2 << " dot " << fv2a << " = " << fv2.dot(fv2a) << endl;
   cout << "length" << fv2 << " = " << fv2.length() << endl;
   cout << "distance " << fv2 << "," << fv2a+fv2
        << " = " << fv2.distance(fv2a+fv2) << endl;
   cout << "distance " << fv2 << "," << fv2a
        << " = " << fv2.distance(fv2a) << endl;
   cout << "normalize(" << fv2 << ") = ";
   cout << fv2.normalize() << ", " << fv2 << endl;
}

//----------------------------------------------------------------------------
void testVec3f()
{
   // SP tests
   cout << "**** sglVec3<float> ****" << endl;
   sglVec3<float>  fv2;
   cout << "def ctor: " << fv2 << endl;
   sglVec3f fv2a(1.2, 2.3, 4.5);
   cout << "ctor(f,f,f): " << fv2a << endl;

   double z = -1.2, y=4.5, x = -3.4;
   sglVec3f fv2b(x,y,z);
   cout << "ctor(d,d,d): " << fv2b << endl;

   double v[3] = {-5.4, 3.2123123123, 7.8888888888};
   sglVec3f fv2c(v);
   cout << "ctor(d v[3]): " << fv2c << endl;
   
   int v1[3] = {1,5,9};
   sglVec3f fv2d(v1);
   cout << "ctor(i v[3]): " << fv2d << endl;
   
   unsigned int v2[3] = {1,5,8};
   sglVec3f fv2e(v2);
   cout << "ctor(ui v[3]): " << fv2e << endl;

   fv2.set((int) 1, (int)-3, (int)5);
   cout << "set(i,i,i): " << fv2 << endl;
   fv2.set((unsigned int) 1, (unsigned int)-3, (unsigned int)5);
   cout << "set(ui,ui,ui): " << fv2 << endl;
   fv2.set(-3.0, 1.14159, 7.8888);
   cout << "set(f,f,f): " << fv2 << endl;
   fv2.set(x,y,z);
   cout << "set(d,d,d): " << fv2 << endl;

   sglVec3d dv2a(3.141888888, -7.8777777777, 1.11111999);
   sglVec3f fv2f = dv2a;
   cout << "v2f = v2d: " << fv2f << endl;
   sglVec3f fv2g = fv2f;
   cout << "v2f = v2f: " << fv2g << endl;
   
   fv2 = fv2g;
   cout << "v2f = v2f: " << fv2 << endl;
   fv2 = fv2;
   cout << "self assignment: " << fv2 << endl;

   unsigned int ix=0, iy=1;
   cout << "indexing: " << fv2[ix] << ' ' << fv2[iy] << endl;
   float fx=0.6, fy=1;
   cout << "indexing: " << fv2[fx] << ' ' << fv2[fy] << endl;

   cout << "fv2 == fv2 -> " << (fv2 == fv2) << endl;
   cout << "fv2 == fv2a -> " << (fv2 == fv2a) << endl;
   cout << "fv2 != fv2a -> " << (fv2 != fv2a) << endl;
   cout << "fv2 == dv2 -> " << (fv2 == dv2a) << endl;
   cout << "fv2 < fv2a -> " << (fv2 < fv2a) << endl;
   cout << "fv2 > fv2a -> " << (fv2 > fv2a) << endl;

   fv2b = fv2 + fv2a;
   cout << fv2 << " + " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b += fv2a;
   cout << fv2 << " += " << fv2a << " -> " << fv2b << endl;
   fv2b.add(fv2, fv2a);
   cout << fv2 << " add " << fv2a << " = " << fv2b << endl;

   fv2b = fv2 - fv2a;
   cout << fv2 << " - " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b -= fv2a;
   cout << fv2 << " -= " << fv2a << " -> " << fv2b << endl;
   fv2b.sub(fv2, fv2a);
   cout << fv2 << " sub " << fv2a << " = " << fv2b << endl;

   fv2b = fv2*2.0;
   cout << fv2 << " * 2.0 = " << fv2b << endl;
   fv2b = fv2;
   fv2b *= 2.0;
   cout << fv2 << " *= 2.0 -> " << fv2b << endl;
   fv2b.scale(fv2, 2.0);
   cout << fv2 << " scale by 2.0 = " << fv2b << endl;

   cout << fv2 << " - " << fv2a << " = ";
   fv2.sub(fv2, fv2a);
   cout << fv2 << endl;
   cout << fv2 << " - " << fv2 << " = ";
   fv2.sub(fv2, fv2);
   cout << fv2 << endl;

   fv2.cross(fv2a, fv2b);
   cout << fv2a << " x " << fv2b << " = " << fv2 << endl;
   cout << fv2 << " x " << fv2a << " = ";
   fv2.cross(fv2a);
   cout << fv2 << endl;
   fv2 = fv2a;
   fv2.cross(fv2);
   cout << fv2a << " x " << fv2a << " = " << fv2 << endl;

   fv2 = fv2a;
   cout << fv2 << " dot " << fv2 << " = " << fv2.dot(fv2) << endl;
   cout << fv2 << " dot " << fv2a << " = " << fv2.dot(fv2a) << endl;
   cout << "length" << fv2 << " = " << fv2.length() << endl;
   cout << "distance " << fv2 << "," << fv2a+fv2
        << " = " << fv2.distance(fv2a+fv2) << endl;
   cout << "distance " << fv2 << "," << fv2a
        << " = " << fv2.distance(fv2a) << endl;

   cout << "normalize(" << fv2 << ") = ";
   cout << fv2.normalize() << ", " << fv2 << endl;
}

//----------------------------------------------------------------------------
void testVec3d()
{
   // DP tests
   cout << "**** sglVec3<double> ****" << endl;
   sglVec3d  fv2;
   cout << "def ctor: " << fv2 << endl;
   sglVec3d fv2a(1.2, 2.3, 4.5);
   cout << "ctor(f,f,f): " << fv2a << endl;

   double z = -1.2, y=4.5, x = -3.4;
   sglVec3d fv2b(z, y, x);
   cout << "ctor(d,d,d): " << fv2b << endl;

   double v[3] = {-5.4, 3.2123123123, 1.11111888};
   sglVec3d fv2c(v);
   cout << "ctor(d v[3]): " << fv2c << endl;
   
   int v1[3] = {1,5,9};
   sglVec3d fv2d(v1);
   cout << "ctor(i v[3]): " << fv2d << endl;
   
   unsigned int v2[3] = {1,5,9};
   sglVec3d fv2e(v2);
   cout << "ctor(ui v[3]): " << fv2e << endl;

   fv2.set((int) 1, (int)-3, (int)5);
   cout << "set(i,i,i): " << fv2 << endl;
   fv2.set((unsigned int) 1, (unsigned int)-3, (unsigned int)5);
   cout << "set(ui,ui,ui): " << fv2 << endl;
   fv2.set(-3.0, 1.14159, 2.454545);
   cout << "set(f,f,f): " << fv2 << endl;
   fv2.set(x,y,z);
   cout << "set(d,d,d): " << fv2 << endl;

   sglVec3d dv2a(3.141888888, -7.8777777777, 1.111119999);
   sglVec3d fv2f = dv2a;
   cout << "v2f = v2d: " << fv2f << endl;
   sglVec3d fv2g = fv2f;
   cout << "v2f = v2f: " << fv2g << endl;
   
   fv2 = fv2g;
   cout << "v2f = v2f: " << fv2 << endl;
   fv2 = fv2;
   cout << "self assignment: " << fv2 << endl;

   unsigned int ix=0, iy=1;
   cout << "indexing: " << fv2[ix] << ' ' << fv2[iy] << endl;
   float fx=0.6, fy=1;
   cout << "indexing: " << fv2[fx] << ' ' << fv2[fy] << endl;

   cout << "fv2 == fv2 -> " << (fv2 == fv2) << endl;
   cout << "fv2 == fv2a -> " << (fv2 == fv2a) << endl;
   cout << "fv2 != fv2a -> " << (fv2 != fv2a) << endl;
   cout << "fv2 == dv2 -> " << (fv2 == dv2a) << endl;
   cout << "fv2 < fv2a -> " << (fv2 < fv2a) << endl;
   cout << "fv2 > fv2a -> " << (fv2 > fv2a) << endl;

   fv2b = fv2 + fv2a;
   cout << fv2 << " + " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b += fv2a;
   cout << fv2 << " += " << fv2a << " -> " << fv2b << endl;
   fv2b.add(fv2, fv2a);
   cout << fv2 << " add " << fv2a << " = " << fv2b << endl;

   fv2b = fv2 - fv2a;
   cout << fv2 << " - " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b -= fv2a;
   cout << fv2 << " -= " << fv2a << " -> " << fv2b << endl;
   fv2b.sub(fv2, fv2a);
   cout << fv2 << " sub " << fv2a << " = " << fv2b << endl;

   fv2b = fv2*2.0;
   cout << fv2 << " * 2.0 = " << fv2b << endl;
   fv2b = fv2;
   fv2b *= 2.0;
   cout << fv2 << " *= 2.0 -> " << fv2b << endl;
   fv2b.scale(fv2, 2.0);
   cout << fv2 << " scale by 2.0 = " << fv2b << endl;

   cout << fv2 << " - " << fv2a << " = ";
   fv2.sub(fv2, fv2a);
   cout << fv2 << endl;
   cout << fv2 << " - " << fv2 << " = ";
   fv2.sub(fv2, fv2);
   cout << fv2 << endl;

   fv2.cross(fv2a, fv2b);
   cout << fv2a << " x " << fv2b << " = " << fv2 << endl;
   cout << fv2 << " x " << fv2a << " = ";
   fv2.cross(fv2a);
   cout << fv2 << endl;
   fv2 = fv2a;
   fv2.cross(fv2);
   cout << fv2a << " x " << fv2a << " = " << fv2 << endl;

   fv2 = fv2a;
   cout << fv2 << " dot " << fv2 << " = " << fv2.dot(fv2) << endl;
   cout << fv2 << " dot " << fv2a << " = " << fv2.dot(fv2a) << endl;
   cout << "length" << fv2 << " = " << fv2.length() << endl;
   cout << "distance " << fv2 << "," << fv2a+fv2
        << " = " << fv2.distance(fv2a+fv2) << endl;
   cout << "distance " << fv2 << "," << fv2a
        << " = " << fv2.distance(fv2a) << endl;
   cout << "normalize(" << fv2 << ") = ";
   cout << fv2.normalize() << ", " << fv2 << endl;
}

//----------------------------------------------------------------------------
void testVec4f()
{
   // SP tests
   cout << "**** sglVec4<float> ****" << endl;
   sglVec4<float>  fv2;
   cout << "def ctor: " << fv2 << endl;
   sglVec4f fv2a(1.2, 2.3, 4.5, 6.7);
   cout << "ctor(f,f,f,f): " << fv2a << endl;

   double w = 6.7, z = -1.2, y=4.5, x = -3.4;
   sglVec4f fv2b(x,y,z,w);
   cout << "ctor(d,d,d): " << fv2b << endl;

   double v[4] = {-5.4, 3.2123123123, 7.8888888888, 2.1};
   sglVec4f fv2c(v);
   cout << "ctor(d v[4]): " << fv2c << endl;
   
   int v1[4] = {1,5,9,3};
   sglVec4f fv2d(v1);
   cout << "ctor(i v[4]): " << fv2d << endl;
   
   unsigned int v2[4] = {1,5,8,2};
   sglVec4f fv2e(v2);
   cout << "ctor(ui v[4]): " << fv2e << endl;

   fv2.set((int) 1, (int)-3, (int)5, (int)7);
   cout << "set(i,i,i): " << fv2 << endl;
   fv2.set((unsigned int) 1, (unsigned int)-3, (unsigned int)5, (unsigned int)7);
   cout << "set(ui,ui,ui): " << fv2 << endl;
   fv2.set(-3.0, 1.14159, 7.8888, -2.3333);
   cout << "set(f,f,f): " << fv2 << endl;
   fv2.set(x,y,z,w);
   cout << "set(d,d,d): " << fv2 << endl;

   sglVec4d dv2a(3.141888888, -7.8777777777, 1.11111999, 1.0);
   sglVec4f fv2f = dv2a;
   cout << "v2f = v2d: " << fv2f << endl;
   sglVec4f fv2g = fv2f;
   cout << "v2f = v2f: " << fv2g << endl;
   
   fv2 = fv2g;
   cout << "v2f = v2f: " << fv2 << endl;
   fv2 = fv2;
   cout << "self assignment: " << fv2 << endl;

   unsigned int ix=0, iy=1;
   cout << "indexing: " << fv2[ix] << ' ' << fv2[iy] << endl;
   float fx=0.6, fy=1;
   cout << "indexing: " << fv2[fx] << ' ' << fv2[fy] << endl;

   cout << "fv2 == fv2 -> " << (fv2 == fv2) << endl;
   cout << "fv2 == fv2a -> " << (fv2 == fv2a) << endl;
   cout << "fv2 != fv2a -> " << (fv2 != fv2a) << endl;
   cout << "fv2 == dv2 -> " << (fv2 == dv2a) << endl;
   cout << "fv2 < fv2a -> " << (fv2 < fv2a) << endl;
   cout << "fv2 > fv2a -> " << (fv2 > fv2a) << endl;

   fv2b = fv2 + fv2a;
   cout << fv2 << " + " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b += fv2a;
   cout << fv2 << " += " << fv2a << " -> " << fv2b << endl;
   fv2b.add(fv2, fv2a);
   cout << fv2 << " add " << fv2a << " = " << fv2b << endl;

   fv2b = fv2 - fv2a;
   cout << fv2 << " - " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b -= fv2a;
   cout << fv2 << " -= " << fv2a << " -> " << fv2b << endl;
   fv2b.sub(fv2, fv2a);
   cout << fv2 << " sub " << fv2a << " = " << fv2b << endl;

   fv2b = fv2*2.0;
   cout << fv2 << " * 2.0 = " << fv2b << endl;
   fv2b = fv2;
   fv2b *= 2.0;
   cout << fv2 << " *= 2.0 -> " << fv2b << endl;
   fv2b.scale(fv2, 2.0);
   cout << fv2 << " scale by 2.0 = " << fv2b << endl;

   cout << fv2 << " - " << fv2a << " = ";
   fv2.sub(fv2, fv2a);
   cout << fv2 << endl;
   cout << fv2 << " - " << fv2 << " = ";
   fv2.sub(fv2, fv2);
   cout << fv2 << endl;

   fv2 = fv2a;
   cout << fv2 << " dot " << fv2 << " = " << fv2.dot(fv2) << endl;
   cout << fv2 << " dot " << fv2a << " = " << fv2.dot(fv2a) << endl;
   cout << "length" << fv2 << " = " << fv2.length() << endl;
   cout << "distance " << fv2 << "," << fv2a+fv2
        << " = " << fv2.distance(fv2a+fv2) << endl;
   cout << "distance " << fv2 << "," << fv2a
        << " = " << fv2.distance(fv2a) << endl;

   cout << "normalize(" << fv2 << ") = ";
   cout << fv2.normalize() << ", " << fv2 << endl;
}

//----------------------------------------------------------------------------
void testVec4d()
{
   // DP tests
   cout << "**** sglVec4<double> ****" << endl;
   sglVec4d  fv2;
   cout << "def ctor: " << fv2 << endl;
   sglVec4d fv2a(1.2, 2.3, 4.5, 6.7);
   cout << "ctor(f,f,f): " << fv2a << endl;

   double w = 6.7, z = -1.2, y=4.5, x = -3.4;
   sglVec4d fv2b(x,y,z,w);
   cout << "ctor(d,d,d): " << fv2b << endl;

   double v[4] = {-5.4, 3.2123123123, 1.11111888, 2.1};
   sglVec4d fv2c(v);
   cout << "ctor(d v[4]): " << fv2c << endl;
   
   int v1[4] = {1,5,9,3};
   sglVec4d fv2d(v1);
   cout << "ctor(i v[4]): " << fv2d << endl;
   
   unsigned int v2[4] = {1,5,8,2};
   sglVec4d fv2e(v2);
   cout << "ctor(ui v[4]): " << fv2e << endl;

   fv2.set((int) 1, (int)-3, (int)5, 7);
   cout << "set(i,i,i): " << fv2 << endl;
   fv2.set((unsigned int) 1, (unsigned int)-3, (unsigned int)5, (unsigned int)7);
   cout << "set(ui,ui,ui): " << fv2 << endl;
   fv2.set(-3.0, 1.14159, 7.8888, -2.3333);
   cout << "set(f,f,f): " << fv2 << endl;
   fv2.set(x,y,z,w);
   cout << "set(d,d,d): " << fv2 << endl;

   sglVec4d dv2a(3.141888888, -7.8777777777, 1.111119999, 1.0);
   sglVec4d fv2f = dv2a;
   cout << "v2f = v2d: " << fv2f << endl;
   sglVec4d fv2g = fv2f;
   cout << "v2f = v2f: " << fv2g << endl;
   
   fv2 = fv2g;
   cout << "v2f = v2f: " << fv2 << endl;
   fv2 = fv2;
   cout << "self assignment: " << fv2 << endl;

   unsigned int ix=0, iy=1;
   cout << "indexing: " << fv2[ix] << ' ' << fv2[iy] << endl;
   float fx=0.6, fy=1;
   cout << "indexing: " << fv2[fx] << ' ' << fv2[fy] << endl;

   cout << "fv2 == fv2 -> " << (fv2 == fv2) << endl;
   cout << "fv2 == fv2a -> " << (fv2 == fv2a) << endl;
   cout << "fv2 != fv2a -> " << (fv2 != fv2a) << endl;
   cout << "fv2 == dv2 -> " << (fv2 == dv2a) << endl;
   cout << "fv2 < fv2a -> " << (fv2 < fv2a) << endl;
   cout << "fv2 > fv2a -> " << (fv2 > fv2a) << endl;

   fv2b = fv2 + fv2a;
   cout << fv2 << " + " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b += fv2a;
   cout << fv2 << " += " << fv2a << " -> " << fv2b << endl;
   fv2b.add(fv2, fv2a);
   cout << fv2 << " add " << fv2a << " = " << fv2b << endl;

   fv2b = fv2 - fv2a;
   cout << fv2 << " - " << fv2a << " = " << fv2b << endl;
   fv2b = fv2;
   fv2b -= fv2a;
   cout << fv2 << " -= " << fv2a << " -> " << fv2b << endl;
   fv2b.sub(fv2, fv2a);
   cout << fv2 << " sub " << fv2a << " = " << fv2b << endl;

   fv2b = fv2*2.0;
   cout << fv2 << " * 2.0 = " << fv2b << endl;
   fv2b = fv2;
   fv2b *= 2.0;
   cout << fv2 << " *= 2.0 -> " << fv2b << endl;
   fv2b.scale(fv2, 2.0);
   cout << fv2 << " scale by 2.0 = " << fv2b << endl;

   cout << fv2 << " - " << fv2a << " = ";
   fv2.sub(fv2, fv2a);
   cout << fv2 << endl;
   cout << fv2 << " - " << fv2 << " = ";
   fv2.sub(fv2, fv2);
   cout << fv2 << endl;

   fv2 = fv2a;
   cout << fv2 << " dot " << fv2 << " = " << fv2.dot(fv2) << endl;
   cout << fv2 << " dot " << fv2a << " = " << fv2.dot(fv2a) << endl;
   cout << "length" << fv2 << " = " << fv2.length() << endl;
   cout << "distance " << fv2 << "," << fv2a+fv2
        << " = " << fv2.distance(fv2a+fv2) << endl;
   cout << "distance " << fv2 << "," << fv2a
        << " = " << fv2.distance(fv2a) << endl;
   cout << "normalize(" << fv2 << ") = ";
   cout << fv2.normalize() << ", " << fv2 << endl;
}
