/*****************************************************************************
 * SGL: A Scene Graph Library
 *
 * Copyright (C) 1997-2001  Scott McMillan   All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************
 *     File: testVecMat.cpp
 *  Created: 1999
 *  Summary: test vector-matrix operations
 *****************************************************************************/

#include <sglVec2.hpp>
#include <sglVec3.hpp>
#include <sglVec4.hpp>
#include <sglVector.hpp>

#include <sglMat2.hpp>
#include <sglMat3.hpp>
#include <sglMat4.hpp>
#include <sglMatrix.hpp>

extern void testVec2f();
extern void testVec2d();
extern void testVec3f();
extern void testVec3d();
extern void testVec4f();
extern void testVec4d();

extern void testMat2f();
extern void testMat2d();
extern void testMat3f();
extern void testMat3d();
extern void testMat4f();
extern void testMat4d();

//----------------------------------------------------------------------------
void testVecMat2()
{
   sglMat2f mf;
   sglMat2d md;
   sglVec2f vfa, vfb;
   sglVec2d vda, vdb;

   mf.buildRotation(45.0*M_PI/180.0);
   md.buildRotation(45.0*M_PI/180.0);
   vfa.set(1.0, 1.0);
   vda.set(1.0, 1.0);

   cout << "******************* testVecMat2 **********************" << endl;

   // **** operator*= ****

   // float v, float m
   vfb = vfa;
   mul(vfb, mf, vfb);
   cout << "vf = vf*mf: " << vfb << endl;

   // double v, float m
   vdb = vda;
   mul(vdb, mf, vdb);
   cout << "vd = vd*mf: " << vdb << endl;

   // float v, double m
   vfb = vfa;
   mul(vfb, md, vfb);
   cout << "vf = vf*md: " << vfb << endl;

   // double v, double m
   vdb = vda;
   mul(vdb, md, vdb);
   cout << "vd = vd*md: " << vdb << endl;

   // **** mul ****

   // float v, float m
   mul(vfa, mf, vfb);
   cout << "vf = mul(vf, mf): " << vfb << endl;

   // double v, float m
   mul(vda, mf, vdb);
   cout << "vd = mul(vd, mf): " << vdb << endl;

   // float v, double m
   mul(vfa, md, vfb);
   cout << "vf = mul(vf, md): " << vfb << endl;

   // double v, double m
   mul(vda, md, vdb);
   cout << "vd = mul(vd, md): " << vdb << endl;

   // **** mulTranspose ****

   // float v, float m
   mulTranspose(vfa, mf, vfb);
   cout << "vf = mulTranspose(vf, mf): " << vfb << endl;

   // double v, float m
   mulTranspose(vda, mf, vdb);
   cout << "vd = mulTranspose(vd, mf): " << vdb << endl;

   // float v, double m
   mulTranspose(vfa, md, vfb);
   cout << "vf = mulTranspose(vf, md): " << vfb << endl;

   // double v, double m
   mulTranspose(vda, md, vdb);
   cout << "vd = mulTranspose(vd, md): " << vdb << endl;
}

//----------------------------------------------------------------------------
void testVecMat3()
{
   sglMat3f mf;
   sglMat3d md;
   sglVec3f vfa, vfb;
   sglVec3d vda, vdb;

   mf.buildRotation(0.0, 0.0, 1.0, 45.0*M_PI/180.0);
   md.buildRotation(0.0, 0.0, 1.0, 45.0*M_PI/180.0);
   vfa.set(1.0, 1.0, 1.0);
   vda.set(1.0, 1.0, 1.0);

   cout << "******************* testVecMat3 **********************" << endl;

   // **** operator*= ****

   // float v, float m
   vfb = vfa;
   mul(vfb, mf, vfb);
   cout << "vf *= mf: " << vfb << endl;

   // double v, float m
   vdb = vda;
   mul(vdb, mf, vdb);
   cout << "vd *= mf: " << vdb << endl;

   // float v, double m
   vfb = vfa;
   mul(vfb, md, vfb);
   cout << "vf *= md: " << vfb << endl;

   // double v, double m
   vdb = vda;
   mul(vdb, md, vdb);
   cout << "vd *= md: " << vdb << endl;

   // **** mul ****

   // float v, float m
   mul(vfa, mf, vfb);
   cout << "vf = mul(vf, mf): " << vfb << endl;

   // double v, float m
   mul(vda, mf, vdb);
   cout << "vd = mul(vd, mf): " << vdb << endl;

   // float v, double m
   mul(vfa, md, vfb);
   cout << "vf = mul(vf, md): " << vfb << endl;

   // double v, double m
   mul(vda, md, vdb);
   cout << "vd = mul(vd, md): " << vdb << endl;

   // **** mul(v2,z) ****
   sglVec2f vfa2(1.0, 1.0);
   sglVec2d vda2(1.0, 1.0);

   // float v, float m
   mul(vfa2, 1.0f, mf, vfb);
   cout << "vf = mul(vf2, 1, mf): " << vfb << endl;

   // double v, float m
   mul(vda2, 1.0, mf, vdb);
   cout << "vd = mul(vd2, 1, mf): " << vdb << endl;

   // float v, double m
   mul(vfa2, 1.0f, md, vfb);
   cout << "vf = mul(vf2, 1, md): " << vfb << endl;

   // double v, double m
   mul(vda2, 1.0, md, vdb);
   cout << "vd = mul(vd2, 1, md): " << vdb << endl;

   // **** mulTranspose ****

   // float v, float m
   mulTranspose(vfa, mf, vfb);
   cout << "vf = mulTranspose(vf, mf): " << vfb << endl;

   // double v, float m
   mulTranspose(vda, mf, vdb);
   cout << "vd = mulTranspose(vd, mf): " << vdb << endl;

   // float v, double m
   mulTranspose(vfa, md, vfb);
   cout << "vf = mulTranspose(vf, md): " << vfb << endl;

   // double v, double m
   mulTranspose(vda, md, vdb);
   cout << "vd = mulTranspose(vd, md): " << vdb << endl;

   // **** mulTranspose(v2,z) ****

   // float v, float m
   mulTranspose(vfa2, 1.0f, mf, vfb);
   cout << "vf = mul(vf2, 1, mf): " << vfb << endl;

   // double v, float m
   mulTranspose(vda2, 1.0, mf, vdb);
   cout << "vd = mul(vd2, 1, mf): " << vdb << endl;

   // float v, double m
   mulTranspose(vfa2, 1.0f, md, vfb);
   cout << "vf = mul(vf2, 1, md): " << vfb << endl;

   // double v, double m
   mulTranspose(vda2, 1.0, md, vdb);
   cout << "vd = mul(vd2, 1, md): " << vdb << endl;
}

//----------------------------------------------------------------------------
void testVecMat4()
{
   sglMat4f mf;
   sglMat4d md;
   sglVec4f vfa, vfb;
   sglVec4d vda, vdb;

   mf.buildRotation(0.0, 0.0, 1.0, 45.0*M_PI/180.0);
   md.buildRotation(0.0, 0.0, 1.0, 45.0*M_PI/180.0);
   vfa.set(1.0, 1.0, 1.0, 1.0);
   vda.set(1.0, 1.0, 1.0, 1.0);

   cout << "******************* testVecMat4 **********************" << endl;

   // **** operator*= ****

   // float v, float m
   vfb = vfa;
   mul(vfb, mf, vfb);
   cout << "vf *= mf: " << vfb << endl;

   // double v, float m
   vdb = vda;
   mul(vdb, mf, vdb);
   cout << "vd *= mf: " << vdb << endl;

   // float v, double m
   vfb = vfa;
   mul(vfb, md, vfb);
   cout << "vf *= md: " << vfb << endl;

   // double v, double m
   vdb = vda;
   mul(vdb, md, vdb);
   cout << "vd *= md: " << vdb << endl;

   // **** mul ****

   // float v, float m
   mul(vfa, mf, vfb);
   cout << "vf = mul(vf, mf): " << vfb << endl;

   // double v, float m
   mul(vda, mf, vdb);
   cout << "vd = mul(vd, mf): " << vdb << endl;

   // float v, double m
   mul(vfa, md, vfb);
   cout << "vf = mul(vf, md): " << vfb << endl;

   // double v, double m
   mul(vda, md, vdb);
   cout << "vd = mul(vd, md): " << vdb << endl;

   // **** mul(v2,z) ****
   sglVec3f vfa2(1.0, 1.0, 1.0);
   sglVec3d vda2(1.0, 1.0, 1.0);

   // float v, float m
   mul(vfa2, 1.0f, mf, vfb);
   cout << "vf = mul(vf3, 1, mf): " << vfb << endl;

   // double v, float m
   mul(vda2, 1.0, mf, vdb);
   cout << "vd = mul(vd3, 1, mf): " << vdb << endl;

   // float v, double m
   mul(vfa2, 1.0f, md, vfb);
   cout << "vf = mul(vf3, 1, md): " << vfb << endl;

   // double v, double m
   mul(vda2, 1.0, md, vdb);
   cout << "vd = mul(vd3, 1, md): " << vdb << endl;

   // **** mulTranspose ****

   // float v, float m
   mulTranspose(vfa, mf, vfb);
   cout << "vf = mulTranspose(vf, mf): " << vfb << endl;

   // double v, float m
   mulTranspose(vda, mf, vdb);
   cout << "vd = mulTranspose(vd, mf): " << vdb << endl;

   // float v, double m
   mulTranspose(vfa, md, vfb);
   cout << "vf = mulTranspose(vf, md): " << vfb << endl;

   // double v, double m
   mulTranspose(vda, md, vdb);
   cout << "vd = mulTranspose(vd, md): " << vdb << endl;

   // **** mulTranspose(v3,z) ****

   // float v, float m
   mulTranspose(vfa2, 1.0f, mf, vfb);
   cout << "vf = mul(vf3, 1, mf): " << vfb << endl;

   // double v, float m
   mulTranspose(vda2, 1.0, mf, vdb);
   cout << "vd = mul(vd3, 1, mf): " << vdb << endl;

   // float v, double m
   mulTranspose(vfa2, 1.0f, md, vfb);
   cout << "vf = mul(vf3, 1, md): " << vfb << endl;

   // double v, double m
   mulTranspose(vda2, 1.0, md, vdb);
   cout << "vd = mul(vd3, 1, md): " << vdb << endl;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
int main()
{
   //testVec2f();
   //testVec2d();
   //testVec3f();
   //testVec3d();
   //testVec4f();
   //testVec4d();

   //testMat2f();
   //testMat2d();
   //testMat3f();
   //testMat3d();
   //testMat4f();
   //testMat4d();

   //testVecMat2();
   //testVecMat3();
   testVecMat4();

   return 0;
}
